@@ -1,6 +1,6 @@
 /****************************************************************
  *								*
- * Copyright (c) 2001-2019 Fidelity National Information	*
+ * Copyright (c) 2001-2022 Fidelity National Information	*
  * Services, Inc. and/or its subsidiaries. All rights reserved.	*
  *								*
  *	This source code contains the intellectual property	*
@@ -51,12 +51,15 @@
 #include "bmm_find_free.h"
 
 /* Include prototypes */
+#include "wcs_mm_recover.h"
 #include "t_qread.h"
 #include "t_write_map.h"
 #include "bit_clear.h"
 #include "send_msg.h"
 #include "bm_getfree.h"
 #include "gdsfilext.h"
+#include "anticipatory_freeze.h"
+#include "interlock.h"
 
 GBLREF sgmnt_addrs	*cs_addrs;
 GBLREF sgmnt_data_ptr_t	cs_data;
@@ -66,26 +69,31 @@ GBLREF unsigned char	rdfail_detail;
 GBLREF uint4		dollar_tlevel;
 GBLREF uint4		update_array_size, cumul_update_array_size;
 GBLREF unsigned int	t_tries;
+GBLREF jnlpool_addrs_ptr_t	jnlpool;
 
 error_def(ERR_DBBADFREEBLKCTR);
 error_def(ERR_DBMBMINCFREFIXED);
 
-block_id bm_getfree(block_id hint, boolean_t *blk_used, unsigned int cw_work, cw_set_element *cs, int *cw_depth_ptr)
+block_id bm_getfree(block_id hint_arg, boolean_t *blk_used, unsigned int cw_work, cw_set_element *cs, int *cw_depth_ptr)
 {
 	cw_set_element	*cs1;
 	sm_uc_ptr_t	bmp;
-	block_id	bml, hint_cycled, hint_limit, total_blks, lcnt, local_maps, offset;
+	block_id	bml, extension_size, hint, hint_cycled, hint_limit, lcnt, local_maps, offset;
 	block_id_ptr_t	b_ptr;
+	gd_region	*baseDBreg;
 	int		cw_set_top, depth;
 	unsigned int	n_decrements = 0;
 	trans_num	ctn;
-	int4		free_bit, map_size;
-	uint4		space_needed, status;
+	int4		free_bit, map_size, new_allocation, status;
+	ublock_id	total_blks;
+	uint4		space_needed, was_crit;
+	sgmnt_addrs	*baseDBcsa;
 	srch_blk_status	blkhist;
+	DCL_THREADGBL_ACCESS;
 
+	SETUP_THREADGBL_ACCESS;
 	total_blks = (dba_mm == cs_data->acc_meth) ? cs_addrs->total_blks : cs_addrs->ti->total_blks;
-	if (hint >= total_blks)		/* for TP, hint can be > total_blks */
-		hint = 1;
+	hint = (((ublock_id)hint_arg >= total_blks) ? 1 : hint_arg);	/* avoid any chance of treating TP chain.flag as a sign */
 	hint_cycled = DIVIDE_ROUND_UP(total_blks, BLKS_PER_LMAP);
 	hint_limit = DIVIDE_ROUND_DOWN(hint, BLKS_PER_LMAP);
 	local_maps = hint_cycled + 2;	/* for (up to) 2 wraps */
@@ -100,8 +108,89 @@ block_id bm_getfree(block_id hint, boolean_t *blk_used, unsigned int cw_work, cw
 				hint = 1;
 				continue;
 			}
-			if (SS_NORMAL != (status = GDSFILEXT(cs_data->extension_size, total_blks, TRANS_IN_PROG_TRUE)))
-				return (status);
+			was_crit = cs_addrs->now_crit;
+			if (!was_crit)
+			{	/* We are working up to a file extension, which requires crit anyway, we need a consistent
+				 * view of our last minute checks, and in case we are updating statsdb_allocation we want
+				 * that to be atomic with the extension itself. Strictly speaking, statsdb_allocation isn't
+				 * associated with the current (statsdb) region, but barring a concurrent MUPIP SET this should
+				 * be sufficient.
+				 */
+#				ifdef DEBUG
+				if ((WBTEST_ENABLED(WBTEST_MM_CONCURRENT_FILE_EXTEND) && dollar_tlevel
+						&& !MEMCMP_LIT(gv_cur_region->rname, "DEFAULT"))
+					|| (WBTEST_ENABLED(WBTEST_WSSTATS_PAUSE) && (10 == gtm_white_box_test_case_count)
+						&& !MEMCMP_LIT(gv_cur_region->rname, "DEFAULT")))
+				{	/* Sync with copy in gdsfilext()
+					 * Unset the env shouldn't affect the parent, it reads env just once at process startup.
+					 */
+					if(WBTEST_ENABLED(WBTEST_WSSTATS_PAUSE))
+						unsetenv("gtm_white_box_test_case_enable");
+					SYSTEM("$gtm_dist/mumps -run $gtm_wbox_mrtn");
+					assert(1 == cs_addrs->nl->wbox_test_seq_num);	/* should have been set by mubfilcpy */
+					/* signal mupip backup to stop sleeping in mubfilcpy */
+					cs_addrs->nl->wbox_test_seq_num = 2;
+				}
+#				endif
+				while (!cs_addrs->now_crit)
+				{
+					grab_crit(gv_cur_region, WS_12);
+					if (FROZEN_CHILLED(cs_addrs))
+						DO_CHILLED_AUTORELEASE(cs_addrs, cs_data);
+					assert(FROZEN(cs_data) || !cs_addrs->jnlpool || (cs_addrs->jnlpool == jnlpool));
+					if (FROZEN(cs_data) || IS_REPL_INST_FROZEN)
+					{
+						rel_crit(gv_cur_region);
+						while (FROZEN(cs_data) || IS_REPL_INST_FROZEN)
+						{
+							hiber_start(1000);
+							if (FROZEN_CHILLED(cs_addrs) && CHILLED_AUTORELEASE(cs_addrs))
+								break;
+						}
+					}
+				}
+			}
+			if (total_blks != cs_addrs->ti->total_blks)
+			{	/* File extension or MM switch detected. Rather than try to reset the loop, just recurse. */
+				CHECK_MM_DBFILEXT_REMAP_IF_NEEDED(cs_addrs, gv_cur_region);
+				rel_crit(gv_cur_region);
+				return (dba_mm == cs_data->acc_meth)
+						? FILE_EXTENDED
+						: bm_getfree(hint_arg, blk_used, cw_work, cs, cw_depth_ptr);
+			}
+			if (IS_STATSDB_CSA(cs_addrs))
+			{       /* Double the allocation size for statsdb regions to reduce tp_restart calls */
+				assert(cs_addrs->total_blks == cs_addrs->ti->total_blks);
+				extension_size = cs_addrs->total_blks;
+			} else
+				extension_size = cs_data->extension_size;
+#			ifdef DEBUG
+			TREF(in_bm_getfree_gdsfilext) = TRUE;
+#			endif
+			status = GDSFILEXT(extension_size, total_blks, TRANS_IN_PROG_TRUE);
+#			ifdef DEBUG
+			TREF(in_bm_getfree_gdsfilext) = FALSE;
+#			endif
+			if (SS_NORMAL != status)
+			{
+				if (!was_crit)
+					rel_crit(gv_cur_region);
+				if (EXTEND_SUSPECT == status)
+					continue;
+				return status;
+			}
+			if (IS_STATSDB_CSA(cs_addrs))
+			{	/* Save the new allocation size to the database file header.
+				 * Use this allocation size the next time we create statsdb for the region.
+				 */
+				new_allocation = (int4)(total_blks * 2);
+				STATSDBREG_TO_BASEDBREG(gv_cur_region, baseDBreg);
+				baseDBcsa = &FILE_INFO(baseDBreg)->s_addrs;
+				assert(new_allocation > baseDBcsa->hdr->statsdb_allocation);
+				baseDBcsa->hdr->statsdb_allocation = new_allocation;
+			}
+			if (!was_crit)
+				rel_crit(gv_cur_region);
 			if (dba_mm == cs_data->acc_meth)
 				return (FILE_EXTENDED);
 			hint = total_blks;
@@ -154,7 +243,7 @@ block_id bm_getfree(block_id hint, boolean_t *blk_used, unsigned int cw_work, cw
 			}
 		} else
 		{
-			for (depth = *cw_depth_ptr - 1; depth >= cw_work;  depth--)
+			for (depth = *cw_depth_ptr - 1; depth >= cw_work; depth--)
 			{	/* do non-tp back to front, because of adjacency */
 				if (bml == (cs + depth)->blk)
 				{
@@ -199,7 +288,7 @@ block_id bm_getfree(block_id hint, boolean_t *blk_used, unsigned int cw_work, cw
 			free_bit = NO_FREE_SPACE;
 		if (NO_FREE_SPACE != free_bit)
 			break;
-		if ((hint = bml + BLKS_PER_LMAP) >= total_blks)		/* if map is full, start at 1st blk in next map */
+		if ((hint = (bml + BLKS_PER_LMAP)) >= total_blks)	/* if map is full, start at 1st blk in next map */
 		{	/* wrap - second one should force an extend for sure */
 			hint = 1;
 			if (hint_cycled)
@@ -235,12 +324,12 @@ block_id bm_getfree(block_id hint, boolean_t *blk_used, unsigned int cw_work, cw
 		}
 		BLK_ADDR(b_ptr, space_needed, block_id);
 		memset(b_ptr, 0, space_needed);
-		*b_ptr = free_bit;
+		*b_ptr = (block_id)free_bit;
 		blkhist.blk_num = bml;
 		blkhist.buffaddr = bmp;	/* cycle and cr have already been assigned from t_qread */
 		t_write_map(&blkhist, (uchar_ptr_t)b_ptr, ctn, 1); /* last parameter 1 is what cs->reference_cnt gets set to */
 	}
-	return bml + free_bit;
+	return (bml + free_bit);
 }
 
 /* This routine returns whether the free_blocks counter in the file-header is ok (TRUE) or not (FALSE).
@@ -294,7 +383,7 @@ boolean_t	is_free_blks_ctr_ok(void)
 	if (cs_addrs->ti->free_blocks != free_blocks)
 	{
 		send_msg_csa(CSA_ARG(cs_addrs) VARLSTCNT(6) ERR_DBBADFREEBLKCTR, 4, DB_LEN_STR(gv_cur_region),
-				cs_addrs->ti->free_blocks, free_blocks);
+				&(cs_addrs->ti->free_blocks), &free_blocks);
 		cs_addrs->ti->free_blocks = free_blocks;
 		return FALSE;
 	}
