@@ -1,6 +1,6 @@
 /****************************************************************
  *								*
- * Copyright (c) 2001-2020 Fidelity National Information	*
+ * Copyright (c) 2001-2022 Fidelity National Information	*
  * Services, Inc. and/or its subsidiaries. All rights reserved.	*
  *								*
  *	This source code contains the intellectual property	*
@@ -34,6 +34,7 @@
 #include "rtn_src_chksum.h"
 #include "gtmmsg.h"
 #include "iosp.h"	/* for SS_NORMAL */
+#include "start_fetches.h"
 
 #define HOPELESS_COMPILE 128
 
@@ -44,12 +45,11 @@ GBLREF char			cg_phase;	/* code generation phase */
 GBLREF command_qualifier	cmd_qlf;
 GBLREF hash_table_str		*complits_hashtab;
 GBLREF int			mlmax;
-GBLREF int4			curr_fetch_count;
 GBLREF mcalloc_hdr 		*mcavailptr, *mcavailbase;
 GBLREF mline			mline_root;
 GBLREF spdesc			indr_stringpool, rts_stringpool, stringpool;
 GBLREF src_line_struct 	src_head;
-GBLREF triple			t_orig, *curr_fetch_trip, *curr_fetch_opr;
+GBLREF triple			t_orig;
 GBLREF unsigned char		source_file_name[];
 GBLREF unsigned short		source_name_len;
 
@@ -76,6 +76,7 @@ boolean_t compiler_startup(void)
 	gtm_rtn_src_chksum_ctx	checksum_ctx;
 	mstr			str;
 	size_t			mcallocated, alloc;
+	size_t			stlen;
 	mcalloc_hdr		*lastmca, *nextmca;
 	DCL_THREADGBL_ACCESS;
 
@@ -124,15 +125,7 @@ boolean_t compiler_startup(void)
 	mstr_native_align = FALSE; /* TODO: remove this line and  uncomment the above line */
 	cg_phase = CGP_NOSTATE;
 	TREF(source_error_found) = errknt = 0;
-	if (!open_source_file())
-	{
-		mstr_native_align = save_mstr_native_align;
-		REVERT;
-		assert(indr_stringpool.base == stringpool.base);
-		indr_stringpool = stringpool;
-		stringpool = rts_stringpool;
-		return FALSE;
-	}
+	open_source_file();
 	rtn_src_chksum_init(&checksum_ctx);
 	cg_phase = CGP_PARSE;
 	creating_list_file = (cmd_qlf.qlf & CQ_LIST) || (cmd_qlf.qlf & CQ_CROSS_REFERENCE);
@@ -147,8 +140,8 @@ boolean_t compiler_startup(void)
 	null_lab = get_mladdr(&null_mident);
 	null_lab->ml = &mline_root;
 	mlmax++;
-	curr_fetch_trip = curr_fetch_opr = newtriple(OC_LINEFETCH);
-	curr_fetch_count = 0;
+	(TREF(fetch_control)).curr_fetch_trip = (TREF(fetch_control)).curr_fetch_opr = newtriple(OC_LINEFETCH);
+	(TREF(fetch_control)).curr_fetch_count = 0;
 	TREF(code_generated) = FALSE;
 	line_count = 1;
 	total_source_len = 0;
@@ -161,10 +154,12 @@ boolean_t compiler_startup(void)
 		/* Accumulate list of M source lines */
 		sl = (src_line_struct *)mcalloc(SIZEOF(src_line_struct));
 		dqrins(&src_head, que, sl);
-		sl->str.addr = mcalloc(n + 1);		/* +1 for zero termination */
+		stlen = n + 1;
+		sl->str.addr = mcalloc(stlen);		/* == n+1 for zero termination */
 		sl->str.len = n;
 		sl->line = TREF(source_line);
-		memcpy(sl->str.addr, (TREF(source_buffer)).addr, n + 1);
+		assert(NULL != sl->str.addr);
+		memcpy(sl->str.addr, (TREF(source_buffer)).addr, stlen);
 		total_source_len += n;
 		cp = (unsigned char *)((TREF(source_buffer)).addr + n - 1);
 		NEWLINE_TO_NULL(*cp); /* avoid SPOREOL errors due to trailing newlines */
