@@ -1,6 +1,6 @@
 /****************************************************************
  *								*
- * Copyright (c) 2001-2019 Fidelity National Information	*
+ * Copyright (c) 2001-2021 Fidelity National Information	*
  * Services, Inc. and/or its subsidiaries. All rights reserved.	*
  *								*
  *	This source code contains the intellectual property	*
@@ -75,7 +75,7 @@ void dse_f_blk(void)
 	do
 	{
 		if (!(bp = t_qread(look, &dummy_int, &dummy_cr)))
-			rts_error_csa(CSA_ARG(NULL) VARLSTCNT(1) ERR_DSEBLKRDFAIL);
+			RTS_ERROR_CSA_ABT(NULL, VARLSTCNT(1) ERR_DSEBLKRDFAIL);
 		if (((blk_hdr_ptr_t)bp)->bver > BLK_ID_32_VER) /* Check blk version to see if using 32 or 64 bit block_id */
 		{
 #			ifdef BLK_NUM_64BIT
@@ -84,7 +84,7 @@ void dse_f_blk(void)
 #			else
 			DSE_REL_CRIT_AS_APPROPRIATE(was_crit, was_hold_onto_crit, nocrit_present, cs_addrs, gv_cur_region);
 			REVERT;
-			rts_error_csa(CSA_ARG(cs_addrs) VARLSTCNT(1) ERR_DSEINVALBLKID);
+			RTS_ERROR_CSA_ABT(cs_addrs, VARLSTCNT(1) ERR_DSEINVALBLKID);
 #			endif
 		} else
 		{
@@ -126,7 +126,7 @@ void dse_f_blk(void)
 #			else
 			DSE_REL_CRIT_AS_APPROPRIATE(was_crit, was_hold_onto_crit, nocrit_present, cs_addrs, gv_cur_region);
 			REVERT;
-			rts_error_csa(CSA_ARG(cs_addrs) VARLSTCNT(1) ERR_DSEINVALBLKID);
+			RTS_ERROR_CSA_ABT(cs_addrs, VARLSTCNT(1) ERR_DSEINVALBLKID);
 #			endif
 		} else
 			GET_BLK_ID_32(look, blk_id);
@@ -141,8 +141,8 @@ void dse_f_blk(void)
 		{
 			if (patch_find_sibs)
 			{
-				util_out_print("!/!_Left sibling!_Current block!_Right sibling", TRUE);
-				util_out_print("!_none!_!_0x!XL!_none",TRUE, patch_find_blk);
+				util_out_print("!/!_Left sibling!_!_Current block!_!_Right sibling", TRUE);
+				util_out_print("!_none!_!_!_0x!16@XQ!_none",TRUE, &patch_find_blk);
 			} else
 			{
 				assert(1 == patch_path[0]);	/* OK to assert because pro prints */
@@ -165,8 +165,8 @@ void dse_f_blk(void)
 				{
 					if (patch_find_sibs)
 					{
-						util_out_print("!/!_Left sibling!_Current block!_Right sibling", TRUE);
-						util_out_print("!_none!_!_0x!XL!_none",TRUE, patch_find_blk);
+						util_out_print("!/!_Left sibling!_!_Current block!_!_Right sibling", TRUE);
+						util_out_print("!_none!_!_!_0x!16@XQ!_none",TRUE, &patch_find_blk);
 					} else
 					{
 						patch_path_count--;
@@ -267,14 +267,14 @@ void dse_f_blk(void)
 	}
 	if (patch_find_sibs)
 	{	/* the cross-branch sib action could logically go in dse_order but is here 'cause it only gets used when needed */
-		util_out_print("!/!_Left sibling!_Current block!_Right sibling", TRUE);
+		util_out_print("!/!_Left sibling!_!_Current block!_!_Right sibling", TRUE);
 		if (!patch_left_sib)
 		{
 			for (last = 0, lvl = (patch_find_root_search ? patch_dir_path_count : patch_path_count) - 1; 0 <= --lvl;)
 			{
 				if (!(sp = t_qread(patch_find_root_search ? patch_path[lvl] : patch_path1[lvl], &dummy_int,
 					&dummy_cr)))
-						rts_error_csa(CSA_ARG(NULL) VARLSTCNT(1) ERR_DSEBLKRDFAIL);
+						RTS_ERROR_CSA_ABT(NULL, VARLSTCNT(1) ERR_DSEBLKRDFAIL);
 				if (((blk_hdr_ptr_t)sp)->bver > BLK_ID_32_VER) /* Check to see if using 32 or 64 bit block_id */
 				{
 #					ifdef BLK_NUM_64BIT
@@ -284,7 +284,7 @@ void dse_f_blk(void)
 					DSE_REL_CRIT_AS_APPROPRIATE(was_crit, was_hold_onto_crit,
 							nocrit_present, cs_addrs, gv_cur_region);
 					REVERT;
-					rts_error_csa(CSA_ARG(cs_addrs) VARLSTCNT(1) ERR_DSEINVALBLKID);
+					RTS_ERROR_CSA_ABT(cs_addrs, VARLSTCNT(1) ERR_DSEINVALBLKID);
 #					endif
 				} else
 				{
@@ -295,8 +295,8 @@ void dse_f_blk(void)
 					s_top = sp + cs_addrs->hdr->blk_size;
 				else if (SIZEOF(blk_hdr) > ((blk_hdr_ptr_t)sp)->bsiz)
 				{
-					util_out_print("Error: sibling search hit problem blk 0x!XL",
-						TRUE, patch_find_root_search ? patch_path[lvl] : patch_path1[lvl]);
+					util_out_print("Error: sibling search hit problem blk 0x!16@XQ",
+						TRUE, patch_find_root_search ? &(patch_path[lvl]) : &(patch_path1[lvl]));
 					lvl = -1;
 					break;
 				} else
@@ -312,7 +312,7 @@ void dse_f_blk(void)
 					DSE_REL_CRIT_AS_APPROPRIATE(was_crit, was_hold_onto_crit,
 							nocrit_present, cs_addrs, gv_cur_region);
 					REVERT;
-					rts_error_csa(CSA_ARG(cs_addrs) VARLSTCNT(1) ERR_DSEINVALBLKID);
+					RTS_ERROR_CSA_ABT(cs_addrs, VARLSTCNT(1) ERR_DSEINVALBLKID);
 #					endif
 				} else
 					GET_BLK_ID_32(look, srp - SIZEOF(block_id_32));
@@ -337,28 +337,28 @@ void dse_f_blk(void)
 						DSE_REL_CRIT_AS_APPROPRIATE(was_crit, was_hold_onto_crit,
 								nocrit_present, cs_addrs, gv_cur_region);
 						REVERT;
-						rts_error_csa(CSA_ARG(cs_addrs) VARLSTCNT(1) ERR_DSEINVALBLKID);
+						RTS_ERROR_CSA_ABT(cs_addrs, VARLSTCNT(1) ERR_DSEINVALBLKID);
 #						endif
 					} else
 						GET_BLK_ID_32(look, srp - SIZEOF(block_id_32));
 				}
 				if ((patch_find_root_search ? patch_path[lvl] : patch_path1[lvl]) != look)
 				{
-					util_out_print("Error: sibling search hit problem blk 0x!XL",
-						TRUE, patch_find_root_search ? patch_path[lvl] : patch_path1[lvl]);
+					util_out_print("Error: sibling search hit problem blk 0x!16@XQ",
+						TRUE, patch_find_root_search ? &(patch_path[lvl]) : &(patch_path1[lvl]));
 					last = 0;
 					lvl = (patch_find_root_search ? patch_dir_path_count : patch_path_count);
 				} else if (last >= cs_addrs->ti->total_blks)
 				{	/* should never come here as block was previously OK, but this is dse so be careful */
-					util_out_print("Error: sibling search got 0x!XL which exceeds total blocks 0x!XL",
-						       TRUE, last, cs_addrs->ti->total_blks);
+					util_out_print("Error: sibling search got 0x!16@XQ which exceeds total blocks 0x!16@XQ",
+						       TRUE, &last, &(cs_addrs->ti->total_blks));
 					last = 0;
 					lvl = (patch_find_root_search ? patch_dir_path_count : patch_path_count);
 				}
 				for (lvl++; lvl < (patch_find_root_search ? patch_dir_path_count : patch_path_count); lvl++)
 				{
 					if (!(sp = t_qread(last, &dummy_int, &dummy_cr)))
-						rts_error_csa(CSA_ARG(NULL) VARLSTCNT(1) ERR_DSEBLKRDFAIL);
+						RTS_ERROR_CSA_ABT(NULL, VARLSTCNT(1) ERR_DSEBLKRDFAIL);
 					if (((blk_hdr_ptr_t)sp)->bver > BLK_ID_32_VER) /* Check if using 32 or 64 bit block_id */
 					{
 #						ifdef BLK_NUM_64BIT
@@ -368,7 +368,7 @@ void dse_f_blk(void)
 						DSE_REL_CRIT_AS_APPROPRIATE(was_crit, was_hold_onto_crit,
 								nocrit_present, cs_addrs, gv_cur_region);
 						REVERT;
-						rts_error_csa(CSA_ARG(cs_addrs) VARLSTCNT(1) ERR_DSEINVALBLKID);
+						RTS_ERROR_CSA_ABT(cs_addrs, VARLSTCNT(1) ERR_DSEINVALBLKID);
 #						endif
 					} else
 					{
@@ -379,7 +379,7 @@ void dse_f_blk(void)
 						s_top = sp + cs_addrs->hdr->blk_size;
 					else if (SIZEOF(blk_hdr) > ((blk_hdr_ptr_t)sp)->bsiz)
 					{
-						util_out_print("Error: sibling search hit problem blk 0x!XL", TRUE, last);
+						util_out_print("Error: sibling search hit problem blk 0x!16@XQ", TRUE, &last);
 						last = 0;
 						break;
 					} else
@@ -398,14 +398,15 @@ void dse_f_blk(void)
 						DSE_REL_CRIT_AS_APPROPRIATE(was_crit, was_hold_onto_crit,
 								nocrit_present, cs_addrs, gv_cur_region);
 						REVERT;
-						rts_error_csa(CSA_ARG(cs_addrs) VARLSTCNT(1) ERR_DSEINVALBLKID);
+						RTS_ERROR_CSA_ABT(cs_addrs, VARLSTCNT(1) ERR_DSEINVALBLKID);
 #						endif
 					} else
 						GET_BLK_ID_32(last, s_top - SIZEOF(block_id_32));
 					if (last >= cs_addrs->ti->total_blks)
 					{
-						util_out_print("Error: sibling search got 0x!XL which exceeds total blocks 0x!XL",
-							TRUE, last, cs_addrs->ti->total_blks);
+						util_out_print(
+							"Error: sibling search got 0x!16@XQ which exceeds total blocks 0x!16@XQ",
+							TRUE, &last, &(cs_addrs->ti->total_blks));
 						break;
 					}
 				}
@@ -413,17 +414,17 @@ void dse_f_blk(void)
 			patch_left_sib = last;
 		}
 		if (patch_left_sib)
-			util_out_print("!_0x!XL", FALSE, patch_left_sib);
+			util_out_print("!_0x!16@XQ", FALSE, &patch_left_sib);
 		else
-			util_out_print("!_none!_", FALSE);
-		util_out_print("!_0x!XL!_", FALSE, patch_find_blk);
+			util_out_print("!_none!_!_", FALSE);
+		util_out_print("!_0x!16@XQ!_", FALSE, &patch_find_blk);
 		if (!patch_right_sib)
 		{
 			for (lvl = (patch_find_root_search ? patch_dir_path_count : patch_path_count) - 1; 0 <= --lvl;)
 			{
 				if (!(sp = t_qread(patch_find_root_search ? patch_path[lvl] : patch_path1[lvl], &dummy_int,
 					&dummy_cr)))
-					rts_error_csa(CSA_ARG(NULL) VARLSTCNT(1) ERR_DSEBLKRDFAIL);
+					RTS_ERROR_CSA_ABT(NULL, VARLSTCNT(1) ERR_DSEBLKRDFAIL);
 				if (((blk_hdr_ptr_t)sp)->bver > BLK_ID_32_VER) /* Check to see if using 32 or 64 bit block_id */
 				{
 #					ifdef BLK_NUM_64BIT
@@ -433,7 +434,7 @@ void dse_f_blk(void)
 					DSE_REL_CRIT_AS_APPROPRIATE(was_crit, was_hold_onto_crit,
 							nocrit_present, cs_addrs, gv_cur_region);
 					REVERT;
-					rts_error_csa(CSA_ARG(cs_addrs) VARLSTCNT(1) ERR_DSEINVALBLKID);
+					RTS_ERROR_CSA_ABT(cs_addrs, VARLSTCNT(1) ERR_DSEINVALBLKID);
 #					endif
 				} else
 				{
@@ -444,8 +445,8 @@ void dse_f_blk(void)
 					s_top = sp + cs_addrs->hdr->blk_size;
 				else if (SIZEOF(blk_hdr) > ((blk_hdr_ptr_t)sp)->bsiz)
 				{
-					util_out_print("Error: sibling search hit problem blk 0x!XL",
-						TRUE, patch_find_root_search ? patch_path[lvl] : patch_path1[lvl]);
+					util_out_print("Error: sibling search hit problem blk 0x!16@XQ",
+						TRUE, patch_find_root_search ? &(patch_path[lvl]) : &(patch_path1[lvl]));
 					lvl = -1;
 					break;
 				} else
@@ -458,14 +459,14 @@ void dse_f_blk(void)
 					DSE_REL_CRIT_AS_APPROPRIATE(was_crit, was_hold_onto_crit,
 							nocrit_present, cs_addrs, gv_cur_region);
 					REVERT;
-					rts_error_csa(CSA_ARG(cs_addrs) VARLSTCNT(1) ERR_DSEINVALBLKID);
+					RTS_ERROR_CSA_ABT(cs_addrs, VARLSTCNT(1) ERR_DSEINVALBLKID);
 #					endif
 				} else
 					GET_BLK_ID_32(look, s_top - SIZEOF(block_id_32));
 				if (look >= cs_addrs->ti->total_blks)
 				{
-					util_out_print("Error: sibling search got 0x!XL which exceeds total blocks 0x!XL",
-						       TRUE, look, cs_addrs->ti->total_blks);
+					util_out_print("Error: sibling search got 0x!16@XQ which exceeds total blocks 0x!16@XQ",
+						       TRUE, &look, &(cs_addrs->ti->total_blks));
 					lvl = -1;
 					break;
 				}
@@ -491,28 +492,29 @@ void dse_f_blk(void)
 						DSE_REL_CRIT_AS_APPROPRIATE(was_crit, was_hold_onto_crit,
 								nocrit_present, cs_addrs, gv_cur_region);
 						REVERT;
-						rts_error_csa(CSA_ARG(cs_addrs) VARLSTCNT(1) ERR_DSEINVALBLKID);
+						RTS_ERROR_CSA_ABT(cs_addrs, VARLSTCNT(1) ERR_DSEINVALBLKID);
 #						endif
 					} else
 						GET_BLK_ID_32(look, srp - SIZEOF(block_id_32));
 					if (look >= cs_addrs->ti->total_blks)
 					{
-						util_out_print("Error: sibling search got 0x!XL which exceeds total blocks 0x!XL",
-							       TRUE, look, cs_addrs->ti->total_blks);
+						util_out_print(
+							"Error: sibling search got 0x!16@XQ which exceeds total blocks 0x!16@XQ",
+							TRUE, &look, &(cs_addrs->ti->total_blks));
 						break;
 					}
 				}
 				if ((patch_find_root_search ? patch_path[lvl] : patch_path1[lvl]) != last)
 				{
-					util_out_print("Error: sibling search hit problem blk 0x!XL",
-						TRUE, patch_find_root_search ? patch_path[lvl] : patch_path1[lvl]);
+					util_out_print("Error: sibling search hit problem blk 0x!16@XQ",
+						TRUE, patch_find_root_search ? &(patch_path[lvl]) : &(patch_path1[lvl]));
 					look = 0;
 					lvl = (patch_find_root_search ? patch_dir_path_count : patch_path_count);
 				}
 				for (lvl++; lvl < (patch_find_root_search ? patch_dir_path_count : patch_path_count); lvl++)
 				{
 					if (!(sp = t_qread(look, &dummy_int, &dummy_cr)))	/* NOTE assignment */
-						rts_error_csa(CSA_ARG(NULL) VARLSTCNT(1) ERR_DSEBLKRDFAIL);
+						RTS_ERROR_CSA_ABT(NULL, VARLSTCNT(1) ERR_DSEBLKRDFAIL);
 					if (((blk_hdr_ptr_t)sp)->bver > BLK_ID_32_VER) /* Check if using 32 or 64 bit block_id */
 					{
 #						ifdef BLK_NUM_64BIT
@@ -522,7 +524,7 @@ void dse_f_blk(void)
 						DSE_REL_CRIT_AS_APPROPRIATE(was_crit, was_hold_onto_crit,
 								nocrit_present, cs_addrs, gv_cur_region);
 						REVERT;
-						rts_error_csa(CSA_ARG(cs_addrs) VARLSTCNT(1) ERR_DSEINVALBLKID);
+						RTS_ERROR_CSA_ABT(cs_addrs, VARLSTCNT(1) ERR_DSEINVALBLKID);
 #						endif
 					} else
 					{
@@ -553,7 +555,7 @@ void dse_f_blk(void)
 						DSE_REL_CRIT_AS_APPROPRIATE(was_crit, was_hold_onto_crit,
 								nocrit_present, cs_addrs, gv_cur_region);
 						REVERT;
-						rts_error_csa(CSA_ARG(cs_addrs) VARLSTCNT(1) ERR_DSEINVALBLKID);
+						RTS_ERROR_CSA_ABT(cs_addrs, VARLSTCNT(1) ERR_DSEINVALBLKID);
 #						endif
 					} else
 						GET_BLK_ID_32(look, srp - SIZEOF(block_id_32));
@@ -570,7 +572,7 @@ void dse_f_blk(void)
 			patch_right_sib = look;
 		}
 		if (patch_right_sib)
-			util_out_print("0x!XL!/", TRUE, patch_right_sib);
+			util_out_print("0x!16@XQ!/", TRUE, &patch_right_sib);
 		else
 			util_out_print("none!/", TRUE);
 		DSE_REL_CRIT_AS_APPROPRIATE(was_crit, was_hold_onto_crit, nocrit_present, cs_addrs, gv_cur_region);
@@ -643,4 +645,3 @@ CONDITION_HANDLER(dse_f_blk_ch)
 		DSE_REL_CRIT_AS_APPROPRIATE(was_crit, was_hold_onto_crit, nocrit_present, cs_addrs, gv_cur_region);
 	NEXTCH;
 }
-
