@@ -1,6 +1,6 @@
 /****************************************************************
  *								*
- * Copyright (c) 2001-2018 Fidelity National Information	*
+ * Copyright (c) 2001-2020 Fidelity National Information	*
  * Services, Inc. and/or its subsidiaries. All rights reserved.	*
  *								*
  *	This source code contains the intellectual property	*
@@ -23,7 +23,8 @@
 #include "gtm_string.h"
 #include "gtm_utf8.h"
 
-GBLREF	boolean_t	run_time;
+GBLREF boolean_t	run_time;
+GBLREF triple		t_orig;
 
 error_def(ERR_EXPR);
 error_def(ERR_MAXARGCNT);
@@ -58,11 +59,12 @@ int eval_expr(oprtype *a)
 	mval		tmp_mval;
 	int i = 0;
 	unsigned short	type;
-
 	DCL_THREADGBL_ACCESS;
+
 	SETUP_THREADGBL_ACCESS;
+	CHKTCHAIN(TREF(curtchain), exorder, TRUE);	/* defined away in mdq.h except with DEBUG_TRIPLES */
 	if (!expratom(&optyp_1))
-	{	/* If didn't already add an error of our own, do so now with catch all expression error */
+	{	/* If didn't already add an error of our own, do so now to catch all expression errors */
 		if (!ALREADY_RTERROR)
 			stx_error(ERR_EXPR);
 		return EXPR_FAIL;
@@ -80,7 +82,15 @@ int eval_expr(oprtype *a)
 				for (ref = (TREF(curtchain))->exorder.bl; oc_tab[ref->opcode].octype & OCT_BOOL;
 					ref = ref->exorder.bl)
 						;
+				assert(ref->exorder.bl != ref);
+				if (&t_orig == ref->exorder.fl)
+				{
+					ref1 = maketriple(OC_NOOP);
+					dqins(ref, exorder, ref1);
+				}
+				assert(&t_orig != ref->exorder.fl);
 				TREF(expr_start) = TREF(expr_start_orig) = ref;
+				CHKTCHAIN(TREF(curtchain), exorder, TRUE); /* defined away in mdq.h except with DEBUG_TRIPLES */
 			}
 			switch (bin_opcode)
 			{
@@ -120,10 +130,11 @@ int eval_expr(oprtype *a)
 					stringpool.free = (unsigned char *)tmp_mval.str.addr + tmp_mval.str.len;
 					s2n(&tmp_mval);		/* things rely on the compiler doing literals with complete types */
 					ref1->operand[0] = put_lit(&tmp_mval);
-					optyp_1 = ref1->operand[0];
-					replaced = TRUE;
 					unuse_literal(&m1->v);
 					unuse_literal(&m2->v);
+					dqdel(optyp_1.oprval.tref, exorder);
+					optyp_1 = ref1->operand[0];
+					replaced = TRUE;
 					op_count--;
 				} else
 				{
@@ -231,12 +242,14 @@ int eval_expr(oprtype *a)
 			} else
 			{
 				advancewindow();
+				CHKTCHAIN(TREF(curtchain), exorder, TRUE); /* defined away in mdq.h except with DEBUG_TRIPLES */
 				if (!expratom(&optyp_2))
 				{
 					stx_error(ERR_RHMISSING);
 					return EXPR_FAIL;
 				}
 			}
+			CHKTCHAIN(TREF(curtchain), exorder, TRUE);	/* defined away in mdq.h except with DEBUG_TRIPLES */
 			coerce(&optyp_2, type);
 			ref1 = optyp_1.oprval.tref;
 			if (((OC_VAR == ref1->opcode) || (OC_GETINDX == ref1->opcode))
