@@ -1,6 +1,6 @@
 /****************************************************************
  *								*
- * Copyright (c) 2001-2020 Fidelity National Information	*
+ * Copyright (c) 2001-2022 Fidelity National Information	*
  * Services, Inc. and/or its subsidiaries. All rights reserved.	*
  *								*
  *	This source code contains the intellectual property	*
@@ -25,6 +25,8 @@
 LITREF mval		literal_minusone, literal_one, literal_zero;
 LITREF octabstruct	oc_tab[];
 
+error_def(ERR_NUMOFLOW);
+
 void ex_tail(oprtype *opr)
 /* work a non-leaf operand toward final form
  * contains code to do arthimetic on literals at compile time
@@ -83,6 +85,13 @@ void ex_tail(oprtype *opr)
 			MV_FORCE_NUMD(v0);
 			v1 = &t1->operand[0].oprval.mlit->v;
 			MV_FORCE_NUMD(v1);
+			if (!(MV_NM & v1->mvtype) || !(MV_NM & v0->mvtype))
+			{	/* if we don't have a useful number we can't do useful math */
+				TREF(last_source_column) += (TK_EOL == TREF(director_token)) ? -2 : 2;	/* improve hints */
+				rts_error_csa(CSA_ARG(NULL) VARLSTCNT(1) ERR_NUMOFLOW);
+				assert(TREF(rts_error_in_parse));
+				return;
+			}
 			v = (mval *)mcalloc(SIZEOF(mval));
 			switch (c)
 			{
@@ -91,10 +100,13 @@ void ex_tail(oprtype *opr)
 				break;
 			case OC_DIV:
 			case OC_IDIV:
+			case OC_MOD:
 				if (!(MV_NM & v1->mvtype) || (0 != v1->m[1]))
 				{
 					if (OC_DIV == c)
 						op_div(v0, v1, v);
+					else if (OC_MOD == c)
+						flt_mod(v0, v1, v);
 					else
 						op_idiv(v0, v1, v);
 				} else				/* divide by literal 0 is a technique so let it go to run time*/
@@ -102,9 +114,8 @@ void ex_tail(oprtype *opr)
 				break;
 			case OC_EXP:
 				op_exp(v0, v1, v);
-				break;
-			case OC_MOD:
-				flt_mod(v0, v1, v);
+				if ((1 == v->sgn) && (MV_INT & v->mvtype) && (0 == v->m[0]))
+					v = NULL;		/* flag value from op_exp indicates DIVZERO, so leave to run time */
 				break;
 			case OC_MUL:
 				op_mul(v0, v1, v);
@@ -117,16 +128,15 @@ void ex_tail(oprtype *opr)
 				break;
 			}
 			RETURN_IF_RTS_ERROR;
-			if (NULL == v)				/* leaving divide by literal 0 to create a run time error */
-				break;				/* from while */
-			unuse_literal(v0);			/* drop original literals only after deciding whether to defer */
+			if ((NULL == v) || (!v->mvtype))
+				break;		/* leave divide by zero or missing mvtype from NUMOFLOW to cause run time errors */
+			unuse_literal(v0);	/* drop original literals only after deciding whether to defer */
 			unuse_literal(v1);
 			dqdel(t0, exorder);
 			dqdel(t1, exorder);
-			assert(v->mvtype);
 			n2s(v);
-			s2n(v);					/* compiler must leave literals with both numeric and string */
-			t->opcode = OC_LIT;			/* replace the original operator triple with new literal */
+			s2n(v);			/* compiler must leave literals with both numeric and string */
+			t->opcode = OC_LIT;	/* replace the original operator triple with new literal */
 			put_lit_s(v, t);
 			t->operand[1].oprclass = NO_REF;
 			assert(opr->oprval.tref == t);
