@@ -1,6 +1,6 @@
 /****************************************************************
  *								*
- * Copyright (c) 2001-2020 Fidelity National Information	*
+ * Copyright (c) 2001-2022 Fidelity National Information	*
  * Services, Inc. and/or its subsidiaries. All rights reserved.	*
  *								*
  *	This source code contains the intellectual property	*
@@ -34,6 +34,7 @@ error_def(ERR_FCNSVNEXPECTED);
 error_def(ERR_FNOTONSYS);
 error_def(ERR_INVFCN);
 error_def(ERR_INVSVN);
+error_def(ERR_NUMOFLOW);
 error_def(ERR_RPARENMISSING);
 error_def(ERR_SIDEEFFECTEVAL);
 error_def(ERR_VAREXPECTED);
@@ -91,6 +92,7 @@ LITDEF nametabent svn_names[] =
 	,{ 4, "ZJOB" }
 	,{ 4, "ZKEY" }
 	,{ 6, "ZLEVEL" }
+	,{10, "ZMALLOCLIM" }
 	,{13, "ZMAXTPTIMEOUT" }
 	,{ 8, "ZMLKHASH" }
 	,{ 5, "ZMODE" }
@@ -136,7 +138,7 @@ LITDEF nametabent svn_names[] =
 LITDEF unsigned char svn_index[27] = {
 	 0,  0,  0,  0,  2,  8,  8,  8, 10,	/* a b c d e f g h i */
 	12, 14 ,16, 16, 16, 16, 16, 18, 20,	/* j k l m n o p q r */
-	22, 28, 34 ,34, 34, 34, 35, 36, 100	/* s t u v w x y z ~ */
+	22, 28, 34 ,34, 34, 34, 35, 36, 101	/* s t u v w x y z ~ */
 };
 
 /* These entries correspond to the entries in the svn_names array */
@@ -155,7 +157,7 @@ LITDEF svn_data_type svn_data[] =
 	,{ SV_REFERENCE, FALSE, ALL_SYS }, { SV_REFERENCE, FALSE, ALL_SYS }
 	,{ SV_STACK, FALSE, ALL_SYS }, { SV_STACK, FALSE, ALL_SYS }
 	,{ SV_STORAGE, FALSE, ALL_SYS }, { SV_STORAGE, FALSE, ALL_SYS }
-	,{ SV_SYSTEM, FALSE, ALL_SYS }, { SV_SYSTEM, FALSE, ALL_SYS }
+	,{ SV_SYSTEM, TRUE, ALL_SYS }, { SV_SYSTEM, TRUE, ALL_SYS }
 	,{ SV_TEST, FALSE, ALL_SYS }, { SV_TEST, FALSE, ALL_SYS }
 	,{ SV_TLEVEL, FALSE, ALL_SYS }, { SV_TLEVEL, FALSE, ALL_SYS }
 	,{ SV_TRESTART, FALSE, ALL_SYS }, { SV_TRESTART, FALSE, ALL_SYS }
@@ -186,6 +188,7 @@ LITDEF svn_data_type svn_data[] =
 	,{ SV_ZJOB, FALSE, ALL_SYS }
 	,{ SV_ZKEY, FALSE , ALL_SYS }
 	,{ SV_ZLEVEL, FALSE, ALL_SYS }
+	,{ SV_ZMALLOCLIM, TRUE, ALL_SYS}
 	,{ SV_ZMAXTPTIME, TRUE, ALL_SYS }
 	,{ SV_ZMLKHASH, FALSE, ALL_SYS }
 	,{ SV_ZMODE, FALSE, ALL_SYS }
@@ -263,6 +266,7 @@ LITDEF nametabent fun_names[] =
 	,{8, "ZAHANDLE"}
 	,{6, "ZASCII"}
 	,{11,"ZATRANSFORM"}
+	,{9, "ZAUDITLOG"}
 	,{7, "ZBITAND"}
 	,{9, "ZBITCOUNT"}
 	,{8, "ZBITFIND"}
@@ -312,7 +316,7 @@ LITDEF unsigned char fun_index[27] =
 {
 	 0,  2,  2,  4,  6,  8, 12, 14, 14,	/* a b c d e f g h i */
 	17, 19, 19, 21, 21, 25, 27, 29, 35,	/* j k l m n o p q r */
-	39, 43, 47, 47, 49, 49, 49, 49, 95	/* s t u v w x y z ~ */
+	39, 43, 47, 47, 49, 49, 49, 49, 96	/* s t u v w x y z ~ */
 };
 
 /* Each entry corresponds to an entry in fun_names */
@@ -348,6 +352,7 @@ LITDEF fun_data_type fun_data[] =
 	,{ OC_FNZAHANDLE, ALL_SYS }
 	,{ OC_FNZASCII, ALL_SYS }
 	,{ OC_FNZATRANSFORM, ALL_SYS }
+	,{ OC_FNZAUDITLOG, ALL_SYS }
 	,{ OC_FNZBITAND, ALL_SYS }
 	,{ OC_FNZBITCOUN, ALL_SYS }
 	,{ OC_FNZBITFIND, ALL_SYS }
@@ -425,6 +430,7 @@ GBLDEF int (*fun_parse[])(oprtype *, opctype) =		/* contains addresses so can't
 	f_zahandle,
 	f_ascii,
 	f_zatransform,
+	f_one_mval,
 	f_two_mval,			/* $ZBITAND */
 	f_one_mval,			/* $ZBITCOUNT */
 	f_fnzbitfind,
@@ -602,6 +608,13 @@ int expritem(oprtype *a)
 					}
 				}
 				advancewindow();
+				if (((OC_FNZCHAR == fun_data[index].opcode) || (OC_FNCHAR == fun_data[index].opcode))
+					&& (MV_NUM_APPROX == ((MV_NM | MV_NUM_APPROX) & (TREF(director_mval)).mvtype)))
+				{	/* [z]f_char need an error based s2n failing to create a valid numeric argument */
+					assert((0 == (TREF(director_mval)).e) && !val_iscan(&(TREF(director_mval))));
+					TREF(last_source_column) += (TK_EOL == TREF(director_token)) ? -2 : 2;  /* improve hints */
+					stx_error(ERR_NUMOFLOW);
+				}
 				advancewindow();
 				if (!parse_warn)
 				{
