@@ -1,6 +1,7 @@
 /****************************************************************
  *								*
- *	Copyright 2001, 2012 Fidelity Information Services, Inc	*
+ * Copyright (c) 2001-2018 Fidelity National Information	*
+ * Services, Inc. and/or its subsidiaries. All rights reserved.	*
  *								*
  *	This source code contains the intellectual property	*
  *	of its copyright holder(s), and is made available	*
@@ -9,8 +10,8 @@
  *								*
  ****************************************************************/
 
-#ifndef _FNORDER_H_INC_
-#define _FNORDER_H_INC_
+#ifndef FNORDER_H_INCLUDED
+#define FNORDER_H_INCLUDED
 
 enum order_obj {
 	GLOBAL = 0,
