@@ -1,6 +1,6 @@
 /****************************************************************
  *								*
- * Copyright (c) 2001-2020 Fidelity National Information	*
+ * Copyright (c) 2001-2022 Fidelity National Information	*
  * Services, Inc. and/or its subsidiaries. All rights reserved.	*
  *								*
  *	This source code contains the intellectual property	*
@@ -135,6 +135,7 @@
 #define DEFAULT_ZERROR_LEN	(SIZEOF(DEFAULT_ZERROR_STR) - 1)
 #include "gtm_libaio.h"
 #include "deferred_events_queue.h"
+#include "deferred_events.h"
 #include "dm_audit_log.h"
 
 GBLDEF	gd_region		*db_init_region;
@@ -145,7 +146,6 @@ GBLDEF	sgmnt_addrs		*cs_addrs_list;	/* linked list of csa corresponding to all c
 GBLDEF	unsigned short	proc_act_type;
 GBLDEF	volatile bool	ctrlc_pending;
 GBLDEF	bool		undef_inhibit;
-GBLDEF	volatile int4	ctrap_action_is;
 GBLDEF	bool		out_of_time;
 GBLDEF	io_pair		io_curr_device;		/* current device	*/
 GBLDEF	io_pair		io_std_device;		/* standard device	*/
@@ -153,8 +153,10 @@ GBLDEF	io_log_name	*dollar_principal;	/* pointer to log name GTM$PRINCIPAL if de
 GBLDEF	boolean_t	prin_dm_io;		/* used by op_dmode so mdb_condition_handler and iorm_readfl know it's active */
 GBLDEF	boolean_t	prin_in_dev_failure;	/* used in I/O to perform NOPRINCIO detection on input */
 GBLDEF	boolean_t	prin_out_dev_failure;	/* used in I/O to perform NOPRINCIO detection on output */
-GBLDEF	uint4		parallel_freeze_online;
+GBLDEF	boolean_t	wcs_noasyncio;		/* used in wcs_wtstart to disable asyncio in the forward rollback/recover phase */
 GBLDEF	io_desc		*active_device;
+GBLDEF	bool		pin_shared_memory;
+GBLDEF	bool		hugetlb_shm_enabled;
 GBLDEF	bool		error_mupip,
 			file_backed_up,
 			gv_replopen_error,
@@ -181,8 +183,7 @@ GBLDEF	boolean_t	is_updproc,
 			is_replicator,		/* TRUE => this process can write jnl records to the jnlpool for replicated db */
 			dollar_truth = TRUE,
 			gtm_stdxkill,		/* TRUE => Use M Standard X-KILL - FALSE use historical GTM X-KILL (default) */
-			in_timed_tn,		/* TRUE => Timed TP transaction in progress */
-			tp_timeout_deferred;	/* TRUE => A TP timeout has occurred but is deferred */
+			in_timed_tn;		/* TRUE => Timed TP transaction in progress */
 GBLDEF	uint4		is_updhelper;		/* = UPD_HELPER_READER if reader helper, = UPD_HELPER_WRITER if writer helper,
 						 * = 0 otherwise.
 						 */
@@ -220,15 +221,14 @@ GBLDEF	mval		dollar_zgbldir,
 			dollar_zsource = DEFINE_MVAL_STRING(MV_STR, 0, 0, 0, NULL, 0, 0),
 			dollar_zstatus,
 			ztrap_pop2level = DEFINE_MVAL_STRING(MV_NM | MV_INT, 0, 0, 0, 0, 0, 0),
-			zstep_action,
-			dollar_system,
+			dollar_system, dollar_system_initial,
 			dollar_estack_delta = DEFINE_MVAL_STRING(MV_STR, 0, 0, 0, NULL, 0, 0),
 			dollar_zerror = DEFINE_MVAL_STRING(MV_STR, 0, 0, DEFAULT_ZERROR_LEN, DEFAULT_ZERROR_STR, 0, 0),
 			dollar_zyerror,
 			dollar_ztexit = DEFINE_MVAL_STRING(MV_STR, 0, 0, 0, NULL, 0, 0);
 GBLDEF  uint4		dollar_zjob;
 GBLDEF	mval		dollar_zinterrupt;
-GBLDEF	boolean_t	dollar_zininterrupt;
+GBLDEF	volatile boolean_t	dollar_zininterrupt;
 GBLDEF	boolean_t	dollar_ztexit_bool; /* Truth value of dollar_ztexit when coerced to boolean */
 GBLDEF	boolean_t	dollar_zquit_anyway;
 GBLDEF	mv_stent	*mv_chain;
@@ -306,7 +306,6 @@ GBLDEF	boolean_t	oldjnlclose_started;
 /* DEFERRED EVENTS */
 GBLDEF	bool		licensed = TRUE;
 
-GBLDEF	volatile int4		num_deferred;
 GBLDEF	volatile	int4	fast_lock_count;	/* Used in wcs_stale */
 /* REPLICATION RELATED GLOBALS */
 GBLDEF	gtmsource_options_t	gtmsource_options;
@@ -316,7 +315,7 @@ GBLDEF	boolean_t		is_tracing_on;
 GBLDEF	void			(*tp_timeout_start_timer_ptr)(int4 tmout_sec) = tp_start_timer_dummy;
 GBLDEF	void			(*tp_timeout_clear_ptr)(void) = tp_clear_timeout_dummy;
 GBLDEF	void			(*tp_timeout_action_ptr)(void) = tp_timeout_action_dummy;
-GBLDEF	void			(*ctrlc_handler_ptr)() = ctrlc_handler_dummy;
+GBLDEF	void			(*ctrlc_handler_ptr)(void) = ctrlc_handler_dummy;
 GBLDEF	int			(*op_open_ptr)(mval *v, mval *p, mval *t, mval *mspace) = op_open_dummy;
 GBLDEF	void			(*unw_prof_frame_ptr)(void) = unw_prof_frame_dummy;
 /* Initialized only in gtm_startup() */
@@ -396,13 +395,10 @@ GBLDEF	char			cli_err_str[MAX_CLI_ERR_STR] = "";	/* Parse Error message buffer *
 GBLDEF	char			*cli_err_str_ptr;
 GBLDEF	boolean_t		gtm_pipe_child;
 GBLDEF	io_desc			*gtm_err_dev;
-/* this array is indexed by file descriptor */
-/* Latch variable for Unix implementations. Used in SUN and HP */
-GBLDEF	global_latch_t		defer_latch;
-GBLDEF	global_latch_t		outofband_queue_latch;
 GBLDEF	int			num_additional_processors;
 GBLDEF	int			gtm_errno = -1;		/* holds the errno (unix) in case of an rts_error */
 GBLDEF	int4			error_condition;
+GBLDEF	int4			pre_drvlongjmp_error_condition;
 GBLDEF	global_tlvl_info	*global_tlvl_info_head;
 GBLDEF	buddy_list		*global_tlvl_info_list;
 GBLDEF	boolean_t		job_try_again;
@@ -454,13 +450,12 @@ GBLDEF	ipcs_mesg	db_ipcs;		/* For requesting gtmsecshr to update ipc fields */
 GBLDEF	gd_region	*ftok_sem_reg;		/* Last region for which ftok semaphore is grabbed */
 GBLDEF	int		gtm_non_blocked_write_retries; /* number of retries for non-blocked write to pipe */
 GBLDEF	boolean_t	write_after_image;	/* true for after-image jnlrecord writing by recover/rollback */
-GBLDEF	int		iott_write_error;
 GBLDEF	int4		write_filter;
 GBLDEF	boolean_t	need_no_standalone;
 GBLDEF	int4		zdir_form = ZDIR_FORM_FULLPATH; /* $ZDIR shows full path including DEVICE and DIRECTORY */
 GBLDEF	mval		dollar_zdir = DEFINE_MVAL_STRING(MV_STR, 0, 0, 0, NULL, 0, 0);
 GBLDEF	int * volatile	var_on_cstack_ptr; /* volatile pointer to int; volatile so that nothing gets optimized out */
-GBLDEF	hash_table_int4	cw_stagnate;
+GBLDEF	hash_table_int8	cw_stagnate;
 GBLDEF	boolean_t	cw_stagnate_reinitialized;
 
 GBLDEF	uint4		pat_everything[] = { 0, 2, PATM_E, 1, 0, PAT_MAX_REPEAT, 0, PAT_MAX_REPEAT, 1 }; /* pattern = ".e" */
@@ -671,6 +666,7 @@ GBLDEF	boolean_t	is_uchar_wcs_code[] =	/* uppercase failure codes that imply dat
 #undef CDB_SC_UCHAR_ENTRY
 #undef CDB_SC_LCHAR_ENTRY
 };
+GBLDEF	int	sizeof_is_uchar_wcs_code = SIZEOF(is_uchar_wcs_code);	/* 4SCA */
 GBLDEF	boolean_t	is_lchar_wcs_code[] =	/* lowercase failure codes that imply database cache related problem */
 {	/* if any of the following failure codes are seen in the final retry, wc_blocked will be set to trigger cache recovery */
 #define	CDB_SC_NUM_ENTRY(code, final_retry_ok, value)
@@ -681,6 +677,7 @@ GBLDEF	boolean_t	is_lchar_wcs_code[] =	/* lowercase failure codes that imply dat
 #undef CDB_SC_UCHAR_ENTRY
 #undef CDB_SC_LCHAR_ENTRY
 };
+GBLDEF	int	sizeof_is_lchar_wcs_code = SIZEOF(is_lchar_wcs_code);	/* 4SCA */
 GBLDEF	boolean_t	is_final_retry_code_num[] =	/* failure codes that are possible in final retry : numeric */
 {
 #define	CDB_SC_NUM_ENTRY(code, final_retry_ok, value)			final_retry_ok,
@@ -691,6 +688,7 @@ GBLDEF	boolean_t	is_final_retry_code_num[] =	/* failure codes that are possible
 #undef CDB_SC_UCHAR_ENTRY
 #undef CDB_SC_LCHAR_ENTRY
 };
+GBLDEF	int		sizeof_is_final_retry_code_num = SIZEOF(is_final_retry_code_num);	/* 4SCA */
 GBLDEF	boolean_t	is_final_retry_code_uchar[] =	/* failure codes that are possible in final retry : upper case */
 {
 #define	CDB_SC_NUM_ENTRY(code, final_retry_ok, value)
@@ -701,6 +699,7 @@ GBLDEF	boolean_t	is_final_retry_code_uchar[] =	/* failure codes that are possibl
 #undef CDB_SC_UCHAR_ENTRY
 #undef CDB_SC_LCHAR_ENTRY
 };
+GBLDEF	int		sizeof_is_final_retry_code_uchar = SIZEOF(is_final_retry_code_uchar);	/* 4SCA */
 GBLDEF	boolean_t	is_final_retry_code_lchar[] =	/* failure codes that are possible in final retry : lower case */
 {
 #define	CDB_SC_NUM_ENTRY(code, final_retry_ok, value)
@@ -711,6 +710,7 @@ GBLDEF	boolean_t	is_final_retry_code_lchar[] =	/* failure codes that are possibl
 #undef CDB_SC_UCHAR_ENTRY
 #undef CDB_SC_LCHAR_ENTRY
 };
+GBLDEF	int		sizeof_is_final_retry_code_lchar = SIZEOF(is_final_retry_code_lchar);	/* 4SCA */
 GBLDEF	boolean_t	gvdupsetnoop = TRUE;	/* if TRUE, duplicate SETs do not change GDS block (and therefore no PBLK journal
 						 * records will be written) although the database transaction number will be
 						 * incremented and logical SET journal records will be written. By default, this
@@ -768,6 +768,7 @@ GBLDEF	int4		cws_reorg_remove_index;			/* see mu_swap_blk.c for comments on the
 GBLDEF	block_id	cws_reorg_remove_array[CWS_REORG_REMOVE_ARRAYSIZE];
 GBLDEF	uint4		log_interval;
 GBLDEF	uint4		gtm_principal_editing_defaults;	/* ext_cap flags if tt */
+GBLDEF	enum db_ver	gtm_db_create_ver;		/* database creation version */
 GBLDEF	boolean_t	in_repl_inst_edit;		/* used by an assert in repl_inst_read/repl_inst_write */
 GBLDEF	boolean_t	in_repl_inst_create;		/* used by repl_inst_read/repl_inst_write */
 GBLDEF	boolean_t	holds_sem[NUM_SEM_SETS][NUM_SRC_SEMS];	/* whether a particular replication semaphore is being held
@@ -953,6 +954,7 @@ GBLDEF	boolean_t	donot_INVOKE_MUMTSTART;		/* Set to TRUE whenever an implicit TS
 							 * mdb_condition_handler (invoked by INVOKE_RESTART macro when it does
 							 * an rts_error) checks it never gets invoked while this is set.
 							 */
+GBLDEF	char		asccurtime[10];			/* used in deferred_events.h by macro SHOWTIME timestamping debug output */
 #endif
 GBLDEF	boolean_t	block_is_free;			/* Set to TRUE if the caller wants to let t_qread know that the block it is
 							 * attempting to read is actually a FREE block
@@ -1078,6 +1080,7 @@ GBLDEF	is_anticipatory_freeze_needed_t		is_anticipatory_freeze_needed_fnptr;
 GBLDEF	set_anticipatory_freeze_t		set_anticipatory_freeze_fnptr;
 GBLDEF	boolean_t	is_jnlpool_creator;
 GBLDEF	char		gtm_dist[GTM_PATH_MAX];		/* Value of $gtm_dist env variable */
+GBLDEF	unsigned int	gtm_dist_len = 0;		/* Length of validated $gtm_dist */
 GBLDEF	boolean_t	gtm_dist_ok_to_use = FALSE;	/* Whether or not we can use $gtm_dist */
 GBLDEF	semid_queue_elem	*keep_semids;		/* Access semaphores that should be kept because shared memory is up */
 GBLDEF	boolean_t		dmterm_default;		/* Retain default line terminators in the direct mode */
@@ -1092,7 +1095,9 @@ GBLDEF	boolean_t	gtm_main_thread_id_set;		/* Indicates whether the thread ID is
 							 */
 GBLDEF	boolean_t	gtm_jvm_process;		/* Indicates whether we are running with JVM or stand-alone. */
 #endif
-GBLDEF	size_t		gtm_max_storalloc;		/* Maximum that GTM allows to be allocated - used for testing */
+GBLDEF	size_t		zmalloclim;			/* ISV memory warning of MALLOCCRIT in bytes */
+GBLDEF	boolean_t	malloccrit_issued;		/* MEMORY error limit set at time of MALLOCCRIT */
+GBLDEF	xfer_set_handlers_fnptr_t	xfer_set_handlers_fnptr;	/* see comment in deferred_events.h about this typedef */
 GBLDEF	boolean_t	ipv4_only;			/* If TRUE, only use AF_INET. Reflects the value of the gtm_ipv4_only
 							 * environment variable, so is process wide.
 							 */
@@ -1196,11 +1201,6 @@ GBLDEF	char		io_setup_errstr[IO_SETUP_ERRSTR_ARRAYSIZE];
 GBLDEF	void		(*mupip_exit_fp)(int4 errnum);	/* Function pointer to mupip_exit() in MUPIP but points to a routine
 							 * that assert fails if run from non-MUPIP builds.
 							 */
-
-/* Direct mode auditing related global variables */
-GBLDEF	boolean_t	dollar_zaudit;			/* Intrinsic that indicates whether direct mode
-							 * auditing (i.e. APD) is enabled.
-							 * TRUE => Auditing is enabled.
-							 */
-GBLDEF	dm_audit_info	*audit_conn;			/* Stores the APD logger's connection information */
 GBLDEF	boolean_t	gtm_nofflf;			/* GTM-9136 TRUE only to suppress LF after FF in "write #" */
+GBLDEF	void 		(*primary_exit_handler)(void);	/* If non-NULL, may be used to reestablish atexit() handler */
+GBLDEF	dm_audit_info	audit_conn[MAX_AUD_CONN];	/* AUdit logging connection information */
