@@ -1,6 +1,6 @@
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;								;
-; Copyright (c) 2001-2019 Fidelity National Information		;
+; Copyright (c) 2001-2020 Fidelity National Information		;
 ; Services, Inc. and/or its subsidiaries. All rights reserved.	;
 ;								;
 ;	This source code contains the intellectual property	;
@@ -78,24 +78,24 @@ GDEINIT
 	;	SIZEOF("gd_segment")		; --> size of the "gd_segment" structure (defined in gdsfhead.h)
 	;
 	i (gtm64=FALSE) d
-	. s SIZEOF("am_offset")=332		; --> offset of "acc_meth" field in the "gd_segment" structure
+	. s SIZEOF("am_offset")=336		; --> offset of "acc_meth" field in the "gd_segment" structure
 	. s SIZEOF("file_spec")=256		; --> maximum size (in bytes) of a file name specified in gde command line
 	. s SIZEOF("gd_contents")=80		; --> size of the "gd_addr" structure (defined in gdsfhead.h)
 	. s SIZEOF("gd_header")=16		; --> 16-byte header structure at offset 0 of .gld (12 byte label, 4-byte filesize)
 	. s SIZEOF("gd_map")=16			; --> size of the "gd_binding" structure (defined in gdsfhead.h)
-	. s SIZEOF("gd_region")=412		; --> size of the "gd_region"  structure (defined in gdsfhead.h)
+	. s SIZEOF("gd_region")=416		; --> size of the "gd_region"  structure (defined in gdsfhead.h)
 	. s SIZEOF("gd_region_padding")=0	; --> padding at end of "gd_region" structure (4-bytes for 64-bit platforms)
-	. s SIZEOF("gd_segment")=368		; --> size of the "gd_segment" structure (defined in gdsfhead.h)
+	. s SIZEOF("gd_segment")=372		; --> size of the "gd_segment" structure (defined in gdsfhead.h)
 	e  d
 	. s SIZEOF("am_offset")=340		; --> offset of "acc_meth" field in the "gd_segment" structure
 	. s SIZEOF("file_spec")=256		; --> maximum size (in bytes) of a file name specified in gde command line
 	. s SIZEOF("gd_contents")=120		; --> size of the "gd_addr" structure (defined in gdsfhead.h)
 	. s SIZEOF("gd_header")=16		; --> 16-byte header structure at offset 0 of .gld (12 byte label, 4-byte filesize)
 	. s SIZEOF("gd_map")=24			; --> size of the "gd_binding" structure (defined in gdsfhead.h)
-	. s SIZEOF("gd_region")=424		; --> size of the "gd_region"  structure (defined in gdsfhead.h)
-	. s SIZEOF("gd_region_padding")=4	; --> padding at end of "gd_region" structure (4-bytes for 64-bit platforms)
+	. s SIZEOF("gd_region")=432		; --> size of the "gd_region"  structure (defined in gdsfhead.h)
+	. s SIZEOF("gd_region_padding")=8	; --> padding at end of "gd_region" structure (4-bytes for 64-bit platforms)
 	. s SIZEOF("gd_segment")=384		; --> size of the "gd_segment" structure (defined in gdsfhead.h)
-	d Init^GDEINITSZ			; --> current gvstats size comes from routine generated by scantypedefs
+	d gvstats
 	s SIZEOF("blk_hdr")=16
 	s SIZEOF("dsk_blk")=512
 	s SIZEOF("gd_gblname")=40
@@ -141,7 +141,7 @@ GDEINIT
 	s minreg("AUTODB")=0,maxreg("AUTODB")=1
 	s minreg("AUTOSWITCHLIMIT")=16384,maxreg("AUTOSWITCHLIMIT")=8388607
 	s minreg("BEFORE_IMAGE")=0,maxreg("BEFORE_IMAGE")=1
-	s minreg("BUFFER_SIZE")=2307,maxreg("BUFFER_SIZE")=32768
+	s minreg("BUFFER_SIZE")=2307,maxreg("BUFFER_SIZE")=1048576
 	s minreg("COLLATION_DEFAULT")=0,maxreg("COLLATION_DEFAULT")=255
 	s minreg("EPOCH_INTERVAL")=1,maxreg("EPOCH_INTERVAL")=32767
 	s minreg("EPOCHTAPER")=0,maxreg("EPOCHTAPER")=1
@@ -168,11 +168,12 @@ GDEINIT
 	n minsegcommon,maxsegcommon
 	s minsegcommon("ALLOCATION")=10,maxsegcommon("ALLOCATION")=TWO(30)-TWO(25) ; supports 992M blocks
 	s minsegcommon("BLOCK_SIZE")=SIZEOF("dsk_blk"),maxsegcommon("BLOCK_SIZE")=HEX(4)-SIZEOF("dsk_blk")
-	s minsegcommon("EXTENSION_COUNT")=0,maxsegcommon("EXTENSION_COUNT")=HEX(4)-1
+	s minsegcommon("EXTENSION_COUNT")=0,maxsegcommon("EXTENSION_COUNT")=HEX(5)-1
 	s minsegcommon("LOCK_SPACE")=10,maxsegcommon("LOCK_SPACE")=262144
 	s minsegcommon("MUTEX_SLOTS")=64	; keep this in sync with MIN_CRIT_ENTRY in gdsbt.h
 	s maxsegcommon("MUTEX_SLOTS")=32768	; keep this in sync with MAX_CRIT_ENTRY in gdsbt.h
 	s minsegcommon("RESERVED_BYTES")=0,maxsegcommon("RESERVED_BYTES")=HEX(4)-SIZEOF("dsk_blk")
+	s minsegcommon("FULLBLKWRT")=0,maxsegcommon("FULLBLKWRT")=2
 ; bg
 	m minseg("BG")=minsegcommon,maxseg("BG")=maxsegcommon	; copy over all common stuff into BG access method first
 	; now add BG specific overrides; GTM64_WC_MAX_BUFFS defined in gdsbt.h
@@ -186,6 +187,7 @@ GDEINIT
 	; So if we are reading an older format .gld file, we can use these as the default values.
 	s defseg("ALLOCATION")=100
 	s defseg("ASYNCIO")=0
+	s defseg("FULLBLKWRT")=0
 	s defseg("BLOCK_SIZE")=4096
 	s defseg("BUCKET_SIZE")=""
 	s defseg("DEFER_ALLOCATE")=1
@@ -244,6 +246,8 @@ syntabi:
 	s syntab("ADD","SEGMENT","ALLOCATION")="REQUIRED"
 	s syntab("ADD","SEGMENT","ALLOCATION","TYPE")="TNUMBER"
 	s syntab("ADD","SEGMENT","ASYNCIO")="NEGATABLE"
+	s syntab("ADD","SEGMENT","FULLBLKWRT")="REQUIRED"
+	s syntab("ADD","SEGMENT","FULLBLKWRT","TYPE")="TNUMBER"
 	s syntab("ADD","SEGMENT","BLOCK_SIZE")="REQUIRED"
 	s syntab("ADD","SEGMENT","BLOCK_SIZE","TYPE")="TNUMBER"
 	s syntab("ADD","SEGMENT","BUCKET_SIZE")="REQUIRED"
@@ -310,6 +314,8 @@ syntabi:
 	s syntab("CHANGE","SEGMENT","ALLOCATION")="REQUIRED"
 	s syntab("CHANGE","SEGMENT","ALLOCATION","TYPE")="TNUMBER"
 	s syntab("CHANGE","SEGMENT","ASYNCIO")="NEGATABLE"
+	s syntab("CHANGE","SEGMENT","FULLBLKWRT")="REQUIRED"
+	s syntab("CHANGE","SEGMENT","FULLBLKWRT","TYPE")="TNUMBER"
 	s syntab("CHANGE","SEGMENT","BLOCK_SIZE")="REQUIRED"
 	s syntab("CHANGE","SEGMENT","BLOCK_SIZE","TYPE")="TNUMBER"
 	s syntab("CHANGE","SEGMENT","BUCKET_SIZE")="REQUIRED"
@@ -367,6 +373,8 @@ syntabi:
 	s syntab("TEMPLATE","SEGMENT","ALLOCATION")="REQUIRED"
 	s syntab("TEMPLATE","SEGMENT","ALLOCATION","TYPE")="TNUMBER"
 	s syntab("TEMPLATE","SEGMENT","ASYNCIO")="NEGATABLE"
+	s syntab("TEMPLATE","SEGMENT","FULLBLKWRT")="REQUIRED"
+	s syntab("TEMPLATE","SEGMENT","FULLBLKWRT","TYPE")="TNUMBER"
 	s syntab("TEMPLATE","SEGMENT","BLOCK_SIZE")="REQUIRED"
 	s syntab("TEMPLATE","SEGMENT","BLOCK_SIZE","TYPE")="TNUMBER"
 	s syntab("TEMPLATE","SEGMENT","BUCKET_SIZE")="REQUIRED"
@@ -431,8 +439,8 @@ syntabi:
 	s syntab("VERIFY","TEMPLATE")=""
 	q
 UNIX:
-	s hdrlab="GTCGBDUNX012"         ; must be concurrently maintained in gbldirnam.h!!!
-	i (gtm64=TRUE) s hdrlab="GTCGBDUNX112" ; the high order digit is a 64-bit flag
+	s hdrlab="GTCGBDUNX015"         ; must be concurrently maintained in gbldirnam.h!!!
+	i (gtm64=TRUE) s hdrlab="GTCGBDUNX115" ; the high order digit is a 64-bit flag
 	s tfile="$gtmgbldir"
 	s accmeth="\BG\MM"
 	s helpfile="$gtm_dist/gdehelp.gld"
@@ -444,3 +452,8 @@ UNIX:
 	s filexfm="filespec"
 	s sep="TKDASH"
 	q
+gvstats		; Obtain the gvstats size from the output of ZSHOW "G"
+	n zshow
+	zsh "G":zshow
+	s SIZEOF("gvstats")=($length(zshow("G",0),",")-2)*8	; --> Field count, minus GLD/REG fields, times 8 bytes per stat.
+	q
