@@ -1,6 +1,7 @@
 /****************************************************************
  *								*
- *	Copyright 2001, 2009 Fidelity Information Services, Inc	*
+ * Copyright (c) 2001-2018 Fidelity National Information	*
+ * Services, Inc. and/or its subsidiaries. All rights reserved.	*
  *								*
  *	This source code contains the intellectual property	*
  *	of its copyright holder(s), and is made available	*
@@ -43,12 +44,9 @@ void gtm_tempnam(char *dir, char *prefix, char *fullname)
 	{
 		prefix = def_prefix;
 		len = SIZEOF(def_prefix) - 1;
-		memcpy(ptr, def_prefix, len);
 	} else
-	{
 		len = STRLEN(prefix);
-		memcpy(ptr, prefix, len);
-	}
+	memcpy(ptr, prefix, len);
 	ptr += len;
-	SPRINTF(ptr, "_%d.tmp", temp_file_counter++);
+	SNPRINTF(ptr, MAX_FN_LEN - len, "_%d.tmp", temp_file_counter++);
 }
