@@ -1,6 +1,6 @@
 /****************************************************************
  *								*
- * Copyright (c) 2010-2020 Fidelity National Information	*
+ * Copyright (c) 2010-2022 Fidelity National Information	*
  * Services, Inc. and/or its subsidiaries. All rights reserved.	*
  *								*
  *	This source code contains the intellectual property	*
@@ -18,7 +18,7 @@
  *
  *   1. Make organization defensible. Possible organizations:
  *      a. Group like-types alphabetically.
- *      b. Gorup related items (such as compiler, replication, etc.
+ *      b. Group related items (such as compiler, replication, etc.
  *   2. Because references will use an offset into this structure and since the "immediate offset" in
  *      compiler generated instructions is usually limited to "smaller" values like less than 16K or
  *      32K or whatever (platform dependent), items near the top of the table should be reserved used for
@@ -35,7 +35,7 @@
 THREADGBLDEF(grabbing_crit, 			gd_region *)			/* Region currently grabbing crit in (if any) */
 
 /* Compiler */
-THREADGBLDEF(blkmod_fail_level,		int4)				/* TP trace reporting element */
+THREADGBLDEF(blkmod_fail_level,			int4)				/* TP trace reporting element */
 THREADGBLDEF(blkmod_fail_type,			int4)				/* TP trace reporting element */
 THREADGBLDEF(block_level,			int4)				/* used to check embedded subroutine levels */
 THREADGBLDEF(boolchain,				triple)				/* anchor for chain used by bx_boolop  */
@@ -68,13 +68,14 @@ THREADGBLDEF(linkage_last,			struct linkage_entry *)		/* Last added entry */
 #ifdef USHBIN_SUPPORTED
 THREADGBLDEF(objhash_state,			hash128_state_t)		/* Seed value - progressive hash of object file */
 #endif
+THREADGBLDEF(discard,				boolean_t)                      /* parsing after literal FALSE postconditional */
 THREADGBLDEF(pos_in_chain,			triple)				/* anchor used to restart after a parsing error */
 THREADGBLDEF(rts_error_in_parse,		boolean_t)			/* flag to stop parsing current line */
 THREADGBLDEF(s2n_intlit, 			boolean_t)			/* type info from s2n for advancewindow */
 THREADGBLDEF(routine_source_offset,		uint4)				/* offset of M source within literal text pool */
 THREADGBLDEF(saw_side_effect,			boolean_t)			/* need side effect handling other than naked */
 THREADGBLDEF(shift_side_effects, 		int)				/* flag shifting of side-effects ahead of boolean
-										 * evalation */
+										 * evaluation */
 THREADGBLDEF(side_effect_base,			boolean_t *)			/* anchor side effect array: bin ops & func args */
 THREADGBLDEF(side_effect_depth,			uint4)				/* current high water of side effect expr array */
 THREADGBLDEF(side_effect_handling,		int)				/* side effect handling in actuallists, function
@@ -88,16 +89,18 @@ THREADGBLDEF(window_ident,			mstr)				/* current scanner mident from advancewind
 THREADGBLDEF(window_mval,			mval)				/* current scanner mval from advancewindow */
 THREADGBLDEF(window_token,			char)				/* current scanner token from advancewindow */
 THREADGBLDEF(xecute_literal_parse,		boolean_t)			/* flag TRUE when trying what its name says */
+THREADGBLDEF(fetch_control,			fetch_ctrl)			/* structure for managing lvn fetches */
 
 /* Database */
 THREADGBLDEF(dbinit_max_delta_secs,		uint4)				/* max time before we bail out in db_init */
 THREADGBLDEF(dollar_zmaxtptime, 		int4)				/* tp timeout in seconds */
-THREADGBLDEF(ztimeout_deferred,			boolean_t)			/* ztimeout occured but was deferred */
-THREADGBLDEF(in_ztimeout,			boolean_t)
-THREADGBLDEF(ztimeout_set_xfer,			volatile boolean_t)		/* A ztimeout set the xfer table intercepts */
-THREADGBLDEF(dollar_ztimeout,			dollar_ztimeout_struct)
-THREADGBLDEF(save_xfer_root,			volatile save_xfer_entry*)	 /*  *= NULL;*/
-THREADGBLDEF(save_xfer_tail,			volatile save_xfer_entry*) 	/*= NULL;*/
+THREADGBLAR1DEF(save_xfer_root,		save_xfer_entry, DEFERRED_EVENTS)	/* array of deferred events with the zeroth
+										 * acting as the root of a queue identified by the
+										 * _ptr item immediately following
+										 */
+THREADGBLDEF(save_xfer_root_ptr,		save_xfer_entry*) 		/* pointer to anchor; see immediately prior item */
+THREADGBLDEF(ztimeout_timer_on,		boolean_t)			/* ztimeout has a gt_timer pending */
+THREADGBLDEF(dollar_ztimeout,			dollar_ztimeout_struct)		/* holds ztimeout state info */
 THREADGBLDEF(donot_write_inctn_in_wcs_recover,	boolean_t)			/* TRUE if wcs_recover should NOT write INCTN */
 THREADGBLDEF(gbuff_limit,			mval)				/* holds a GTM_POOLLIMIT value for REORG or DBG */
 THREADGBLDEF(gd_targ_tn,			trans_num)			/* number that is incremented for every gvcst_spr*
@@ -222,6 +225,11 @@ THREADGBLDEF(max_lcl_coll_xform_bufsiz,		int)				/* max size of local collation
 THREADGBLDEF(replgbl,				replgbl_t)			/* set of global variables needed by the source
 										 * server */
 THREADGBLDEF(tqread_nowait,			boolean_t)			/* avoid sleeping in t_qread if TRUE */
+/* Auditing related global variables */
+THREADGBLDEF(dollar_zaudit, 			boolean_t)			/* Intrinsic that indicates whether direct mode
+										 * auditing (i.e. APD) is enabled.
+										 * TRUE => Auditing is enabled. */
+THREADGBLDEF(is_zauditlog, 			boolean_t)			/* Index for audit_conn*/
 /* Miscellaneous */
 THREADGBLDEF(arlink_enabled,			boolean_t)			/* TRUE if any zroutines segment is autorelink
 										 * enabled. */
@@ -238,7 +246,8 @@ THREADGBLDEF(dollar_zmode,			mval)				/* run mode indicator */
 THREADGBLDEF(dollar_zonlnrlbk,			int)				/* ISV (incremented for every online rollback) */
 THREADGBLDEF(dollar_zclose,			int)				/* ISV (set to close status for PIPE device) */
 THREADGBLDEF(dollar_zroutines,			mstr)				/* routine search list */
-THREADGBLDEF(dollar_zstep,			mval)				/* $zstep - zstep action */
+THREADGBLDEF(dollar_zstep,			mval)				/* $zstep - action repository for zstep */
+THREADGBLDEF(zstep_action,			mval)				/* $zstep - actual action for zstep */
 THREADGBLDEF(dollar_ztrap,			mval)				/* $ztrap - recursive try error action */
 THREADGBLDEF(error_on_jnl_file_lost,		unsigned int)			/* controls error handling done by jnl_file_lost.
 										 * 0 (default) : Turn off journaling and continue.
@@ -539,6 +548,9 @@ THREADGBLDEF(gtm_test_autorelink_always,	boolean_t)	/*  DEBUG-only option to ena
 THREADGBLDEF(fork_without_child_wait,		boolean_t)	/*  we did a FORK but did not wait for child to detach from
 								 *  inherited shm so shm_nattch could be higher than we expect.
 								 */
+THREADGBLDEF(in_bm_getfree_gdsfilext,           boolean_t)	/* bm_getfree() did a preemptive crit grab before doing a
+								 * file extension.
+								 */
 #endif	/* #ifdef DEBUG */
 /* (DEBUG_ONLY relevant points reproduced from the comment at the top of this file)
  *   5. It is important for ANY DEBUG_ONLY fields to go at the VERY END. Failure to do this breaks gtmpcat.
