@@ -1,6 +1,6 @@
 /****************************************************************
  *								*
- * Copyright (c) 2010-2018 Fidelity National Information	*
+ * Copyright (c) 2010-2021 Fidelity National Information	*
  * Services, Inc. and/or its subsidiaries. All rights reserved.	*
  *								*
  *	This source code contains the intellectual property	*
@@ -178,7 +178,7 @@ error_def(ERR_GTMASSERT);
 
 void gtm_threadgbl_init(void)
 {
-	void	*lcl_gtm_threadgbl;
+	void	*lcl_gtm_threadgbl;	/* DCL_THREADGBL_ACCESS */
 
 	if (SIZEOF(gtm_threadgbl_true_t) != size_gtm_threadgbl_struct)
 	{	/* Size mismatch with gtm_threadgbl_deftypes.h - no error handling yet available so do
@@ -192,7 +192,7 @@ void gtm_threadgbl_init(void)
 		FPRINTF(stderr, "GTM-F-GTMASSERT gtm_threadgbl is already initialized\n");
 		EXIT(ERR_GTMASSERT);
 	}
-	gtm_threadgbl = lcl_gtm_threadgbl = malloc(size_gtm_threadgbl_struct);
+	gtm_threadgbl = malloc(size_gtm_threadgbl_struct);
 	if (NULL == gtm_threadgbl)
 	{	/* Storage was not allocated for some reason - no error handling yet still */
 		perror("GTM-F-MEMORY Unable to allocate startup thread structure");
@@ -200,6 +200,7 @@ void gtm_threadgbl_init(void)
 	}
 	memset(gtm_threadgbl, 0, size_gtm_threadgbl_struct);
 	gtm_threadgbl_true = (gtm_threadgbl_true_t *)gtm_threadgbl;
+	lcl_gtm_threadgbl = gtm_threadgbl;	/* SETUP_THREADGBL_ACCESS */
 
 	/* Add specific initializations if other than 0s here using the TREF() family of macros: */
 	(TREF(director_ident)).addr = TADR(director_string);
