@@ -1,6 +1,7 @@
 /****************************************************************
  *								*
- *	Copyright 2001, 2013 Fidelity Information Services, Inc	*
+ * Copyright (c) 2001-2017 Fidelity National Information	*
+ * Services, Inc. and/or its subsidiaries. All rights reserved.	*
  *								*
  *	This source code contains the intellectual property	*
  *	of its copyright holder(s), and is made available	*
@@ -11,6 +12,8 @@
 
 #include "mdef.h"
 
+#include "gtm_stdio.h"
+
 #include "gtm_string.h"
 #include "gdsroot.h"
 #include "gdsblk.h"
@@ -45,6 +48,17 @@
 #include "hashtab_mname.h"
 #include "gvnh_spanreg.h"
 #include "change_reg.h"
+#include "io.h"
+#include "gtmio.h"
+
+#ifdef EXTRACT_HASHT_GLOBAL
+# include "gv_trigger_common.h"	/* for IS_GVKEY_HASHT_GBLNAME and HASHT_GBL_CHAR1 macros */
+# ifdef GTM_TRIGGER
+#  include "gv_trigger.h"
+#  include "targ_alloc.h"
+#  include "gvcst_protos.h"	/* for gvcst_root_search prototype used in GVTR_SWITCH_REG_AND_HASHT_BIND_NAME macro */
+# endif
+#endif
 
 #define	MAX_GMAP_ENTRIES_PER_ITER	2 /* maximum increase (could even be negative) in gmap array size per call to global_map */
 
@@ -89,6 +103,11 @@ void gv_select(char *cli_buff, int n_len, boolean_t freeze, char opname[], glist
 #	else
 	hash_table_int4		ext_hash;
 #	endif
+#	ifdef EXTRACT_HASHT_GLOBAL
+	gvnh_reg_t		*hashgbl_gvnh_reg = NULL;
+	gd_region		*r_top;
+	sgmnt_addrs		*csa;
+#	endif
 
 	memset(gmap, 0, SIZEOF(gmap));
 	gmap_size = SIZEOF(gmap) / SIZEOF(gmap[0]);
@@ -210,8 +229,10 @@ void gv_select(char *cli_buff, int n_len, boolean_t freeze, char opname[], glist
 	}
 	if (freeze)
 	{
-		GTM64_ONLY(init_hashtab_int8(&ext_hash, 0, HASHTAB_COMPACT, HASHTAB_SPARE_TABLE);)
-		NON_GTM64_ONLY(init_hashtab_int4(&ext_hash, 0, HASHTAB_COMPACT, HASHTAB_SPARE_TABLE);)
+		GTM64_ONLY(init_hashtab_int8(&ext_hash, gmap_size * (100.0 / HT_LOAD_FACTOR),
+					HASHTAB_COMPACT, HASHTAB_SPARE_TABLE);)
+		NON_GTM64_ONLY(init_hashtab_int4(&ext_hash, gmap_size * (100.0 / HT_LOAD_FACTOR),
+					HASHTAB_COMPACT, HASHTAB_SPARE_TABLE);)
 	}
 	gl_head->next = NULL;
 	gl_tail = gl_head;
@@ -231,9 +252,44 @@ void gv_select(char *cli_buff, int n_len, boolean_t freeze, char opname[], glist
 			 */
 			if (MAX_MIDENT_LEN < curr_gbl_name.str.len)
 				curr_gbl_name.str.len = MAX_MIDENT_LEN;
+#			if defined(GTM_TRIGGER) && defined(EXTRACT_HASHT_GLOBAL)
+			if (HASHT_GBL_CHAR1 == curr_gbl_name.str.addr[0])
+			{	/* Global names starting with "#" e.g. ^#t. Consider these as spanning across
+				 * all regions of the gbldir for the purposes of the caller (MUPIP EXTRACT
+				 * or MUPIP REORG or MUPIP SIZE).
+				 */
+				if (NULL == hashgbl_gvnh_reg)
+				{
+					hashgbl_gvnh_reg = (gvnh_reg_t *)malloc(SIZEOF(gvnh_reg_t));
+					/* initialize gvnh_reg fields to NULL specifically gvnh_reg->gvspan
+					 * as this is used by the caller of gv_select.
+					 */
+					memset(hashgbl_gvnh_reg, 0, SIZEOF(gvnh_reg_t));
+				}
+				/* At this time, only ^#t is supported. In the future ^#k or some such globals
+				 * might be supported for other purposes. The following code needs to change if/when
+				 * that happens.
+				 */
+				assert(IS_GVKEY_HASHT_GBLNAME(curr_gbl_name.str.len, curr_gbl_name.str.addr));
+				if (!gd_header)
+					gvinit();
+				for (reg = gd_header->regions, r_top = reg + gd_header->n_regions; reg < r_top; reg++)
+				{
+					if (IS_STATSDB_REGNAME(reg))
+						continue;
+					GVTR_SWITCH_REG_AND_HASHT_BIND_NAME(reg);
+					csa = cs_addrs;
+					if (NULL == csa)	/* not BG or MM access method */
+						continue;
+					gv_select_reg((void *)&ext_hash, freeze, reg_max_rec, reg_max_key,
+									reg_max_blk, restrict_reg, hashgbl_gvnh_reg, &gl_tail);
+				}
+				break;
+			}
+#			endif
 			COMPUTE_HASH_MSTR(curr_gbl_name.str, hash_code);
 			op_gvname_fast(VARLSTCNT(2) hash_code, &curr_gbl_name);
-			assert((dba_bg == REG_ACC_METH(gv_cur_region)) || (dba_mm == REG_ACC_METH(gv_cur_region)));
+			assert(IS_REG_BG_OR_MM(gv_cur_region));
 				/* for dba_cm or dba_usr, op_gvname_fast/gv_bind_name/gv_init_reg would have errored out */
 			gvname.hash_code = hash_code;
 			gvname.var_name = curr_gbl_name.str;
@@ -246,8 +302,9 @@ void gv_select(char *cli_buff, int n_len, boolean_t freeze, char opname[], glist
 				gv_select_reg((void *)&ext_hash, freeze, reg_max_rec, reg_max_key, reg_max_blk,
 										restrict_reg, gvnh_reg, &gl_tail);
 			else
-			{	/* If global spans multiple regions, make sure gv_targets corresponding to ALL spanned regions
-				 * are allocated and gv_target->root is also initialized accordingly for all the spanned regions.
+			{	/* If global spans multiple regions, make sure gv_targets corresponding to ALL
+				 * spanned regions are allocated and gv_target->root is also initialized
+				 * accordingly for all the spanned regions.
 				 */
 				gvnh_spanreg_subs_gvt_init(gvnh_reg, gd_header, NULL);
 				maxi = gvspan->max_reg_index;
@@ -264,7 +321,7 @@ void gv_select(char *cli_buff, int n_len, boolean_t freeze, char opname[], glist
 						gv_cur_region = reg;
 						change_reg();
 						gv_select_reg((void *)&ext_hash, freeze, reg_max_rec, reg_max_key,
-										reg_max_blk, restrict_reg, gvnh_reg, &gl_tail);
+									reg_max_blk, restrict_reg, gvnh_reg, &gl_tail);
 					}
 				}
 			}
@@ -326,7 +383,7 @@ STATICFNDEF void gv_select_reg(void *ext_hash, boolean_t freeze, int *reg_max_re
 	        NON_GTM64_ONLY(if (add_hashtab_int4((hash_table_int4 *)ext_hash, (uint4 *)&gv_cur_region,
 											HT_VALUE_DUMMY, &tabent)))
                 {
-                        if (cs_addrs->hdr->freeze)
+                        if (FROZEN(cs_addrs->hdr))
                         {
                                 gtm_putmsg_csa(CSA_ARG(cs_addrs) VARLSTCNT(4) ERR_FREEZE, 2, REG_LEN_STR(gv_cur_region));
                                 mupip_exit(ERR_MUNOFINISH);
@@ -339,7 +396,7 @@ STATICFNDEF void gv_select_reg(void *ext_hash, boolean_t freeze, int *reg_max_re
 					DB_LEN_STR(gv_cur_region));
 				mupip_exit(ERR_MUNOFINISH);
 			}
-			while (REG_ALREADY_FROZEN == region_freeze(gv_cur_region, TRUE, FALSE, FALSE))
+			while (REG_ALREADY_FROZEN == region_freeze(gv_cur_region, TRUE, FALSE, FALSE, FALSE, TRUE))
 			{
 				hiber_start(1000);
 				if (mu_ctrly_occurred || mu_ctrlc_occurred)
@@ -348,7 +405,6 @@ STATICFNDEF void gv_select_reg(void *ext_hash, boolean_t freeze, int *reg_max_re
                                 	mupip_exit(ERR_MUNOFINISH);
 				}
 			}
-			wcs_flu(WCSFLU_FLUSH_HDR | WCSFLU_WRITE_EPOCH | WCSFLU_SYNC_EPOCH);
                 }
         }
 	if (*reg_max_rec < cs_data->max_rec_size)
