@@ -1,6 +1,7 @@
 /****************************************************************
  *								*
- *	Copyright 2001, 2011 Fidelity Information Services, Inc	*
+ * Copyright (c) 2001-2018 Fidelity National Information	*
+ * Services, Inc. and/or its subsidiaries. All rights reserved.	*
  *								*
  *	This source code contains the intellectual property	*
  *	of its copyright holder(s), and is made available	*
@@ -9,8 +10,8 @@
  *								*
  ****************************************************************/
 
-#ifndef __GV_XFORM_KEY_H__
-#define __GV_XFORM_KEY_H__
+#ifndef GV_XFORM_KEY_H_INCLUDED
+#define GV_XFORM_KEY_H_INCLUDED
 
 void gv_xform_key(gv_key *keyp, boolean_t xback);
 
