@@ -1,6 +1,6 @@
 /****************************************************************
  *								*
- * Copyright (c) 2001-2020 Fidelity National Information	*
+ * Copyright (c) 2001-2022 Fidelity National Information	*
  * Services, Inc. and/or its subsidiaries. All rights reserved.	*
  *								*
  *	This source code contains the intellectual property	*
@@ -25,6 +25,7 @@
 #include "fileinfo.h"
 #include "gdsbt.h"
 #include "gdsfhead.h"
+#include "db_header_conversion.h"
 #include "gdsblk.h"
 #include "gdskill.h"
 #include "gdscc.h"
@@ -171,6 +172,7 @@ GBLREF	boolean_t		jnlpool_init_needed;
 GBLREF	jnlpool_addrs_ptr_t	jnlpool;
 GBLREF	jnlpool_addrs_ptr_t	jnlpool_head;
 GBLREF	uint4			process_id;
+GBLREF	int4			pre_drvlongjmp_error_condition;
 
 LITREF char			gtm_release_name[];
 LITREF int4			gtm_release_name_len;
@@ -356,7 +358,7 @@ void gvcst_init(gd_region *reg, gd_addr *addr)
 	ua_list			*tmp_ua;
 	time_t			curr_time;
 	uint4			curr_time_uint4, next_warn_uint4;
-	unsigned int            minus1 = (unsigned)-1;
+	gtm_uint8		minus1 = (gtm_uint8)-1;
 	enum db_acc_method	reg_acc_meth;
 	boolean_t		onln_rlbk_cycle_mismatch = FALSE;
 	boolean_t		replpool_valid = FALSE, replfilegdid_valid = FALSE, jnlpool_found = FALSE;
@@ -457,7 +459,7 @@ void gvcst_init(gd_region *reg, gd_addr *addr)
 			if (!ftok_sem_lock(baseDBreg, FALSE))
 			{	/* Use FTOK of the base db as a lock, the same lock obtained when statsdb is auto deleted */
 				assert(FALSE);
-				rts_error_csa(CSA_ARG(baseDBcsa) VARLSTCNT(4) ERR_DBFILERR, 2, DB_LEN_STR(baseDBreg));
+				RTS_ERROR_CSA_ABT(baseDBcsa, VARLSTCNT(4) ERR_DBFILERR, 2, DB_LEN_STR(baseDBreg));
 			}
 			ESTABLISH(gvcst_init_autoDB_ch);	/* ch reverses the DEFER and lock in the opposite order */
 			if (0 == baseDBnl->statsdb_fname_len)
@@ -500,8 +502,8 @@ void gvcst_init(gd_region *reg, gd_addr *addr)
 				assert(TREF(gvcst_statsDB_open_ch_active));	/* below error goes to syslog and not to user */
 				baseDBreg->reservedDBFlags |= RDBF_NOSTATS;	/* Disable STATS in base DB */
 				baseDBcsa->reservedDBFlags |= RDBF_NOSTATS;
-				rts_error_csa(CSA_ARG(NULL) VARLSTCNT(8) ERR_DBFILERR, 2, DB_LEN_STR(baseDBreg), ERR_STATSDBFNERR,
-					      2, errrsn_text_len, errrsn_text);
+				RTS_ERROR_CSA_ABT(NULL, VARLSTCNT(8) ERR_DBFILERR, 2, DB_LEN_STR(baseDBreg), ERR_STATSDBFNERR,
+					2, errrsn_text_len, errrsn_text);
 			}
 			COPY_STATSDB_FNAME_INTO_STATSREG(reg, baseDBnl->statsdb_fname, baseDBnl->statsdb_fname_len);
 			seg = reg->dyn.addr;
@@ -515,13 +517,18 @@ void gvcst_init(gd_region *reg, gd_addr *addr)
 				seg->blk_size = STATSDB_BLK_SIZE;
 				/* Similar code for a few other critical fields that need initialization before "mu_cre_file" */
 				seg->allocation = STATSDB_ALLOCATION;
-				seg->ext_blk_count = STATSDB_EXTENSION;
 				reg->max_key_size = STATSDB_MAX_KEY_SIZE;
 				reg->max_rec_size = STATSDB_MAX_REC_SIZE;
 				/* The below is directly inherited from the base db so no macro/assert like above fields */
 				seg->mutex_slots = NUM_CRIT_ENTRY(baseDBcsa->hdr);
 				reg->mumps_can_bypass = TRUE;
 			}
+			if (0 != baseDBcsa->hdr->statsdb_allocation)
+			{	/* The statsdb allocation for this region has been extended before.
+				 * Use the extended allocation size from the file header.
+				 */
+				seg->allocation = baseDBcsa->hdr->statsdb_allocation;
+			}
 			for ( ; ; )     /* for loop only to let us break from error cases without having a deep if-then-else */
 			{	/* with the ftok lock in place, check if the db is already created */
 				if (baseDBnl->statsdb_created)
@@ -575,6 +582,8 @@ void gvcst_init(gd_region *reg, gd_addr *addr)
 					}
 					/* Open worked - read file header from it*/
 					LSEEKREAD(fd, 0, (char *)&statsDBcsd, SIZEOF(sgmnt_data), rc);
+					if (0 == memcmp(statsDBcsd.label, V6_GDS_LABEL, GDS_LABEL_SZ - 1))
+						db_header_upconv(&statsDBcsd);
 					if (0 > rc)
 					{	/* Wasn't enough data to be a file header - not a statsDB */
 						statsdb_rcerr = STATSDB_NOTSTATSDB;
@@ -669,8 +678,8 @@ void gvcst_init(gd_region *reg, gd_addr *addr)
 					if (!ftok_sem_release(baseDBreg, FALSE, FALSE))
 					{	/* Release the lock before unwinding back */
 						assert(FALSE);
-						rts_error_csa(CSA_ARG(baseDBcsa) VARLSTCNT(4) ERR_DBFILERR, 2,
-							      DB_LEN_STR(baseDBreg));
+						RTS_ERROR_CSA_ABT(baseDBcsa, VARLSTCNT(4) ERR_DBFILERR, 2,
+							DB_LEN_STR(baseDBreg));
 					}
 					/* For those errors that need a special error message, take care of that here
 					 * now that we've released the lock.
@@ -684,50 +693,50 @@ void gvcst_init(gd_region *reg, gd_addr *addr)
 							 * associated with that baseDB. First check if this IS a statsdb.
 							 */
 							if (IS_RDBF_STATSDB(&statsDBcsd))
-								rts_error_csa(CSA_ARG(NULL) VARLSTCNT(8) ERR_STATSDBINUSE,
-									      6, DB_LEN_STR(reg),
-									      statsDBcsd.basedb_fname_len,
-									      statsDBcsd.basedb_fname,
-									      DB_LEN_STR(baseDBreg));
+								RTS_ERROR_CSA_ABT(NULL, VARLSTCNT(8) ERR_STATSDBINUSE,
+									6, DB_LEN_STR(reg),
+									statsDBcsd.basedb_fname_len,
+									statsDBcsd.basedb_fname,
+									DB_LEN_STR(baseDBreg));
 							else
-								rts_error_csa(CSA_ARG(NULL) VARLSTCNT(6) ERR_INVSTATSDB,
-									      4, DB_LEN_STR(reg), REG_LEN_STR(reg));
+								RTS_ERROR_CSA_ABT(NULL, VARLSTCNT(6) ERR_INVSTATSDB,
+									4, DB_LEN_STR(reg), REG_LEN_STR(reg));
 							break;			/* For the compiler */
 						case STATSDB_OPNERR:
-							rts_error_csa(CSA_ARG(NULL) VARLSTCNT(5) ERR_DBOPNERR, 2,
-								      DB_LEN_STR(reg), save_errno);
+							RTS_ERROR_CSA_ABT(NULL, VARLSTCNT(5) ERR_DBOPNERR, 2,
+								DB_LEN_STR(reg), save_errno);
 							break;			/* For the compiler */
 						case STATSDB_READERR:
-							rts_error_csa(CSA_ARG(NULL) VARLSTCNT(5) ERR_DBFILERR, 2,
-								      DB_LEN_STR(reg), save_errno);
+							RTS_ERROR_CSA_ABT(NULL, VARLSTCNT(5) ERR_DBFILERR, 2,
+								DB_LEN_STR(reg), save_errno);
 							break;			/* For the compiler */
 						case STATSDB_NOTSTATSDB:
-							rts_error_csa(CSA_ARG(NULL) VARLSTCNT(6) ERR_INVSTATSDB, 4,
-								      DB_LEN_STR(reg), REG_LEN_STR(reg));
+							RTS_ERROR_CSA_ABT(NULL, VARLSTCNT(6) ERR_INVSTATSDB, 4,
+								DB_LEN_STR(reg), REG_LEN_STR(reg));
 							break;			/* For the compiler */
 						case STATSDB_UNLINKERR:
-							rts_error_csa(CSA_ARG(NULL) VARLSTCNT(8) ERR_SYSCALL, 5,
-								      LEN_AND_LIT("unlink()"), CALLFROM, save_errno);
+							RTS_ERROR_CSA_ABT(NULL, VARLSTCNT(8) ERR_SYSCALL, 5,
+								LEN_AND_LIT("unlink()"), CALLFROM, save_errno);
 							break;			/* For the compiler */
 						case STATSDB_RECREATEERR:
-							rts_error_csa(CSA_ARG(NULL) VARLSTCNT(5) ERR_DBOPNERR, 2,
-								      DB_LEN_STR(reg), save_errno);
+							RTS_ERROR_CSA_ABT(NULL, VARLSTCNT(5) ERR_DBOPNERR, 2,
+								DB_LEN_STR(reg), save_errno);
 							break;			/* For the compiler */
 						case STATSDB_CLOSEERR:
-							rts_error_csa(CSA_ARG(NULL) VARLSTCNT(8) ERR_SYSCALL, 5,
-								      LEN_AND_LIT("close()"), CALLFROM, save_errno);
+							RTS_ERROR_CSA_ABT(NULL, VARLSTCNT(8) ERR_SYSCALL, 5,
+								LEN_AND_LIT("close()"), CALLFROM, save_errno);
 							break;			/* For the compiler */
 						case STATSDB_SHMRMIDERR:
-							rts_error_csa(CSA_ARG(NULL) VARLSTCNT(8) ERR_SYSCALL, 5,
-								      LEN_AND_LIT("shm_rmid()"), CALLFROM, save_errno);
+							RTS_ERROR_CSA_ABT(NULL, VARLSTCNT(8) ERR_SYSCALL, 5,
+								LEN_AND_LIT("shm_rmid()"), CALLFROM, save_errno);
 							break;			/* For the compiler */
 						case STATSDB_SEMRMIDERR:
-							rts_error_csa(CSA_ARG(NULL) VARLSTCNT(8) ERR_SYSCALL, 5,
-								      LEN_AND_LIT("sem_rmid()"), CALLFROM, save_errno);
+							RTS_ERROR_CSA_ABT(NULL, VARLSTCNT(8) ERR_SYSCALL, 5,
+								LEN_AND_LIT("sem_rmid()"), CALLFROM, save_errno);
 							break;			/* For the compiler */
 						case STATSDB_FTOKSEMRMIDERR:
-							rts_error_csa(CSA_ARG(NULL) VARLSTCNT(8) ERR_SYSCALL, 5,
-								      LEN_AND_LIT("ftok sem_rmid()"), CALLFROM, save_errno);
+							RTS_ERROR_CSA_ABT(NULL, VARLSTCNT(8) ERR_SYSCALL, 5,
+								LEN_AND_LIT("ftok sem_rmid()"), CALLFROM, save_errno);
 							break;			/* For the compiler */
 						default:
 							assertpro(FALSE);
@@ -736,13 +745,14 @@ void gvcst_init(gd_region *reg, gd_addr *addr)
 					{	/* Unwind back to ESTABLISH_NORET where did gvcst_init() call to open this
 						 * statsDB which now won't open.
 						 */
-						rts_error_csa(CSA_ARG(NULL) VARLSTCNT(1) ERR_DRVLONGJMP);
+						pre_drvlongjmp_error_condition = error_condition;
+						RTS_ERROR_CSA_ABT(NULL, VARLSTCNT(1) ERR_DRVLONGJMP);
 					} else
 					{	/* We are not nested so can give the appropriate error ourselves */
-						rts_error_csa(CSA_ARG(NULL) VARLSTCNT(8) ERR_DBFILERR, 2,
-							      DB_LEN_STR(reg), ERR_TEXT, 2,
-							      RTS_ERROR_TEXT("See preceding errors written to syserr"
-									     " and/or syslog for details"));
+						RTS_ERROR_CSA_ABT(NULL, VARLSTCNT(8) ERR_DBFILERR, 2,
+							DB_LEN_STR(reg), ERR_TEXT, 2,
+							RTS_ERROR_TEXT("See preceding errors written to syserr"
+							" and/or syslog for details"));
 					}
 				}
 				break;
@@ -751,7 +761,7 @@ void gvcst_init(gd_region *reg, gd_addr *addr)
 			if (!ftok_sem_release(baseDBreg, FALSE, FALSE))
 			{
 				assert(FALSE);
-				rts_error_csa(CSA_ARG(baseDBcsa) VARLSTCNT(4) ERR_DBFILERR, 2, DB_LEN_STR(baseDBreg));
+				RTS_ERROR_CSA_ABT(baseDBcsa, VARLSTCNT(4) ERR_DBFILERR, 2, DB_LEN_STR(baseDBreg));
 			}
 			ENABLE_INTERRUPTS(INTRPT_IN_GVCST_INIT, prev_intrpt_state);
 		} else
@@ -762,12 +772,17 @@ void gvcst_init(gd_region *reg, gd_addr *addr)
 			{	/* see comment above on why we this initialization for a statsDB */
 				seg->blk_size = STATSDB_BLK_SIZE;
 				seg->allocation = STATSDB_ALLOCATION;
-				seg->ext_blk_count = STATSDB_EXTENSION;
 				reg->max_key_size = STATSDB_MAX_KEY_SIZE;
 				reg->max_rec_size = STATSDB_MAX_REC_SIZE;
 				seg->mutex_slots = NUM_CRIT_ENTRY(baseDBcsa->hdr);
 				reg->mumps_can_bypass = TRUE;
 			}
+			if (0 != baseDBcsa->hdr->statsdb_allocation)
+			{	/* The statsdb allocation for this region has been extended before.
+				 * Use the extended allocation size from the file header.
+				 */
+				seg->allocation = baseDBcsa->hdr->statsdb_allocation;
+			}
 		}
 	}
 #	ifdef DEBUG
@@ -783,7 +798,7 @@ void gvcst_init(gd_region *reg, gd_addr *addr)
 	assert(SIZEOF(th_rec) == (SIZEOF(bt_rec) - SIZEOF(bt->blkque)));
 	assert(SIZEOF(cache_rec) == (SIZEOF(cache_state_rec) + SIZEOF(cr->blkque)));
 	DEBUG_ONLY(assert_jrec_member_offsets());
-	assert(MAX_DB_BLK_SIZE < (1 << NEXT_OFF_MAX_BITS));	/* Ensure a off_chain record's next_off member
+	assert(MAX_DB_BLK_SIZE < (1ULL << NEXT_OFF_MAX_BITS));	/* Ensure a off_chain record's next_off member
 								 * can work with all possible block sizes */
 	set_num_additional_processors();
 #	ifdef DEBUG
@@ -816,8 +831,6 @@ void gvcst_init(gd_region *reg, gd_addr *addr)
 	 *	th_base (SIZEOF(que_ent) into an odd bt_rec)
 	 *	bt_base
 	 *	(n_bts * bt_rec)
-	 *	LOCK_BLOCK (lock_space)
-	 *	(lock_space_size)
 	 *	cs_addrs->acc_meth.bg.cache_state
 	 *	(cache_que_heads)
 	 *	(bt_buckets * cache_rec)
@@ -828,10 +841,10 @@ void gvcst_init(gd_region *reg, gd_addr *addr)
 	 *	(node_local)
 	 *	[jnl_name
 	 *	jnl_buffer]
-	 * MM
-	 *	file contents
 	 *	LOCK_BLOCK (lock_space)
 	 *	(lock_space_size)
+	 * MM
+	 *	file contents
 	 *	cs_addrs->acc_meth.mm.mmblk_state
 	 *	(mmblk_que_heads)
 	 *	(bt_buckets * mmblk_rec)
@@ -842,6 +855,8 @@ void gvcst_init(gd_region *reg, gd_addr *addr)
 	 *	(node_local)
 	 *	[jnl_name
 	 *	jnl_buffer]
+	 *	LOCK_BLOCK (lock_space)
+	 *	(lock_space_size)
 	 * high address
 	 */
  	/* Ensure first 3 members (upto now_running) of node_local are at the same offset for any version.
@@ -1010,7 +1025,7 @@ void gvcst_init(gd_region *reg, gd_addr *addr)
 	{
 		assert(FALSE);	/* we don't know of a practical way to get errors in each of the for-loop attempts above */
 		/* "db_init" returned with an unexpected error. Issue a generic error to note this out-of-design state */
-		rts_error_csa(CSA_ARG(NULL) VARLSTCNT(6) ERR_REGOPENFAIL, 4, REG_LEN_STR(reg), DB_LEN_STR(reg));
+		RTS_ERROR_CSA_ABT(NULL, VARLSTCNT(6) ERR_REGOPENFAIL, 4, REG_LEN_STR(reg), DB_LEN_STR(reg));
 	}
 	/* At this point, we have initialized the database, but haven't yet set reg->open to TRUE. If any rts_errors happen in
 	 * the meantime, there are no condition handlers established to handle the rts_error. More importantly, it is non-trivial
@@ -1052,7 +1067,7 @@ void gvcst_init(gd_region *reg, gd_addr *addr)
 	if (!csd->fully_upgraded && curr_time_uint4 > next_warn_uint4
 	    && COMPSWAP_LOCK(&csd->next_upgrd_warn.time_latch, next_warn_uint4, 0, (curr_time_uint4 + UPGRD_WARN_INTERVAL), 0))
 	{	/* The msg is due and we have successfully updated the next time interval */
-		if (GDSVCURR != csd->desired_db_format)
+		if (GDSVCURR != csd->desired_db_format)	/* TODO: what should this check? */
 			send_msg_csa(CSA_ARG(csa) VARLSTCNT(4) ERR_DBVERPERFWARN1, 2, DB_LEN_STR(reg));
 		else
 			send_msg_csa(CSA_ARG(csa) VARLSTCNT(4) ERR_DBVERPERFWARN2, 2, DB_LEN_STR(reg));
