@@ -1,6 +1,7 @@
 /****************************************************************
  *								*
- *	Copyright 2001 Sanchez Computer Associates, Inc.	*
+ * Copyright (c) 2001-2020 Fidelity National Information	*
+ * Services, Inc. and/or its subsidiaries. All rights reserved.	*
  *								*
  *	This source code contains the intellectual property	*
  *	of its copyright holder(s), and is made available	*
@@ -12,6 +13,6 @@
 #ifndef GVCST_MAP_BUILD_INCLUDED
 #define GVCST_MAP_BUILD_INCLUDED
 
-void gvcst_map_build(uint4 *array, sm_uc_ptr_t base_addr, cw_set_element *cs, trans_num ctn);
+void gvcst_map_build(block_id *array, sm_uc_ptr_t base_addr, cw_set_element *cs, trans_num ctn);
 
 #endif /* GVCST_MAP_BUILD_INCLUDED */
