@@ -1,6 +1,6 @@
 /****************************************************************
  *								*
- * Copyright (c) 2016-2020 Fidelity National Information	*
+ * Copyright (c) 2016-2022 Fidelity National Information	*
  * Services, Inc. and/or its subsidiaries. All rights reserved.	*
  *								*
  *	This source code contains the intellectual property	*
@@ -61,6 +61,7 @@ GBLREF boolean_t	created_core;
 GBLREF boolean_t	dont_want_core;
 GBLREF gd_addr		*gd_header;
 GBLREF mstr		extnam_str;
+GBLREF int4		pre_drvlongjmp_error_condition;
 DEBUG_ONLY(GBLREF boolean_t    ok_to_UNWIND_in_exit_handling;)
 
 STATICDEF intrpt_state_t	gvcst_statsDB_open_ch_intrpt_ok_state;
@@ -204,8 +205,8 @@ void gvcst_init_statsDB(gd_region *baseDBreg, boolean_t do_statsdb_init)
 				 * stats in the gld (OPEN_BASEREG_IF_STATSREG) use this opportunity to raise an error.
 				 */
 				if (!statsDBcsa->orig_read_write)
-					rts_error_csa(CSA_ARG(statsDBcsa) VARLSTCNT(4) ERR_DBPRIVERR, 2,
-											DB_LEN_STR(statsDBreg_located));
+					RTS_ERROR_CSA_ABT(statsDBcsa, VARLSTCNT(4) ERR_DBPRIVERR, 2,
+						DB_LEN_STR(statsDBreg_located));
 			}
 			break;
 		}
@@ -339,7 +340,7 @@ void gvcst_init_statsDB(gd_region *baseDBreg, boolean_t do_statsdb_init)
 		statsDBcsa = &FILE_INFO(statsDBreg)->s_addrs;
 		if (!statsDBcsa->orig_read_write)
 			/* Database was opened read/only as this process has no privs to write to it - raise error */
-			rts_error_csa(CSA_ARG(statsDBcsa) VARLSTCNT(4) ERR_DBPRIVERR, 2, DB_LEN_STR(statsDBreg));
+			RTS_ERROR_CSA_ABT(statsDBcsa, VARLSTCNT(4) ERR_DBPRIVERR, 2, DB_LEN_STR(statsDBreg));
 		if (!statsDBcsa->statsDB_setup_completed)
 		{	/* If initialization was never completed, do it now */
 			assert(IS_STATSDB_REG(statsDBreg));
@@ -403,7 +404,6 @@ void gvcst_init_statsDB(gd_region *baseDBreg, boolean_t do_statsdb_init)
 	 */
 	if (longjmp_done2 || longjmp_done1)
 	{
-		assert(0 != error_condition);
 		/* Check if we got a TPRETRY (an internal error) in "gvcst_init" above. If so drive parent condition handler to
 		 * trigger restart. For any other error conditions, do not do anything more as we want statsdb open to silently
 		 * switch to nostats in that case.
@@ -427,6 +427,8 @@ void gvcst_init_statsDB(gd_region *baseDBreg, boolean_t do_statsdb_init)
 			baseDBreg->reservedDBFlags |= RDBF_NOSTATS;
 			baseDBcsa->reservedDBFlags |= RDBF_NOSTATS;
 		}
+		if (ERR_DRVLONGJMP == error_condition)
+			SET_ERROR_CONDITION(pre_drvlongjmp_error_condition);
 	}
 	return;
 }
