@@ -1,6 +1,6 @@
 /****************************************************************
  *								*
- * Copyright (c) 2001-2019 Fidelity National Information	*
+ * Copyright (c) 2001-2022 Fidelity National Information	*
  * Services, Inc. and/or its subsidiaries. All rights reserved.	*
  *								*
  *	This source code contains the intellectual property	*
@@ -27,7 +27,8 @@
 #include <rtnhdr.h>
 #include "stack_frame.h"
 #include "mv_stent.h"
-#include "outofband.h"
+#include "have_crit.h"
+#include "deferred_events_queue.h"
 #include "gtm_netdb.h"
 #include "gtm_ipv6.h"
 #include "gtm_unistd.h"
@@ -36,7 +37,6 @@
 
 #define	ESTABLISHED	"ESTABLISHED"
 
-GBLREF	boolean_t		dollar_zininterrupt;
 GBLREF	d_socket_struct		*newdsocket;	/* in case jobinterrupt */
 GBLREF	int			socketus_interruptus;
 GBLREF	int4			gtm_max_sockets;
@@ -44,6 +44,7 @@ GBLREF	mv_stent		*mv_chain;
 GBLREF	stack_frame		*frame_pointer;
 GBLREF	unsigned char		*stackbase, *stacktop, *msp, *stackwarn;
 GBLREF	volatile int4		outofband;
+GBLREF	volatile boolean_t	dollar_zininterrupt;
 
 error_def(ERR_GETNAMEINFO);
 error_def(ERR_GETSOCKNAMERR);
@@ -51,8 +52,6 @@ error_def(ERR_GETSOCKOPTERR);
 error_def(ERR_OPENCONN);
 error_def(ERR_SETSOCKOPTERR);
 error_def(ERR_SOCKINIT);
-error_def(ERR_STACKCRIT);
-error_def(ERR_STACKOFLOW);
 error_def(ERR_TEXT);
 error_def(ERR_ZINTRECURSEIO);
 
@@ -63,7 +62,8 @@ boolean_t iosocket_connect(socket_struct *sockptr, int4 msec_timeout, boolean_t
 	int4		errlen, last_errno, save_errno;
 	int		d_socket_struct_len, res, nfds, sockerror;
 	fd_set		writefds;
-	boolean_t	no_time_left = FALSE;
+	boolean_t	no_time_left = FALSE, first_time = TRUE, timer_started = FALSE, oneshot;
+	boolean_t	timer_done;	/* for TIMEOUT_INIT */
 	boolean_t	need_connect, need_socket, need_select;
 	short		len;
 	io_desc		*iod;
@@ -73,6 +73,7 @@ boolean_t iosocket_connect(socket_struct *sockptr, int4 msec_timeout, boolean_t
 	struct timeval	*sel_time;
 	mv_stent	*mv_zintdev;
 	struct addrinfo *remote_ai_ptr, *raw_ai_ptr, *local_ai_ptr;
+	intrpt_state_t	prev_intrpt_state;
 	int		errcode, real_errno;
 	char		ipaddr[SA_MAXLEN + 1];
 	char		port_buffer[NI_MAXSERV];
@@ -130,6 +131,7 @@ boolean_t iosocket_connect(socket_struct *sockptr, int4 msec_timeout, boolean_t
 		add_int_to_abs_time(&cur_time, msec_timeout, &end_time);
 	}
 	real_sockintr->end_time_valid = sockintr->end_time_valid = FALSE;
+	oneshot = (0 == msec_timeout);
 	last_errno = 0;
 
 	remote_ai_ptr = (struct addrinfo*)(&(sockptr->remote.ai));
@@ -168,9 +170,12 @@ boolean_t iosocket_connect(socket_struct *sockptr, int4 msec_timeout, boolean_t
 			{
 				if (NULL != sockptr->remote.ai_head)
 				{
+					DEFER_INTERRUPTS(INTRPT_IN_FUNC_WITH_MALLOC, prev_intrpt_state);
 					freeaddrinfo(sockptr->remote.ai_head);
+					ENABLE_INTERRUPTS(INTRPT_IN_FUNC_WITH_MALLOC, prev_intrpt_state);
 					sockptr->remote.ai_head = NULL;
 				}
+				SOCKET_FREE(sockptr);
 				assertpro(-2 != real_errno);
 				errptr = (char *)STRERROR(real_errno);
 				errlen = STRLEN(errptr);
@@ -181,6 +186,7 @@ boolean_t iosocket_connect(socket_struct *sockptr, int4 msec_timeout, boolean_t
 			if (remote_ai_ptr != raw_ai_ptr)
 				memcpy(remote_ai_ptr, raw_ai_ptr, SIZEOF(struct addrinfo));
 			need_socket = FALSE;
+			first_time = TRUE;	/* for initial connect */
 			temp_1 = 1;
 			if (socket_local != sockptr->protocol)
 			{
@@ -198,9 +204,12 @@ boolean_t iosocket_connect(socket_struct *sockptr, int4 msec_timeout, boolean_t
 					sockptr->sd = FD_INVALID;
 					if (NULL != sockptr->remote.ai_head)
 					{
+						DEFER_INTERRUPTS(INTRPT_IN_FUNC_WITH_MALLOC, prev_intrpt_state);
 						freeaddrinfo(sockptr->remote.ai_head);
+						ENABLE_INTERRUPTS(INTRPT_IN_FUNC_WITH_MALLOC, prev_intrpt_state);
 						sockptr->remote.ai_head = NULL;
 					}
+					SOCKET_FREE(sockptr);
 					rts_error_csa(CSA_ARG(NULL) VARLSTCNT(7) ERR_SETSOCKOPTERR, 5,
 						  RTS_ERROR_LITERAL("SO_REUSEADDR"), save_errno, errlen, errptr);
 					return FALSE;
@@ -217,9 +226,12 @@ boolean_t iosocket_connect(socket_struct *sockptr, int4 msec_timeout, boolean_t
 					sockptr->sd = FD_INVALID;
 					if (NULL != sockptr->remote.ai_head)
 					{
+						DEFER_INTERRUPTS(INTRPT_IN_FUNC_WITH_MALLOC, prev_intrpt_state);
 						freeaddrinfo(sockptr->remote.ai_head);
+						ENABLE_INTERRUPTS(INTRPT_IN_FUNC_WITH_MALLOC, prev_intrpt_state);
 						sockptr->remote.ai_head = NULL;
 					}
+					SOCKET_FREE(sockptr);
 					rts_error_csa(CSA_ARG(NULL) VARLSTCNT(7) ERR_SETSOCKOPTERR, 5,
 						RTS_ERROR_LITERAL("TCP_NODELAY"), save_errno, errlen, errptr);
 					return FALSE;
@@ -237,13 +249,17 @@ boolean_t iosocket_connect(socket_struct *sockptr, int4 msec_timeout, boolean_t
 						sockptr->sd = FD_INVALID;
 						if (NULL != sockptr->remote.ai_head)
 						{
+							DEFER_INTERRUPTS(INTRPT_IN_FUNC_WITH_MALLOC, prev_intrpt_state);
 							freeaddrinfo(sockptr->remote.ai_head);
+							ENABLE_INTERRUPTS(INTRPT_IN_FUNC_WITH_MALLOC, prev_intrpt_state);
 							sockptr->remote.ai_head = NULL;
 						}
+						SOCKET_FREE(sockptr);
 						rts_error_csa(CSA_ARG(NULL) VARLSTCNT(7) ERR_SETSOCKOPTERR, 5,
 							RTS_ERROR_LITERAL("SO_RCVBUF"), save_errno, errlen, errptr);
 						return FALSE;
 					}
+					sockptr->options_state.rcvbuf |= SOCKOPTIONS_USER;
 				} else
 				{
 					sockbuflen = SIZEOF(sockptr->bufsiz);
@@ -257,13 +273,25 @@ boolean_t iosocket_connect(socket_struct *sockptr, int4 msec_timeout, boolean_t
 						sockptr->sd = FD_INVALID;
 						if (NULL != sockptr->remote.ai_head)
 						{
+							DEFER_INTERRUPTS(INTRPT_IN_FUNC_WITH_MALLOC, prev_intrpt_state);
 							freeaddrinfo(sockptr->remote.ai_head);
+							ENABLE_INTERRUPTS(INTRPT_IN_FUNC_WITH_MALLOC, prev_intrpt_state);
 							sockptr->remote.ai_head = NULL;
 						}
+						SOCKET_FREE(sockptr);
 						rts_error_csa(CSA_ARG(NULL) VARLSTCNT(7) ERR_GETSOCKOPTERR, 5,
 							RTS_ERROR_LITERAL("SO_RCVBUF"), save_errno, errlen, errptr);
 						return FALSE;
 					}
+					sockptr->options_state.rcvbuf |= SOCKOPTIONS_SYSTEM;
+				}
+				if (0 != (SOCKOPTIONS_PENDING & sockptr->options_state.sndbuf))
+				{
+					if (-1 == iosocket_setsockopt(sockptr, "SO_SNDBUF", SO_SNDBUF, SOL_SOCKET,
+						&sockptr->iobfsize, sizeof(sockptr->iobfsize), TRUE))
+						return FALSE;
+					sockptr->options_state.sndbuf |= SOCKOPTIONS_USER;
+					sockptr->options_state.sndbuf &= ~SOCKOPTIONS_PENDING;
 				}
 			}
 		}
@@ -272,11 +300,29 @@ boolean_t iosocket_connect(socket_struct *sockptr, int4 msec_timeout, boolean_t
 		{
 			/* Use plain connect to allow jobinterrupt */
 			assert(FD_INVALID != sockptr->sd);
+			if (first_time && (NO_M_TIMEOUT != msec_timeout))
+			{	/* start timer if not end_time, if timeout is 0 just a short wait */
+				DBGSOCK((stdout, "socconn: if first_time: msec_timeout = %d\n", msec_timeout));
+				sys_get_curr_time(&cur_time);
+				cur_time = sub_abs_time(&end_time, &cur_time);	/* calculate remaining time */
+				if (0 != msec_timeout)
+					msec_timeout = (int4)((cur_time.at_sec * MILLISECS_IN_SEC) +
+					/* Round up in order to prevent premature timeouts */
+						DIVIDE_ROUND_UP(cur_time.at_usec, MICROSECS_IN_MSEC));
+				if (0 >= msec_timeout)
+				{
+					no_time_left = TRUE;
+					msec_timeout = 100 /* enough time to connect if no problems */;
+					DBGSOCK((stdout, "socconn: first_time msec_timeout was 0, now = %d\n", msec_timeout));
+				}
+				first_time = FALSE;
+				timer_started = TRUE;
+				TIMEOUT_INIT(timer_done, msec_timeout);
+			 }
 			res = connect(sockptr->sd, SOCKET_REMOTE_ADDR(sockptr), remote_ai_ptr->ai_addrlen);	 /* BYPASSOK */
 			if (res < 0)
 			{
 				save_errno = errno;
-				no_time_left = FALSE;
 				need_connect = TRUE;
 				switch (save_errno)
 				{
@@ -286,7 +332,9 @@ boolean_t iosocket_connect(socket_struct *sockptr, int4 msec_timeout, boolean_t
 						need_connect = FALSE;
 						break;
 					case EINTR:
-						if (outofband && 0 != msec_timeout)
+						DBGSOCK((stdout, "socconn: EINTR: outofband = %d, oneshot = %d\n",
+							outofband, oneshot));
+						if (outofband && !oneshot)
 						{	/* handle outofband unless zero timeout */
 							save_errno = 0;
 							need_socket = need_connect = FALSE;
@@ -295,23 +343,28 @@ boolean_t iosocket_connect(socket_struct *sockptr, int4 msec_timeout, boolean_t
 					case EINPROGRESS:
 					case EALREADY:
 						need_socket = need_connect = FALSE;
-						if (0 != msec_timeout)
+						if (!oneshot)
+						{
+							DBGSOCK((stdout, "socconn: EINPROGESS: errno = %d\n", save_errno));
 							need_select = TRUE;
+						}
 					/* fall through */
 					case ETIMEDOUT:	/* the other side bound but not listening */
 					case ECONNREFUSED:
 					case ENOENT:    /* LOCAL socket not there */
-						if (!no_time_left && (0 != msec_timeout) && (NO_M_TIMEOUT != msec_timeout))
+						if (!no_time_left && (!oneshot) && (NO_M_TIMEOUT != msec_timeout))
 						{
 							sys_get_curr_time(&cur_time);
 							cur_time = sub_abs_time(&end_time, &cur_time);
+							DBGSOCK((stdout, "socconn: ENOENT: errno = %d, msec_timeout= %d\n",
+								save_errno, msec_timeout));
 							msec_timeout = (int4)(cur_time.at_sec * MILLISECS_IN_SEC +
 								/* Round up in order to prevent premature timeouts */
 								DIVIDE_ROUND_UP(cur_time.at_usec, MICROSECS_IN_MSEC));
 							if (0 >= msec_timeout)
 								msec_timeout = 0;
 						}
-						if (0 == msec_timeout)
+						if (oneshot || (0 == msec_timeout))
 							no_time_left = TRUE;
 						else if (!no_time_left)
 						{
@@ -334,6 +387,11 @@ boolean_t iosocket_connect(socket_struct *sockptr, int4 msec_timeout, boolean_t
 				}
 			} /* if connect failed */
 		}
+		if (timer_started)
+		{
+			timer_started = FALSE;
+			TIMEOUT_DONE(timer_done);
+		}
 		if (need_select)
 		{
 			sockerror = 0;
@@ -421,17 +479,22 @@ boolean_t iosocket_connect(socket_struct *sockptr, int4 msec_timeout, boolean_t
 			}
 			if (NULL != sockptr->remote.ai_head)
 			{
+				DEFER_INTERRUPTS(INTRPT_IN_FUNC_WITH_MALLOC, prev_intrpt_state);
 				freeaddrinfo(sockptr->remote.ai_head);
+				ENABLE_INTERRUPTS(INTRPT_IN_FUNC_WITH_MALLOC, prev_intrpt_state);
 				sockptr->remote.ai_head = NULL;
 			}
 			errptr = (char *)STRERROR(save_errno);
 			if (dev_open == iod->state)
 			{
-				iod->dollar.za = 9;
+				iod->dollar.za = ZA_IO_ERR;
 				SET_DOLLARDEVICE_ONECOMMA_ERRSTR(dsocketptr->iod, errptr, errlen);
 			}
 			if (sockptr->ioerror)
-				rts_error_csa(CSA_ARG(NULL) VARLSTCNT(6) ERR_OPENCONN, 0, ERR_TEXT, 2, errlen, errptr);
+			{
+				SOCKET_FREE(sockptr);
+				RTS_ERROR_CSA_ABT(NULL, VARLSTCNT(6) ERR_OPENCONN, 0, ERR_TEXT, 2, errlen, errptr);
+			}
 			errno = save_errno;
 			return FALSE;
 		}
@@ -439,10 +502,18 @@ boolean_t iosocket_connect(socket_struct *sockptr, int4 msec_timeout, boolean_t
 		{
 			if (NULL != sockptr->remote.ai_head)
 			{
+				DEFER_INTERRUPTS(INTRPT_IN_FUNC_WITH_MALLOC, prev_intrpt_state);
 				freeaddrinfo(sockptr->remote.ai_head);
+				ENABLE_INTERRUPTS(INTRPT_IN_FUNC_WITH_MALLOC, prev_intrpt_state);
 				sockptr->remote.ai_head = NULL;
 			}
-			return FALSE;	/* caller will close socket */
+			if (!need_socket)
+			{
+				close(sockptr->sd);
+				sockptr->sd = FD_INVALID;
+			}
+			SOCKET_FREE(sockptr);
+			return FALSE;
 		}
 		if (res < 0 && outofband)	/* if connected delay outofband */
 		{
@@ -455,7 +526,15 @@ boolean_t iosocket_connect(socket_struct *sockptr, int4 msec_timeout, boolean_t
 					close(sockptr->sd);
 					sockptr->sd = FD_INVALID;
 				}
-				outofband_action(FALSE);
+				if (NULL != sockptr->remote.ai_head)
+				{
+					DEFER_INTERRUPTS(INTRPT_IN_FUNC_WITH_MALLOC, prev_intrpt_state);
+					freeaddrinfo(sockptr->remote.ai_head);
+					ENABLE_INTERRUPTS(INTRPT_IN_FUNC_WITH_MALLOC, prev_intrpt_state);
+					sockptr->remote.ai_head = NULL;
+				}
+				SOCKET_FREE(sockptr);
+				async_action(FALSE);
 				assertpro(FALSE);
 			}
 			if (need_connect)
@@ -490,8 +569,8 @@ boolean_t iosocket_connect(socket_struct *sockptr, int4 msec_timeout, boolean_t
 			socketus_interruptus++;
 			DBGSOCK((stdout, "socconn: mv_stent queued - endtime: %d/%d  interrupts: %d\n",
 				 end_time.at_sec, end_time.at_usec, socketus_interruptus));
-			outofband_action(FALSE);
-			assertpro(FALSE);      /* Should *never* return from outofband_action */
+			async_action(FALSE);
+			assertpro(FALSE);      /* Should *never* return from async_action */
 			return FALSE;   /* For the compiler.. */
 		}
 		hiber_start(100);
@@ -509,7 +588,9 @@ boolean_t iosocket_connect(socket_struct *sockptr, int4 msec_timeout, boolean_t
 	assert(FALSE == need_socket);
 	if (NULL != sockptr->remote.ai_head)
 	{
+		DEFER_INTERRUPTS(INTRPT_IN_FUNC_WITH_MALLOC, prev_intrpt_state);
 		freeaddrinfo(sockptr->remote.ai_head);
+		ENABLE_INTERRUPTS(INTRPT_IN_FUNC_WITH_MALLOC, prev_intrpt_state);
 		sockptr->remote.ai_head = NULL;
 	}
 	local_ai_ptr = &(sockptr->local.ai);
@@ -562,9 +643,14 @@ boolean_t iosocket_connect(socket_struct *sockptr, int4 msec_timeout, boolean_t
 		}
 		STRNDUP(ipaddr, SA_MAXLEN, sockptr->remote.saddr_ip);
 		strncpy(&iod->dollar.key[len], sockptr->remote.saddr_ip, DD_BUFLEN - 1 - len);
-		keepalive_opt = TREF(gtm_socket_keepalive_idle);	/* deviceparameter would give more granular control */
-		if (keepalive_opt && !iosocket_tcp_keepalive(sockptr, keepalive_opt, action))
-			return FALSE;				/* iosocket_tcp_keepalive issues rts_error rather than return */
+	if ((SOCKOPTIONS_PENDING & sockptr->options_state.alive)
+		|| (SOCKOPTIONS_PENDING & sockptr->options_state.cnt)
+		|| (SOCKOPTIONS_PENDING & sockptr->options_state.intvl))
+		keepalive_opt = SOCKOPTIONS_FROM_STRUCT;
+	else
+		keepalive_opt = TREF(gtm_socket_keepalive_idle);	/* deviceparameter takes precedence */
+	if (keepalive_opt && !iosocket_tcp_keepalive(sockptr, keepalive_opt, action, TRUE))
+		return FALSE;		/* iosocket_tcp_keepalive issues rts_error rather than return */
 	} else
 	{	/* getsockname does not return info for AF_UNIX connected socket so copy from remote side */
 		local_ai_ptr->ai_socktype = sockptr->remote.ai.ai_socktype;
