@@ -1,6 +1,6 @@
 /****************************************************************
  *								*
- * Copyright (c) 2001-2019 Fidelity National Information	*
+ * Copyright (c) 2001-2022 Fidelity National Information	*
  * Services, Inc. and/or its subsidiaries. All rights reserved.	*
  *								*
  *	This source code contains the intellectual property	*
@@ -35,6 +35,21 @@
 #define TCP_WRITE		1
 #define TCP_READ		2
 
+/* bits for readyforwhat in socket_struct */
+#define SOCKREADY_READ		1
+#define SOCKREADY_WRITE		2
+
+/* bits for pendingevent in socket_struct */
+#define SOCKPEND_READ		1	/* from poll */
+#define SOCKPEND_WRITE		2
+#define SOCKPEND_BUFFER		4	/* something in input buffer */
+
+/* bits for options_state in socket_struct */
+#define SOCKOPTIONS_SYSTEM	1	/* value from getsockopt */
+#define SOCKOPTIONS_USER	2	/* value specified by user */
+#define SOCKOPTIONS_PENDING	4	/* setsockopt needed */
+#define	SOCKOPTIONS_FROM_STRUCT	-1	/* flag for iosocket_tcp_keepalive.c */
+
 typedef struct
 {
 	uid_t   mem;
@@ -49,14 +64,29 @@ typedef struct
  */
 /* #define DEBUG_SOCK */
 #ifdef DEBUG_SOCK
+#  include "gtmio.h"
 #  define DBGSOCK(X) DBGFPF(X)
 #  define DBGSOCK_ONLY(X) X
 #else
 #  define DBGSOCK(X)
 #  define DBGSOCK_ONLY(X)
 #endif
-#define DBGSOCK2(X)
-#define DBGSOCK_ONLY2(X)
+#ifdef DEBUG_SOCKWAIT
+#  define DBGSOCKWAIT(X) DBGFPF(X)
+#  define DBGSOCKWAIT_ONLY(X) X
+#else
+#  define DBGSOCKWAIT(X)
+#  define DBGSOCKWAIT_ONLY(X)
+#endif
+/* #define DEBUG_SOCK2 */
+#ifdef DEBUG_SOCK2
+#  include "gtmio.h"
+#  define DBGSOCK2(X) DBGFPF(X)
+#  define DBGSOCK_ONLY2(X) X
+#else
+#  define DBGSOCK2(X)
+#  define DBGSOCK_ONLY2(X)
+#endif
 
 /* About the length of the delimiter string. While we are allocating lots of space here for the maximum representation
  * of 64 delimiters each of 64 chars MB chars, the fact is that the iop option processing actually limits the string
@@ -74,7 +104,6 @@ typedef struct
 #define DEFAULT_LISTEN_DEPTH		1
 #define	DEFAULT_SOCKET_BUFFER_SIZE	0x400
 #define	MAX_SOCKET_BUFFER_SIZE		0x100000
-#define	MAX_INTERNAL_SOCBUF_SIZE	0x100000
 /* Next three fields relate to the time that a variable length unterminated read will wait to see
  * if there is more input coming in before it gives up and returns what it has to the user. This
  * time is specified in milliseconds. This value used to be 200ms but that was deemed too long on
@@ -90,18 +119,28 @@ typedef struct
 /* For buffered output, wait this long for socket to be ready to output */
 #define DEFAULT_WRITE_WAIT		200
 
-#define SOCKWRTERROR(IOD, SOCKPTR, GTMERR, SYSERR) 								\
-MBSTART { 													\
+#define SOCKWRTERROR(IOD, SOCKPTR, GTMERR, SYSERR, ERRORTEXT, TLSLIT) 						\
+MBSTART {													\
 	int	ERRLEN; 											\
 	char	*ERRPTR; 											\
-	IOD->dollar.za = 9; 											\
+	IOD->dollar.za = ZA_IO_ERR; 										\
 														\
 	ISSUE_NOPRINCIO_IF_NEEDED(IOD, TRUE, !SOCKPTR->ioerror);	/* TRUE indicates WRITE */		\
-	ERRPTR = (char *)STRERROR(SYSERR); 									\
+	assert((0 != SYSERR) || (NULL != ERRORTEXT));								\
+	if (NULL == ERRORTEXT)											\
+		ERRPTR = (char *)STRERROR(SYSERR); 								\
+	else													\
+		ERRPTR = (char *)ERRORTEXT;									\
 	SET_DOLLARDEVICE_ONECOMMA_ERRSTR(IOD, ERRPTR, ERRLEN);							\
-	assert(ERR_SOCKWRITE == GTMERR);									\
+	assert((ERR_SOCKWRITE == GTMERR) || (ERR_TLSIOERROR == GTMERR));					\
 	if (SOCKPTR->ioerror)											\
-		rts_error_csa(CSA_ARG(NULL) VARLSTCNT(6) GTMERR, 0, ERR_TEXT, 2, ERRLEN, ERRPTR);		\
+	{													\
+		if (ERR_SOCKWRITE == GTMERR)									\
+			rts_error_csa(CSA_ARG(NULL) VARLSTCNT(6) GTMERR, 0, ERR_TEXT, 2, ERRLEN, ERRPTR);	\
+		else												\
+			rts_error_csa(CSA_ARG(NULL) VARLSTCNT(8) ERR_TLSIOERROR, 2, LEN_AND_LIT(TLSLIT),	\
+				ERR_TEXT, 2, ERRLEN, ERRPTR);							\
+	}													\
 } MBEND
 
 #define SOCKET_ALLOC(SOCKPTR)										\
@@ -141,28 +180,68 @@ MBSTART { 													\
 	SOCKPTR->buffered_length = SOCKPTR->buffered_offset = 0;		\
 }
 
+#define SOCKET_OBUFFER_INIT(SOCKETPTR, SIZE, WAIT_TIME, FLUSH_TIME)			\
+{											\
+	if (0 == SOCKETPTR->obuffer_size)						\
+	{										\
+		SOCKETPTR->obuffer_size = (0 != SIZE) ? SIZE : SOCKETPTR->buffer_size;	\
+		SOCKETPTR->obuffer = (char *)malloc(SOCKETPTR->obuffer_size);		\
+		SOCKETPTR->obuffer_length = SOCKETPTR->obuffer_offset = 0;		\
+	}										\
+	SOCKETPTR->obuffer_wait_time = WAIT_TIME;					\
+	SOCKETPTR->obuffer_flush_time = FLUSH_TIME;					\
+}
+
 #define SOCKET_FREE(SOCKPTR)							\
 {										\
-	if (NULL != SOCKPTR->buffer) 						\
-		free(SOCKPTR->buffer);						\
-	if (NULL != SOCKPTR->zff.addr)						\
+	if (NULL != SOCKPTR)							\
 	{									\
-		if ((NULL != SOCKPTR->ozff.addr) && (SOCKPTR->ozff.addr != SOCKPTR->zff.addr))				\
-			free(SOCKPTR->ozff.addr);				\
-		free(SOCKPTR->zff.addr);					\
+		if (NULL != SOCKPTR->buffer) 					\
+		{								\
+			free(SOCKPTR->buffer);					\
+			SOCKPTR->buffer = NULL;					\
+		}								\
+		if (NULL != SOCKPTR->obuffer) 					\
+		{								\
+			free(SOCKPTR->obuffer);					\
+			SOCKPTR->obuffer = NULL;				\
+		}								\
+		if (NULL != SOCKPTR->zff.addr)					\
+		{								\
+			if ((NULL != SOCKPTR->ozff.addr) && (SOCKPTR->ozff.addr != SOCKPTR->zff.addr))			\
+				free(SOCKPTR->ozff.addr);			\
+			free(SOCKPTR->zff.addr);				\
+			SOCKPTR->zff.addr = SOCKPTR->ozff.addr = NULL;		\
+		}								\
+		if (NULL != SOCKPTR->local.sa)					\
+		{								\
+			free(SOCKPTR->local.sa);				\
+			SOCKPTR->local.sa = NULL;				\
+		}								\
+		if (NULL != SOCKPTR->remote.sa)					\
+		{								\
+			free(SOCKPTR->remote.sa);				\
+			SOCKPTR->remote.sa = NULL;				\
+		}								\
+		if (NULL != SOCKPTR->local.saddr_ip)				\
+		{								\
+			free(SOCKPTR->local.saddr_ip);				\
+			SOCKPTR->local.saddr_ip = NULL;				\
+		}								\
+		if (NULL != SOCKPTR->remote.saddr_ip)				\
+		{								\
+			free(SOCKPTR->remote.saddr_ip);				\
+			SOCKPTR->remote.saddr_ip = NULL;			\
+		}								\
+		if (NULL != SOCKPTR->parenthandle)				\
+		{								\
+			free(SOCKPTR->parenthandle);				\
+			SOCKPTR->parenthandle = NULL;				\
+		}								\
+		iosocket_delimiter((unsigned char *)NULL, 0, SOCKPTR, TRUE);	\
+		free(SOCKPTR);							\
+		SOCKPTR = NULL;							\
 	}									\
-	if (NULL != SOCKPTR->local.sa)						\
-		free(SOCKPTR->local.sa);					\
-	if (NULL != SOCKPTR->remote.sa)						\
-		free(SOCKPTR->remote.sa);					\
-	if (NULL != SOCKPTR->local.saddr_ip)					\
-		free(SOCKPTR->local.saddr_ip);					\
-	if (NULL != SOCKPTR->remote.saddr_ip)					\
-		free(SOCKPTR->remote.saddr_ip);					\
-	if (NULL != SOCKPTR->parenthandle)					\
-		free(SOCKPTR->parenthandle);					\
-	iosocket_delimiter((unsigned char *)NULL, 0, SOCKPTR, TRUE);		\
-	free(SOCKPTR);								\
 }
 
 #define SOCKET_DUP(SOCKPTR, NEWSOCKPTR)										\
@@ -174,6 +253,7 @@ MBSTART { 													\
 		NEWSOCKPTR->buffered_length = NEWSOCKPTR->buffered_offset = 0;					\
 		NEWSOCKPTR->buffer = (char *)malloc(SOCKPTR->buffer_size);					\
 	}													\
+	assert((NULL == SOCKPTR->obuffer) && (0 == SOCKPTR->obuffer_size));					\
 	if ((0 != SOCKPTR->zff.len) && (NULL != SOCKPTR->zff.addr))						\
 	{													\
 		NEWSOCKPTR->zff.addr = (char *)malloc(MAX_ZFF_LEN);						\
@@ -323,12 +403,15 @@ typedef struct socket_struct_type
 	mstr				ozff;			/* UTF-16 if chset is UTF-16 else copy  of zff */
 	uint4				lastaction;		/* waitcycle  count */
 	uint4				readycycle;		/* when was ready */
-	boolean_t			pendingevent;		/* if listening, needs accept */
+	uint4				pendingevent;		/* bitmask, if listening, needs accept */
 	enum socket_creator		howcreated;
 	char				*parenthandle;		/* listening socket this created from */
 	size_t				obuffer_size;		/* size of the output buffer for this socket */
 	size_t				obuffer_length;		/* length of output in this buffer */
 	size_t				obuffer_offset;		/* offset of the buffered output to buffer head */
+	size_t				obuffer_stopped_at;	/* non blocking output blocked at this offset */
+	size_t				obuffer_stopped_left;	/* non blocking remaining output */
+	volatile boolean_t		obuffer_in_use;		/* obuffer should be used for TLS or non blocking */
 	volatile boolean_t		obuffer_timer_set;	/* timer scheduled to flush buffer */
 	volatile boolean_t		obuffer_output_active;	/* in buffer output now */
 	int				obuffer_flush_time;	/* flush output buffer after this many milliseconds */
@@ -343,6 +426,29 @@ typedef struct socket_struct_type
 	boolean_t			tlswriteblocked;
 	short				tlspolldirection;	/* what TLS wants */
 #endif
+	boolean_t			nonblocked_output;	/* default is blocking output */
+	boolean_t			output_blocked;		/* if set error all writes */
+	int				output_failures;	/* when nonblocked output */
+	int				max_output_retries;	/* when nonblocked output */
+	int				args_written;		/* number of WRITE arguments output since CLEAR */
+	size_t				lastarg_size;		/* number of bytes in last arg processed */
+	ssize_t				lastarg_sent;		/* number of bytes actually written */
+	int				readyforwhat;		/* bit mask SOCKREADY_READ and/or SOCKREADY_WRITE */
+	uint4				current_events;		/* bitmask SOCKPEND_ only within iosocket_wait, see pendingevent */
+	struct
+	{
+		unsigned int		alive	: 3;
+		unsigned int		idle	: 3;
+		unsigned int		cnt	: 3;
+		unsigned int		intvl	: 3;
+		unsigned int		sndbuf	: 3;
+		unsigned int		rcvbuf	: 3;
+	} options_state;
+	int				keepalive;		/* SO_KEEPALIVE */
+	int				keepidle;		/* TCP_KEEPIDLE */
+	int				keepcnt;		/* TCP_KEEPCNT */
+	int				keepintvl;		/* TCP_KEEPINTVL */
+	int				iobfsize;		/* SO_SNDBUF */
 } socket_struct;
 
 typedef struct socket_interrupt_type
@@ -355,6 +461,7 @@ typedef struct socket_interrupt_type
 	boolean_t                       end_time_valid;
 	boolean_t			ibfsize_specified;
 	struct d_socket_struct_type	*newdsocket;
+	int				wait_for_what;		/* bit mask */
 } socket_interrupt;
 
 typedef struct d_socket_struct_type
@@ -391,8 +498,12 @@ boolean_t iosocket_listen_sock(socket_struct *socketptr, unsigned short len);
 void iosocket_close_one(d_socket_struct *dsocketptr, int index);
 int iosocket_accept(d_socket_struct *dsocketptr, socket_struct *socketptr, boolean_t selectfirst);
 ssize_t iosocket_output_buffer(socket_struct *socketptr);
-void iosocket_buffer_error(socket_struct *socketptr);
-boolean_t iosocket_tcp_keepalive(socket_struct *socketptr, int keepalive_opt, char * act);
+int iosocket_buffer_error(socket_struct *socketptr);
+boolean_t iosocket_tcp_keepalive(socket_struct *socketptr, int keepalive_opt, char * act, boolean_t freesocket);
+boolean_t iosocket_getsockopt(socket_struct *socketptr, char *optname, int option,
+	int level, void *optvalue,  GTM_SOCKLEN_TYPE *optvaluelen, boolean_t freesocket);
+int iosocket_setsockopt(socket_struct *socketptr, char *optname, int option,
+	int level, void *optvalue, GTM_SOCKLEN_TYPE optvaluelen, boolean_t freesocket);
 #ifdef GTM_TLS
 void    iosocket_tls(mval *optionmval, int4 timeoutarg, mval *tlsid, mval *password, mval *extraarg);
 #endif
