@@ -1,6 +1,6 @@
 /****************************************************************
  *								*
- * Copyright (c) 2001-2015 Fidelity National Information 	*
+ * Copyright (c) 2001-2022 Fidelity National Information	*
  * Services, Inc. and/or its subsidiaries. All rights reserved.	*
  *								*
  *	This source code contains the intellectual property	*
@@ -13,8 +13,7 @@
 #ifndef JOBEXAM_PROCESS_INCLUDED
 #define JOBEXAM_PROCESS_INCLUDED
 
-void jobexam_process(mval *dump_file_name, mval *dump_file_spec);
-void jobexam_dump(mval *dump_file_name, mval *dump_file_spec, char *fatal_file_name_buff);
+void jobexam_process(mval *dump_file_name, mval *fmt, mval *dump_file_spec);
+void jobexam_dump(mval *dump_file_name, mval *dump_file_spec, char *fatal_file_name_buff, mval *fmt);
 
 #endif
-
