@@ -1,6 +1,6 @@
 /****************************************************************
  *								*
- * Copyright (c) 2001-2018 Fidelity National Information	*
+ * Copyright (c) 2001-2019 Fidelity National Information	*
  * Services, Inc. and/or its subsidiaries. All rights reserved.	*
  *								*
  *	This source code contains the intellectual property	*
@@ -37,17 +37,17 @@ error_def(ERR_CTRLC);
 
 mlk_shrblk_ptr_t mlk_shrblk_sort(mlk_shrblk_ptr_t head);
 
-void lke_show_memory(mlk_shrblk_ptr_t bhead, char *prefix)
+void lke_show_memory(mlk_pvtctl_ptr_t pctl, mlk_shrblk_ptr_t bhead, char *prefix)
 {
 	mlk_shrblk_ptr_t	b, bnext, parent, children;
 	mlk_shrsub_ptr_t	dsub;
 	mlk_prcblk_ptr_t	pending;
 	char			temp[MAX_ZWR_KEY_SZ + 1];
 	char			new_prefix[KDIM + 2];
-	hash128_state_t		hs;
+	mlk_subhash_state_t	hs;
 	uint4			total_len;
-	gtm_uint16		hashres;
-	uint4			hash;
+	mlk_subhash_res_t	hashres;
+	mlk_subhash_val_t	hash;
 
 	SNPRINTF(new_prefix, KDIM + 2, "	%s", prefix);
 	for (b = bhead, bnext = 0; bnext != bhead; b = bnext)
@@ -61,16 +61,16 @@ void lke_show_memory(mlk_shrblk_ptr_t bhead, char *prefix)
 		PRINTF("%s%s : [shrblk] %p : [shrsub] %p (len=%d) : [shrhash] %x : [parent] %p : [children] %p : [pending] %p : "
 				"[owner] %u : [auxowner] %" PRIuPTR "\n",
 			prefix, temp, b, dsub, dsub->length, b->hash, parent, children, pending, b->owner, b->auxowner);
-		HASH128_STATE_INIT(hs, 0);
+		MLK_SUBHASH_INIT_PVTCTL(pctl, hs);
 		total_len = 0;
 		mlk_shrhash_val_build(b, &total_len, &hs);
-		gtmmrhash_128_result(&hs, total_len, &hashres);
-		hash = (uint4)hashres.one;
+		MLK_SUBHASH_FINALIZE(hs, total_len, hashres);
+		hash = MLK_SUBHASH_RES_VAL(hashres);
 		if (hash != b->hash)		/* Should never happen; only here in case things get mangled. */
 			PRINTF("\t\t: [computed shrhash] %x\n", hash);
 		FFLUSH(stdout);
 		if (b->children)
-			lke_show_memory((mlk_shrblk_ptr_t)R2A(b->children), new_prefix);
+			lke_show_memory(pctl, (mlk_shrblk_ptr_t)R2A(b->children), new_prefix);
 		bnext = (mlk_shrblk_ptr_t)R2A(b->rsib);
 	}
 }
@@ -96,6 +96,8 @@ void lke_show_hashtable(mlk_pvtctl_ptr_t pctl)
 		FFLUSH(stdout);
 	}
 	PRINTF("\t: [num_buckets] %d\n", num_buckets);
+	if (0 != pctl->ctl->hash_seed)
+		PRINTF("\t: [seed] %" PRIu64 "\n", pctl->ctl->hash_seed);
 	FFLUSH(stdout);
 }
 
@@ -127,7 +129,7 @@ bool	lke_showtree(struct CLB 	*lnk,
 	tree = (mlk_shrblk_ptr_t)R2A(ctl->blkroot);
 	if (memory)
 	{
-		lke_show_memory(tree, "	");
+		lke_show_memory(pctl, tree, "	");
 		if (shr_sub_size)
 			(*shr_sub_size) = ctl->subfree - ctl->subbase;
 		lke_show_hashtable(pctl);
