@@ -1,6 +1,6 @@
 /****************************************************************
  *								*
- * Copyright (c) 2001-2017 Fidelity National Information	*
+ * Copyright (c) 2001-2022 Fidelity National Information	*
  * Services, Inc. and/or its subsidiaries. All rights reserved.	*
  *								*
  *	This source code contains the intellectual property	*
@@ -22,9 +22,7 @@
 #include "advancewindow.h"
 #include "cmd.h"
 #include "namelook.h"
-
-GBLREF int4 		curr_fetch_count;
-GBLREF triple		*curr_fetch_opr, *curr_fetch_trip;
+#include "start_fetches.h"
 
 LITREF nametabent 	svn_names[];
 LITREF svn_data_type 	svn_data[];
@@ -52,14 +50,14 @@ int m_new(void)
 	{
 	case TK_IDENT:
 		var = get_mvaddr(&(TREF(window_ident)));
-		if (var->last_fetch != curr_fetch_trip)
-		{
+		if ((var->last_fetch != (TREF(fetch_control)).curr_fetch_trip) && !TREF(discard))
+		{	/* This block is identical to one in put_mvar */
 			fetch = newtriple(OC_PARAMETER);
-			curr_fetch_opr->operand[1] = put_tref(fetch);
+			(TREF(fetch_control)).curr_fetch_opr->operand[1] = put_tref(fetch);
 			fetch->operand[0] = put_ilit(var->mvidx);
-			curr_fetch_count++;
-			curr_fetch_opr = fetch;
-			var->last_fetch = curr_fetch_trip;
+			((TREF(fetch_control)).curr_fetch_count)++;
+			(TREF(fetch_control)).curr_fetch_opr = fetch;
+			var->last_fetch = (TREF(fetch_control)).curr_fetch_trip;
 		}
 		tmp = maketriple(OC_NEWVAR);
 		tmp->operand[0] = put_ilit(var->mvidx);
@@ -73,7 +71,7 @@ int m_new(void)
 		ref->operand[0] = tmparg;
 		ref->operand[1] = put_ilit((mint) indir_new);
 		ins_triple(ref);
-		start_fetches(OC_FETCH);
+		MID_LINE_REFETCH;
 		return TRUE;
 	case TK_DOLLAR:
 		advancewindow();
