@@ -1,6 +1,7 @@
 /****************************************************************
  *								*
- *	Copyright 2001, 2014 Fidelity Information Services, Inc	*
+ * Copyright (c) 2001-2020 Fidelity National Information	*
+ * Services, Inc. and/or its subsidiaries. All rights reserved.	*
  *								*
  *	This source code contains the intellectual property	*
  *	of its copyright holder(s), and is made available	*
@@ -10,20 +11,22 @@
  ****************************************************************/
 
 #include "mdef.h"
+#include "gtm_string.h"
 #include "compiler.h"
 #include "mdq.h"
 #include "mmemory.h"
 
-GBLREF short int source_line;
 GBLREF int	 source_column;
 
 triple *maketriple(opctype op)
 {
 	triple *x;
+        DCL_THREADGBL_ACCESS;
 
+        SETUP_THREADGBL_ACCESS;
 	x = (triple *)mcalloc(SIZEOF(triple));
 	x->opcode = op;
-	x->src.line = source_line;
+	x->src.line = TREF(source_line);
 	x->src.column = source_column;
 	dqinit(&(x->backptr), que);
 	dqinit(&(x->jmplist), que);
