@@ -1,6 +1,6 @@
 /****************************************************************
  *								*
- * Copyright (c) 2001-2020 Fidelity National Information	*
+ * Copyright (c) 2001-2022 Fidelity National Information	*
  * Services, Inc. and/or its subsidiaries. All rights reserved.	*
  *								*
  *	This source code contains the intellectual property	*
@@ -40,7 +40,8 @@
 #include "lv_val.h"
 #include <rtnhdr.h>
 #include "mv_stent.h"
-#include "outofband.h"
+#include "have_crit.h"
+#include "deferred_events_queue.h"
 #include "stack_frame.h"
 #include "stringpool.h"
 #include "buddy_list.h"		/* needed for tp.h */
@@ -73,19 +74,19 @@
 #include "tp_change_reg.h"
 #include "alias.h"
 #include "create_fatal_error_zshow_dmp.h"
-#include "have_crit.h"
 #include "ztimeout_routines.h"
-# include "iormdef.h"
-# include "ftok_sems.h"
-# include "gtm_putmsg_list.h"
+#include "iormdef.h"
+#include "ftok_sems.h"
+#include "gtm_putmsg_list.h"
 #include "gvt_inline.h"
+#include "deferred_events.h"
 #ifdef GTM_TRIGGER
-# include "gv_trigger.h"
-# include "gtm_trigger.h"
+#include "gv_trigger.h"
+#include "gtm_trigger.h"
 #endif
 
-GBLREF	boolean_t		created_core, dont_want_core, in_gvcst_incr, prin_dm_io, prin_in_dev_failure, prin_out_dev_failure,
-				run_time;
+GBLREF	boolean_t		created_core, dont_want_core, hup_on, in_gvcst_incr, prin_dm_io, prin_in_dev_failure,
+				prin_out_dev_failure, run_time;
 GBLREF	boolean_t		ztrap_explicit_null;		/* whether $ZTRAP was explicitly set to NULL in this frame */
 GBLREF	dollar_ecode_type	dollar_ecode;			/* structure containing $ECODE related information */
 GBLREF	dollar_stack_type	dollar_stack;
@@ -129,22 +130,13 @@ GBLREF	jnl_gbls_t		jgbl;
 error_def(ERR_ASSERT);
 error_def(ERR_CTRAP);
 error_def(ERR_CTRLC);
-error_def(ERR_CTRLY);
-error_def(ERR_GTMASSERT);
-error_def(ERR_GTMASSERT2);
-error_def(ERR_GTMCHECK);
-error_def(ERR_GTMERREXIT);
 error_def(ERR_JOBINTRRETHROW);
 error_def(ERR_JOBINTRRQST);
 error_def(ERR_LABELMISSING);
 error_def(ERR_MEMORY);
 error_def(ERR_NOEXCNOZTRAP);
 error_def(ERR_NOTPRINCIO);
-error_def(ERR_OUTOFSPACE);
 error_def(ERR_REPEATERROR);
-error_def(ERR_RESTART);
-error_def(ERR_RTSLOC);
-error_def(ERR_SRCLOCUNKNOWN);
 error_def(ERR_STACKCRIT);
 error_def(ERR_STACKOFLOW);
 error_def(ERR_TERMHANGUP);
@@ -155,7 +147,7 @@ error_def(ERR_TPSTACKCRIT);
 error_def(ERR_TPSTACKOFLOW);
 error_def(ERR_TPTIMEOUT);
 error_def(ERR_UNSOLCNTERR);
-error_def(ERR_VMSMEMORY);
+error_def(ERR_ZINTRECURSEIO);
 error_def(ERR_ZTIMEOUT);
 
 boolean_t clean_mum_tstart(void);
@@ -207,7 +199,9 @@ boolean_t clean_mum_tstart(void)
 {
 	stack_frame	*save_zyerr_frame, *fp, *fpprev;
 	boolean_t	save_check_flag;
+	DCL_THREADGBL_ACCESS;
 
+	SETUP_THREADGBL_ACCESS;
 	if (NULL != zyerr_frame)
 	{
 		while ((NULL != frame_pointer) && (NULL != zyerr_frame))
@@ -221,6 +215,7 @@ boolean_t clean_mum_tstart(void)
 			indr_stringpool = stringpool;
 			stringpool = rts_stringpool;
 		}
+		TREF(compile_time) = FALSE;	/* Switching back to run-time */
 		return TRUE;
 	}
 	return (NULL != err_act);
@@ -254,6 +249,7 @@ CONDITION_HANDLER(mdb_condition_handler)
 	sgmnt_addrs		*csa;
 	stack_frame		*fp;
 	boolean_t		error_in_zyerror;
+	boolean_t		compile_time;
 	boolean_t		repeat_error, etrap_handling, reset_mpc;
 	int			level, rc;
 	boolean_t		reserve_sock_dev = FALSE;
@@ -453,6 +449,7 @@ CONDITION_HANDLER(mdb_condition_handler)
 		 *    in the local variable code which would cause immediate shutdown with no cleanup).
 		 * Note that we will bypass check 2 if GDL_ZSHOWDumpOnSignal debug flag is on
 		 */
+		outofband = no_event;
 		SET_PROCESS_EXITING_TRUE;	/* So zshow doesn't push stuff on stack to "protect" it when
 						 * we potentially already have a stack overflow */
 		CANCEL_TIMERS;			/* No unsafe interruptions now that we are dying */
@@ -580,7 +577,7 @@ CONDITION_HANDLER(mdb_condition_handler)
 		 * of this writing, there are no conditions that these should validly be called in this
 		 * situation so this check is more for the future.
 		 */
-		assert(((int)ERR_CTRLY != SIGNAL) && ((int)ERR_CTRLC != SIGNAL) && ((int)ERR_CTRAP != SIGNAL)
+		assert(((int)ERR_CTRLC != SIGNAL) && ((int)ERR_CTRAP != SIGNAL)
 		       && ((int)ERR_JOBINTRRQST != SIGNAL) && ((int)ERR_JOBINTRRETHROW != SIGNAL));
 		gtm_trigger_fini(TRUE, FALSE);
 		DBGEHND((stderr, "mdb_condition_handler: Current trigger frame unwound so error is thrown"
@@ -589,8 +586,9 @@ CONDITION_HANDLER(mdb_condition_handler)
 #	endif
  	err_dev = active_device;
 	active_device = (io_desc *)NULL;
+	compile_time = TREF(compile_time);
 	dm_action = (prin_dm_io || (frame_pointer->old_frame_pointer->type & SFT_DM)
-		|| (TREF(compile_time) && (frame_pointer->type & SFT_DM)));
+		|| (compile_time && (frame_pointer->type & SFT_DM)));
 	/* The errors are said to be transcendental when they occur during compilation/execution
 	 * of the error trap ({z,e}trap, device exception) or $zinterrupt. The errors in other
 	 * indirect code frames (zbreak, zstep, xecute etc.) aren't defined to be trancendental
@@ -601,21 +599,10 @@ CONDITION_HANDLER(mdb_condition_handler)
 	src_line_d.len = 0;
 	if (sighup != outofband)	/* because prin_out_dev_failure is not yet set - it will be below */
 		flush_pio();
-	if ((int)ERR_CTRLY == SIGNAL)
+	if ((int)ERR_CTRLC == SIGNAL)
 	{
-		outofband_clear();
-		/* Verify not indirect or that context is unchanged before reset context */
-		assert(NULL != frame_pointer->restart_pc);
-		assert((!(SFF_INDCE & frame_pointer->flags)) || (frame_pointer->restart_ctxt == frame_pointer->ctxt));
-		DBGEHND((stderr, "mdb_condition_handler(1): Resetting frame 0x"lvaddr" mpc/context with restart_pc/ctxt "
-			 "0x"lvaddr"/0x"lvaddr" - frame has type 0x%04lx\n", frame_pointer, frame_pointer->restart_pc,
-			 frame_pointer->restart_ctxt, frame_pointer->type));
-		frame_pointer->mpc = frame_pointer->restart_pc;
-		frame_pointer->ctxt = frame_pointer->restart_ctxt;
-		MUM_TSTART;
-	} else  if ((int)ERR_CTRLC == SIGNAL)
-	{
-		outofband_clear();
+		xfer_reset_if_setter(ctrlc);
+		outofband = no_event;
 		if (!trans_action && !dm_action)
 		{	/* Verify not indirect or that context is unchanged before reset context */
 			assert(NULL != frame_pointer->restart_pc);
@@ -658,11 +645,20 @@ CONDITION_HANDLER(mdb_condition_handler)
 	} else if (((int)ERR_CTRAP == SIGNAL) || ((int)ERR_TERMHANGUP == SIGNAL))
 	{
 		if (outofband && !repeat_error)
-		{	/* This has already been done if we are re-throwing the error */
-			outofband_clear();
+		{	/* don't need to do this again if re-throwing the error */
+			assert((((int)ERR_TERMHANGUP == SIGNAL) ? sighup : ctrap) == outofband);
 			if ((int)ERR_TERMHANGUP == SIGNAL)
+			{
+				assert(hup_on || prin_in_dev_failure);
+				SAVE_ZSTATUS_INTO_RTS_STRINGPOOL(dollar_ecode.error_last_b_line, NULL);
 				prin_in_dev_failure = prin_out_dev_failure = TRUE;
+				hup_on = FALSE;	/* normally there's only 1, but sigproc could cause multiple; app can reset it  */
+			}
+			TAREF1(save_xfer_root, outofband).event_state = active;
+			real_xfer_reset(outofband);
+			TAREF1(save_xfer_root, outofband).event_state = not_in_play;	/* sighup & ctrap both headed to trap */
 		}
+		outofband = no_event;
 		if (!trans_action && !(frame_pointer->type & SFT_DM))
 		{
 			if (!repeat_error)
@@ -785,7 +781,7 @@ CONDITION_HANDLER(mdb_condition_handler)
 		}
 		MUM_TSTART_FRAME_CHECK;
 		MUM_TSTART;
-	} else  if ((int)ERR_JOBINTRRQST == SIGNAL)
+	} else if ((int)ERR_JOBINTRRQST == SIGNAL)
 	{	/* Verify not indirect or that context is unchanged before reset context */
 		assert(NULL != frame_pointer->restart_pc);
 		assert((!(SFF_INDCE & frame_pointer->flags)) || (frame_pointer->restart_ctxt == frame_pointer->ctxt));
@@ -794,9 +790,12 @@ CONDITION_HANDLER(mdb_condition_handler)
 			 frame_pointer->restart_ctxt, frame_pointer->type));
 		frame_pointer->mpc = frame_pointer->restart_pc;
 		frame_pointer->ctxt = frame_pointer->restart_ctxt;
-		assert(!dollar_zininterrupt);
-		dollar_zininterrupt = TRUE;	/* Note done before outofband is cleared to prevent nesting */
-		outofband_clear();
+		assert(jobinterrupt == outofband);
+		dollar_zininterrupt = TRUE;     /* Note down before outofband is cleared to prevent nesting */
+		TAREF1(save_xfer_root, jobinterrupt).event_state = active;
+		xfer_reset_if_setter(jobinterrupt);
+		assert(not_in_play != TAREF1(save_xfer_root, jobinterrupt).event_state);
+		TAREF1(save_xfer_root, jobinterrupt).event_state = active;
 		proc_act_type = SFT_ZINTR | SFT_COUNT;	/* trans_code will invoke jobinterrupt_process for us */
 		MUM_TSTART;
 	} else  if ((int)ERR_JOBINTRRETHROW == SIGNAL)
@@ -814,7 +813,9 @@ CONDITION_HANDLER(mdb_condition_handler)
 		push_stck(cp, 0, (void **)&stackwarn, MVST_STCK_SP);
 	} else if ((int)ERR_ZTIMEOUT == SIGNAL)
 	{	/* Verify not indirect or that context is unchanged before reset context */
-		if (((TREF(dollar_ztimeout)).ztimeout_vector.str.len))
+		DBGDFRDEVNT((stderr, "%d %s: ztimeout about to use vector: %s\n", __LINE__, __FILE__,
+			(TREF(dollar_ztimeout)).ztimeout_vector.str.len ? "TRUE" : "FALSE"));
+		if ((TREF(dollar_ztimeout)).ztimeout_vector.str.len)
 		{
 			assert(NULL != frame_pointer->restart_pc);
 			assert((!(SFF_INDCE & frame_pointer->flags)) || (frame_pointer->restart_ctxt == frame_pointer->ctxt));
@@ -969,6 +970,7 @@ CONDITION_HANDLER(mdb_condition_handler)
 					if ((SFT_ZINTR & fp->type) && (SFF_INDCE & fp->flags))
 					{ /* Not modified by a goto or zgoto entry */
 						assert(SFF_INDCE & fp->flags);
+						assert(dollar_zininterrupt);
 						fp->mpc = fp->ctxt;
 						fp->flags &= SSF_NORET_VIA_MUMTSTART_OFF; /* Frame enterable now with mpc reset */
 						GTMTRIG_ONLY(
@@ -1103,7 +1105,7 @@ CONDITION_HANDLER(mdb_condition_handler)
 		if (dm_action)
 		{
 			PRN_ERROR;
-			if (TREF(compile_time) && (((int)ERR_LABELMISSING) != SIGNAL))
+			if (compile_time && (((int)ERR_LABELMISSING) != SIGNAL))
 				show_source_line(TRUE);
 		}
 	}
@@ -1116,10 +1118,15 @@ CONDITION_HANDLER(mdb_condition_handler)
 		 */
 		DBGEHND((stderr, "mdb_condition_handler: trans_code_cleanup() or jobinterrupt_process_cleanup being "
 			 "dispatched\n"));
-		if (!(SFT_ZINTR & proc_act_type))
+		if (!(SFT_ZINTR & proc_act_type) && !(SFT_ZTIMEOUT & proc_act_type))  	/* ztimeout vector precompiled */
+		{
+			assert(!dollar_zininterrupt || ((int)ERR_ZINTRECURSEIO == SIGNAL));
 			trans_code_cleanup();
-		else
+		} else if (!(SFT_ZTIMEOUT & proc_act_type))
+		{
+			assert(dollar_zininterrupt);
 			jobinterrupt_process_cleanup();
+		}
 		MUM_TSTART_FRAME_CHECK;
 		MUM_TSTART;
 	}
