@@ -1,6 +1,6 @@
 /****************************************************************
  *								*
- * Copyright (c) 2001-2019 Fidelity National Information	*
+ * Copyright (c) 2001-2021 Fidelity National Information	*
  * Services, Inc. and/or its subsidiaries. All rights reserved.	*
  *								*
  *	This source code contains the intellectual property	*
@@ -316,10 +316,13 @@ LITDEF int4 gtm_release_stamp_len = SIZEOF(GTM_RELEASE_STAMP) - 1;
  * the internal context of incremental/stream backup so the value should never appear where
  * this table is being indexed (suggest asserts for index being > 0 at usage points).
  */
-LITDEF char *gtm_dbversion_table[] =
+LITDEF char *gtm_dbversion_table[] =	/* gtmpcat.m has a table, dbversion, that must be maintained to mirror this one */
 {
 	"V4",
-	"V6"
+	"V6",
+	"V6p",	/* GDSV6p - upgrading to V7 and therefore has unusual temporary characteristics */
+	"V7m",	/* GDSV7m - upgraded from V6 and therfore has different start_vbn */
+	"V7"
 };
 
 LITDEF int4 ten_pwr[NUM_DEC_DG_1L+1] = {1, 10, 100, 1000, 10000, 100000, 1000000, 10000000, 100000000, 1000000000};
