@@ -1,6 +1,6 @@
 /****************************************************************
  *								*
- * Copyright (c) 2001-2019 Fidelity National Information	*
+ * Copyright (c) 2001-2021 Fidelity National Information	*
  * Services, Inc. and/or its subsidiaries. All rights reserved.	*
  *								*
  *	This source code contains the intellectual property	*
@@ -447,6 +447,7 @@ boolean_t mu_reorg(glist *gl_ptr, glist *exclude_glist_ptr, boolean_t *resume,
 							GVCST_BMP_MARK_FREE(&kill_set_list, ret_tn, inctn_mu_reorg,
 									inctn_bmp_mark_free_mu_reorg, inctn_opcode, cs_addrs)
 							DECR_KIP(cs_data, cs_addrs, kip_csa);
+							DEFERRED_EXIT_REORG_CHECK;
 							if (detailed_log)
 								log_detailed_log("KIL", &(gv_target->hist), NULL, level,
 									&kill_set_list, ret_tn);
@@ -541,7 +542,7 @@ boolean_t mu_reorg(glist *gl_ptr, glist *exclude_glist_ptr, boolean_t *resume,
 				 * Histories are sent as gv_target->hist and reorg_gv_target->hist.
 				 */
 				mu_reorg_in_swap_blk = TRUE;
-				status = mu_swap_blk(level, &dest_blk_id, &kill_set_list, exclude_glist_ptr);
+				status = mu_swap_blk(level, &dest_blk_id, &kill_set_list, exclude_glist_ptr, 0); /* not upgrade */
 				mu_reorg_in_swap_blk = FALSE;
 				if (cdb_sc_oprnotneeded == status)
 				{
@@ -577,6 +578,7 @@ boolean_t mu_reorg(glist *gl_ptr, glist *exclude_glist_ptr, boolean_t *resume,
 						GVCST_BMP_MARK_FREE(&kill_set_list, ret_tn, inctn_mu_reorg,
 								inctn_bmp_mark_free_mu_reorg, inctn_opcode, cs_addrs)
 						DECR_KIP(cs_data, cs_addrs, kip_csa);
+						DEFERRED_EXIT_REORG_CHECK;
 						if (detailed_log)
 							log_detailed_log("KIL", &(gv_target->hist), NULL, level,
 								&kill_set_list, ret_tn);
@@ -675,6 +677,7 @@ boolean_t mu_reorg(glist *gl_ptr, glist *exclude_glist_ptr, boolean_t *resume,
 				GVCST_BMP_MARK_FREE(&kill_set_list, ret_tn, inctn_mu_reorg,
 						inctn_bmp_mark_free_mu_reorg, inctn_opcode, cs_addrs)
 				DECR_KIP(cs_data, cs_addrs, kip_csa);
+				DEFERRED_EXIT_REORG_CHECK;
 				if (detailed_log)
 					log_detailed_log("KIL", &(gv_target->hist), NULL, level, &kill_set_list, ret_tn);
 				blks_reused += kill_set_list.used;
@@ -704,17 +707,19 @@ void reorg_finish(block_id dest_blk_id, block_id blks_processed, block_id blks_k
 {
 	t_abort(gv_cur_region, cs_addrs);
 	file_extended = cs_data->trans_hist.total_blks - file_extended;
-	util_out_print("Blocks processed    : !SL ", FLUSH, blks_processed);
-	util_out_print("Blocks coalesced    : !SL ", FLUSH, blks_coalesced);
-	util_out_print("Blocks split        : !SL ", FLUSH, blks_split);
-	util_out_print("Blocks swapped      : !SL ", FLUSH, blks_swapped);
-	util_out_print("Blocks freed        : !SL ", FLUSH, blks_killed);
-	util_out_print("Blocks reused       : !SL ", FLUSH, blks_reused);
+	util_out_print("Blocks processed    : !@UQ ", FLUSH, &blks_processed);
+	util_out_print("Blocks coalesced    : !@UQ ", FLUSH, &blks_coalesced);
+	util_out_print("Blocks split        : !@UQ ", FLUSH, &blks_split);
+	util_out_print("Blocks swapped      : !@UQ ", FLUSH, &blks_swapped);
+	util_out_print("Blocks freed        : !@UQ ", FLUSH, &blks_killed);
+	util_out_print("Blocks reused       : !@UQ ", FLUSH, &blks_reused);
 	if (0 > lvls_reduced)
-		util_out_print("Levels Increased    : !SL ", FLUSH, -lvls_reduced);
-	else if (0 < lvls_reduced)
-		util_out_print("Levels Eliminated   : !SL ", FLUSH, lvls_reduced);
-	util_out_print("Blocks extended     : !SL ", FLUSH, file_extended);
+	{
+		lvls_reduced = -1 * lvls_reduced;
+		util_out_print("Levels Increased    : !@UQ ", FLUSH, &lvls_reduced);
+	} else if (0 < lvls_reduced)
+		util_out_print("Levels Eliminated   : !@UQ ", FLUSH, &lvls_reduced);
+	util_out_print("Blocks extended     : !@UQ ", FLUSH, &file_extended);
 	cs_addrs->reorg_last_dest = dest_blk_id;
 	/* next attempt for this global will start from the beginning, if RESUME option is present */
 	cs_data->reorg_restart_block = 0;
