@@ -1,6 +1,6 @@
 /****************************************************************
  *								*
- * Copyright (c) 2005-2020 Fidelity National Information	*
+ * Copyright (c) 2005-2021 Fidelity National Information	*
  * Services, Inc. and/or its subsidiaries. All rights reserved.	*
  *								*
  *	This source code contains the intellectual property	*
@@ -91,6 +91,7 @@ error_def(ERR_DBFILERR);
 error_def(ERR_DBRDONLY);
 error_def(ERR_DSEBLKRDFAIL);
 error_def(ERR_DYNUPGRDFAIL);
+error_def(ERR_GTMCURUNSUPP);
 error_def(ERR_MUNOACTION);
 error_def(ERR_MUNOFINISH);
 error_def(ERR_MUREORGFAIL);
@@ -143,6 +144,9 @@ void	mu_reorg_upgrd_dwngrd(void)
 	unsigned char		save_cw_set_depth;
 	uint4			lcl_update_trans;
 	unix_db_info		*udi;
+
+	mupip_exit(ERR_GTMCURUNSUPP);
+
 	DCL_THREADGBL_ACCESS;
 
 	SETUP_THREADGBL_ACCESS;
@@ -227,8 +231,8 @@ void	mu_reorg_upgrd_dwngrd(void)
 			status = ERR_MUNOFINISH;
 			continue;
 		}
-		assert(GDSVCURR == GDSV6); /* so we trip this assert in case GDSVCURR changes without a change to this module */
-		new_db_format = (upgrade ? GDSV6 : GDSV4);
+		assert(GDSVCURR >= GDSV6); /* so we trip this assert in case GDSVCURR changes without a change to this module */
+		new_db_format = (upgrade ? GDSV7m : GDSV6p);	/* this downgrade is not currently implemented  */
 		grab_crit(reg, WS_8);
 		curr_tn = csd->trans_hist.curr_tn;
 		/* set the desired db format in the file header to the appropriate version, increment transaction number */
@@ -361,7 +365,7 @@ void	mu_reorg_upgrd_dwngrd(void)
 			}
 			bml_sm_buff = t_qread(curbmp, (sm_int_ptr_t)&cycle, &cr); /* now that in crit, note down stable buffer */
 			if (NULL == bml_sm_buff)
-				rts_error_csa(CSA_ARG(csa) VARLSTCNT(1) ERR_DSEBLKRDFAIL);
+				RTS_ERROR_CSA_ABT(csa, VARLSTCNT(1) ERR_DSEBLKRDFAIL);
 			ondsk_blkver = cr->ondsk_blkver;	/* note down db fmt on disk for bitmap block */
 			/* Take a copy of the shared memory bitmap buffer into process-private memory before releasing crit.
 			 * We are interested in those blocks that are currently marked as USED in the bitmap.
@@ -433,7 +437,7 @@ void	mu_reorg_upgrd_dwngrd(void)
 						{
 							if (!upgrade && (ERR_DYNUPGRDFAIL == status1))
 							{
-								assert(GDSV4 == new_db_format);
+								assert(GDSV7m == new_db_format);
 								ondsk_blkver = new_db_format;
 							} else
 							{
@@ -718,7 +722,7 @@ void	mu_reorg_upgrd_dwngrd(void)
 			TRUE, REG_LEN_STR(reg), reorg_stats.blks_converted_nonbmp);
 		if (reorg_entiredb && (SS_NORMAL == status1) && (0 != blocks_left))
 		{	/* file-header counter does not match what reorg on the entire database expected to see */
-			gtm_putmsg_csa(CSA_ARG(csa) VARLSTCNT(4) ERR_DBBTUWRNG, 2, expected_blks2upgrd, actual_blks2upgrd);
+			gtm_putmsg_csa(CSA_ARG(csa) VARLSTCNT(4) ERR_DBBTUWRNG, 2, &expected_blks2upgrd, &actual_blks2upgrd);
 			util_out_print("Region !AD : Run MUPIP INTEG (without FAST qualifier) to fix the counter",
 				TRUE, REG_LEN_STR(reg));
 			status1 = ERR_MUNOFINISH;
