@@ -1,6 +1,6 @@
 /****************************************************************
  *								*
- * Copyright (c) 2001-2020 Fidelity National Information	*
+ * Copyright (c) 2001-2021 Fidelity National Information	*
  * Services, Inc. and/or its subsidiaries. All rights reserved.	*
  *								*
  *	This source code contains the intellectual property	*
@@ -52,6 +52,7 @@ mu_swap_blk.c:
 #include "mu_reorg.h"
 #include "cws_insert.h"
 #include "gvt_inline.h"
+#include "tp.h"
 
 /* Include prototypes */
 #include "t_qread.h"
@@ -70,13 +71,13 @@ GBLREF sgmnt_data_ptr_t	cs_data;
 GBLREF char		*update_array, *update_array_ptr;
 GBLREF uint4		update_array_size;	/* for the BLK_* macros */
 GBLREF uint4		t_err;
-GBLREF cw_set_element 	cw_set[];
-GBLREF unsigned char    cw_map_depth;
+GBLREF cw_set_element	cw_set[];
+GBLREF unsigned char	cw_map_depth;
 GBLREF unsigned char	cw_set_depth;
 GBLREF unsigned char	rdfail_detail;
-GBLREF unsigned int     t_tries;
-GBLREF gv_key 		*gv_currkey;
-GBLREF hash_table_int4	cw_stagnate;
+GBLREF unsigned int	t_tries;
+GBLREF gv_key		*gv_currkey;
+GBLREF hash_table_int8	cw_stagnate;
 
 /******************************************************************************************
 Input Parameters:
@@ -87,9 +88,9 @@ Output Parameters:
 	*exclude_glist_ptr: List of globals not to be moved for a swap destination
 Input/Output Parameters:
 	gv_target : as working block's history
-	reorg_gv_target->hist : as desitnitions block's history
+	reorg_gv_target->hist : as destination block's history
  ******************************************************************************************/
-enum cdb_sc mu_swap_blk(int level, block_id *pdest_blk_id, kill_set *kill_set_ptr, glist *exclude_glist_ptr)
+enum cdb_sc mu_swap_blk(int level, block_id *pdest_blk_id, kill_set *kill_set_ptr, glist *exclude_glist_ptr, block_id upg_mv_block)
 {
 	unsigned char		x_blk_lmap;
 	unsigned short		temp_ushort;
@@ -98,12 +99,12 @@ enum cdb_sc mu_swap_blk(int level, block_id *pdest_blk_id, kill_set *kill_set_pt
 	int			piece_len1, piece_len2, first_offset, second_offset,
 				work_blk_size, work_parent_size, dest_blk_size, dest_parent_size;
 	int			dest_child_cycle;
-	int			blk_seg_cnt, blk_size;
+	int			blk_seg_cnt, blk_size, work_blk_id_sz, dest_blk_id_sz;
 	trans_num		ctn;
 	int			key_len, key_len_dir;
 	block_id		dest_blk_id, work_blk_id, child1, child2;
 	enum cdb_sc		status;
-	srch_hist 		*dest_hist_ptr, *dir_hist_ptr;
+	srch_hist		*dest_hist_ptr, *dir_hist_ptr;
 	cache_rec_ptr_t		dest_child_cr;
 	blk_segment		*bs1, *bs_ptr;
 	sm_uc_ptr_t		saved_blk, work_blk_ptr, work_parent_ptr, dest_parent_ptr, dest_blk_ptr,
@@ -111,10 +112,11 @@ enum cdb_sc mu_swap_blk(int level, block_id *pdest_blk_id, kill_set *kill_set_pt
 	boolean_t		gbl_target_was_set, blk_was_free, deleted;
 	gv_namehead		*save_targ;
 	srch_blk_status		bmlhist, destblkhist, *hist_ptr, *work_blk_stat;
-	unsigned char    	save_cw_set_depth;
+	unsigned char		save_cw_set_depth;
 	cw_set_element		*tmpcse;
 	jnl_buffer_ptr_t	jbbp; /* jbbp is non-NULL only if before-image journaling */
 	unsigned int		bsiz;
+	boolean_t		work_long_blk_id, dest_long_blk_id, temp_long_blk_id;
 	DCL_THREADGBL_ACCESS;
 
 	SETUP_THREADGBL_ACCESS;
@@ -131,6 +133,9 @@ enum cdb_sc mu_swap_blk(int level, block_id *pdest_blk_id, kill_set *kill_set_pt
 	work_blk_ptr = work_blk_stat->buffaddr;
 	work_blk_size = ((blk_hdr_ptr_t)work_blk_ptr)->bsiz;
 	work_blk_id = work_blk_stat->blk_num;
+	assert(!upg_mv_block || (upg_mv_block == work_blk_id));
+	work_long_blk_id = IS_64_BLK_ID(work_blk_ptr);
+	work_blk_id_sz = SIZEOF_BLK_ID(work_long_blk_id);
 	if (blk_size < work_blk_size)
 	{
 		assert(t_tries < CDB_STAGNATE);
@@ -149,9 +154,13 @@ enum cdb_sc mu_swap_blk(int level, block_id *pdest_blk_id, kill_set *kill_set_pt
 		 * Still a grand-child can go to its grand-parent. This is rare and following code can handle it.
 		 */
 		if (dest_blk_id == gv_target->hist.h[level+1].blk_num)
+		{
+			assert(0 == upg_mv_block);
 			continue;
+		}
 		if (cs_data->trans_hist.total_blks <= dest_blk_id || dest_blk_id == work_blk_id)
 		{
+			assert(0 == upg_mv_block);
 			*pdest_blk_id = dest_blk_id;
 			return cdb_sc_oprnotneeded;
 		}
@@ -178,7 +187,7 @@ enum cdb_sc mu_swap_blk(int level, block_id *pdest_blk_id, kill_set *kill_set_pt
 		/* reset whatever blocks the previous iteration of this for loop had filled in the cw_stagnate hashtable */
 		for ( ; cws_reorg_remove_index > 0; cws_reorg_remove_index--)
 		{
-			deleted = delete_hashtab_int4(&cw_stagnate, (uint4 *)&cws_reorg_remove_array[cws_reorg_remove_index]);
+			deleted = delete_hashtab_int8(&cw_stagnate, (ublock_id *)&cws_reorg_remove_array[cws_reorg_remove_index]);
 			assert(deleted);
 		}
 		/* read corresponding bitmap block before attempting to read destination  block.
@@ -192,24 +201,28 @@ enum cdb_sc mu_swap_blk(int level, block_id *pdest_blk_id, kill_set *kill_set_pt
 			assert(CDB_STAGNATE > t_tries);
 			return cdb_sc_badbitmap;
 		}
-		if (BLK_FREE != x_blk_lmap)
+		if ((BLK_FREE != x_blk_lmap) || (0 != upg_mv_block))
 		{	/* x_blk_lmap is either BLK_BUSY or BLK_RECYCLED. In either case, we need to read destination block
 			 * in case we later detect that the before-image needs to be written.
 			 */
 			if (!(dest_blk_ptr = t_qread(dest_blk_id, (sm_int_ptr_t)&destblkhist.cycle, &destblkhist.cr)))
 			{
+				assert(0 == upg_mv_block);
 				assert(t_tries < CDB_STAGNATE);
 				return (enum cdb_sc)rdfail_detail;
 			}
 			destblkhist.blk_num = dest_blk_id;
 			destblkhist.buffaddr = dest_blk_ptr;
 			destblkhist.level = dest_blk_level = ((blk_hdr_ptr_t)dest_blk_ptr)->levl;
+			dest_long_blk_id = IS_64_BLK_ID(dest_blk_ptr);
+			dest_blk_id_sz = SIZEOF_BLK_ID(dest_long_blk_id);
 		}
 		if (BLK_BUSY != x_blk_lmap)
 		{	/* x_blk_map is either BLK_FREE or BLK_RECYCLED both of which mean the block is not used in the bitmap */
 			blk_was_free = TRUE;
 			break;
-		}
+		} else
+			assert(0 == upg_mv_block);
 		/* dest_blk_id might contain a *-record only.
 		 * So follow the pointer to go to the data/index block, which has a non-* key to search.
 		 */
@@ -222,9 +235,10 @@ enum cdb_sc mu_swap_blk(int level, block_id *pdest_blk_id, kill_set *kill_set_pt
 		rec_base = dest_blk_ptr + SIZEOF(blk_hdr);
 		GET_RSIZ(rec_size1, rec_base);
 		tblk_ptr = dest_blk_ptr;
-		while ((BSTAR_REC_SIZE == rec_size1) && (0 != nslevel))
+		temp_long_blk_id = dest_long_blk_id;
+		while ((bstar_rec_size(temp_long_blk_id) == rec_size1) && (0 != nslevel))
 		{
-			GET_BLK_ID(child1, rec_base + SIZEOF(rec_hdr));
+			READ_BLK_ID(temp_long_blk_id, &child1, rec_base + SIZEOF(rec_hdr));
 			if (0 == child1 || child1 > cs_data->trans_hist.total_blks - 1)
 			{
 				assert(t_tries < CDB_STAGNATE);
@@ -235,6 +249,7 @@ enum cdb_sc mu_swap_blk(int level, block_id *pdest_blk_id, kill_set *kill_set_pt
 				assert(t_tries < CDB_STAGNATE);
 				return (enum cdb_sc)rdfail_detail;
 			}
+			temp_long_blk_id = IS_64_BLK_ID(tblk_ptr);
 			/* leaf of a killed GVT can have block header only.   Skip those blocks */
 			if (SIZEOF(blk_hdr) >= ((blk_hdr_ptr_t)tblk_ptr)->bsiz)
 				break;
@@ -244,7 +259,10 @@ enum cdb_sc mu_swap_blk(int level, block_id *pdest_blk_id, kill_set *kill_set_pt
 		}
 		/* leaf of a killed GVT can have block header only.   Skip those blocks */
 		if (SIZEOF(blk_hdr) >= ((blk_hdr_ptr_t)tblk_ptr)->bsiz)
+		{
+			assert((0 == upg_mv_block) || (SIZEOF(blk_hdr) == ((blk_hdr_ptr_t)tblk_ptr)->bsiz));
 			continue;
+		}
 		/* get length of global variable name (do not read subscript) for dest_blk_id */
 		key_len_dir = get_gblname_len(tblk_ptr, rec_base + SIZEOF(rec_hdr));
 		/* key_len = length of 1st key value (including subscript) for dest_blk_id */
@@ -264,8 +282,9 @@ enum cdb_sc mu_swap_blk(int level, block_id *pdest_blk_id, kill_set *kill_set_pt
 		memcpy(&((TREF(gv_reorgkey))->base[0]), rec_base + SIZEOF(rec_hdr), key_len_dir);
 		(TREF(gv_reorgkey))->base[key_len_dir] = 0;
 		(TREF(gv_reorgkey))->end = key_len_dir;
-		if (exclude_glist_ptr->next)
+		if ((NULL != exclude_glist_ptr) && exclude_glist_ptr->next)
 		{	/* exclude blocks for globals in the list of EXCLUDE option */
+			assert(0 == upg_mv_block);
 			if  (in_exclude_list(&((TREF(gv_reorgkey))->base[0]), key_len_dir - 1, exclude_glist_ptr))
 				continue;
 		}
@@ -290,25 +309,25 @@ enum cdb_sc mu_swap_blk(int level, block_id *pdest_blk_id, kill_set *kill_set_pt
 		}
 		if (dir_hist_ptr->h[0].curr_rec.match != (TREF(gv_reorgkey))->end + 1)
 		{	/* may be in a kill_set of another process */
+			assert((0 == upg_mv_block) || (dir_hist_ptr->h[0].curr_rec.match == (TREF(gv_reorgkey))->end + 1));
 			RESET_GV_TARGET_LCL_AND_CLR_GBL(save_targ, DO_GVT_GVKEY_CHECK);
 			continue;
 		}
-		for (wlevel = 0; wlevel <= dir_hist_ptr->depth &&
-			dir_hist_ptr->h[wlevel].blk_num != dest_blk_id; wlevel++);
+		for (wlevel = 0; (wlevel <= dir_hist_ptr->depth) && (dir_hist_ptr->h[wlevel].blk_num != dest_blk_id); wlevel++)
+			;
 		if (dir_hist_ptr->h[wlevel].blk_num == dest_blk_id)
-		{	/* do not swap a dir_tree block */
+		{	/* do not use a dir_tree block as the destination of a swap */
+			assert(0 == upg_mv_block);
 			RESET_GV_TARGET_LCL_AND_CLR_GBL(save_targ, DO_GVT_GVKEY_CHECK);
 			continue;
 		}
-		/* gv_reorgkey will now have the first key from dest_blk_id,
-		 * or, from a descendant of dest_blk_id (in case it had a *-key only).
-		 */
+		/* gv_reorgkey gets the first key from dest_blk_id, or if that's a *-key, from some descendent block */
 		memcpy(&((TREF(gv_reorgkey))->base[0]), rec_base + SIZEOF(rec_hdr), key_len);
 		(TREF(gv_reorgkey))->end = key_len - 1;
 		key_base = dir_hist_ptr->h[0].buffaddr + dir_hist_ptr->h[0].curr_rec.offset + SIZEOF(rec_hdr);
 		key_len_dir = get_key_len(dir_hist_ptr->h[0].buffaddr, key_base);
 		/* Get root of GVT for dest_blk_id */
-		GET_BLK_ID(gv_target->root, key_base + key_len_dir);
+		READ_BLK_ID(IS_64_BLK_ID(dir_hist_ptr->h[0].buffaddr), &(gv_target->root), key_base + key_len_dir);
 		if ((0 == gv_target->root) || (gv_target->root > (cs_data->trans_hist.total_blks - 1)))
 		{
 			assert(t_tries < CDB_STAGNATE);
@@ -320,9 +339,12 @@ enum cdb_sc mu_swap_blk(int level, block_id *pdest_blk_id, kill_set *kill_set_pt
 		gv_target->clue.end = 0;
 		status = gvcst_search(TREF(gv_reorgkey), dest_hist_ptr);
 		RESET_GV_TARGET_LCL_AND_CLR_GBL(save_targ, DO_GVT_GVKEY_CHECK);
-		if (dest_blk_level >= dest_hist_ptr->depth || /* do not swap in root level */
-			dest_hist_ptr->h[dest_blk_level].blk_num != dest_blk_id) /* must be in a kill set of another process. */
+		if (dest_blk_level >= dest_hist_ptr->depth			/* do not swap in root level */
+			|| dest_hist_ptr->h[dest_blk_level].blk_num != dest_blk_id) /* must be in a kill set of another process. */
+		{
+			assert(0 == upg_mv_block);
 			continue;
+		}
 		if ((cdb_sc_normal != status) || (dest_hist_ptr->h[nslevel].curr_rec.match != ((TREF(gv_reorgkey))->end + 1)))
 		{
 			assert(t_tries < CDB_STAGNATE);
@@ -340,14 +362,17 @@ enum cdb_sc mu_swap_blk(int level, block_id *pdest_blk_id, kill_set *kill_set_pt
 		dest_blk_size = ((blk_hdr_ptr_t)dest_blk_ptr)->bsiz;
 		dest_parent_ptr = dest_hist_ptr->h[dest_blk_level+1].buffaddr;
 		dest_parent_size = ((blk_hdr_ptr_t)dest_parent_ptr)->bsiz;
+		dest_long_blk_id = IS_64_BLK_ID(dest_blk_ptr);
+		dest_blk_id_sz = SIZEOF_BLK_ID(dest_long_blk_id);
 		break;
 	}
-	/*===== End of infinite loop to find the destination block =====*/
+	/*===== End of indefinite loop to find the destination block =====*/
 	/*-----------------------------------------------------
 	   Now modify blocks for swapping. Maximum of 4 blocks.
 	   -----------------------------------------------------*/
 	if (!blk_was_free)
 	{	/* 1: dest_blk_id into work_blk_id */
+		assert(0 == upg_mv_block);
 		BLK_INIT(bs_ptr, bs1);
 		BLK_SEG(bs_ptr, dest_blk_ptr + SIZEOF(blk_hdr), dest_blk_size - SIZEOF(blk_hdr));
 		if (!BLK_FINI (bs_ptr,bs1))
@@ -361,11 +386,12 @@ enum cdb_sc mu_swap_blk(int level, block_id *pdest_blk_id, kill_set *kill_set_pt
 		t_write(&gv_target->hist.h[level], (unsigned char *)bs1, 0, 0, dest_blk_level, TRUE, TRUE, GDS_WRITE_KILLTN);
 	}
 	/* 2: work_blk_id into dest_blk_id */
-	if (!blk_was_free && work_blk_id == dest_hist_ptr->h[dest_blk_level+1].blk_num)
+	if (!blk_was_free && (work_blk_id == dest_hist_ptr->h[dest_blk_level+1].blk_num))
 	{	/* work_blk_id will be swapped with its child.
 		 * This is the only vertical swap.  Here working block goes to its child.
 		 * Working block cannot goto its parent because of traversal
 		 */
+		assert((0 == upg_mv_block));
 		if (dest_blk_level + 1 != level || dest_parent_size != work_blk_size)
 		{
 			assert(t_tries < CDB_STAGNATE);
@@ -384,10 +410,11 @@ enum cdb_sc mu_swap_blk(int level, block_id *pdest_blk_id, kill_set *kill_set_pt
 			return cdb_sc_blkmod;
 		}
 		piece_len1 =  first_offset + rec_size1;
-		BLK_SEG(bs_ptr, saved_blk + SIZEOF(blk_hdr), piece_len1 - SIZEOF(block_id) - SIZEOF(blk_hdr));
-		BLK_ADDR(bn_ptr, SIZEOF(block_id), unsigned char);
-		PUT_BLK_ID(bn_ptr, work_blk_id); /* since work_blk_id will now be the child of dest_blk_id */
-		BLK_SEG(bs_ptr, bn_ptr, SIZEOF(block_id));
+		BLK_SEG(bs_ptr, saved_blk + SIZEOF(blk_hdr), piece_len1 - dest_blk_id_sz - SIZEOF(blk_hdr));
+		BLK_ADDR(bn_ptr, dest_blk_id_sz, unsigned char);
+		/* since work_blk_id will now be the child of dest_blk_id */
+		WRITE_BLK_ID(dest_long_blk_id, work_blk_id, bn_ptr);
+		BLK_SEG(bs_ptr, bn_ptr, dest_blk_id_sz);
 		BLK_SEG(bs_ptr, saved_blk + piece_len1, dest_parent_size - piece_len1);
 		if (!BLK_FINI(bs_ptr, bs1))
 		{
@@ -448,6 +475,7 @@ enum cdb_sc mu_swap_blk(int level, block_id *pdest_blk_id, kill_set *kill_set_pt
 					 * "old_block->bsiz". Restart if we ever access a buffer whose size is greater
 					 * than the db block size.
 					 */
+					 assert(0 == upg_mv_block);
 					bsiz = ((blk_hdr_ptr_t)(tmpcse->old_block))->bsiz;
 					if (bsiz > blk_size)
 					{
@@ -457,9 +485,10 @@ enum cdb_sc mu_swap_blk(int level, block_id *pdest_blk_id, kill_set *kill_set_pt
 					JNL_GET_CHECKSUM_ACQUIRED_BLK(tmpcse, cs_data, cs_addrs, tmpcse->old_block, bsiz);
 				}
 			}
-			assert(GDSVCURR == tmpcse->ondsk_blkver);	/* should have been set by t_create above */
+			assert(cs_data->desired_db_format == tmpcse->ondsk_blkver);	/* should have been set by t_create above */
 		} else
 		{
+			assert(0 == upg_mv_block);
 			hist_ptr = &dest_hist_ptr->h[dest_blk_level];
 			assert(dest_blk_id == hist_ptr->blk_num);
 			assert(dest_blk_ptr == hist_ptr->buffaddr);
@@ -468,6 +497,7 @@ enum cdb_sc mu_swap_blk(int level, block_id *pdest_blk_id, kill_set *kill_set_pt
 	}
 	if (!blk_was_free)
 	{	/* 3: Parent of destination block (may be parent of working block too) */
+		assert(0 == upg_mv_block);
 		if (gv_target->hist.h[level+1].blk_num == dest_hist_ptr->h[dest_blk_level+1].blk_num)
 		{	/* dest parent == work_blk parent */
 			BLK_INIT(bs_ptr, bs1);
@@ -490,27 +520,28 @@ enum cdb_sc mu_swap_blk(int level, block_id *pdest_blk_id, kill_set *kill_set_pt
 			}
 			GET_RSIZ(rec_size1, dest_parent_ptr + first_offset);
 			GET_RSIZ(rec_size2, dest_parent_ptr + second_offset);
-			if (dest_parent_size < first_offset + rec_size1 ||
-				dest_parent_size < second_offset + rec_size2 ||
-				BSTAR_REC_SIZE >= rec_size1 || BSTAR_REC_SIZE > rec_size2)
+			if ((dest_parent_size < first_offset + rec_size1) ||
+				(dest_parent_size < second_offset + rec_size2) ||
+				(bstar_rec_size(dest_long_blk_id) >= rec_size1) ||
+				(bstar_rec_size(dest_long_blk_id) > rec_size2))
 			{
 				assert(t_tries < CDB_STAGNATE);
 				NONTP_TRACE_HIST_MOD(work_blk_stat, t_blkmod_mu_swap_blk);
 				return cdb_sc_blkmod;
 			}
-			piece_len1 =  first_offset + rec_size1 - SIZEOF(block_id);
-			piece_len2 =  second_offset + rec_size2 - SIZEOF(block_id);
-			GET_BLK_ID(child1, dest_parent_ptr + piece_len1);
-			GET_BLK_ID(child2, dest_parent_ptr + piece_len2);
+			piece_len1 =  first_offset + rec_size1 - dest_blk_id_sz;
+			piece_len2 =  second_offset + rec_size2 - dest_blk_id_sz;
+			READ_BLK_ID(dest_long_blk_id, &child1, dest_parent_ptr + piece_len1);
+			READ_BLK_ID(dest_long_blk_id, &child2, dest_parent_ptr + piece_len2);
 			BLK_SEG(bs_ptr, dest_parent_ptr + SIZEOF(blk_hdr), piece_len1 - SIZEOF(blk_hdr));
-			BLK_ADDR(bn_ptr, SIZEOF(block_id), unsigned char);
-			PUT_BLK_ID(bn_ptr, child2);
-			BLK_SEG(bs_ptr, bn_ptr, SIZEOF(block_id));
+			BLK_ADDR(bn_ptr, dest_blk_id_sz, unsigned char);
+			WRITE_BLK_ID(dest_long_blk_id, child2, bn_ptr);
+			BLK_SEG(bs_ptr, bn_ptr, dest_blk_id_sz);
 			BLK_SEG(bs_ptr, dest_parent_ptr + first_offset + rec_size1,
-				second_offset + rec_size2 - SIZEOF(block_id) - first_offset - rec_size1);
-			BLK_ADDR(bn_ptr, SIZEOF(block_id), unsigned char);
-			PUT_BLK_ID(bn_ptr, child1);
-			BLK_SEG(bs_ptr, bn_ptr, SIZEOF(block_id));
+				second_offset + rec_size2 - dest_blk_id_sz - first_offset - rec_size1);
+			BLK_ADDR(bn_ptr, dest_blk_id_sz, unsigned char);
+			WRITE_BLK_ID(dest_long_blk_id, child1, bn_ptr);
+			BLK_SEG(bs_ptr, bn_ptr, dest_blk_id_sz);
 			BLK_SEG(bs_ptr, dest_parent_ptr + second_offset + rec_size2,
 				dest_parent_size - second_offset - rec_size2);
 			if (!BLK_FINI(bs_ptr,bs1))
@@ -528,20 +559,20 @@ enum cdb_sc mu_swap_blk(int level, block_id *pdest_blk_id, kill_set *kill_set_pt
 			 */
 			BLK_INIT(bs_ptr, bs1);
 			GET_RSIZ(rec_size1, dest_parent_ptr + dest_hist_ptr->h[dest_blk_level+1].curr_rec.offset);
-			if (dest_parent_size < rec_size1 +  dest_hist_ptr->h[dest_blk_level+1].curr_rec.offset ||
-				BSTAR_REC_SIZE > rec_size1)
+			if ((dest_parent_size < (rec_size1 + dest_hist_ptr->h[dest_blk_level+1].curr_rec.offset)) ||
+				(bstar_rec_size(dest_long_blk_id) > rec_size1))
 			{
 				assert(t_tries < CDB_STAGNATE);
 				NONTP_TRACE_HIST_MOD(work_blk_stat, t_blkmod_mu_swap_blk);
 				return cdb_sc_blkmod;
 			}
 			BLK_SEG (bs_ptr, dest_parent_ptr + SIZEOF(blk_hdr),
-			    dest_hist_ptr->h[dest_blk_level+1].curr_rec.offset + rec_size1 - SIZEOF(blk_hdr) - SIZEOF(block_id));
-			BLK_ADDR(bn_ptr, SIZEOF(block_id), unsigned char);
-			PUT_BLK_ID(bn_ptr, work_blk_id);
-			BLK_SEG(bs_ptr, bn_ptr, SIZEOF(block_id));
+			    dest_hist_ptr->h[dest_blk_level+1].curr_rec.offset + rec_size1 - SIZEOF(blk_hdr) - dest_blk_id_sz);
+			BLK_ADDR(bn_ptr, dest_blk_id_sz, unsigned char);
+			WRITE_BLK_ID(dest_long_blk_id, work_blk_id, bn_ptr);
+			BLK_SEG(bs_ptr, bn_ptr, dest_blk_id_sz);
 			BLK_SEG(bs_ptr, dest_parent_ptr + dest_hist_ptr->h[dest_blk_level+1].curr_rec.offset + rec_size1,
-				dest_parent_size - dest_hist_ptr->h[dest_blk_level+1].curr_rec.offset  - rec_size1);
+				dest_parent_size - dest_hist_ptr->h[dest_blk_level+1].curr_rec.offset - rec_size1);
 			if (!BLK_FINI(bs_ptr,bs1))
 			{
 				assert(t_tries < CDB_STAGNATE);
@@ -557,18 +588,19 @@ enum cdb_sc mu_swap_blk(int level, block_id *pdest_blk_id, kill_set *kill_set_pt
 	if (blk_was_free || gv_target->hist.h[level+1].blk_num != dest_hist_ptr->h[dest_blk_level+1].blk_num)
 	{	/* Parent block of working blk should correctly point the working block. Working block went to dest_blk_id  */
 		GET_RSIZ(rec_size1, (work_parent_ptr + gv_target->hist.h[level+1].curr_rec.offset));
-		if (work_parent_size < rec_size1 +  gv_target->hist.h[level+1].curr_rec.offset || BSTAR_REC_SIZE > rec_size1)
+		if (work_parent_size < rec_size1 +  gv_target->hist.h[level+1].curr_rec.offset ||
+				bstar_rec_size(work_long_blk_id) > rec_size1)
 		{
-			assert(t_tries < CDB_STAGNATE);
+			assert((t_tries < CDB_STAGNATE) && (0 == upg_mv_block));
 			NONTP_TRACE_HIST_MOD(work_blk_stat, t_blkmod_mu_swap_blk);
 			return cdb_sc_blkmod;
 		}
 		BLK_INIT(bs_ptr, bs1);
 		BLK_SEG(bs_ptr, work_parent_ptr + SIZEOF(blk_hdr),
-			gv_target->hist.h[level+1].curr_rec.offset + rec_size1 - SIZEOF(blk_hdr) - SIZEOF(block_id));
-		BLK_ADDR(bn_ptr, SIZEOF(block_id), unsigned char);
-		PUT_BLK_ID(bn_ptr, dest_blk_id);
-		BLK_SEG(bs_ptr, bn_ptr, SIZEOF(block_id));
+			gv_target->hist.h[level+1].curr_rec.offset + rec_size1 - SIZEOF(blk_hdr) - work_blk_id_sz);
+		BLK_ADDR(bn_ptr, work_blk_id_sz, unsigned char);
+		WRITE_BLK_ID(work_long_blk_id, dest_blk_id, bn_ptr);
+		BLK_SEG(bs_ptr, bn_ptr, work_blk_id_sz);
 		BLK_SEG(bs_ptr, work_parent_ptr + gv_target->hist.h[level+1].curr_rec.offset + rec_size1,
 			work_parent_size - gv_target->hist.h[level+1].curr_rec.offset - rec_size1);
 		if (!BLK_FINI(bs_ptr, bs1))
@@ -614,6 +646,8 @@ enum cdb_sc mu_swap_blk(int level, block_id *pdest_blk_id, kill_set *kill_set_pt
 		kill_set_ptr->blk[kill_set_ptr->used].level = 0;
 		kill_set_ptr->blk[kill_set_ptr->used++].block = work_blk_id;
 	}
+	if (0 != upg_mv_block)
+		reorg_gv_target->hist.h[dest_blk_level] = destblkhist;
 	*pdest_blk_id = dest_blk_id;
 	return cdb_sc_normal;
 }
