@@ -1,6 +1,7 @@
 /****************************************************************
  *								*
- *	Copyright 2001 Sanchez Computer Associates, Inc.	*
+ * Copyright (c) 2001-2022 Fidelity National Information	*
+ * Services, Inc. and/or its subsidiaries. All rights reserved.	*
  *								*
  *	This source code contains the intellectual property	*
  *	of its copyright holder(s), and is made available	*
@@ -12,6 +13,6 @@
 #ifndef MUCBLKINI_INCLUDED
 #define MUCBLKINI_INCLUDED
 
-void mucblkini(void);
+void mucblkini(enum db_ver desired_db_ver);
 
 #endif /* MUCBLKINI_INCLUDED */
