@@ -1,6 +1,6 @@
 /****************************************************************
  *								*
- * Copyright (c) 2001-2019 Fidelity National Information	*
+ * Copyright (c) 2001-2022 Fidelity National Information	*
  * Services, Inc. and/or its subsidiaries. All rights reserved.	*
  *								*
  *	This source code contains the intellectual property	*
@@ -74,7 +74,7 @@ error_def(ERR_JNLALLOCGROW);
 error_def(ERR_MUNOACTION);
 error_def(ERR_TEXT);
 
-void mucregini(block_id blk_init_size)
+void mucregini(block_id blk_init_size, enum db_ver desired_db_ver)
 {
 	int4			status;
 	block_id		i;
@@ -86,17 +86,37 @@ void mucregini(block_id blk_init_size)
 	gd_region		*baseDBreg;
 	gd_segment		*seg;
 	sgmnt_data_ptr_t	csd;
+	char			*label;
+	enum db_ver		majorver;
+	enum mdb_ver		minorver;
+	trans_num		max_tn_default;
+	gtm_int8		master_map_size;
 
 	csd = cs_data;
-	MEMCPY_LIT(csd->label, GDS_LABEL);
-	csd->desired_db_format = GDSVCURR;
+	if (GDSVCURR == desired_db_ver)
+	{	/* Current version defaults */
+		label		= GDS_LABEL;
+		majorver	= GDSVCURR;
+		minorver	= GDSMVCURR;
+		master_map_size	= MASTER_MAP_SIZE_DFLT;
+		max_tn_default	= MAX_TN_DFLT;
+	} else
+	{	/* Prior version settings. TODO allow for alternative settings */
+		label		= V6_GDS_LABEL;
+		majorver	= GDSV6;
+		minorver	= GDSMV63014;
+		master_map_size	= MASTER_MAP_SIZE_V6;
+		max_tn_default	= MAX_TN_V6;
+	}
+	memcpy(csd->label, label, strnlen(label, sizeof(csd->label)));
+	csd->desired_db_format = majorver;
 	csd->fully_upgraded = TRUE;
 	csd->db_got_to_v5_once = TRUE;	/* no V4 format blocks that are non-upgradeable */
-	csd->minor_dbver = GDSMVCURR;
-	csd->certified_for_upgrade_to = GDSVCURR;
-	csd->creation_db_ver = GDSVCURR;
-	csd->creation_mdb_ver = GDSMVCURR;
-	csd->master_map_len = MASTER_MAP_SIZE_DFLT;
+	csd->minor_dbver = minorver;
+	csd->certified_for_upgrade_to = majorver;
+	csd->creation_db_ver = majorver;
+	csd->creation_mdb_ver = minorver;
+	csd->master_map_len = master_map_size;
 	csd->bplmap = BLKS_PER_LMAP;
 	seg = gv_cur_region->dyn.addr;
 	assert(seg->blk_size <= MAX_DB_BLK_SIZE);
@@ -250,7 +270,7 @@ void mucregini(block_id blk_init_size)
 
 	csd->max_update_array_size = csd->max_non_bm_update_array_size
 					= (int4)ROUND_UP2(MAX_NON_BITMAP_UPDATE_ARRAY_SIZE(csd), UPDATE_ARRAY_ALIGN_SIZE);
-	csd->max_update_array_size += (int4)ROUND_UP2(MAX_BITMAP_UPDATE_ARRAY_SIZE, UPDATE_ARRAY_ALIGN_SIZE);
+	csd->max_update_array_size += (int4)ROUND_UP2(MAX_BITMAP_UPDATE_ARRAY_SIZE(csd), UPDATE_ARRAY_ALIGN_SIZE);
 	/* bt_malloc(cs_addrs) Done by db_init at file open time -- not needed here */
 	if (dba_bg == REG_ACC_METH(gv_cur_region))
 		csd->flush_time[0] = TIM_FLU_MOD_BG;
@@ -268,6 +288,7 @@ void mucregini(block_id blk_init_size)
 	time(&ctime);
 	assert(SIZEOF(ctime) >= SIZEOF(int4));
 	csd->creation_time4 = (int4)ctime;	/* Need only lower order 4-bytes of current time (in case system time is 8-bytes) */
+	csd->statsdb_allocation = STATSDB_ALLOCATION;
 	if (IS_STATSDB_REG(gv_cur_region))
 	{	/* Copy basedb fname into statsdb file header (needed by MUPIP RUNDOWN -FILE statsdb-file-name) */
 		STATSDBREG_TO_BASEDBREG(gv_cur_region, baseDBreg);
@@ -289,14 +310,14 @@ void mucregini(block_id blk_init_size)
 			(*mupip_exit_fp)(ERR_MUNOACTION);
 		}
 	}
-	mucblkini();
+	mucblkini(desired_db_ver);
 	th->mm_tn = 0;
 	th->early_tn = 1;
 	th->curr_tn = 1;	/* in order to use INCREMENT_CURR_TN macro here, the logic has to be made complicated.
 				 * this is because the macro relies on max_tn/max_tn_warn being set and that does not happen
 				 * until a few lines later. hence keeping it simple here by doing a plain assignment of curr_tn.
 				 */
-	csd->max_tn = MAX_TN_DFLT;
+	csd->max_tn = max_tn_default;
 	SET_TN_WARN(csd, csd->max_tn_warn);
 	SET_LATCH_GLOBAL(&csd->next_upgrd_warn.time_latch, LOCK_AVAILABLE);
 }
