@@ -1,6 +1,6 @@
 /****************************************************************
  *								*
- * Copyright (c) 2001-2020 Fidelity National Information	*
+ * Copyright (c) 2001-2022 Fidelity National Information	*
  * Services, Inc. and/or its subsidiaries. All rights reserved.	*
  *								*
  *	This source code contains the intellectual property	*
@@ -9,9 +9,7 @@
  *	the license, please stop and do not read further.	*
  *								*
  ****************************************************************/
-
 #include "mdef.h"
-
 #include "gtm_fcntl.h"
 #include "gtm_stat.h"
 #include "gtm_unistd.h"
@@ -22,7 +20,6 @@
 #include "gtm_stdlib.h"
 #include "gtm_tempnam.h"
 #include "gtm_time.h"
-
 #ifdef __MVS__
 #include "gtm_zos_io.h"
 #endif
@@ -84,14 +81,12 @@
 #include "repl_sem.h"
 #include "gtm_sem.h"
 #include "anticipatory_freeze.h"
-
+#include "mdq.h"
 #define PATH_DELIM		'/'
-
 #define TMPDIR_ACCESS_MODE	(R_OK | W_OK | X_OK)
 
 GBLDEF  boolean_t	backup_started;
 GBLDEF  boolean_t	backup_interrupted;
-
 GBLREF 	bool		record;
 GBLREF 	bool		error_mupip;
 GBLREF 	bool		file_backed_up;
@@ -120,20 +115,20 @@ GBLREF	gd_addr		*gd_header;
 #ifdef DEBUG
 GBLREF  int		process_exiting;		/* Process is on it's way out */
 #endif
-
 GBLREF	boolean_t		jnlpool_init_needed;
 GBLREF	backup_reg_list		*mu_repl_inst_reg_list;
 GBLREF	jnlpool_addrs_ptr_t	jnlpool;
 GBLREF	jnlpool_addrs_ptr_t	jnlpool_head;
 GBLREF	uint4			mutex_per_process_init_pid;
 GBLREF	boolean_t		holds_sem[NUM_SEM_SETS][NUM_SRC_SEMS];
+GBLREF	usr_reg_que		*usr_spec_regions;
 GBLREF	int			pool_init;
-
 LITREF char             gtm_release_name[];
 LITREF int4             gtm_release_name_len;
 
 error_def(ERR_BACKUPCTRL);
 error_def(ERR_BACKUPKILLIP);
+error_def(ERR_BKUPRETRY);
 error_def(ERR_BKUPRUNNING);
 error_def(ERR_NOMORESEMCNT);
 error_def(ERR_DBCCERR);
@@ -169,6 +164,12 @@ error_def(ERR_REPLSTATEERR);
 error_def(ERR_SYSCALL);
 error_def(ERR_TEXT);
 error_def(ERR_FILENAMETOOLONG);
+error_def(ERR_BACKUPSUCCESS);
+error_def(ERR_BACKUPFAIL);
+error_def(ERR_DIRACCESS);
+error_def(ERR_BACKUPREPL);
+error_def(ERR_BACKUPSEQNO);
+error_def(ERR_CMDERR);
 ZOS_ONLY(error_def(ERR_BADTAG);)
 
 static char	* const jnl_parms[] =
@@ -228,14 +229,14 @@ void mupip_backup(void)
 	backup_reg_list		*rptr, *rrptr, *nocritrptr;
 	boolean_t		inc_since_inc , inc_since_rec, result, newjnlfiles, tn_specified,
 				replication_on, newjnlfiles_specified, keep_prev_link, bkdbjnl_disable_specified,
-				bkdbjnl_off_specified;
+				bkdbjnl_off_specified, stopretries;
 	unsigned char		since_buff[50];
 	jnl_create_info		jnl_info;
 	file_control		*fc;
 	char			tempdir_trans_buffer[MAX_TRANS_NAME_LEN],
 				tempnam_prefix[MAX_FN_LEN], tempdir_full_buffer[MAX_FN_LEN + 1];
 	char			*jnl_str_ptr, jnl_str[256], entry[256], prev_jnl_fn[JNL_NAME_SIZE];
-	int			index, jnl_fstat;
+	int			index, jnl_fstat, attemptcnt, maxtries;
 	mstr			tempdir_log, tempdir_trans, *file, *rfile, *replinstfile, tempdir_full, filestr;
 	uint4			jnl_status, temp_file_name_len, tempdir_trans_len, trans_log_name_status;
 	boolean_t		jnl_options[jnl_end_of_list] = {FALSE, FALSE, FALSE}, save_no_prev_link;
@@ -244,7 +245,7 @@ void mupip_backup(void)
 	jnl_tm_t		save_gbl_jrec_time;
 	gd_region		*r_save, *reg;
 	int			sync_io_status;
-	boolean_t		sync_io, sync_io_specified;
+	boolean_t		sync_io, sync_io_specified, showprogress;
 	boolean_t		dummy_ftok_counter_halted;
 	void			*repl_inst_available;
 	struct stat		stat_buf;
@@ -274,6 +275,7 @@ void mupip_backup(void)
 	char			time_str[CTIME_BEFORE_NL + 2];	/* for GET_CUR_TIME macro */
 	struct perm_diag_data	pdd;
 	ZOS_ONLY(int		realfiletag;)
+	usr_reg_que		*region_que_entry;
 
 	/* ==================================== STEP 1. Initialization ======================================= */
 	backup_started = backup_interrupted = FALSE;
@@ -319,6 +321,7 @@ void mupip_backup(void)
 	}
 	online = (TRUE != cli_negated("ONLINE"));
 	record = (CLI_PRESENT == cli_present("RECORD"));
+	showprogress = (CLI_PRESENT == cli_present("SHOWPROGRESS"));
 	newjnlfiles_specified = FALSE;
 	newjnlfiles = TRUE;	/* by default */
 	keep_prev_link = TRUE;
@@ -397,7 +400,7 @@ void mupip_backup(void)
 	if (error_mupip)
 	{
 		mubclnup(NULL, need_to_free_space);
-		util_out_print("!/MUPIP cannot start backup with above errors!/", TRUE);
+		gtm_putmsg_csa(CSA_ARG(NULL) VARLSTCNT(1) ERR_BACKUPFAIL);
 		mupip_exit(ERR_MUNOACTION);
 	}
 	if (mu_ctrly_occurred || mu_ctrlc_occurred)
@@ -487,7 +490,7 @@ void mupip_backup(void)
 	if (TRUE == error_mupip)
 	{
 		mubclnup(NULL, need_to_free_space);
-		util_out_print("!/MUPIP cannot start backup with above errors!/", TRUE);
+		gtm_putmsg_csa(CSA_ARG(NULL) VARLSTCNT(1) ERR_BACKUPFAIL);
 		mupip_exit(ERR_MUNOACTION);
 	}
 	/* =========================== STEP 3. Verify the regions and grab_crit()/freeze them ============ */
@@ -655,14 +658,14 @@ void mupip_backup(void)
 					(0 != ACCESS(tempdir_full.addr, TMPDIR_ACCESS_MODE)))
 				{
 					status = errno;
-					util_out_print("!/Do not have full access to directory for temporary files: !AD", TRUE,
+					gtm_putmsg_csa(CSA_ARG(cs_addrs) VARLSTCNT(4) ERR_DIRACCESS, 2,
 						tempdir_trans.len, tempdir_trans.addr);
 				} else
 					util_out_print("!/Cannot create the temporary file in directory !AD for online backup",
 						TRUE, tempdir_trans.len, tempdir_trans.addr);
 				gtm_putmsg_csa(CSA_ARG(cs_addrs) VARLSTCNT(1) status);
 				SET_ERROR_CONDITION(status);	/* sets "error_condition" & "severity" */
-				util_out_print("!/MUPIP cannot start backup with above errors!/", TRUE);
+				gtm_putmsg_csa(CSA_ARG(NULL) VARLSTCNT(1) ERR_BACKUPFAIL);
 				mubclnup(rptr, need_to_del_tempfile);
 				mupip_exit(status);
 			}
@@ -684,7 +687,7 @@ void mupip_backup(void)
 				util_out_print("!/Error re-opening temporary file created by mkstemp()!/", TRUE);
 				gtm_putmsg_csa(CSA_ARG(cs_addrs) VARLSTCNT(1) status);
 				SET_ERROR_CONDITION(status);	/* sets "error_condition" & "severity" */
-				util_out_print("!/MUPIP cannot start backup with above errors!/", TRUE);
+				gtm_putmsg_csa(CSA_ARG(NULL) VARLSTCNT(1) ERR_BACKUPFAIL);
 				mubclnup(rptr, need_to_del_tempfile);
 				mupip_exit(status);
 			}
@@ -727,7 +730,7 @@ void mupip_backup(void)
 				status = errno;
 				gtm_putmsg_csa(CSA_ARG(cs_addrs) VARLSTCNT(1) status);
 				SET_ERROR_CONDITION(status);	/* sets "error_condition" & "severity" */
-				util_out_print("!/MUPIP cannot start backup with above errors!/", TRUE);
+				gtm_putmsg_csa(CSA_ARG(NULL) VARLSTCNT(1) ERR_BACKUPFAIL);
 				mubclnup(rptr, need_to_del_tempfile);
 				mupip_exit(status);
 			}
@@ -776,7 +779,7 @@ void mupip_backup(void)
 		{	/* pool_init is only set if the source server has setup shared memory */
 			assert(NULL != jnlpool);
 			if (!repl_inst_available)
-				rts_error_csa(CSA_ARG(NULL) VARLSTCNT(1) ERR_REPLINSTUNDEF);
+				RTS_ERROR_CSA_ABT(NULL, VARLSTCNT(1) ERR_REPLINSTUNDEF);
 			if (!jnlpool)
 			{	/* jnlpool_init should have provided but be safe */
 				jnlpool = malloc(SIZEOF(jnlpool_addrs));
@@ -1090,7 +1093,7 @@ repl_inst_bkup_done1:
 						errptr = (char *)STRERROR(save_errno);
 						util_out_print("system : !AZ", TRUE, errptr);
 					}
-					util_out_print("Error doing !AD", TRUE, cmdptr - command, command);
+					gtm_putmsg_csa(CSA_ARG(cs_addrs) VARLSTCNT(4) ERR_CMDERR, 2, cmdptr - command, command);
 					error_mupip = TRUE;
 					goto repl_inst_bkup_done2;
 				}
@@ -1156,11 +1159,9 @@ repl_inst_bkup_done2:
 			}
 			if (!error_mupip)
 			{
-				util_out_print("Replication Instance file !AD backed up in file !AD", TRUE,
-					LEN_AND_STR(udi->fn),
+				gtm_putmsg_csa(CSA_ARG(NULL) VARLSTCNT(6) ERR_BACKUPREPL, 4, LEN_AND_STR(udi->fn),
 					mu_repl_inst_reg_list->backup_file.len, mu_repl_inst_reg_list->backup_file.addr);
-				util_out_print("Journal Seqnos up to 0x!16@XQ are backed up.", TRUE, &jnl_seqno);
-				util_out_print("", TRUE);
+				gtm_putmsg_csa(CSA_ARG(NULL) VARLSTCNT(3) ERR_BACKUPSEQNO, 1, &jnl_seqno);
 			} else
 				util_out_print("Error backing up replication instance file !AD. Moving on to other backups.",
 					TRUE, LEN_AND_STR(udi->fn));
@@ -1427,7 +1428,7 @@ repl_inst_bkup_done2:
 				shmpool_unlock_hdr(gv_cur_region);
 				/* Signal to GT.M processes to start backing up before-images of any GDS block that gets changed
 				 * starting now. To do that set "nbb" to -1 as that is smaller than every valid block number in db
-				 * (starting from 0 upto a max. of 2^28-1 which is lesser than 2^31-1=BACKUP_NOT_IN_PROGRESS)
+				 * (starting from 0 upto a max. of 2^62-1 which is lesser than 2^63-1=BACKUP_NOT_IN_PROGRESS)
 				 */
 				cs_addrs->nl->nbb = -1; /* start */
 				/* Decerement counter so that inhibited KILLs can now proceed */
@@ -1472,31 +1473,67 @@ repl_inst_bkup_done2:
 					LEN_AND_STR(jnl_state_lit[rptr->backup_hdr->jnl_state]));
 		}
 		ENABLE_AST
-		for (rptr = (backup_reg_list *)(grlist);  NULL != rptr;  rptr = rptr->fPtr)
+		/* Set maxtries */
+		if (CLI_PRESENT == cli_present("RETRY"))
 		{
-			if (rptr->not_this_time > keep_going)
-				continue;
-			gv_cur_region = rptr->reg;
-			TP_CHANGE_REG(gv_cur_region);	/* sets cs_addrs and cs_data which mubinccpy/mubfilcpy rely on */
-			if ((cs_addrs->onln_rlbk_cycle != cs_addrs->nl->onln_rlbk_cycle)
-				|| (0 != cs_addrs->nl->onln_rlbk_pid))
-			{	/* A concurrent online rollback happened since we did the gvcst_init or one is going on right now.
-				 * In either case, the BACKUP is unreliable. Cleanup and exit
-				 */
-				error_mupip = TRUE;
-				gtm_putmsg_csa(CSA_ARG(cs_addrs) VARLSTCNT(1) ERR_DBROLLEDBACK);
-				break;
-			}
-			result = (incremental ? mubinccpy(rptr) : mubfilcpy(rptr));
-			if (FALSE == result)
+			cli_get_int("RETRY", &maxtries);
+			maxtries++;
+		}
+		else
+			maxtries = 2;
+		if (incremental)
+			maxtries = 1;
+		dqloop(usr_spec_regions, que, region_que_entry)
+		{
+			for (rptr = (backup_reg_list *)(grlist);  NULL != rptr;  rptr = rptr->fPtr)
 			{
-				if (file_backed_up)
-					util_out_print("Files backed up before above error are OK.", TRUE);
-				error_mupip = TRUE;
-				break;
+				if (rptr->not_this_time > keep_going)
+					continue;
+				TP_CHANGE_REG(rptr->reg); /* sets cs_addrs and cs_data which mubinccpy/mubfilcpy rely on */
+				if ((cs_addrs->onln_rlbk_cycle != cs_addrs->nl->onln_rlbk_cycle)
+					|| (0 != cs_addrs->nl->onln_rlbk_pid))
+				{	/* A concurrent online rollback happened since we did the gvcst_init or one
+					* is going on right now. In either case, the BACKUP is unreliable. Cleanup
+					* and exit.
+					*/
+					error_mupip = TRUE;
+					gtm_putmsg_csa(CSA_ARG(cs_addrs) VARLSTCNT(1) ERR_DBROLLEDBACK);
+					break;
+				}
+				if ((char *)gv_cur_region->rname == (char *)region_que_entry->usr_reg)
+					break; /* Matching region found. Exit the loop*/
 			}
-			if (mu_ctrly_occurred || mu_ctrlc_occurred)
-				break;
+			if (NULL == rptr)
+				continue; /* continue the dqloop */
+			/* If we are doing a timestamp, this is point we take as the date of the backup */
+			if (record)
+			{
+				rptr->backup_hdr->last_start_backup = (gtm_timet) time(NULL);
+			}
+			for (attemptcnt = 1, stopretries = FALSE; (attemptcnt <= maxtries) ; attemptcnt++)
+			{
+				if (1 < attemptcnt)
+				{
+					gtm_putmsg_csa(CSA_ARG(REG2CSA(rptr->reg)) VARLSTCNT(8) ERR_BKUPRETRY, 6,
+							REG_LEN_STR(gv_cur_region), DB_LEN_STR(gv_cur_region),
+							attemptcnt, maxtries);
+				}
+				result = (incremental ? mubinccpy(rptr) : mubfilcpy(rptr, showprogress, attemptcnt, &stopretries));
+				if (result)
+					break; /* break to dqloop */
+
+				if ((attemptcnt == maxtries) || (TRUE == stopretries))
+				{
+					if (file_backed_up)
+						util_out_print("Files backed up before above error are OK.", TRUE);
+					error_mupip = TRUE; /* retry attempts exhausted without success. */
+					break;
+				}
+				if (mu_ctrlc_occurred)
+					break;
+			}
+			if (error_mupip)
+				break; /* break out of dqloop */
 		}
 	} else
 	{
@@ -1514,7 +1551,7 @@ repl_inst_bkup_done2:
 	} else if (TRUE == error_mupip)
 		ret = ERR_MUNOFINISH;
 	else
-		util_out_print("!/!/BACKUP COMPLETED.!/", TRUE);
+		gtm_putmsg_csa(CSA_ARG(NULL) VARLSTCNT(1) ERR_BACKUPSUCCESS);
 	gv_cur_region = NULL;
 	mupip_exit(ret);
 }
