@@ -1,6 +1,6 @@
 /****************************************************************
  *								*
- * Copyright (c) 2001-2020 Fidelity National Information	*
+ * Copyright (c) 2001-2022 Fidelity National Information	*
  * Services, Inc. and/or its subsidiaries. All rights reserved.	*
  *								*
  *	This source code contains the intellectual property	*
@@ -39,7 +39,6 @@
 #include "ipcrmid.h"
 #include "db_ipcs_reset.h"
 #endif
-#include "mu_getlst.h"
 #include "mu_outofband_setup.h"
 #include "mupip_integ.h"
 #include "gtmmsg.h"
@@ -52,6 +51,10 @@
 #include "muextr.h"
 #include "mu_getkey.h"
 #include "gvt_inline.h"
+#include "mdq.h"
+#include "mu_getlst.h"
+#include "gtm_malloc.h"
+#include "compiler.h"
 
 #define MAX_UTIL_LEN			96
 #define APPROX_ALL_ERRORS		1000000
@@ -146,6 +149,7 @@ GBLREF boolean_t		debug_mupip;
 GBLREF gv_key			*mu_end_key;
 GBLREF boolean_t		mu_key;
 GBLREF gv_key			*mu_start_key;
+GBLREF usr_reg_que		*usr_spec_regions;
 
 error_def(ERR_CTRLC);
 error_def(ERR_CTRLY);
@@ -198,6 +202,8 @@ void mupip_integ(void)
 	node_local_ptr_t	baseDBnl;
 	sgmnt_addrs		*tcsa;
 	char 			*db_file_name;
+	usr_reg_que		*usr_spec_regions_integ;
+	usr_reg_que		*region_que_entry;
 	DCL_THREADGBL_ACCESS;
 
 	SETUP_THREADGBL_ACCESS;
@@ -315,7 +321,7 @@ void mupip_integ(void)
 	}
 	if (CLI_PRESENT == cli_present("BLOCK"))
 	{
-		if (0 == cli_get_hex("BLOCK", (uint4 *)&muint_block))
+		if (0 == cli_get_hex64("BLOCK", (gtm_uint8 *)&muint_block))
 			mupip_exit(SS_NORMAL);
 		block = TRUE;
 		disp_map_errors = 0;
@@ -340,22 +346,46 @@ void mupip_integ(void)
 		} else /* Establish the condition handler ONLY if ONLINE INTEG was not requested */
 			ESTABLISH(mu_freeze_ch);
 	}
-	for (total_errors = mu_int_errknt = 0;  ;  total_errors += mu_int_errknt, mu_int_errknt = 0)
+	total_errors = 0;
+	mu_int_errknt = 0;
+	if (!region)
 	{
+		/* Add a dummy region just to be able to execute dqloop() */
+		usr_spec_regions_integ = (usr_reg_que *)malloc(SIZEOF(usr_reg_que));
+		dqinit(usr_spec_regions_integ, que);
+		region_que_entry = (usr_reg_que *)malloc(SIZEOF(usr_reg_que));
+		region_que_entry->usr_reg = "_DUMMYREG";
+		region_que_entry->usr_reg_len = 9;
+                /* Insert on the right side of the queue */
+		dqrins(usr_spec_regions_integ, que, region_que_entry);
+	}
+	else
+		usr_spec_regions_integ = usr_spec_regions;
+	dqloop(usr_spec_regions_integ, que, region_que_entry)
+	{
+		total_errors += mu_int_errknt;
+		mu_int_errknt = 0;
 		if (mu_ctrly_occurred || mu_ctrlc_occurred)
 			break;
 		if (region)
 		{
-			assert(NULL != rptr);
-			reg = rptr->reg;
-			if (!mupfndfil(reg, NULL, LOG_ERROR_TRUE))
+			for (rptr = grlist;  NULL != rptr;  rptr = rptr->fPtr)
 			{
-				mu_int_skipreg_cnt++;
-				rptr = rptr->fPtr;
-				if (NULL == rptr)
-					break;
-				continue;
+				gv_cur_region = rptr->reg;
+				reg = rptr->reg;
+				if ((char *)gv_cur_region->rname == (char *)region_que_entry->usr_reg)
+				{
+					if (!mupfndfil(reg, NULL, LOG_ERROR_TRUE))
+					{
+						mu_int_skipreg_cnt++;
+						continue;
+					}
+					else
+						break;
+				}
 			}
+			if (NULL == rptr)
+				continue; /* continue the dqloop */
 		}
 		memset(mu_int_tot, 0, SIZEOF(mu_int_tot));
 		memset(mu_int_cum, 0, SIZEOF(mu_int_tot));
@@ -419,10 +449,7 @@ void mupip_integ(void)
 			{
 				if (cs_addrs && cs_addrs->hdr && cs_addrs->hdr->recov_interrupted)
 					util_out_print("!/Recover Interrupted flag is TRUE.!/", TRUE);
-				rptr = rptr->fPtr;
-				if (NULL == rptr)
-					break;
-				continue;
+				continue; /* continue the dqloop */
 			}
 			csa = cs_addrs;
 			/* If the region was frozen (INTEG -REG -NOONLINE) then use cs_addrs->hdr for verification of
@@ -672,11 +699,14 @@ void mupip_integ(void)
 			if (!muint_fast && (mu_int_blks_to_upgrd != csd->blks_to_upgrd))
 			{
 				gtm_putmsg_csa(CSA_ARG(csa)
-					VARLSTCNT(4) ERR_DBBTUWRNG, 2, mu_int_blks_to_upgrd, csd->blks_to_upgrd);
+					VARLSTCNT(4) ERR_DBBTUWRNG, 2, &(csd->blks_to_upgrd), &mu_int_blks_to_upgrd);
 				if (gv_cur_region->read_only || mu_int_errknt)
 					mu_int_errknt++;
 				else
+				{
+					csd->blks_to_upgrd = mu_int_blks_to_upgrd;
 					gtm_putmsg_csa(CSA_ARG(csa) VARLSTCNT(1) ERR_DBBTUFIXED);
+				}
 			}
 			if (((0 != mu_int_data.kill_in_prog) || (0 != mu_int_data.abandoned_kills)) && (!mu_map_errs) && !region
 				&& !gv_cur_region->read_only)
@@ -807,9 +837,7 @@ void mupip_integ(void)
 				ss_release(&csa->ss_ctx);
 				CLEAR_SNAPSHOTS_IN_PROG(csa);
 			}
-			rptr = rptr->fPtr;
-			if (NULL == rptr)
-				break;
+				continue; /* continue the dqloop */
 		} else if (!gv_cur_region->read_only)
 		{
 			assert(!online_integ);
@@ -902,7 +930,7 @@ void mupip_integ(void)
 	if ((NULL != tcsa) && (NULL != tcsa->ti) && (0 != tcsa->ti->total_blks) && (NULL != tcsa->hdr)
 		&& (0 != MAXTOTALBLKS(tcsa->hdr)))
 	{
-		if ((NULL != gv_cur_region->dyn.addr) && (NULL != gv_cur_region->dyn.addr->fname))
+		if ((NULL != gv_cur_region->dyn.addr) && (0 != gv_cur_region->dyn.addr->fname_len))
 			db_file_name = (char *)gv_cur_region->dyn.addr->fname;
 		else
 			db_file_name = "";
