@@ -1,6 +1,6 @@
 /****************************************************************
  *								*
- * Copyright (c) 2001-2020 Fidelity National Information	*
+ * Copyright (c) 2001-2022 Fidelity National Information	*
  * Services, Inc. and/or its subsidiaries. All rights reserved.	*
  *								*
  *	This source code contains the intellectual property	*
@@ -35,6 +35,7 @@ GBLDEF	bool		region;
 GBLREF	bool		error_mupip;
 GBLREF	boolean_t	jnlpool_init_needed;
 
+error_def(ERR_GTMCURUNSUPP);
 error_def(ERR_MUNOACTION);
 error_def(ERR_MUNODBNAME);
 
@@ -44,7 +45,7 @@ void mupip_set(void)
 	char		db_fn[MAX_FN_LEN + 1], jnl_fn[JNL_NAME_SIZE + 1];
 	unsigned short	db_fn_len, jnl_fn_len;
 	uint4		status;
-	int 		cli_stat;
+	int		cli_stat;
 	boolean_t	set_journal, set_replication;
 
 	jnlpool_init_needed = TRUE;
@@ -78,6 +79,10 @@ void mupip_set(void)
 		status = mupip_set_jnlfile(jnl_fn, SIZEOF(jnl_fn));
 		mupip_exit(status);
 	}
+	if(CLI_PRESENT == cli_present("VERSION"))
+	{
+		mupip_exit(ERR_GTMCURUNSUPP);
+	}
 	if ((CLI_PRESENT == cli_present("ACCESS_METHOD"))
 		|| (CLI_PRESENT == cli_present("ASYNCIO"))
 		|| (CLI_NEGATED == cli_present("ASYNCIO"))
@@ -112,13 +117,15 @@ void mupip_set(void)
 		|| (CLI_PRESENT == cli_present("SPIN_SLEEP_MASK"))
 		|| (CLI_NEGATED == cli_present("STATS"))
 		|| (CLI_PRESENT == cli_present("STATS"))
+		|| (CLI_PRESENT == cli_present("STATSDB_ALLOCATION"))
 		|| (CLI_PRESENT == cli_present("STDNULLCOLL"))
 		|| (CLI_NEGATED == cli_present("STDNULLCOLL"))
 		|| (CLI_PRESENT == cli_present("TRIGGER_FLUSH_LIMIT"))
 		|| (CLI_PRESENT == cli_present("VERSION"))
 		|| (CLI_PRESENT == cli_present("WAIT_DISK"))
 		|| (CLI_PRESENT == cli_present("WRITES_PER_FLUSH"))
-		|| (CLI_PRESENT == cli_present("FULLBLKWRT")))
+		|| (CLI_PRESENT == cli_present("FULLBLKWRT"))
+		|| (CLI_PRESENT == cli_present("PROBLKSPLIT")))
 	{
 		if (SS_NORMAL != (status = mupip_set_file(db_fn_len, db_fn)))
 			mupip_exit(status);
