@@ -1,6 +1,7 @@
 /****************************************************************
  *								*
- *	Copyright 2001, 2014 Fidelity Information Services, Inc	*
+ * Copyright (c) 2001-2022 Fidelity National Information	*
+ * Services, Inc. and/or its subsidiaries. All rights reserved.	*
  *								*
  *	This source code contains the intellectual property	*
  *	of its copyright holder(s), and is made available	*
@@ -119,10 +120,11 @@ void mubexpfilnam(char *dirname, unsigned int dirlen, backup_reg_list *list);
 #else
 #error Unsupported Platform
 #endif
-bool mubfilcpy(backup_reg_list *list);
+boolean_t mubfilcpy(backup_reg_list *list, boolean_t showprogress, int attemptcnt, boolean_t *stopretries);
 boolean_t mubgetfil(backup_reg_list *list, char *name, unsigned short len);
 bool mubinccpy(backup_reg_list *list);
 void mup_bak_mag(void);
 void mup_bak_pause(void);
-
-
+int handle_err(char errorstr[], int err);
+int countdigits(size_t num);
+boolean_t ink_backup(void);
