@@ -1,6 +1,6 @@
 /****************************************************************
  *								*
- * Copyright (c) 2001-2019 Fidelity National Information	*
+ * Copyright (c) 2001-2022 Fidelity National Information	*
  * Services, Inc. and/or its subsidiaries. All rights reserved.	*
  *								*
  *	This source code contains the intellectual property	*
@@ -775,15 +775,15 @@ finish:
 			}
 			if ((SS_NORMAL == status) && shm_size)
 			{
-				shmid = shmget(IPC_PRIVATE, shm_size, 0600 | IPC_CREAT);
+				shmid = gtm_shmget(IPC_PRIVATE, shm_size, 0600 | IPC_CREAT, TRUE);
 				if (-1 == shmid)
 				{
 					save_errno = errno;
 					SNPRINTF(errstr, SIZEOF(errstr),
 						"shmget() : shmsize=0x%llx", shm_size);
 					MUR_SET_MULTI_PROC_KEY(rctl, multi_proc_key);	/* to print region name prefix */
-					rts_error_csa(CSA_ARG(NULL) VARLSTCNT(8)
-								ERR_SYSCALL, 5, LEN_AND_STR(errstr), CALLFROM, save_errno);
+					RTS_ERROR_CSA_ABT(NULL, VARLSTCNT(8)
+						ERR_SYSCALL, 5, LEN_AND_STR(errstr), CALLFROM, save_errno);
 				}
 				shmPtr = (char *)do_shmat(shmid, 0, 0);
 				if (-1 == (sm_long_t)shmPtr)
@@ -792,8 +792,8 @@ finish:
 					SNPRINTF(errstr, SIZEOF(errstr),
 						"shmat() : shmid=%d shmsize=0x%llx", shmid, shm_size);
 					MUR_SET_MULTI_PROC_KEY(rctl, multi_proc_key);	/* to print region name prefix */
-					rts_error_csa(CSA_ARG(NULL) VARLSTCNT(8)
-								ERR_SYSCALL, 5, LEN_AND_STR(errstr), CALLFROM, save_errno);
+					RTS_ERROR_CSA_ABT(NULL, VARLSTCNT(8)
+						ERR_SYSCALL, 5, LEN_AND_STR(errstr), CALLFROM, save_errno);
 				}
 				shm_rctl->jnlext_shmid = shmid;
 				shm_rctl->jnlext_shm_size = shm_size;
