@@ -1,6 +1,6 @@
 /****************************************************************
  *								*
- * Copyright (c) 2003-2019 Fidelity National Information	*
+ * Copyright (c) 2003-2021 Fidelity National Information	*
  * Services, Inc. and/or its subsidiaries. All rights reserved.	*
  *								*
  *	This source code contains the intellectual property	*
@@ -38,6 +38,7 @@
 #include "gtm_bintim.h"
 #include "gtmmsg.h"
 #include "io.h"
+#include "gtm_stat.h"
 #include "file_input.h"
 
 GBLREF	mur_opt_struct	mur_options;
@@ -64,12 +65,23 @@ error_def(ERR_RSYNCSTRMVAL);
 #define EXCLUDE_CHAR	'~'
 #define STR2PID		asc2i
 #define STR2SEQNO	asc2l
+#define HEXSTR2SEQNO(X)	STRTOL(X, NULL, 16)
+#define HEXSTR2BLKNO(X)	STRTOL(X, NULL, 16)
+#define HEXSTR2PID(X)	STRTOL(X, NULL, 16)
+#define STR2BLKNO	asc2l
 #define	MAX_PID_LEN	10	/* maximum number of decimal digits in the process-id */
 #define REDIRECT_STR	"specify as \"old-file-name=new-file-name,...\""
 #define	WILDCARD_CHAR1	'*'
 #define	WILDCARD_CHAR2	'%'
 #define	ESCAPE_CHAR	'\\'
 
+#ifdef DEBUG
+/* Debug GT.M versions support the ability to dump the PBLKs in ZWRITE format in a journal extract.
+ * Use in conjuction with DSE's ability to display a block IMAGE to see the PBLK contents. See test
+ * commit PBLK2 for more information on how to use this debug-only feature. */
+#define CAN_ZWRITE_EXTRACT_PBLK
+#endif
+
 static	char	default_since_time[] = "0 0:0:00",
 		default_lookback_time[] = "0 0:5:00";
 static char	* const extr_parms[] =
@@ -94,9 +106,13 @@ void	mur_get_options(void)
 	boolean_t	global_exclude;
 	long_list	*ll_ptr, *ll_ptr1;
 	long_long_list	*seqno_list, *seqno_list1;
+#ifdef CAN_ZWRITE_EXTRACT_PBLK
+	long_long_list	*blocklist_list, *blocklist_list1;
+#endif
 	redirect_list	*rl_ptr, *rl_ptr1, *tmp_rl_ptr;
 	select_list	*sl_ptr, *sl_ptr1;
 	boolean_t	interactive, parse_error, uniqname_error = FALSE;
+	struct stat	stat_buf;
 	DCL_THREADGBL_ACCESS;
 
 	SETUP_THREADGBL_ACCESS;
@@ -177,6 +193,7 @@ void	mur_get_options(void)
 	{
 		mur_options.extr[extr_type] = FALSE;
 		mur_options.extr_fn_is_stdout[extr_type] = FALSE;
+		mur_options.extr_fn_is_regfile[extr_type] = TRUE;
 		mur_options.extr_fn_is_devnull[extr_type] = FALSE;
 		status = cli_present(extr_parms[extr_type]);
 		if (CLI_PRESENT == status)
@@ -189,21 +206,28 @@ void	mur_get_options(void)
 				mur_options.extr_fn[extr_type] = (char *)malloc(mur_options.extr_fn_len[extr_type] + 1);
 				strncpy(mur_options.extr_fn[extr_type], qual_buffer, length);
 				/*Remove multiple slashes from journal file*/
-                                mur_options.extr_fn_len[extr_type] = rmv_mul_slsh(mur_options.extr_fn[extr_type],length);
+				mur_options.extr_fn_len[extr_type] = rmv_mul_slsh(mur_options.extr_fn[extr_type],length);
 				mur_options.extr_fn[extr_type][mur_options.extr_fn_len[extr_type]] = '\0';
 				mur_options.extr_fn_is_stdout[extr_type] =
 					(0 == STRNCASECMP(qual_buffer, JNL_STDO_EXTR, SIZEOF(JNL_STDO_EXTR)));
 				mur_options.extr_fn_is_devnull[extr_type] = ((STR_LIT_LEN(DEVNULL) ==
 					mur_options.extr_fn_len[extr_type]) && (0 == STRNCMP_LIT(qual_buffer, DEVNULL)));
+				if (mur_options.extr_fn_is_stdout[extr_type] || mur_options.extr_fn_is_devnull[extr_type])
+					mur_options.extr_fn_is_regfile[extr_type] = FALSE;
+				else {
+					if (0 == Stat(qual_buffer, &stat_buf))
+						mur_options.extr_fn_is_regfile[extr_type] = S_ISREG(stat_buf.st_mode);
+					/* else The file does not exist, MUPIP can create it */
+				}
 				/* Check if this extract filename is same as any filename seen before
-				 * However, Allow /dev/null & -stdout to be repeated
+				 * However, This check is applicable only for regular files.
+				 * Allow the non-regular files, DEVNULL & -stdout to be repeated
 				 */
-				if (!mur_options.extr_fn_is_devnull[extr_type] && !mur_options.extr_fn_is_stdout[extr_type])
+				if (mur_options.extr_fn_is_regfile[extr_type])
 				{
 					for (cnt = 0; cnt < extr_type; cnt++)
 					{
-						if (mur_options.extr[cnt] && !mur_options.extr_fn_is_devnull[cnt] &&
- 							!mur_options.extr_fn_is_stdout[cnt] &&
+						if (mur_options.extr[cnt] &&
 							(mur_options.extr_fn_len[cnt] == mur_options.extr_fn_len[extr_type]) &&
 							(0 == STRNCMP_STR(mur_options.extr_fn[extr_type], mur_options.extr_fn[cnt],
 								mur_options.extr_fn_len[cnt])))
@@ -226,6 +250,7 @@ void	mur_get_options(void)
 			mur_options.extr_fn_len[extr_type] = STRLEN(DEVNULL);
 			mur_options.extr_fn[extr_type] = (char *)malloc(mur_options.extr_fn_len[extr_type] + 1);
 			STRNCPY_STR(mur_options.extr_fn[extr_type], DEVNULL, mur_options.extr_fn_len[extr_type] + 1);
+			mur_options.extr_fn_is_regfile[extr_type] = FALSE;
 		}
 	}
 	if (uniqname_error)
@@ -290,7 +315,10 @@ void	mur_get_options(void)
 			mur_options.lookback_opers_specified = TRUE;
 			length = MAX_LINE;
 			cli_get_str("LOOKBACK_LIMIT.OPERATIONS", qual_buffer, &length);
-			mur_options.lookback_opers = asc2i((uchar_ptr_t)qual_buffer, (int4)length);
+			if (cli_is_hex_explicit((char_ptr_t)qual_buffer))
+				mur_options.lookback_opers = STRTOL((char_ptr_t)qual_buffer, NULL, 16);
+			else
+				mur_options.lookback_opers = asc2i((uchar_ptr_t)qual_buffer, (int4)length);
 		}
 		if (CLI_PRESENT == cli_present("LOOKBACK_LIMIT.TIME"))
 		{
@@ -366,9 +394,9 @@ void	mur_get_options(void)
 			for (tmp_rl_ptr = mur_options.redirect; tmp_rl_ptr != NULL; tmp_rl_ptr = tmp_rl_ptr->next)
 			{
 				if (((tmp_rl_ptr->org_name_len == file_name_expanded_len) &&
-				     (0 == memcmp(tmp_rl_ptr->org_name, file_name_expanded, tmp_rl_ptr->org_name_len))) ||
+					(0 == memcmp(tmp_rl_ptr->org_name, file_name_expanded, tmp_rl_ptr->org_name_len))) ||
 				    ((tmp_rl_ptr->new_name_len == file_name_expanded_len) &&
-				     (0 == memcmp(tmp_rl_ptr->new_name, file_name_expanded, tmp_rl_ptr->new_name_len))))
+					(0 == memcmp(tmp_rl_ptr->new_name, file_name_expanded, tmp_rl_ptr->new_name_len))))
 				{
 					gtm_putmsg_csa(CSA_ARG(NULL) VARLSTCNT(4) ERR_INVREDIRQUAL, 2,
 						LEN_AND_LIT("Duplicate or invalid specification of files"));
@@ -382,11 +410,11 @@ void	mur_get_options(void)
 			entry_ptr = cptr + 1; /* skip the = */
 			file_name_specified_len = length - file_name_specified_len - 1; /* the rest of the entry);*/
 			if (file_name_specified_len > (MAX_FN_LEN + 1))
-                        {
-                                  gtm_putmsg_csa(CSA_ARG(NULL) VARLSTCNT(4) ERR_INVREDIRQUAL, 2,
+			{
+				  gtm_putmsg_csa(CSA_ARG(NULL) VARLSTCNT(4) ERR_INVREDIRQUAL, 2,
 							LEN_AND_LIT("Redirect DB filename too long: greater than 255"));
-                                  mupip_exit(ERR_MUPCLIERR);
-                        }
+				  mupip_exit(ERR_MUPCLIERR);
+			}
 			memcpy(file_name_specified, entry_ptr, file_name_specified_len);
 			*(file_name_specified + file_name_specified_len)= '\0';
 			if (!get_full_path(file_name_specified, file_name_specified_len, file_name_expanded,
@@ -741,6 +769,85 @@ void	mur_get_options(void)
 		free(file_name_specified);
 		free(file_name_expanded);
 	}
+#ifdef CAN_ZWRITE_EXTRACT_PBLK
+	/*-----		-BLOCKID=(list of block numbers)	-----*/
+	while (cli_present("BLOCKID") == CLI_PRESENT)
+	{
+		mur_options.dump_all_blocks = TRUE;
+		length = MAX_LINE;
+		if (!CLI_GET_STR_ALL("BLOCKID", qual_buffer, &length))
+			break;
+		mur_options.dump_all_blocks = FALSE;
+		qual_buffer_ptr = qual_buffer;
+		global_exclude = FALSE;
+		if ('"' == *qual_buffer_ptr )
+		{
+			++qual_buffer_ptr;
+			--length;
+			if ('"' == qual_buffer_ptr[length-1])
+				qual_buffer_ptr[--length] = '\0';
+		}
+		if (EXCLUDE_CHAR == *qual_buffer_ptr)
+		{
+			global_exclude = TRUE;
+			++qual_buffer_ptr;
+			--length;
+		}
+		if ('(' == *qual_buffer_ptr)
+		{
+			++qual_buffer_ptr;
+			--length;
+		} else if (global_exclude)
+		{
+			--qual_buffer_ptr;
+			++length;
+			global_exclude = FALSE;
+		}
+		if (')' == qual_buffer_ptr[length - 1])
+			qual_buffer_ptr[--length] = '\0';
+		for (ctop = qual_buffer_ptr + length; qual_buffer_ptr < ctop;)
+		{
+			if (!cli_get_str_ele(qual_buffer_ptr, entry, &length, FALSE))
+				mupip_exit(ERR_MUPCLIERR);
+			qual_buffer_ptr += length;
+			assert(',' == *qual_buffer_ptr || !(*qual_buffer_ptr));	/* either comma separator or end of option list */
+			if (',' == *qual_buffer_ptr)
+				qual_buffer_ptr++;  /* skip separator */
+			entry_ptr = entry;
+			blocklist_list1 = (long_long_list *)malloc(SIZEOF(long_long_list));
+			blocklist_list1->next = NULL;
+			if (NULL == mur_options.blocklist)
+				mur_options.blocklist = blocklist_list1;
+			else
+				blocklist_list->next = blocklist_list1;
+			blocklist_list = blocklist_list1;
+			if ('"' == entry_ptr[length - 1])
+				--length;
+			if ('"' == *entry_ptr)
+			{
+				entry_ptr++;
+				length--;
+			}
+			if (EXCLUDE_CHAR == *entry_ptr)
+			{
+				++entry_ptr;
+				length--;
+				blocklist_list->exclude = TRUE;
+			} else
+				blocklist_list->exclude = FALSE;
+			if (global_exclude)
+				blocklist_list->exclude = !blocklist_list->exclude;
+			if ((cli_is_hex_explicit((char_ptr_t)entry_ptr) ?	/* Warning : assignment */
+				((blocklist_list->u.blk = HEXSTR2BLKNO((char_ptr_t)entry_ptr+2)) == (seq_num) - 1)
+				: ((blocklist_list->u.blk = STR2BLKNO((uchar_ptr_t)entry_ptr, length)) == (seq_num) - 1)))
+			{
+				gtm_putmsg_csa(CSA_ARG(NULL) VARLSTCNT(4) ERR_INVSEQNOQUAL, 2, length, entry_ptr);
+				mupip_exit(ERR_MUPCLIERR);
+			}
+		}
+		break;
+	}
+#endif
 	/*-----		-SEQNO=(list of sequence numbers)	-----*/
 	if (cli_present("SEQNO") == CLI_PRESENT)
 	{
@@ -806,7 +913,9 @@ void	mur_get_options(void)
 				seqno_list->exclude = FALSE;
 			if (global_exclude)
 				seqno_list->exclude = !seqno_list->exclude;
-			if ((seqno_list->seqno = STR2SEQNO((uchar_ptr_t)entry_ptr, length)) == (seq_num) - 1)
+			if ((cli_is_hex_explicit((char_ptr_t)entry_ptr) ?	/* Warning : assignment */
+				((seqno_list->u.seqno = HEXSTR2SEQNO((char_ptr_t)entry_ptr+2)) == (seq_num) - 1)
+				: ((seqno_list->u.seqno = STR2SEQNO((uchar_ptr_t)entry_ptr, length)) == (seq_num) - 1)))
 			{
 				gtm_putmsg_csa(CSA_ARG(NULL) VARLSTCNT(4) ERR_INVSEQNOQUAL, 2, length, entry_ptr);
 				mupip_exit(ERR_MUPCLIERR);
@@ -880,7 +989,9 @@ void	mur_get_options(void)
 			if (global_exclude)
 				ll_ptr->exclude = !ll_ptr->exclude;
 			if ((MAX_PID_LEN < length) ||
-			    ((ll_ptr->num = STR2PID((uchar_ptr_t)entry_ptr, length)) == (unsigned int) - 1))
+				((cli_is_hex_explicit((char_ptr_t)entry_ptr) ?	/* Warning : assignment */
+				((ll_ptr->num = HEXSTR2PID((char_ptr_t)entry_ptr+2)) == (unsigned int) - 1)
+				: ((ll_ptr->num = STR2PID((uchar_ptr_t)entry_ptr, length)) == (unsigned int) - 1))))
 			{
 				gtm_putmsg_csa(CSA_ARG(NULL) VARLSTCNT(4) ERR_INVIDQUAL, 2, length, entry_ptr);
 				mupip_exit(ERR_MUPCLIERR);
