@@ -1,6 +1,6 @@
 /****************************************************************
  *								*
- * Copyright (c) 2001-2016 Fidelity National Information	*
+ * Copyright (c) 2001-2020 Fidelity National Information	*
  * Services, Inc. and/or its subsidiaries. All rights reserved.	*
  *								*
  *	This source code contains the intellectual property	*
@@ -62,14 +62,17 @@ GBLREF	boolean_t		write_after_image;
 GBLREF	gd_region		*gv_cur_region;
 GBLREF	gv_namehead		*gv_target;
 GBLREF	jnl_fence_control	jnl_fence_ctl; /* Needed to set the token, optimize jnl_write_logical for recover */
-GBLREF 	jnl_gbls_t		jgbl;
-GBLREF  jnl_process_vector	*prc_vec;
-GBLREF 	mur_gbls_t		murgbl;
-GBLREF  mur_opt_struct  	mur_options;
+GBLREF	jnl_gbls_t		jgbl;
+GBLREF	jnl_process_vector	*prc_vec;
+GBLREF	mur_gbls_t		murgbl;
+GBLREF	mur_opt_struct  	mur_options;
 GBLREF	sgmnt_addrs		*cs_addrs;
-GBLREF  sgmnt_data_ptr_t 	cs_data;
+GBLREF	sgmnt_data_ptr_t 	cs_data;
 GBLREF	struct_jrec_tcom	tcom_record;
 GBLREF	uint4			dollar_tlevel;
+#ifdef DEBUG
+GBLREF	boolean_t		forw_recov_lgtrig_only;
+#endif
 
 error_def(ERR_JNLBADRECFMT);
 error_def(ERR_TRIGLOADFAIL);
@@ -121,6 +124,7 @@ uint4	mur_output_record(reg_ctl_list *rctl)
 			break;
 	}
 	jgbl.gbl_jrec_time = rec->prefix.time;
+	/* Since we would have returned for the JRT_ALIGN case above, we can safely use rec->prefix.pini_addr below */
 	pini_addr = rec->prefix.pini_addr;
 	reg = rctl->gd;
 	jctl = rctl->jctl;
@@ -170,6 +174,10 @@ uint4	mur_output_record(reg_ctl_list *rctl)
 	assert(NULL == TREF(gd_targ_gvnh_reg));
 	if (IS_SET_KILL_ZKILL_ZTRIG(rectype))
 	{	/* TP and non-TP has same format */
+		DEBUG_ONLY(forw_recov_lgtrig_only = FALSE;)	/* now that we have seen a non-LGTRIG record, reset this global
+								 * variable which would have been set to TRUE after op_tstart
+								 * done in mur_forward_play_cur_jrec.
+								 */
 		keystr = (jnl_string *)&rec->jrec_set_kill.mumps_node;
 		if (jnl_enabled)
 		{
@@ -338,15 +346,28 @@ uint4	mur_output_record(reg_ctl_list *rctl)
 		assert(csa == (sgmnt_addrs *)&FILE_INFO(reg)->s_addrs);
 		assert(csd == csa->hdr);
 		if (mur_options.forward)
-		{
-			assert(rec->jrec_inctn.prefix.tn == csd->trans_hist.curr_tn || mur_options.notncheck);
-			if (FALSE == ((was_crit = csa->now_crit)))
-				grab_crit(reg);
-			CHECK_TN(csa, csd, csd->trans_hist.curr_tn);	/* can issue rts_error TNTOOLARGE */
-			csd->trans_hist.early_tn = csd->trans_hist.curr_tn + 1;
-			INCREMENT_CURR_TN(csd);
-			if (!was_crit)
-				rel_crit(reg);
+		{	/* It is possible a process got killed after writing a journal record (INCTN, SET etc.) but
+			 * before incrementing csd->trans_hist.curr_tn. In that case, the next process would have done
+			 * a "wcs_recover" to salvage the crit from the dead pid and as part of that would have written
+			 * an INCTN. This INCTN would have the same tn as the previous record (i.e. a DUPTN situation)
+			 * but since this is an INCTN record and not a logical record, we can skip incrementing the
+			 * curr_tn in this case thereby keeping the db curr_tn in sync with the next jnl rec tn.
+			 * In case NOTNCHECK is specified though, the user could present forward recovery with two
+			 * identical jnl files with just one tn (e.g. v54003/C9K08003315 subtest) in which case it is not
+			 * easy to keep the db in sync with the jnl tn so do not skip the curr_tn increment in that case.
+			 */
+			assert((rec->jrec_inctn.prefix.tn == csd->trans_hist.curr_tn)
+				|| ((rec->jrec_inctn.prefix.tn + 1) == csd->trans_hist.curr_tn) || mur_options.notncheck);
+			if (mur_options.notncheck || (rec->jrec_inctn.prefix.tn >= csd->trans_hist.curr_tn))
+			{
+				if (FALSE == ((was_crit = csa->now_crit)))
+					grab_crit(reg, WS_74);
+				CHECK_TN(csa, csd, csd->trans_hist.curr_tn);	/* can issue rts_error TNTOOLARGE */
+				csd->trans_hist.early_tn = csd->trans_hist.curr_tn + 1;
+				INCREMENT_CURR_TN(csd);
+				if (!was_crit)
+					rel_crit(reg);
+			}
 		}
 		break;
 	case JRT_AIMG:
@@ -379,16 +400,16 @@ uint4	mur_output_record(reg_ctl_list *rctl)
 	case JRT_PFIN:
 		if (jnl_enabled)
 		{
-			if (FALSE == ((was_crit = rctl->csa->now_crit)))
-				grab_crit(reg);
+			if (FALSE == ((was_crit = csa->now_crit)))
+				grab_crit(reg, WS_75);
 			/* MUPIP RECOVER should be the only one updating the database so journal state not expected to changes */
 			assert(JNL_ENABLED(csd));
-			jnl_status = jnl_ensure_open();
+			jnl_status = jnl_ensure_open(reg, csa);
 			if (0 == jnl_status)
 			{
 				assert(plst->new_pini_addr == csa->jnl->pini_addr);
 				if (0 != csa->jnl->pini_addr)
-					jnl_put_jrt_pfin(rctl->csa);
+					jnl_write_pfin(csa);
 			} else
 			{
 				if (SS_NORMAL != csa->jnl->status)
