@@ -1,6 +1,6 @@
 /****************************************************************
  *								*
- * Copyright (c) 2001-2018 Fidelity National Information	*
+ * Copyright (c) 2001-2022 Fidelity National Information	*
  * Services, Inc. and/or its subsidiaries. All rights reserved.	*
  *								*
  *	This source code contains the intellectual property	*
@@ -187,8 +187,9 @@ void xi82mval(mval *v, gtm_uint64_t i)
 				exp--;
 			}
 			v->e = exp;
-			v->m[0] = low;
-			v->m[1] = high;
+			assert((MAXPOSINT4 >= low) && (MAXPOSINT4 >= high));
+			v->m[0] = (int4)low;
+			v->m[1] = (int4)high;
 		} else if (i < (gtm_uint64_t)MANT_HI * MANT_HI)
 		{
 			low = i % MANT_HI;
@@ -201,8 +202,9 @@ void xi82mval(mval *v, gtm_uint64_t i)
 				exp--;
 			}
 			v->e = exp;
-			v->m[0] = low;
-			v->m[1] = high;
+			assert((MAXPOSINT4 >= low) && (MAXPOSINT4 >= high));
+			v->m[0] = (int4)low;
+			v->m[1] = (int4)high;
 		} else
 		{	/* The value won't fit in 18 digits, so return a string. */
 			if (v->sgn)
@@ -294,7 +296,7 @@ void double2mval(mval *dst, double src)
 	return;
 }
 
-/* Converts an mval into a 32-bit signed integer, or MAXPOSINT4 on overflow. */
+/* Converts an mval into a 32-bit signed integer, or MAXPOSINT4 on overflow, or MINNEGINT4 on underflow */
 int4 mval2i(mval *v)
 {
 	int4	i;
@@ -310,7 +312,12 @@ int4 mval2i(mval *v)
 		if (exp > EXP_IDX_BIAL)
 		{
 			j = mval2double(v);
-			i = (MAXPOSINT4 >= j) ? (int4)j : MAXPOSINT4;
+			if (MINNEGINT4 > j)
+				i = MINNEGINT4;
+			else if (MAXPOSINT4 < j)
+				i = MAXPOSINT4;
+			else	/* if ((MINNEGINT4 <= j) && (MAXPOSINT4 >= j)) */
+				i = (int4)j;
 		} else if (exp < MV_XBIAS)
 			i = 0;
 		else
