@@ -1,6 +1,6 @@
 /****************************************************************
  *								*
- * Copyright (c) 2001-2020 Fidelity National Information	*
+ * Copyright (c) 2001-2022 Fidelity National Information	*
  * Services, Inc. and/or its subsidiaries. All rights reserved.	*
  *								*
  *	This source code contains the intellectual property	*
@@ -86,6 +86,7 @@ void	op_fntext(mval *label, int int_exp, mval *rtn, mval *ret);
 void	op_fntranslate(mval *src, mval *in_str, mval *out_str, mval *dst);
 void	op_fnview(UNIX_ONLY_COMMA(int numarg) mval *dst, ...);
 void	op_fnzascii(int4 num, mval *in, mval *out);
+void	op_fnzauditlog(mval *src, mval *dst);
 void	op_fnzbitand(mval *dst, mval *bitstr1, mval *bitstr2);
 void	op_fnzbitcoun(mval *dst, mval *bitstr);
 void	op_fnzbitfind(mval *dst, mval *bitstr, int truthval, int pos);
@@ -108,7 +109,7 @@ void	op_fnzextract(int last, int first, mval *src, mval *dest);
 void	op_fnzfile(mval *name, mval *key, mval *ret);
 int4	op_fnzfind(mval *src, mval *del, mint first, mval *dst);
 void	op_fnzj2(mval *src, int len, mval *dst);
-void	op_fnzjobexam(mval *prelimSpec, mval *finalSpec);
+void	op_fnzjobexam(mval *prelimSpec,  mval *fmt, mval *finalSpec);
 void	op_fnzlength(mval *a1, mval *a0);
 void	op_fnzlkid(mint boolex, mval *retval);
 void	op_fnzm(mint x, mval *v);
@@ -135,7 +136,7 @@ void	op_fntranslate_fast(mval *src, mval *rplc, mval *xlate, mval *m_xlate_hash,
 void	op_fntranslate_common(mval *src, mval *dst, mval *rplc, int4 *xlate, hash_table_int4 *xlate_hash);
 void	op_fnztranslate_common(mval *src, mval *dst, int *xlate);
 void	create_byte_xlate_table(mval *srch, mval *rplc, int4 *xlate);
-hash_table_int4	*create_utf8_xlate_table(mval *srch, mval *rplc, int4 *xlate);
+hash_table_int4	*create_utf8_xlate_table(mval *srch, mval *rplc, mstr *m_xlate);
 void	op_fnztrigger(mval *func, mval *arg1, mval *arg2, mval *dst);
 void	op_fnztrnlnm(mval *name, mval *table, int4 ind, mval *mode, mval *case_blind, mval *item, mval *ret);
 void	op_fnztrnlnm(mval *name, mval *table, int4 ind, mval *mode, mval *case_blind, mval *item, mval *ret);
