@@ -1,6 +1,7 @@
 /****************************************************************
  *								*
- *	Copyright 2001, 2014 Fidelity Information Services, Inc	*
+ * Copyright (c) 2001-2022 Fidelity National Information	*
+ * Services, Inc. and/or its subsidiaries. All rights reserved.	*
  *								*
  *	This source code contains the intellectual property	*
  *	of its copyright holder(s), and is made available	*
@@ -12,12 +13,14 @@
 #include "mdef.h"
 
 #include "gtm_string.h"
+#include "gtm_stdio.h"
 #include <math.h>
 
 #include "arit.h"
 #include "stringpool.h"
 #include "op.h"
 #include "mvalconv.h"
+#include "toktyp.h"
 
 #define ACCUR_PERCENT	0.00000000000000055
 #define MAX_M_INT 999999999
@@ -28,6 +31,7 @@ LITREF mval literal_zero;
 
 STATICFNDEF void op_exp_flgchk(mval *mv);
 
+error_def(ERR_DIVZERO);
 error_def(ERR_NEGFRACPWR);
 error_def(ERR_NUMOFLOW);
 
@@ -36,50 +40,48 @@ void op_exp(mval *u, mval* v, mval *p)
 	mval 		u1, *u1_p;
 	double 		accuracy, exponent;
 	double 		x, x1, y, z, z2, z3, z4, z5, id, il;
+	double		savz;
 	int		im0, im1, ie, i, j, j1;
 	boolean_t	fraction = FALSE, in = FALSE;
 	boolean_t	neg = FALSE, even = TRUE;
-	mval    	w, zmv;
-	int4    	n, n1;
+	mval		w, zmv;
+	int4		n, n1;
 	int4		z1_rnd, z2_rnd, pten;
+	int		idx;
+	DCL_THREADGBL_ACCESS;
 
+	SETUP_THREADGBL_ACCESS;
 	u1_p = &u1;
 	memcpy(u1_p, u, SIZEOF(mval));
 	MV_FORCE_NUM(u1_p);
 	MV_FORCE_NUM(v);
-	if ((0 == v->m[1]) && (0 == v->m[0]))
-	{	/* 0**0 = 1 */
-		*p = literal_one;
-		return;
-	}
+	n = v->m[1];
 	if (0 != (v->mvtype & MV_INT))
 	{	/* Integer-ish exponent (could have up to 3 digits to right of decimal pt) */
-		n = v->m[1];
 		if (0 == n)
-		{	/* anything**0 = 1 where anything != 0 */
+		{	/* anything**0 = 1 */
 			*p = literal_one;
 			return;
 		}
 		if (0 != (u1_p->mvtype & MV_INT))
 		{	/* Integer-ish base */
 			if (0 == u1_p->m[1])
-			{	/* 0**anything = 0 */
-				*p = literal_zero;
-		                return;
+			{
+				if (0 <= n)
+					*p = literal_zero;	/* 0**anything non-negative = 0 */
+				else if (TREF(compile_time))
+				{	/* create "impossible" value flag for compiler */
+					*p = literal_zero;
+					p->sgn = 1;		/* negative zero == oxymoron */
+				} else
+					rts_error_csa(CSA_ARG(NULL) VARLSTCNT(1) ERR_DIVZERO);
+				return;
 			}
-		} else if ((0 == u1_p->m[1]) && (0 == u1_p->m[0]))
-		{	/* 0**anything = 0 */
-			*p = literal_zero;
-			return;
-		}
+		} else
+			assert((0 != u1_p->m[1]) || (0 != u1_p->m[0]));
 		n1 = n / MV_BIAS;
 		if ((n1 * MV_BIAS) == n)
 		{	/* True non-fractional exponent */
-			if (0 == v->m[1])
-			{	/* Duplicate of check on line 58? */
-				*p = literal_one;
-				return;
-			}
 			if (0 > n1)
 			{	/* Create inverse due to negative exponent */
 				op_div((mval *)&literal_one, u1_p, &w);
@@ -102,7 +104,7 @@ void op_exp(mval *u, mval* v, mval *p)
 		{	/* Have non-integer exponent (has fractional component) */
 			if (0 != (u1_p->mvtype & MV_INT))
 			{	/* Base is integer-ish */
-                        	if (0 > u1_p->m[1])
+				if (0 > u1_p->m[1])
 				{	/* Base is negative, invalid exponent expression */
 					rts_error_csa(CSA_ARG(NULL) VARLSTCNT(1) ERR_NEGFRACPWR);
 					return;
@@ -115,31 +117,38 @@ void op_exp(mval *u, mval* v, mval *p)
 					return;
 				}
 			}
-                }
-        } else
+		}
+	} else
 	{	/* Exponent NOT integer-ish */
-	        if (0 != (u1_p->mvtype & MV_INT))
-        	{	/* Base is integer-ish */
-                	if (0 > u1_p->m[1])
-                	{	/* Base is negative - make positive but record was negative */
-                        	u1_p->m[1] = -u1_p->m[1];
-	                        neg = TRUE;
-        	        }
+		assert((0 != v->m[1]) || (0 != v->m[0]));
+		if (0 != (u1_p->mvtype & MV_INT))
+		{	/* Base is integer-ish */
 			if (0 == u1_p->m[1])
-			{	/* 0**anything is 0 */
-				*p = literal_zero;
+			{
+				if (!v->sgn)
+					*p = literal_zero;	/* 0**anything non-negative = 0 */
+				else if (TREF(compile_time))
+				{	/* create "impossible" value flag for compiler */
+					*p = literal_zero;
+					p->sgn = 1;		/* negative zero == oxymoron */
+				} else
+					rts_error_csa(CSA_ARG(NULL) VARLSTCNT(1) ERR_DIVZERO);
 				return;
 			}
-        	} else if (u1_p->sgn)
-        	{	/* Base is NOT integer-ish and is negative - clear sign and record with flag */
-                	u1_p->sgn = 0;
-	                neg = TRUE;
-			if ((0 == u1_p->m[1]) && (0 == u1_p->m[0]))
-			{	/* 0**anything is zero */
-				*p = literal_zero;
-				return;
+			if (0 > u1_p->m[1])
+			{	/* Base is negative - make positive but record was negative */
+				u1_p->m[1] = -u1_p->m[1];
+				neg = TRUE;
 			}
-        	}
+		} else
+		{	/* Base is NOT integer-ish */
+			assert((0 != u1_p->m[1]) || (0 != u1_p->m[0]));
+			if (u1_p->sgn)
+			{	/* and is negative - clear sign and record with flag */
+				u1_p->sgn = 0;
+				neg = TRUE;
+			}
+		}
 		if (NUM_DEC_DG_2L > (ie = (v->e - MV_XBIAS)))		/* Note assignment */
 		{	/* Need to determine 2 things:
 			 *   1. Whether this exponent has a fractional part (vs just being large)
@@ -207,8 +216,9 @@ void op_exp(mval *u, mval* v, mval *p)
 #	ifdef UNIX
 	if (HUGE_VAL == z)		/* Infinity return value check */
 	{
+		TREF(last_source_column) += (TK_EOL == TREF(director_token)) ? -2 : 2;	/* improve hints */
 		rts_error_csa(CSA_ARG(NULL) VARLSTCNT(1) ERR_NUMOFLOW);
-	        return;
+		return;
 	}
 #	endif
 	p->sgn = (neg && !even);	/* Positive numbers only from here on out */
@@ -252,14 +262,15 @@ void op_exp(mval *u, mval* v, mval *p)
 	 * Could add checks for zero/infinity here to avoid lengthy (300ish iterations) while loops below.
 	 */
 	n = 0;
-        while (1e16 <= z)
+	savz = z;
+	while (1e16 <= z)
 	{
-                n += 5;
+		n += 5;
 		z *= 1e-5;
 	}
-        while (1e1 > z)
+	while (1e1 > z)
 	{
-                n -= 5;
+		n -= 5;
 		z *= 1e5;
 	}
 	while (1e9 <= z)
@@ -277,19 +288,25 @@ void op_exp(mval *u, mval* v, mval *p)
 	z *= 1e9;
 	z2_rnd = (int)z;
 	assert(((MANT_LO <= z1_rnd) && (MANT_HI > z1_rnd)) && ((0 <= z2_rnd) && (MANT_HI > z2_rnd)));
-	if ((0 == z2_rnd) && (-11 <= n) && (-3 >= n) && (0 == (z1_rnd % ten_pwr[-3 - n])))
-	{	/* This is a second-chance at detection an integer in case something slipped through the earlier
-		 * check. Not expecting it to ever be invoked but is here as a safety net.
-		 */
-		z1_rnd /= ten_pwr[-3-n];
-		p->e = 0;
-		p->mvtype = (MV_NM | MV_INT);
-		p->m[1] = z1_rnd;
-		return;
+	if ( (0 == z2_rnd) && (-11 <= n) && (-3 >= n))
+	{
+		idx = -3 - n;
+		assert((0 <= idx) && (9 >= idx));
+		if ((0 == (z1_rnd % ten_pwr[idx])))
+		{	/* This is a second-chance at detection an integer in case something slipped through the earlier
+		 	 * check. Not expecting it to ever be invoked but is here as a safety net.
+		 	 */
+			 z1_rnd /= ten_pwr[idx];
+			 p->e = 0;
+			 p->mvtype = (MV_NM | MV_INT);
+			 p->m[1] = z1_rnd;
+			 return;
+		}
 	}
 	exponent = MV_XBIAS + n + 9;
 	if (exponent >= EXPHI)
 	{
+		TREF(last_source_column) += (TK_EOL == TREF(director_token)) ? -2 : 2;	/* improve hints */
 		rts_error_csa(CSA_ARG(NULL) VARLSTCNT(1) ERR_NUMOFLOW);
 		return;
 	}
