@@ -1,6 +1,6 @@
 /****************************************************************
  *								*
- * Copyright (c) 2001-2020 Fidelity National Information	*
+ * Copyright (c) 2001-2022 Fidelity National Information	*
  * Services, Inc. and/or its subsidiaries. All rights reserved.	*
  *								*
  *	This source code contains the intellectual property	*
@@ -58,6 +58,7 @@
 #include "cache.h"
 #include "hashtab_objcode.h"
 #include "gvt_inline.h"
+#include "restrict.h"
 
 GBLREF spdesc			stringpool;
 GBLREF int4			cache_hits, cache_fails, max_cache_entries;
@@ -73,7 +74,7 @@ GBLREF jnlpool_addrs_ptr_t	jnlpool;
 GBLREF bool			undef_inhibit;
 GBLREF int4			break_message_mask;
 GBLREF command_qualifier	 cmd_qlf;
-GBLREF tp_frame		*tp_pointer;
+GBLREF tp_frame			*tp_pointer;
 GBLREF uint4			dollar_tlevel;
 GBLREF int4			zdir_form;
 GBLREF boolean_t		badchar_inhibit;
@@ -106,6 +107,7 @@ LITREF	mval		literal_one;
 #define		NO_REPLINST		"No replication instance defined"
 #define		STATS_MAX_DIGITS	MAX_DIGITS_IN_INT8
 #define		STATS_KEYWD_SIZE	(3 + 1 + 1)	/* 3 character mnemonic, colon and comma */
+#define		DEVICE_MAX_STATUS	(9 + 1 + 7)	/* TERMINAL<sp> : CLOSED<sp> */
 
 #define STATS_PUT_PARM(TXT, CNTR, BASE)					\
 {									\
@@ -118,7 +120,7 @@ LITREF	mval		literal_one;
 }
 
 void	op_fnview(int numarg, mval *dst, ...)
-{	boolean_t	save_transform;
+{	boolean_t	save_transform, n_int8 = FALSE;
 	char		instfilename[MAX_FN_LEN + 1 + 1];	/* 1 for possible flag character */
 	collseq		*csp;
 	gd_binding	*map, *start_map, *end_map;
@@ -129,7 +131,8 @@ void	op_fnview(int numarg, mval *dst, ...)
 	gv_namehead	temp_gv_target;
 	gvnh_reg_t	*gvnh_reg;
 	gvnh_spanreg_t	*gvspan;
-	int		n, tl, newlevel, res, reg_index, collver, nct, act, ver;
+	int		apdtype, n, tl, newlevel, res, reg_index, collver, nct, act, ver;
+	block_id	n2 = 0;
 	lv_val		*lv;
 	mstr		tmpstr, commastr, *gblnamestr;
 	mval		*arg1, *arg2, tmpmval;
@@ -139,6 +142,7 @@ void	op_fnview(int numarg, mval *dst, ...)
 	trans_num	gd_targ_tn, *tn_array;
 	unsigned char	*c, *c_top, *key;
 	unsigned char	buff[MAX_ZWR_KEY_SZ];
+	unsigned char	device_status[DEVICE_MAX_STATUS];
 	va_list		var;
 	viewparm	parmblk, parmblk2;
 	viewtab_entry	*vtp;
@@ -174,6 +178,9 @@ void	op_fnview(int numarg, mval *dst, ...)
 		case VTK_BADCHAR:
 			n = badchar_inhibit ? 0 : 1;
 			break;
+		case VTK_YLGDE:
+			n = (RESTRICTED(gde_enable) && RESTRICTED(aza_enable)) ? 1 : 0;
+			break;
 		case VTK_RCHITS:
 			n = 0;
 			break;
@@ -376,7 +383,7 @@ void	op_fnview(int numarg, mval *dst, ...)
 		case VTK_NOISOLATION:
 			if (NOISOLATION_NULL != parmblk.ni_list.type || NULL == parmblk.ni_list.gvnh_list
 			    || NULL != parmblk.ni_list.gvnh_list->next)
-				rts_error_csa(CSA_ARG(NULL)
+				RTS_ERROR_CSA_ABT(NULL,
 					VARLSTCNT(4) ERR_VIEWFN, 2, strlen((const char *)vtp->keyword), vtp->keyword);
 			n = parmblk.ni_list.gvnh_list->gvnh->noisolation;
 			break;
@@ -553,8 +560,10 @@ void	op_fnview(int numarg, mval *dst, ...)
 				gv_init_reg(parmblk.gv_ptr, NULL);
 			csa = &FILE_INFO(parmblk.gv_ptr)->s_addrs;
 			if (NULL != csa->hdr)
-				n = csa->hdr->trans_hist.free_blocks;
-			else
+			{
+				n2 = csa->hdr->trans_hist.free_blocks;
+				n_int8 = TRUE;
+			} else
 				n = -1;
 			break;
 		case VTK_BLTOTAL:
@@ -564,8 +573,9 @@ void	op_fnview(int numarg, mval *dst, ...)
 			csa = &FILE_INFO(parmblk.gv_ptr)->s_addrs;
 			if (NULL != csa->hdr)
 			{
-				n = csa->hdr->trans_hist.total_blks;
-				n -= (n + csa->hdr->bplmap - 1) / csa->hdr->bplmap;
+				n2 = csa->hdr->trans_hist.total_blks;
+				n2 -= (n2 + csa->hdr->bplmap - 1) / csa->hdr->bplmap;
+				n_int8 = TRUE;
 			} else
 				n = -1;
 			break;
@@ -620,22 +630,22 @@ void	op_fnview(int numarg, mval *dst, ...)
 			break;
 		case VTK_YCOLLATE:
 			n = MV_FORCE_INT(parmblk.value);
-			csp = ready_collseq(n);
+			if (0 == n)
+				break;	/* collation sequence # is 0, any version is compatible with it */
+			csp = ready_collseq(n);	/* Negative value treated as zero */
+			if (NULL == csp)
+			{
+				n = -1;
+				break;
+			}
 			if (NULL == arg2)
 			{	/* $VIEW("YCOLLATE",coll) : Determine version corresponding to collation sequence "coll" */
-				if (0 == n)
-					break;	/* collation sequence # is 0, version is 0 in this case */
-				if (csp)
-				{
-					n = (*csp->version)();
-					n &= 0x000000FF;	/* make an unsigned char, essentially */
-				} else
-					n = -1;
+				n = (*csp->version)();
+				n &= 0x000000FF;	/* make an unsigned char, essentially */
 			} else
 			{	/* $VIEW("YCOLLATE",coll,ver) : Check if collsequence "coll" version is compatible with "ver" */
-				if (0 == n)
-					break;	/* collation sequence # is 0, any version is compatible with it */
-				collver = mval2i(arg2);
+				collver = mval2i(arg2);	/* Negative value downcasted to unsigned char */
+				assert(0 <= collver);
 				n = do_verify(csp, n, collver);
 			}
 			break;
@@ -676,7 +686,7 @@ void	op_fnview(int numarg, mval *dst, ...)
 			break;
 		case VTK_YDIRTREE:
 			if (!parmblk.value->str.len)
-				rts_error_csa(CSA_ARG(NULL)
+				RTS_ERROR_CSA_ABT(NULL,
 					VARLSTCNT(4) ERR_VIEWFN, 2, strlen((const char *)vtp->keyword), vtp->keyword);
 			n = extnam_str.len;		/* internal use of op_gvname should not disturb extended reference */
 			op_gvname(VARLSTCNT(1) parmblk.value);
@@ -691,7 +701,7 @@ void	op_fnview(int numarg, mval *dst, ...)
 				if (((NULL != gvspan) && !gvnh_spanreg_ismapped(gvnh_reg, gd_header, reg))
 					|| ((NULL == gvspan) && (reg != gv_cur_region)))
 				{
-					rts_error_csa(CSA_ARG(NULL) VARLSTCNT(6) ERR_GBLNOMAPTOREG, 4,
+					RTS_ERROR_CSA_ABT(NULL, VARLSTCNT(6) ERR_GBLNOMAPTOREG, 4,
 						parmblk.value->str.len, parmblk.value->str.addr, REG_LEN_STR(reg));
 				}
 				if (NULL != gvspan)
@@ -704,14 +714,14 @@ void	op_fnview(int numarg, mval *dst, ...)
 			RESET_GV_TARGET(DO_GVT_GVKEY_CHECK);
 			break;
 		case VTK_YGDS2GVN:
-			n = (NULL != arg2) ? mval2i(arg2) : 0;
+			n = (NULL != arg2) ? mval2i(arg2) : 0;	/* Negative treated as zero */
 			key = gds2gvn(arg1, &buff[0], n);
 			if (!n)
 				key = &buff[0];
 			COPY_ARG_TO_STRINGPOOL(dst, key, &buff[0]);
 			break;
 		case VTK_YGVN2GDS:
-			n = (NULL != arg2) ? mval2i(arg2) : 0;
+			n = (NULL != arg2) ? mval2i(arg2) : 0;	/* Negative treated as zero */
 			gvkey = (gv_key *)&save_currkey.key;
 			key = gvn2gds(arg1, gvkey, n);
 			if (!n)
@@ -814,11 +824,24 @@ void	op_fnview(int numarg, mval *dst, ...)
 			csa = &FILE_INFO(parmblk.gv_ptr)->s_addrs;
 			assert(NULL != csa->hdr);
 			n = !(RDBF_NOSTATS & csa->reservedDBFlags);
+			break;
+		case VTK_DEVICE:
+			n = view_device(&parmblk.value->str, device_status, sizeof(device_status));
+			dst->str.addr =(char *) device_status;
+			dst->str.len = n;
+			s2pool(&dst->str);
+
 			break;
 		default:
-			rts_error_csa(CSA_ARG(NULL) VARLSTCNT(4) ERR_VIEWFN, 2, strlen((const char *)vtp->keyword), vtp->keyword);
+			RTS_ERROR_CSA_ABT(NULL, VARLSTCNT(4) ERR_VIEWFN, 2, strlen((const char *)vtp->keyword), vtp->keyword);
 	}
-	dst->mvtype = vtp->restype;
-	if (MV_NM == vtp->restype)
-		MV_FORCE_MVAL(dst, n);
+
+	if(MV_NM == vtp->restype)
+	{
+		if(n_int8 == TRUE)
+			MV_FORCE_LMVAL(dst, n2);
+		else
+			MV_FORCE_MVAL(dst, n);
+	} else
+		dst->mvtype = vtp->restype;
 }
