@@ -1,6 +1,7 @@
 /****************************************************************
  *								*
- *	Copyright 2001, 2002 Sanchez Computer Associates, Inc.	*
+ * Copyright (c) 2001-2022 Fidelity National Information	*
+ * Services, Inc. and/or its subsidiaries. All rights reserved.	*
  *								*
  *	This source code contains the intellectual property	*
  *	of its copyright holder(s), and is made available	*
@@ -13,9 +14,10 @@
 #include "op.h"
 #include "jobexam_process.h"
 
-void op_fnzjobexam(mval *prelimSpec, mval *finalSpec)
+void op_fnzjobexam(mval *prelimSpec, mval *fmt, mval *finalSpec)
 {
 	MV_FORCE_STR(prelimSpec);
-	jobexam_process(prelimSpec, finalSpec);
+	MV_FORCE_STR(fmt);
+	jobexam_process(prelimSpec, finalSpec, fmt);
 	return;
 }
