@@ -1,6 +1,7 @@
 /****************************************************************
  *								*
- *	Copyright 2009, 2014 Fidelity Information Services, Inc	*
+ * Copyright (c) 2009-2021 Fidelity National Information	*
+ * Services, Inc. and/or its subsidiaries. All rights reserved.	*
  *								*
  *	This source code contains the intellectual property	*
  *	of its copyright holder(s), and is made available	*
@@ -42,7 +43,7 @@ void op_setalsct2alsct(lv_val *srclv, lv_val *dstlv)
 	assert(dstlv);
 	assert(!LV_IS_BASE_VAR(dstlv));	/* Verify subscripted var */
 	if (!(srclv->v.mvtype & MV_ALIASCONT))
-		rts_error_csa(CSA_ARG(NULL) VARLSTCNT(1) ERR_ALIASEXPECTED);
+		RTS_ERROR_CSA_ABT(NULL, VARLSTCNT(1) ERR_ALIASEXPECTED);
 	src_lvref = (lv_val *)srclv->v.str.addr;
 	assert(src_lvref);
 	assert(LV_IS_BASE_VAR(src_lvref));	/* Verify base var */
