@@ -1,6 +1,6 @@
 /****************************************************************
  *								*
- * Copyright (c) 2001-2020 Fidelity National Information	*
+ * Copyright (c) 2001-2022 Fidelity National Information	*
  * Services, Inc. and/or its subsidiaries. All rights reserved.	*
  *								*
  *	This source code contains the intellectual property	*
@@ -51,73 +51,40 @@
 #include "dollar_quit.h"
 #include "ztimeout_routines.h"
 #include "mlkdef.h"
-#ifdef UNIX
-#  include "iormdef.h"
-#  ifdef DEBUG
-#    include "wbox_test_init.h"
-#  endif
+#include "iormdef.h"
+#include "toktyp.h"
+#ifdef DEBUG
+#  include "wbox_test_init.h"
 #endif
 
 #define ESC_OFFSET		4
 #define MAX_COMMAND_LINE_LENGTH	255
 
-GBLREF mval		dollar_zproc;
-GBLREF mval		dollar_zdir;
-GBLREF stack_frame	*frame_pointer;
-GBLREF mval		dollar_estack_delta;
-GBLREF spdesc		stringpool;
-GBLREF io_pair		io_curr_device;
-GBLREF io_log_name	*io_root_log_name;
-GBLREF io_log_name	*dollar_principal;
-GBLREF mval		dollar_zgbldir;
-GBLREF mval		dollar_job;
-GBLREF uint4		dollar_zjob;
-GBLREF mval		dollar_zstatus;
-GBLREF mval		dollar_zsource;
-GBLREF int4		dollar_zsystem;
-GBLREF boolean_t	dollar_zaudit;
-GBLREF int4		dollar_zeditor;
-GBLREF uint4		dollar_tlevel;
-GBLREF uint4		dollar_trestart;
-GBLREF mval		dollar_zerror;
-GBLREF mval		dollar_zyerror;
-GBLREF mval		dollar_system;
-GBLREF mval		dollar_zinterrupt;
-GBLREF boolean_t	dollar_zininterrupt;
-GBLREF int4		zdir_form;
-GBLREF boolean_t	ztrap_explicit_null;		/* whether $ZTRAP was explicitly set to NULL in this frame */
-GBLREF mval		dollar_ztexit;
-GBLREF size_t		totalAlloc;
-GBLREF size_t		totalRmalloc;
-GBLREF size_t		totalUsed;
-GBLREF size_t		totalAllocGta;
-GBLREF size_t		totalRallocGta;
-GBLREF size_t		totalUsedGta;
-GBLREF mstr		dollar_zchset;
-GBLREF mstr		dollar_zpatnumeric;
-GBLREF boolean_t	dollar_zquit_anyway;
-GBLREF io_pair		*io_std_device;
-GBLREF mstr		dollar_zpin;
-GBLREF mstr		dollar_zpout;
-GBLREF int		process_exiting;
+GBLREF boolean_t		dollar_zquit_anyway;
+GBLREF boolean_t		ztrap_explicit_null;		/* whether $ZTRAP was explicitly set to NULL in this frame */
+GBLREF int			process_exiting;
+GBLREF int4			dollar_zeditor, dollar_zsystem, zdir_form;
+GBLREF io_log_name		*dollar_principal, *io_root_log_name;
+GBLREF io_pair			io_curr_device, *io_std_device;
 GBLREF mlk_subhash_val_t	mlk_last_hash;
+GBLREF mstr			dollar_zchset, dollar_zpatnumeric, dollar_zpin, dollar_zpout;
+GBLREF mval			dollar_estack_delta, dollar_job, dollar_system, dollar_zdir, dollar_zerror, dollar_zgbldir;
+GBLREF mval			dollar_zinterrupt, dollar_zproc, dollar_zsource, dollar_zstatus, dollar_ztexit, dollar_zyerror;
+GBLREF size_t			totalAlloc, totalAllocGta, totalRmalloc, totalRallocGta, totalUsed, totalUsedGta, zmalloclim;
+GBLREF spdesc			stringpool;
+GBLREF stack_frame		*frame_pointer;
+GBLREF uint4			dollar_tlevel, dollar_trestart, dollar_zjob;
+GBLREF volatile boolean_t	dollar_zininterrupt;
 
 #ifdef GTM_TRIGGER
-GBLREF	mstr		*dollar_ztname;
-GBLREF	mval		*dollar_ztdata;
-GBLREF	mval		*dollar_ztdelim;
-GBLREF	mval		*dollar_ztoldval;
-GBLREF	mval		*dollar_ztriggerop;
-GBLREF	mval		dollar_ztslate;
-GBLREF	mval		*dollar_ztupdate;
-GBLREF	mval		*dollar_ztvalue;
-GBLREF	mval		dollar_ztwormhole;
-GBLREF	int4		gtm_trigger_depth;
-GBLREF	boolean_t	ztwormhole_used;		/* TRUE if $ztwormhole was used by trigger code */
+GBLREF	boolean_t		ztwormhole_used;		/* TRUE if $ztwormhole was used by trigger code */
+GBLREF	int4			gtm_trigger_depth;
+GBLREF	mstr			*dollar_ztname;
+GBLREF	mval			*dollar_ztdata, *dollar_ztdelim, *dollar_ztoldval, *dollar_ztriggerop;
+GBLREF	mval			dollar_ztslate, *dollar_ztupdate, *dollar_ztvalue, dollar_ztwormhole;
 #endif
 
 error_def(ERR_INVSVN);
-error_def(ERR_TEXT);
 error_def(ERR_UNIMPLOP);
 error_def(ERR_ZDIROUTOFSYNC);
 
@@ -130,14 +97,12 @@ LITREF int4		gtm_release_stamp_len;
 void op_svget(int varnum, mval *v)
 {
 	boolean_t	lcl_compile_time;
-	io_log_name	*tl;
-	int 		count;
-	gtm_uint64_t	ucount;
-	char		*c1, *c2;
-	mval		*mvp;
-#	ifdef UNIX
+	char		*c1, *c2, director_token;
 	d_rm_struct	*d_rm;
-#	endif
+	gtm_uint64_t	ucount;
+	int 		count;
+	io_log_name	*tl;
+	mval		*mvp;
 	DCL_THREADGBL_ACCESS;
 
 	SETUP_THREADGBL_ACCESS;
@@ -283,7 +248,7 @@ void op_svget(int varnum, mval *v)
 			MV_FORCE_MVAL(v, count);
 			break;
 		case SV_ZAUDIT:
-			MV_FORCE_MVAL(v, dollar_zaudit);
+			MV_FORCE_MVAL(v, TREF(dollar_zaudit));
 			break;
 		case SV_ZB:
 			c1 = (char *)io_curr_device.in->dollar.zb;
@@ -505,7 +470,7 @@ void op_svget(int varnum, mval *v)
 			}
 			break;
 #			else
-			rts_error_csa(CSA_ARG(NULL) VARLSTCNT(1) ERR_UNIMPLOP);
+			RTS_ERROR_CSA_ABT(NULL, VARLSTCNT(1) ERR_UNIMPLOP);
 #			endif
 		case SV_ZTDATA:
 #			ifdef GTM_TRIGGER
@@ -516,7 +481,7 @@ void op_svget(int varnum, mval *v)
 			memcpy(v, (NULL != dollar_ztdata) ? dollar_ztdata : &literal_null, SIZEOF(mval));
 			break;
 #			else
-			rts_error_csa(CSA_ARG(NULL) VARLSTCNT(1) ERR_UNIMPLOP);
+			RTS_ERROR_CSA_ABT(NULL, VARLSTCNT(1) ERR_UNIMPLOP);
 #			endif
 		case SV_ZTDELIM:
 #			ifdef GTM_TRIGGER
@@ -527,7 +492,7 @@ void op_svget(int varnum, mval *v)
 				memcpy(v, dollar_ztdelim, SIZEOF(mval));
 			break;
 #			else
-			rts_error_csa(CSA_ARG(NULL) VARLSTCNT(1) ERR_UNIMPLOP);
+			RTS_ERROR_CSA_ABT(NULL, VARLSTCNT(1) ERR_UNIMPLOP);
 #			endif
 		case SV_ZTOLDVAL:
 #			ifdef GTM_TRIGGER
@@ -538,7 +503,7 @@ void op_svget(int varnum, mval *v)
 			memcpy(v, (NULL != dollar_ztoldval) ? dollar_ztoldval : &literal_null, SIZEOF(mval));
 			break;
 #			else
-			rts_error_csa(CSA_ARG(NULL) VARLSTCNT(1) ERR_UNIMPLOP);
+			RTS_ERROR_CSA_ABT(NULL, VARLSTCNT(1) ERR_UNIMPLOP);
 #			endif
 		case SV_ZTRIGGEROP:
 #			ifdef GTM_TRIGGER
@@ -547,7 +512,7 @@ void op_svget(int varnum, mval *v)
 			memcpy(v, (NULL != dollar_ztriggerop) ? dollar_ztriggerop : &literal_null, SIZEOF(mval));
 			break;
 #			else
-			rts_error_csa(CSA_ARG(NULL) VARLSTCNT(1) ERR_UNIMPLOP);
+			RTS_ERROR_CSA_ABT(NULL, VARLSTCNT(1) ERR_UNIMPLOP);
 #			endif
 		case SV_ZTUPDATE:
 #			ifdef GTM_TRIGGER
@@ -558,7 +523,7 @@ void op_svget(int varnum, mval *v)
 				   : &literal_null), SIZEOF(mval));
 			break;
 #			else
-			rts_error_csa(CSA_ARG(NULL) VARLSTCNT(1) ERR_UNIMPLOP);
+			RTS_ERROR_CSA_ABT(NULL, VARLSTCNT(1) ERR_UNIMPLOP);
 #			endif
 		case SV_ZTVALUE:
 #			ifdef GTM_TRIGGER
@@ -569,7 +534,7 @@ void op_svget(int varnum, mval *v)
 			memcpy(v, (NULL != dollar_ztvalue) ? dollar_ztvalue : &literal_null, SIZEOF(mval));
 			break;
 #			else
-			rts_error_csa(CSA_ARG(NULL) VARLSTCNT(1) ERR_UNIMPLOP);
+			RTS_ERROR_CSA_ABT(NULL, VARLSTCNT(1) ERR_UNIMPLOP);
 #			endif
 		case SV_ZTWORMHOLE:
 #			ifdef GTM_TRIGGER
@@ -584,7 +549,7 @@ void op_svget(int varnum, mval *v)
 			ztwormhole_used = TRUE;
 			break;
 #			else
-			rts_error_csa(CSA_ARG(NULL) VARLSTCNT(1) ERR_UNIMPLOP);
+			RTS_ERROR_CSA_ABT(NULL, VARLSTCNT(1) ERR_UNIMPLOP);
 #			endif
 		case SV_ZTSLATE:
 #			ifdef GTM_TRIGGER
@@ -595,14 +560,14 @@ void op_svget(int varnum, mval *v)
 			memcpy(v, mvp, SIZEOF(mval));
 			break;
 #			else
-			rts_error_csa(CSA_ARG(NULL) VARLSTCNT(1) ERR_UNIMPLOP);
+			RTS_ERROR_CSA_ABT(NULL, VARLSTCNT(1) ERR_UNIMPLOP);
 #			endif
 		case SV_ZTLEVEL:
 #			ifdef GTM_TRIGGER
 			MV_FORCE_MVAL(v, gtm_trigger_depth);
 			break;
 #			else
-			rts_error_csa(CSA_ARG(NULL) VARLSTCNT(1) ERR_UNIMPLOP);
+			RTS_ERROR_CSA_ABT(NULL, VARLSTCNT(1) ERR_UNIMPLOP);
 #			endif
 		case SV_ZONLNRLBK:
 #			ifdef UNIX
@@ -610,7 +575,7 @@ void op_svget(int varnum, mval *v)
 			MV_FORCE_MVAL(v, count);
 			break;
 #			else
-			rts_error_csa(CSA_ARG(NULL) VARLSTCNT(1) ERR_UNIMPLOP);
+			RTS_ERROR_CSA_ABT(NULL, VARLSTCNT(1) ERR_UNIMPLOP);
 #			endif
 		case SV_ZCLOSE:
 #			ifdef UNIX
@@ -618,7 +583,7 @@ void op_svget(int varnum, mval *v)
 			MV_FORCE_MVAL(v, count);
 			break;
 #			else
-			rts_error_csa(CSA_ARG(NULL) VARLSTCNT(1) ERR_UNIMPLOP);
+			RTS_ERROR_CSA_ABT(NULL, VARLSTCNT(1) ERR_UNIMPLOP);
 #			endif
 		case SV_ZKEY:
 			get_dlr_zkey(v);
@@ -627,6 +592,9 @@ void op_svget(int varnum, mval *v)
 			count = stringpool.strpllim;
 			MV_FORCE_MVAL(v, count);
 			break;
+		case SV_ZMALLOCLIM:
+			MV_FORCE_UMVAL(v, zmalloclim);
+			break;
 		case SV_ZTIMEOUT:
 			count = get_ztimeout(v);
 			if (-1 == count)
@@ -636,18 +604,21 @@ void op_svget(int varnum, mval *v)
 			i2usmval(v, mlk_last_hash);
 			break;
 		default:
-			rts_error_csa(CSA_ARG(NULL) VARLSTCNT(1) ERR_INVSVN);
+			RTS_ERROR_CSA_ABT(NULL, VARLSTCNT(1) ERR_INVSVN);
 	}
 	if (!(MVTYPE_IS_STRING(v->mvtype)))
 	{	/* in case op_svget is called at compile time; shouldn't hurt much any time */
 		assert(MVTYPE_IS_NUMERIC(v->mvtype));
 		n2s(v);
 	} else if (!(MVTYPE_IS_NUMERIC(v->mvtype)))
-	{
+	{	/* need to stop NUMOFLOW errors from preventing access to ISV values that s2n would flag as out of range */
 		assert(MVTYPE_IS_STRING(v->mvtype));
 		lcl_compile_time = TREF(compile_time);
+		director_token = TREF(director_token);
+		TREF(director_token) = TK_STRLIT;
 		TREF(compile_time) = TRUE;
 		s2n(v);
+		TREF(director_token) = director_token;
 		TREF(compile_time) = lcl_compile_time;
 	}
 }
