@@ -1,6 +1,6 @@
 /****************************************************************
  *								*
- * Copyright (c) 2001-2020 Fidelity National Information	*
+ * Copyright (c) 2001-2022 Fidelity National Information	*
  * Services, Inc. and/or its subsidiaries. All rights reserved.	*
  *								*
  *	This source code contains the intellectual property	*
@@ -231,7 +231,7 @@ void	op_view(int numarg, mval *keyword, ...)
 			if (!(~(BREAK_MASK_END - 1) & msk))
 				break_message_mask = msk;
 			else
-				rts_error_csa(CSA_ARG(NULL) VARLSTCNT(4) ERR_VIEWCMD, 2, RTS_ERROR_MVAL(parmblk.value));
+				RTS_ERROR_CSA_ABT(NULL, VARLSTCNT(4) ERR_VIEWCMD, 2, RTS_ERROR_MVAL(parmblk.value));
 			break;
 		case VTK_DEBUG1:
 			outval.mvtype = MV_STR;
@@ -274,7 +274,7 @@ void	op_view(int numarg, mval *keyword, ...)
 		case VTK_NOFULLBOOL:
 
 			if ((VTK_NOFULLBOOL == vtp->keycode) && (OLD_SE != TREF(side_effect_handling)))
-				rts_error_csa(CSA_ARG(NULL) VARLSTCNT(1) ERR_SEFCTNEEDSFULLB);
+				RTS_ERROR_CSA_ABT(NULL, VARLSTCNT(1) ERR_SEFCTNEEDSFULLB);
 			old_bool = TREF(gtm_fullbool);
 			TREF(gtm_fullbool) = (VTK_FULLBOOL == vtp->keycode) ? FULL_BOOL
 				: (VTK_FULLBOOLWARN == vtp->keycode) ? FULL_BOOL_WARN : GTM_BOOL;
@@ -377,8 +377,8 @@ void	op_view(int numarg, mval *keyword, ...)
 			if (0 == status)
 			{
 				va_end(var);
-				rts_error_csa(CSA_ARG(NULL) VARLSTCNT(4) ERR_PATTABNOTFND, 2, parmblk.value->str.len,
-						parmblk.value->str.addr);
+				RTS_ERROR_CSA_ABT(NULL, VARLSTCNT(4) ERR_PATTABNOTFND, 2, parmblk.value->str.len,
+					parmblk.value->str.addr);
 			}
 			break;
 		case VTK_RESETGVSTATS:
@@ -411,14 +411,14 @@ void	op_view(int numarg, mval *keyword, ...)
 			else
 			{
 				va_end(var);
-				rts_error_csa(CSA_ARG(NULL) VARLSTCNT(1) ERR_REQDVIEWPARM);
+				RTS_ERROR_CSA_ABT(NULL, VARLSTCNT(1) ERR_REQDVIEWPARM);
 			}
 			if (0 == lct)
 				break;	/* 0 value of collation is always good */
 			if ((lct < MIN_COLLTYPE) || (lct > MAX_COLLTYPE))
 			{
 				va_end(var);
-				rts_error_csa(CSA_ARG(NULL) VARLSTCNT(3) ERR_ACTRANGE, 1, lct);
+				RTS_ERROR_CSA_ABT(NULL, VARLSTCNT(3) ERR_ACTRANGE, 1, lct);
 			}
 			was_skip_gtm_putmsg = TREF(skip_gtm_putmsg);
 			TREF(skip_gtm_putmsg) = TRUE;	/* To avoid ready_collseq from doing gtm_putmsg in case of errors.
@@ -429,7 +429,7 @@ void	op_view(int numarg, mval *keyword, ...)
 			if (0 == new_lcl_collseq)
 			{
 				va_end(var);
-				rts_error_csa(CSA_ARG(NULL) VARLSTCNT(3) ERR_COLLATIONUNDEF, 1, lct);
+				RTS_ERROR_CSA_ABT(NULL, VARLSTCNT(3) ERR_COLLATIONUNDEF, 1, lct);
 			}
 			break;
 		case VTK_YDIRTVAL:
@@ -449,7 +449,7 @@ void	op_view(int numarg, mval *keyword, ...)
 			if ((MAX_YDIRTSTR < TREF(view_ydirt_str_len)) || (0 >=  TREF(view_ydirt_str_len)))
 			{
 				va_end(var);
-				rts_error_csa(CSA_ARG(NULL) VARLSTCNT(3) ERR_YDIRTSZ, 1, TREF(view_ydirt_str_len));
+				RTS_ERROR_CSA_ABT(NULL, VARLSTCNT(3) ERR_YDIRTSZ, 1, TREF(view_ydirt_str_len));
 			}
 			if (TREF(view_ydirt_str_len) > 0)
 				memcpy(TREF(view_ydirt_str), parmblk.value->str.addr, TREF(view_ydirt_str_len));
@@ -461,14 +461,14 @@ void	op_view(int numarg, mval *keyword, ...)
 			 * a YDIRTREE update is performed
 			 */
 			if ((MAX_YDIRTSTR < TREF(view_ydirt_str_len)) || (0 >=  TREF(view_ydirt_str_len)))
-				rts_error_csa(CSA_ARG(NULL) VARLSTCNT(3) ERR_YDIRTSZ, 1, TREF(view_ydirt_str_len));
+				RTS_ERROR_CSA_ABT(NULL, VARLSTCNT(3) ERR_YDIRTSZ, 1, TREF(view_ydirt_str_len));
 			if (!parmblk.value->str.len)
-				rts_error_csa(CSA_ARG(NULL)
+				RTS_ERROR_CSA_ABT(NULL,
 					VARLSTCNT(4) ERR_VIEWCMD, 2, parmblk.value->str.len, parmblk.value->str.addr);
 			size = extnam_str.len;		/* internal use of op_gvname should not disturb extended reference */
 			op_gvname(VARLSTCNT(1) parmblk.value);
 			extnam_str.len = size;
-			gvnh_reg = TREF(gd_targ_gvnh_reg);     						/* Set up by op_gvname */
+			gvnh_reg = TREF(gd_targ_gvnh_reg);						/* Set up by op_gvname */
 			arg = (numarg > 1) ? va_arg(var, mval *) : NULL;
 			if (NULL != arg)
 			{
@@ -477,7 +477,7 @@ void	op_view(int numarg, mval *keyword, ...)
 			} else
 				reg = (NULL != gvnh_reg) ? gvnh_reg->gd_reg : gv_cur_region;
 			if (!(IS_ACC_METH_BG_OR_MM(cs_data->acc_meth)))
-				rts_error_csa(CSA_ARG(NULL) VARLSTCNT(4) ERR_DBREMOTE, 2, REG_LEN_STR(reg));
+				RTS_ERROR_CSA_ABT(NULL, VARLSTCNT(4) ERR_DBREMOTE, 2, REG_LEN_STR(reg));
 			gvspan = (NULL == gvnh_reg) ? NULL : gvnh_reg->gvspan;
 			if (NULL != gvspan)
 			{
@@ -494,7 +494,7 @@ void	op_view(int numarg, mval *keyword, ...)
 					lct = 0;
 			}
 			if (!found_reg)
-				rts_error_csa(CSA_ARG(NULL) VARLSTCNT(6) ERR_GBLNOMAPTOREG, 4,
+				RTS_ERROR_CSA_ABT(NULL, VARLSTCNT(6) ERR_GBLNOMAPTOREG, 4,
 					parmblk.value->str.len, parmblk.value->str.addr, REG_LEN_STR(reg));
 			if (lct)
 				rts_error_csa(CSA_ARG(NULL) VARLSTCNT(6) ERR_COLLDATAEXISTS, 4, LEN_AND_LIT("^"),
@@ -520,7 +520,7 @@ void	op_view(int numarg, mval *keyword, ...)
 			else
 			{
 				va_end(var);
-				rts_error_csa(CSA_ARG(NULL) VARLSTCNT(1) ERR_REQDVIEWPARM);
+				RTS_ERROR_CSA_ABT(NULL, VARLSTCNT(1) ERR_REQDVIEWPARM);
 			}
 			nextarg = NULL;
 			ncol = -1;
@@ -544,7 +544,7 @@ void	op_view(int numarg, mval *keyword, ...)
 				if ((lct < MIN_COLLTYPE) || (lct > MAX_COLLTYPE))
 				{
 					va_end(var);
-					rts_error_csa(CSA_ARG(NULL) VARLSTCNT(3) ERR_ACTRANGE, 1, lct);
+					RTS_ERROR_CSA_ABT(NULL, VARLSTCNT(3) ERR_ACTRANGE, 1, lct);
 				}
 			}
 			/* At this point, verify that there is no local data with subscripts */
@@ -559,7 +559,7 @@ void	op_view(int numarg, mval *keyword, ...)
 						if (lv && LV_HAS_CHILD(lv))
 						{
 							va_end(var);
-							rts_error_csa(CSA_ARG(NULL) VARLSTCNT(6) ERR_COLLDATAEXISTS, 4,
+							RTS_ERROR_CSA_ABT(NULL, VARLSTCNT(6) ERR_COLLDATAEXISTS, 4,
 								LEN_AND_LIT("subscripted "), LEN_AND_LIT("local"));
 						}
 					}
@@ -573,7 +573,7 @@ void	op_view(int numarg, mval *keyword, ...)
 					if (0 == new_lcl_collseq)
 					{
 						va_end(var);
-						rts_error_csa(CSA_ARG(NULL) VARLSTCNT(3) ERR_COLLATIONUNDEF, 1, lct);
+						RTS_ERROR_CSA_ABT(NULL, VARLSTCNT(3) ERR_COLLATIONUNDEF, 1, lct);
 					}
 					TREF(local_collseq) = new_lcl_collseq;
 				} else
@@ -588,17 +588,17 @@ void	op_view(int numarg, mval *keyword, ...)
 					}
 				}
 			}
-			if (-1 != ncol)
+			if (0 <= ncol)
 				TREF(local_collseq_stdnull) = (ncol ? TRUE: FALSE);
-			if (-1 != nct)
+			if (0 <= nct)
 				TREF(local_coll_nums_as_strings) = (nct ? TRUE: FALSE);
 			break;
 		case VTK_PATLOAD:
 			if (!load_pattern_table(parmblk.value->str.len, parmblk.value->str.addr))
 			{
 				va_end(var);
-				rts_error_csa(CSA_ARG(NULL) VARLSTCNT(4) ERR_PATLOAD, 2, parmblk.value->str.len,
-						parmblk.value->str.addr);
+				RTS_ERROR_CSA_ABT(NULL, VARLSTCNT(4) ERR_PATLOAD, 2, parmblk.value->str.len,
+					parmblk.value->str.addr);
 			}
 			break;
 		case VTK_NOUNDEF:
@@ -615,7 +615,7 @@ void	op_view(int numarg, mval *keyword, ...)
 			if (zdefactive)
 			{
 				va_end(var);
-				rts_error_csa(CSA_ARG(NULL) VARLSTCNT(1) ERR_ZDEFACTIVE);
+				RTS_ERROR_CSA_ABT(NULL, VARLSTCNT(1) ERR_ZDEFACTIVE);
 			}
 			zdefactive = TRUE;
 			tmpzdefbufsiz = MV_FORCE_INT(parmblk.value);
@@ -637,7 +637,7 @@ void	op_view(int numarg, mval *keyword, ...)
 				if (2 > numarg)
 				{
 					va_end(var);
-					rts_error_csa(CSA_ARG(NULL) VARLSTCNT(1) ERR_TRACEON);
+					RTS_ERROR_CSA_ABT(NULL, VARLSTCNT(1) ERR_TRACEON);
 				}
 				arg = va_arg(var, mval *);
 				MV_FORCE_STR(arg);
@@ -827,7 +827,7 @@ void	op_view(int numarg, mval *keyword, ...)
 						TREF(lvmon_vars_anchor) = NULL;
 						TREF(lvmon_vars_count) = 0;
 						TREF(lvmon_active) = FALSE;			/* No monitoring active now */
-						rts_error_csa(CSA_ARG(NULL) VARLSTCNT(4) ERR_VIEWLVN, 2, arg->str.len,
+						RTS_ERROR_CSA_ABT(NULL, VARLSTCNT(4) ERR_VIEWLVN, 2, arg->str.len,
 							arg->str.addr);
 					}
 					lvmon_var_p->lvmv.var_name.len = arg->str.len;
@@ -847,7 +847,7 @@ void	op_view(int numarg, mval *keyword, ...)
 			break;
 		default:
 			va_end(var);
-			rts_error_csa(CSA_ARG(NULL) VARLSTCNT(4) ERR_VIEWCMD, 2, strlen((const char *)vtp->keyword), vtp->keyword);
+			RTS_ERROR_CSA_ABT(NULL, VARLSTCNT(4) ERR_VIEWCMD, 2, strlen((const char *)vtp->keyword), vtp->keyword);
 	}
 	va_end(var);
 	return;
@@ -926,6 +926,7 @@ STATICFNDEF void view_dbop(unsigned char keycode, viewparm *parmblkptr, mval *th
 				if (!reg->read_only)
 				{
 					JNLPOOL_INIT_IF_NEEDED(cs_addrs, cs_data, cs_addrs->nl, SCNDDBNOUPD_CHECK_FALSE);
+					/* Negatives treated as zero */
 					nbuffs = (NULL != thirdarg) ? MV_FORCE_INT(thirdarg) : cs_addrs->nl->wcs_active_lvl;
 					JNL_ENSURE_OPEN_WCS_WTSTART(cs_addrs, reg, nbuffs, NULL, FALSE, dummy_errno);
 				}
@@ -1044,7 +1045,7 @@ STATICFNDEF void view_dbop(unsigned char keycode, viewparm *parmblkptr, mval *th
 				{
 					cs_addrs->reservedDBFlags &= ~RDBF_NOSTATS;
 					if (0 < dollar_tlevel)		/* Can't do this in TP */
-						rts_error_csa(CSA_ARG(NULL) VARLSTCNT(1) ERR_TPNOSTATSHARE);
+						RTS_ERROR_CSA_ABT(NULL, VARLSTCNT(1) ERR_TPNOSTATSHARE);
 					gvcst_init_statsDB(reg, DO_STATSDB_INIT_TRUE);
 				}
 				break;
@@ -1055,7 +1056,7 @@ STATICFNDEF void view_dbop(unsigned char keycode, viewparm *parmblkptr, mval *th
 				if (statsDBreg->statsDB_setup_completed)
 				{	/* Don't bother to opt-out if not set up - it just confuses things */
 					if (0 < dollar_tlevel)		/* Can't do this in TP */
-						rts_error_csa(CSA_ARG(NULL) VARLSTCNT(1) ERR_TPNOSTATSHARE);
+						RTS_ERROR_CSA_ABT(NULL, VARLSTCNT(1) ERR_TPNOSTATSHARE);
 					gvcst_remove_statsDB_linkage(reg);
 				}
 				break;
