@@ -1,6 +1,6 @@
 /****************************************************************
  *								*
- * Copyright (c) 2001-2020 Fidelity National Information	*
+ * Copyright (c) 2001-2022 Fidelity National Information	*
  * Services, Inc. and/or its subsidiaries. All rights reserved.	*
  *								*
  *	This source code contains the intellectual property	*
@@ -59,7 +59,7 @@ void op_zprevious(mval *v)
 	boolean_t		found, ok_to_change_currkey;
 	enum db_acc_method	acc_meth;
 	gd_addr			*gd_targ;
-	gd_binding		*gd_map_start, *map, *prev_map;
+	gd_binding		*gd_map_start, *map, *prev_map, *rmap;
 	gd_region		*save_gv_cur_region;
 	gv_key_buf		save_currkey;
 	gv_namehead		*save_gv_target;
@@ -180,11 +180,12 @@ void op_zprevious(mval *v)
 			{	/* Non-statsDB region */
 				if (!gv_cur_region->open)
 					gv_init_reg(gv_cur_region, NULL);
-				change_reg();
 				/* Entries in directory tree could have empty GVT in which case move on to previous entry */
 				acc_meth = REG_ACC_METH(gv_cur_region);
 				for ( ; ; )
 				{
+					gv_cur_region = map->reg.addr;
+					change_reg();
 					assert(0 == gv_currkey->prev);	/* or else gvcst_zprevious could get confused */
 					if (IS_ACC_METH_BG_OR_MM(acc_meth))
 					{
@@ -217,17 +218,52 @@ void op_zprevious(mval *v)
 					GV_BIND_NAME_AND_ROOT_SEARCH(gd_targ, &gvname, gvnh_reg);	/* updates "gv_currkey" */
 					assert((NULL != gvnh_reg->gvspan) || (gv_cur_region == map->reg.addr));
 					if (NULL != gvnh_reg->gvspan)
-					{	/* gv_target would NOT have been initialized by GV_BIND_NAME in this case.
-						 * So finish that initialization.
-						 */
-						datamval = &tmpmval;
-						/* The below macro finishes the task of GV_BIND_NAME_AND_ROOT_SEARCH
-						 * 	(e.g. setting gv_cur_region for spanning globals)
+					{	/* For accurate results, we need to see if the node found by gvcst_zprevious()
+						 * has data or that its first data child lies in the same map. Otherwise,
+						 * the node should be excluded.
+						 * To do this, take the proposed key and do a gvcst_query2() on it.
+						 * If the resulting key matches the request and is in the current map,
+						 * use it to derive the result. Otherwise, put things back to where they
+						 * were after the search and continue.
 						 */
+						if ((prev_map->gvkey_len > gv_currkey->end)
+							&& (0 == memcmp(gv_currkey->base, prev_map->gvkey.addr, gv_currkey->end)))
+						{	/* We found the same global name as the previous map's endpoint,
+							 * so search forward from there to make sure we found something
+							 * in this map by borrowing the endpoint's subscript(s).
+							 */
+							memcpy(gv_currkey->base + gv_currkey->end,
+									prev_map->gvkey.addr + gv_currkey->end,
+									prev_map->gvkey_len - gv_currkey->end);
+							gv_currkey->end = prev_map->gvkey_len;
+							gv_currkey->base[gv_currkey->end] = KEY_DELIMITER;
+						}
 						GV_BIND_SUBSNAME_IF_GVSPAN(gvnh_reg, gd_targ, gv_currkey, gvnh_reg->gd_reg);
-						op_gvdata(datamval);
-						if (MV_FORCE_INT(datamval))
-							break;
+						found = gv_target->root && gvcst_query2();
+						if (found && (gv_altkey->end >= gvname.var_name.len + 1)
+							&& (0 == memcmp(gv_currkey->base, gv_altkey->base,
+										gvname.var_name.len + 1)))
+						{
+							rmap = gv_srch_map_linear_backward(map, (char *)gv_altkey->base,
+												gv_altkey->end - 1);
+							if (map == rmap)
+							{
+								for (gv_altkey->end = 0;
+										gv_altkey->base[gv_altkey->end];
+										gv_altkey->end++)
+									;
+								gv_altkey->base[++gv_altkey->end] = KEY_DELIMITER;
+								found = TRUE;
+								break;
+							} else
+								found = FALSE;
+						} else
+							found = FALSE;
+						/* We may have added subscripts from the previous map above, so restore
+						 * gv_currkey to the name alone by using the gvname we saved above.
+						 */
+						gv_currkey->end = gvname.var_name.len + 1;
+						gv_currkey->base[gv_currkey->end] = KEY_DELIMITER;
 					} else
 					{	/* else gv_target->root would have been initialized by
 						 * GV_BIND_NAME_AND_ROOT_SEARCH
@@ -240,13 +276,13 @@ void op_zprevious(mval *v)
 					break;
 			} else
 				found = FALSE;	/* Skip statsDB region completely. So set found to FALSE */
-			/* If previous map corresponds to a spanning global, then do not update gv_currkey as that would
-			 * effectively cause the spanning global to be skipped. If gvkey_len == gvname_len + 1 it is NOT
-			 * a spanning global map entry.
+			/* At this point, gv_currkey may be one of:
+			 * -- The initial gv_currkey, or one from a previous iteration. (map to statsdb region)
+			 * -- The result of gvcst_zprevious(), possibly mangled/restored (gvspan + prev_map name match case)
 			 */
 			assert(prev_map->gvkey_len >= (prev_map->gvname_len + 1));
-			if ((prev_map > (gd_map_start + 1)) && (prev_map->gvkey_len == (prev_map->gvname_len + 1)))
-			{
+			if (prev_map > (gd_map_start + 1))
+			{	/* Base the continued search on the previous map endpoint. */
 				assert(strlen(prev_map->gvkey.addr) == prev_map->gvname_len);
 				gv_currkey->end = prev_map->gvname_len + 1;
 				assert(gv_currkey->end <= (MAX_MIDENT_LEN + 1));
@@ -254,8 +290,15 @@ void op_zprevious(mval *v)
 				assert(KEY_DELIMITER == gv_currkey->base[gv_currkey->end - 1]);
 				gv_currkey->base[gv_currkey->end] = KEY_DELIMITER;
 				assert(gv_currkey->top > gv_currkey->end);	/* ensure we are within allocated bounds */
+				if (prev_map->gvkey_len != (prev_map->gvname_len + 1))
+				{	/* The prior map endpoint has subscripts, and we may want to match its name, so start the
+					 * search after the name.
+					 */
+					GVKEY_INCREMENT_ORDER(gv_currkey);
+				}
 			}
 		}
+		change_reg();
 		v->mvtype = 0; /* so stp_gcol (if invoked below) can free up space currently occupied (BYPASSOK)
 				* by this to-be-overwritten mval */
 		if (found)
