@@ -1,6 +1,6 @@
 /****************************************************************
  *								*
- * Copyright (c) 2001-2019 Fidelity National Information	*
+ * Copyright (c) 2001-2022 Fidelity National Information	*
  * Services, Inc. and/or its subsidiaries. All rights reserved.	*
  *								*
  *	This source code contains the intellectual property	*
@@ -47,7 +47,6 @@ typedef struct
 			TOPCHAR, MIN, MAX, SIZE, TOTAL_MIN, TOTAL_MAX, COUNT, LASTPATPTR, LAST_INFINITE,	\
 			MIN_DFA, ALTMIN, ALTMAX, INSTR, INCHAR, DFA)						\
 {														\
-	gtm_uint64_t		bound;										\
 	int			cursize, i;									\
 														\
 	PATMASKPTR = OUTCHAR;											\
@@ -67,14 +66,17 @@ typedef struct
 		MAX[COUNT] = PAT_MAX_REPEAT;									\
 		SIZE[COUNT] = cursize;										\
 		TOTAL_MIN += BOUND_MULTIPLY(MIN[COUNT], SIZE[COUNT], bound);					\
+		assert(0 <= TOTAL_MIN);										\
 		if (TOTAL_MIN > PAT_MAX_REPEAT)									\
 			TOTAL_MIN = PAT_MAX_REPEAT;								\
 		TOTAL_MAX += BOUND_MULTIPLY(MAX[COUNT], SIZE[COUNT], bound);					\
+		assert(0 <= TOTAL_MAX);										\
 		if (TOTAL_MAX > PAT_MAX_REPEAT)									\
 			TOTAL_MAX = PAT_MAX_REPEAT;								\
 		LASTPATPTR = PATMASKPTR;									\
 		LAST_INFINITE = TRUE;										\
 		COUNT++;											\
+		assert(0 < COUNT);										\
 	} else													\
 	{													\
 		OUTCHAR = PATMASKPTR;										\
@@ -134,53 +136,42 @@ typedef struct
 
 int patstr(mstr *instr, ptstr *obj, unsigned char **relay)
 {
-	pat_strlit		strlit;
-	boolean_t		dfa, done, infinite;
-	boolean_t		dfa_fixed_len, fixed_len, prev_fixed_len, split_atom, start_dfa;
-	int4			lower_bound, upper_bound, alloclen;
-	gtm_uint64_t		bound;
-	unsigned char		curchar, symbol, *inchar, *in_top, *buffptr;
-	uint4			pattern_mask, last_leaf_mask, y_max, mbit;
-	uint4			*patmaskptr;
-	int			atom_map, count, total_min, total_max;
-	int			min[MAX_PATTERN_ATOMS], max[MAX_PATTERN_ATOMS], size[MAX_PATTERN_ATOMS];
-	struct leaf		leaves, *lv_ptr;
-	struct e_table		expand, *exp_ptr;
-	int			exp_temp[CHAR_CLASSES];
-	int			leaf_num, curr_leaf_num, min_dfa, curr_min_dfa, sym_num;
-	int 			seqcnt, charpos, leafcnt, cursize;
-	int4			bitpos;
-	alternation		init_alt;
-	alternation		*cur_alt;
-	mstr			alttail;
-	int4			status;
-	int4			altactive = 0;
-	int4			altend;
-	char			*saveinstr;
-	int			chidx;
-	int			bit;
-	int			seq;
-	int			altmin, altmax;
-	int			saw_delimiter = 0;
-	int4			altlen, bytelen;
-	int4			allmask;
-	boolean_t		last_infinite;
-	boolean_t		done_free;
-	unsigned char		*let_go;
-	uint4			*fstchar, *lastpatptr, *outchar, *topchar;
-	int			any_alt = FALSE;
-	int			altcount, altsimplify;
-	int			low_in, high_in, size_in, jump;
-	boolean_t		topseen = FALSE;/* If TRUE it means we found inchar to be == in_top and so did NOT scan the NEXT
-						 * byte in inchar (to be stored in curchar). Therefore from this point onwards,
-						 * "curchar" should never be used in this function. This is also asserted below.
-						 */
+	alternation	*cur_alt, init_alt;
+	boolean_t	dfa, dfa_fixed_len, done, done_free, fixed_len, infinite, last_infinite,
+			prev_fixed_len, split_atom, start_dfa;
+	boolean_t	topseen = FALSE;/* If TRUE it means we found inchar to be == in_top and so did NOT scan the NEXT
+					 * byte in inchar (to be stored in curchar). Therefore from this point onwards,
+					 * "curchar" should never be used in this function. This is also asserted below.
+					 */
+	char		*saveinstr;
+	gtm_int64_t	bound;
+	int		atom_map, altmax, any_alt, altcount, altmin, altsimplify, bit,
+			charpos, chidx, count, curr_leaf_num, curr_min_dfa, cursize,
+			exp_temp[CHAR_CLASSES], high_in, jump, leafcnt, leaf_num, low_in,
+			max[MAX_PATTERN_ATOMS], min[MAX_PATTERN_ATOMS], min_dfa,
+			saw_delimiter, seq, seqcnt, size[MAX_PATTERN_ATOMS], size_in, sym_num, total_max, total_min;
+	int4		allmask, alloclen, altactive, altend, altlen, bitpos, bytelen, lower_bound, status, upper_bound;
+	mstr		alttail;
+	pat_strlit	strlit;
+	static int4	recurse_cnt;
+	struct e_table	expand, *exp_ptr;
+	struct leaf	leaves, *lv_ptr;
+	uint4		*fstchar, last_leaf_mask, *lastpatptr, mbit, *outchar, *patmaskptr, pattern_mask, *topchar, y_max;
+	unsigned char	*buffptr, curchar, *inchar, *in_top, *let_go, symbol;
 
 	if (0 == instr->len)		/* empty pattern string. Cant do much */
 	{
-		instr->addr++;	/* Return 1 byte more for compile_pattern to properly compute erroring M source column */
+		instr->addr++;		/* Return 1 byte more for compile_pattern to properly compute erroring M source column */
 		return ERR_PATCODE;
 	}
+	if (NULL == relay)	/* while there might be one or more heurstics producing a correct result from weird patterns */
+		recurse_cnt = MAX_PATTERN_ATOMS;	/* stop excessive recursion; works for any reasonable case; see GTM-9476*/
+	else			/* this game limits excess (see GTM-9469) recursing in a relatively cheap and effective way */
+	{
+		if (!recurse_cnt--)
+			return ERR_PATMAXLEN;	/* source column must be beyond report bounds, so no worries about its accuracy */
+	}
+	any_alt = altactive = saw_delimiter = 0;
 	memset(&leaves, 0, SIZEOF(leaves));
 	memset(&expand, 0, SIZEOF(expand));
 	init_alt.next = NULL;
