@@ -1,6 +1,7 @@
 /****************************************************************
  *								*
- *	Copyright 2001, 2009 Fidelity Information Services, Inc	*
+ * Copyright (c) 2001-2018 Fidelity National Information	*
+ * Services, Inc. and/or its subsidiaries. All rights reserved.	*
  *								*
  *	This source code contains the intellectual property	*
  *	of its copyright holder(s), and is made available	*
@@ -9,8 +10,8 @@
  *								*
  ****************************************************************/
 
-#ifndef __PRINT_EXIT_STATS_H__
-#define __PRINT_EXIT_STATS_H__
+#ifndef PRINT_EXIT_STATS_H_INCLUDED
+#define PRINT_EXIT_STATS_H_INCLUDED
 
 void print_exit_stats(void);
 #endif
