@@ -1,6 +1,6 @@
 /****************************************************************
  *								*
- * Copyright (c) 2001-2020 Fidelity National Information	*
+ * Copyright (c) 2001-2022 Fidelity National Information	*
  * Services, Inc. and/or its subsidiaries. All rights reserved.	*
  *								*
  *	This source code contains the intellectual property	*
@@ -133,7 +133,9 @@ freeze_status	region_freeze_main(gd_region *region, boolean_t freeze, boolean_t
 				 * We need to grab crit in a specific order to avoid deadlocks.
 				 */
 				while (pfrms->region_index > pfrms->pfms->grab_crit_counter)
-				{	/* Keep flushing while waiting to grab crit in a specific order. */
+				{	/* Keep pre-flushing while waiting to grab crit in a specific order, to make
+					 * parallel FREEZE -ONLINE less disruptive.
+					 */
 					JNL_ENSURE_OPEN_WCS_WTSTART(csa, region, csd->n_bts, NULL, FALSE, dummy_errno);
 				}
 			}
@@ -147,8 +149,8 @@ freeze_status	region_freeze_main(gd_region *region, boolean_t freeze, boolean_t
 			{
 				if (pfrms)
 				{
-					INCR_CNT(&pfrms->pfms->grab_crit_counter, &pfrms->pfms->region_frozen_latch);
-					INCR_CNT(&pfrms->pfms->region_frozen_counter, &pfrms->pfms->region_frozen_latch);
+					INCR_CNT(&pfrms->pfms->grab_crit_counter, &pfrms->pfms->grab_crit_latch);
+					INCR_REG_FROZEN_COUNT(pfrms->pfms, FALSE);
 				}
 				rel_crit(region);
 			}
@@ -161,8 +163,8 @@ freeze_status	region_freeze_main(gd_region *region, boolean_t freeze, boolean_t
 			{
 				if (pfrms)
 				{
-					INCR_CNT(&pfrms->pfms->grab_crit_counter, &pfrms->pfms->region_frozen_latch);
-					INCR_CNT(&pfrms->pfms->region_frozen_counter, &pfrms->pfms->region_frozen_latch);
+					INCR_CNT(&pfrms->pfms->grab_crit_counter, &pfrms->pfms->grab_crit_latch);
+					INCR_REG_FROZEN_COUNT(pfrms->pfms, FALSE);
 				}
 				rel_crit(region);
 			}
@@ -200,7 +202,7 @@ freeze_status	region_freeze_main(gd_region *region, boolean_t freeze, boolean_t
 		}
 		if (pfrms)
 		{	/* Increment counter and grab the next crit */
-			INCR_CNT(&pfrms->pfms->grab_crit_counter, &pfrms->pfms->region_frozen_latch);
+			INCR_CNT(&pfrms->pfms->grab_crit_counter, &pfrms->pfms->grab_crit_latch);
 		}
 		/* if can't ever be true when override is true. */
 		if (MAX_CRIT_TRY <= sleep_counter)
@@ -209,7 +211,7 @@ freeze_status	region_freeze_main(gd_region *region, boolean_t freeze, boolean_t
 			if (!was_crit)
 			{
 				if (pfrms)
-					INCR_CNT(&pfrms->pfms->region_frozen_counter, &pfrms->pfms->region_frozen_latch);
+					INCR_REG_FROZEN_COUNT(pfrms->pfms, FALSE);
 				rel_crit(region);
 			}
 			return REG_HAS_KIP;
@@ -263,7 +265,7 @@ freeze_status	region_freeze_main(gd_region *region, boolean_t freeze, boolean_t
 			if (!was_crit)
 			{
 				if (pfrms)
-					INCR_CNT(&pfrms->pfms->region_frozen_counter, &pfrms->pfms->region_frozen_latch);
+					INCR_REG_FROZEN_COUNT(pfrms->pfms, FALSE);
 				rel_crit(region);
 			}
 			rel_latch(&cnl->freeze_latch);
@@ -287,10 +289,12 @@ freeze_status	region_freeze_main(gd_region *region, boolean_t freeze, boolean_t
 		if (!was_crit)
 		{
 			if (pfrms)
-			{
-				INCR_CNT(&pfrms->pfms->region_frozen_counter, &pfrms->pfms->region_frozen_latch);
-				while (pfrms->pfms->region_frozen_counter < pfrms->pfms->ntasks)
-					SLEEP_USEC(10, FALSE);
+			{	/* Wait for all regions to grab crit before releasing it on ours.
+				 * This ensures that we had a point where all regions were stopped before restarting with ONLINE.
+				 */
+				INCR_REG_FROZEN_COUNT(pfrms->pfms, TRUE);
+				if (csd->asyncio)
+					wcs_wtfini(region, CHECK_IS_PROC_ALIVE_FALSE, NULL);
 			}
 			rel_crit(region);
 		}
@@ -357,6 +361,8 @@ freeze_status region_freeze_post(gd_region *region)
 	int			dummy_errno;
 	boolean_t		was_crit;
 	freeze_status		rval;
+	int			slp_cnt;
+	boolean_t		is_logged = FALSE;
 
 	csa = &FILE_INFO(region)->s_addrs;
 	csd = csa->hdr;
@@ -391,8 +397,30 @@ freeze_status region_freeze_post(gd_region *region)
 		}
 		csa->needs_post_freeze_online_clean = FALSE;
 	}
+	slp_cnt = 0;
 	if (csa->needs_post_freeze_flushsync)
 	{
+		/* we can't wcs_flu() until donotflush_dbjnl != TRUE because another process
+		 * like online rollback might be fooling with things, previously this
+		 * was not checked, and wcs_flu() would assert out on donotflush_dbjnl.
+		 * The assert below warns us in dbg something is iffy, if we are in pro
+		 * and we still don't see FALSE on donotflush_dbjnl,
+		 * we call wcs_flu() regardless, which the previous code did too.*/
+		while (TRUE == csa->nl->donotflush_dbjnl)
+		{
+			assert(1000 > slp_cnt);
+			if (1000 <= slp_cnt)
+				break;
+			if (!(slp_cnt % 100))
+				is_logged = FALSE;
+			if (!is_logged)
+			{
+				send_msg_csa(CSA_ARG(csa) VARLSTCNT(4) ERR_TEXT, 2,
+					LEN_AND_LIT("Freeze spinning for clean DB header"));
+				is_logged = TRUE;
+			}
+			wcs_sleep(++slp_cnt); /* wcs_sleep maxes out at 10ms so this loop should run in ~ 10sec worst case */
+		}
 		if (!wcs_flu(WCSFLU_FLUSH_HDR | WCSFLU_WRITE_EPOCH | WCSFLU_SYNC_EPOCH))
 			rval = REG_FLUSH_ERROR;
 		csa->needs_post_freeze_flushsync = FALSE;
