@@ -1,6 +1,6 @@
 /****************************************************************
  *								*
- * Copyright (c) 2001-2020 Fidelity National Information	*
+ * Copyright (c) 2001-2022 Fidelity National Information	*
  * Services, Inc. and/or its subsidiaries. All rights reserved.	*
  *								*
  *	This source code contains the intellectual property	*
@@ -91,6 +91,7 @@
 GBLDEF	int			repl_max_send_buffsize, repl_max_recv_buffsize;
 GBLREF  boolean_t               is_rcvr_server;
 GBLREF  boolean_t               is_src_server;
+GBLREF  void 			(*primary_exit_handler)(void);
 #ifdef GTM_TLS
 GBLREF	gtm_tls_ctx_t		*tls_ctx;
 GBLREF	char			dl_err[MAX_ERRSTR_LEN];
@@ -124,6 +125,13 @@ error_def(ERR_TLSINIT);
 error_def(ERR_TLSIOERROR);
 error_def(ERR_TLSCONNINFO);
 
+/* Use the OS appropriate errno when inducing WBTEST_REPLCOMM_SEND_SRC. Linux has ECOMM but AIX does not */
+#ifdef ECOMM
+#	define	WBTEST_REPLCOMM_SEND_SRC_ERRNO	ECOMM
+#else
+#	define	WBTEST_REPLCOMM_SEND_SRC_ERRNO	ENETUNREACH
+#endif
+
 #define REPL_TRACE_BUFF(TRACE_BUFF, TRACE_BUFF_POS, IO_BUFF, IO_SIZE, MAX_TRACE_SIZE)			\
 {													\
 	if (IO_SIZE > MAX_TRACE_SIZE)									\
@@ -145,7 +153,7 @@ error_def(ERR_TLSCONNINFO);
 
 int fd_ioready(int sock_fd, int poll_direction, int timeout)
 {
-	int		save_errno, status, EAGAIN_cnt = 0;
+	int		save_errno, status, EAGAIN_cnt = 0, ENOMEM_cnt = 0, REPL_MAXPOLLFAIL_cnt = 0;
 #	ifdef USE_POLL
 	struct pollfd	fds;
 #	else
@@ -174,9 +182,8 @@ int fd_ioready(int sock_fd, int poll_direction, int timeout)
 	{
 		save_errno = ERRNO;
 		if (EINTR == save_errno)
-		{	/* Give it another shot. But, halve the timeout so we don't keep doing this forever. */
-			timeout = timeout >> 1;
-		} else if (EAGAIN == save_errno)
+			timeout = timeout >> 1; /* Give it another shot but reduce the timeout by half */
+		else if (EAGAIN == save_errno)
 		{	/* Resource starved system; relinquish the processor in the hope that we may get the required resources
 			 * next time around.
 			 */
@@ -186,14 +193,23 @@ int fd_ioready(int sock_fd, int poll_direction, int timeout)
 						"starved. EAGAIN returned from select()/poll() %d times\n", EAGAIN_cnt);
 			}
 			rel_quant();	/* this seems legit */
-		} else
+		} else if (ENOMEM == save_errno)
+		{
+			if (0 == ++ENOMEM_cnt % REPL_COMM_LOG_ENOMEM_INTERVAL)
+			{
+				repl_log(stderr, TRUE, TRUE, "Communication subsytem warning: No memory available for "
+						"polling. ENOMEM returned from select()/poll() %d times\n", ENOMEM_cnt);
+			}
 			return -1;
-		/* Just in case select() modifies the incoming arguments, restore fd_set and timeout_spec */
+		}
+		/* Restore fd_set. Also, on Linux, select() modifies timeout, so restore timeout to the GT.M controlled timeout */
 		SELECT_ONLY(
 			assert(0 == timeout_spec.tv_sec);
-			timeout_spec.tv_usec = timeout;	/* Note: timeout is the reduced value (in case of EINTR) */
+			timeout_spec.tv_usec = timeout;
 			FD_SET(sock_fd, &fds);
 		)
+		if (REPL_MAXPOLLFAIL < ++REPL_MAXPOLLFAIL_cnt)
+			return -1;
 	}
 	return status;
 }
@@ -271,9 +287,10 @@ int repl_send(int sock_fd, unsigned char *buff, int *send_len, int timeout GTMTL
 			if (-1 == save_errno)
 			{	/* This indicates an error from TLS/SSL layer and not from a system call. */
 				assert(repl_tls.enabled);
-				errptr = gtm_tls_get_error();
-				rts_error_csa(CSA_ARG(NULL) VARLSTCNT(8) ERR_TLSIOERROR, 2, LEN_AND_LIT("send"), ERR_TEXT, 2,
-						LEN_AND_STR(errptr));
+				errptr = gtm_tls_get_error(repl_tls.sock);
+				gtm_putmsg_csa(CSA_ARG(NULL) VARLSTCNT(8) MAKE_MSG_WARNING(ERR_TLSIOERROR), 2, LEN_AND_LIT("send"),
+						ERR_TEXT, 2, LEN_AND_STR(errptr));
+				return save_errno;
 			}
 #			else
 			save_errno = ERRNO;
@@ -281,7 +298,7 @@ int repl_send(int sock_fd, unsigned char *buff, int *send_len, int timeout GTMTL
 			if (WBTEST_ENABLED(WBTEST_REPLCOMM_SEND_SRC) && is_src_server)
 			{
 				repl_log(stderr, TRUE, TRUE,"Changing save errno\n");
-				save_errno = 70;
+				save_errno = WBTEST_REPLCOMM_SEND_SRC_ERRNO; /* ECOMM on Linux and ENETUNREACH on AIX */
 			}
 			assert((EMSGSIZE != save_errno) && (EWOULDBLOCK != save_errno));
 			if (EINTR == save_errno)
@@ -407,9 +424,10 @@ int repl_recv(int sock_fd, unsigned char *buff, int *recv_len, int timeout GTMTL
 			if (-1 == save_errno)
 			{
 				assert(repl_tls.enabled);
-				errptr = gtm_tls_get_error();
-				rts_error_csa(CSA_ARG(NULL) VARLSTCNT(8) ERR_TLSIOERROR, 2, LEN_AND_LIT("recv"), ERR_TEXT, 2,
-						LEN_AND_STR(errptr));
+				errptr = gtm_tls_get_error(repl_tls.sock);
+				gtm_putmsg_csa(CSA_ARG(NULL) VARLSTCNT(8) MAKE_MSG_WARNING(ERR_TLSIOERROR), 2, LEN_AND_LIT("recv"),
+						ERR_TEXT, 2, LEN_AND_STR(errptr));
+				return save_errno;
 			}
 #			else
 			save_errno = ERRNO;
@@ -440,7 +458,7 @@ int repl_recv(int sock_fd, unsigned char *buff, int *recv_len, int timeout GTMTL
                  (WBTEST_ENABLED(WBTEST_REPLCOMM_ERR_SRC) && is_src_server))
 		{
 			repl_log(stderr, TRUE, TRUE, "Changing save_errno\n");
-			save_errno = 113;
+			save_errno = EHOSTUNREACH;
 		}
 		if (((WBTEST_ENABLED(WBTEST_FETCHCOMM_ERR)) || (WBTEST_ENABLED(WBTEST_FETCHCOMM_HISTINFO)))
 								&& !(is_rcvr_server || is_src_server))
@@ -591,7 +609,8 @@ void repl_log_tls_info(FILE *logfp, gtm_tls_socket_t *socket)
 
 	if (0 != gtm_tls_get_conn_info(repl_tls.sock, &conn_info))
 	{
-		gtm_putmsg_csa(CSA_ARG(NULL) VARLSTCNT(6) ERR_TLSCONNINFO, 0, ERR_TEXT, 2, LEN_AND_STR(gtm_tls_get_error()));
+		gtm_putmsg_csa(CSA_ARG(NULL) VARLSTCNT(6) ERR_TLSCONNINFO, 0,
+				ERR_TEXT, 2, LEN_AND_STR(gtm_tls_get_error(repl_tls.sock)));
 		return;
 	}
 	repl_log(logfp, FALSE, TRUE, "TLS/SSL Session:\n");
@@ -630,7 +649,7 @@ void repl_do_tls_init(FILE *logfp)
 	if (SS_NORMAL != (status = gtm_tls_loadlibrary()))
 	{
 		if (!PLAINTEXT_FALLBACK)
-			rts_error_csa(CSA_ARG(NULL) VARLSTCNT(6) ERR_TLSDLLNOOPEN, 0, ERR_TEXT, 2, LEN_AND_STR(dl_err));
+			RTS_ERROR_CSA_ABT(NULL, VARLSTCNT(6) ERR_TLSDLLNOOPEN, 0, ERR_TEXT, 2, LEN_AND_STR(dl_err));
 		else
 			gtm_putmsg_csa(CSA_ARG(NULL) VARLSTCNT(6) MAKE_MSG_WARNING(ERR_TLSDLLNOOPEN), 0, ERR_TEXT, 2,
 					LEN_AND_STR(dl_err));
@@ -638,11 +657,14 @@ void repl_do_tls_init(FILE *logfp)
 	} else if (NULL == (tls_ctx = gtm_tls_init(GTM_TLS_API_VERSION, 0)))
 	{
 		if (!PLAINTEXT_FALLBACK)
-			rts_error_csa(CSA_ARG(NULL) VARLSTCNT(6) ERR_TLSINIT, 0, ERR_TEXT, 2, LEN_AND_STR(gtm_tls_get_error()));
+			RTS_ERROR_CSA_ABT(NULL, VARLSTCNT(6) ERR_TLSINIT, 0, ERR_TEXT, 2, LEN_AND_STR(gtm_tls_get_error(NULL)));
 		else
 			gtm_putmsg_csa(CSA_ARG(NULL) VARLSTCNT(6) MAKE_MSG_WARNING(ERR_TLSINIT), 0, ERR_TEXT, 2,
-					LEN_AND_STR(gtm_tls_get_error()));
+					LEN_AND_STR(gtm_tls_get_error(NULL)));
 		issue_fallback_warning = TRUE;
+	} else if (primary_exit_handler)
+	{
+		atexit(primary_exit_handler);
 	}
 	if (issue_fallback_warning)
 	{
