@@ -1,6 +1,7 @@
 /****************************************************************
  *								*
- *	Copyright 2001, 2013 Fidelity Information Services, Inc	*
+ * Copyright (c) 2001-2022 Fidelity National Information	*
+ * Services, Inc. and/or its subsidiaries. All rights reserved.	*
  *								*
  *	This source code contains the intellectual property	*
  *	of its copyright holder(s), and is made available	*
@@ -13,6 +14,10 @@
 
 #include "arit.h"
 #include "stringpool.h"
+#include "toktyp.h"
+#include "cgp.h"
+
+GBLREF char		cg_phase;	/* code generation phase */
 
 #define DIGIT(x)	((x >='0') && (x <= '9'))
 #define NUM_MASK	(MV_NM | MV_INT | MV_NUM_APPROX)
@@ -115,7 +120,7 @@ char *s2n (mval *u)
 	}
 	digit = (0 != z) || (0 != y) || (0 != zero);
 	x = 0;
-	exp = ('E' == *c) && digit;
+	exp = (c < eos) && ('E' == *c) && digit;
 	if (exp && ((c + 1) < eos))
 	{
 		w = c;	/* save pointer to return in case expression following E is not a valid exponent */
@@ -149,7 +154,7 @@ char *s2n (mval *u)
 		i += j;
 		if ((0 == u->m[0]) && (6 >= x) && (0 <= i))
 		{
-			u->mvtype |= (tail || (1 < sign) || ((0 != zero) && (1 != u->str.len)))
+			u->mvtype |= (tail || (1 < sign) || ((0 != zero) && (1 != u->str.len)))			/* approx or not? */
 				? (MV_NM | MV_INT | MV_NUM_APPROX) : (MV_NM | MV_INT);
 			if (0 > j)
 				u->m[1] /= ((sign & 1) ? -ten_pwr[-j] : ten_pwr[-j]);
@@ -167,13 +172,14 @@ char *s2n (mval *u)
 			{
 				u->mvtype &= ~NUM_MASK;
 				if (!TREF(compile_time))
-					rts_error_csa(CSA_ARG(NULL) VARLSTCNT(1) ERR_NUMOFLOW);
+					RTS_ERROR_CSA_ABT(NULL, VARLSTCNT(1) ERR_NUMOFLOW);
+				u->mvtype |= MV_NUM_APPROX; /* breadcrumb for experitem to help f_[z]char() with NUMOFLOW error */
 			} else
 			{
 				u->e = x;
 				u->sgn = sign & 1;
-				u->mvtype |= (tail || (1 < sign) || ((0 != zero) && (1 != u->str.len)))
-					? (MV_NM | MV_NUM_APPROX) : MV_NM;
+				u->mvtype |= (tail || (1 < sign) || ((0 != zero) && (1 != u->str.len)))		/* approx or not? */
+					  ? (MV_NM | MV_NUM_APPROX) : MV_NM;		/* similar to above but not MV_INT */
 			}
 		}
 		assert(MANT_HI > u->m[1]);
