@@ -1,6 +1,6 @@
 /****************************************************************
  *								*
- * Copyright (c) 2001-2019 Fidelity National Information	*
+ * Copyright (c) 2001-2022 Fidelity National Information	*
  * Services, Inc. and/or its subsidiaries. All rights reserved.	*
  *								*
  *	This source code contains the intellectual property	*
@@ -16,8 +16,9 @@
 #include "stp_parms.h"
 #include "gtm_string.h"
 
-GBLREF mstr **stp_array;
-GBLREF uint4 stp_array_size;
+GBLREF boolean_t	retry_if_expansion_fails;
+GBLREF mstr		**stp_array;
+GBLREF uint4		stp_array_size;
 
 void stp_expand_array(void)
 {
@@ -28,6 +29,7 @@ void stp_expand_array(void)
 	stp_array_size += STP_MAXITEMS;
 	assert(stp_array_size > n);
 	a = stp_array;
+	retry_if_expansion_fails = FALSE;	/* this memory request comes in the middle of stp_gcol, so is doomed if it fails */
 	stp_array = (mstr **) malloc(stp_array_size * SIZEOF(mstr *));
 	memcpy((uchar_ptr_t)stp_array, (uchar_ptr_t)a, n * SIZEOF(mstr *));
 	free(a);
