@@ -1,6 +1,6 @@
 /****************************************************************
  *								*
- * Copyright (c) 2001-2020 Fidelity National Information	*
+ * Copyright (c) 2001-2022 Fidelity National Information	*
  * Services, Inc. and/or its subsidiaries. All rights reserved.	*
  *								*
  *	This source code contains the intellectual property	*
@@ -87,7 +87,7 @@ GBLREF lvzwrite_datablk		*lvzwrite_block;
 GBLREF mliteral			literal_chain;
 GBLREF mstr			*comline_base, **stp_array;
 GBLREF mval			dollar_system, dollar_zerror, dollar_zgbldir, dollar_zstatus;
-GBLREF mval			dollar_zyerror, zstep_action, dollar_zinterrupt, dollar_zsource, dollar_ztexit;
+GBLREF mval			dollar_zyerror, dollar_zinterrupt, dollar_zsource, dollar_ztexit;
 GBLREF mv_stent			*mv_chain;
 GBLREF sgm_info			*first_sgm_info;
 GBLREF spdesc			indr_stringpool, rts_stringpool, stringpool;
@@ -104,6 +104,7 @@ GBLREF hash_table_str		*complits_hashtab;
 GBLREF mval			*alias_retarg;
 GBLREF io_pair			*io_std_device;
 GTMTRIG_ONLY(GBLREF mval 	dollar_ztwormhole;)
+GTMTRIG_ONLY(GBLREF mval 	dollar_ztslate;)
 DEBUG_ONLY(GBLREF   boolean_t	ok_to_UNWIND_in_exit_handling;)
 
 
@@ -368,7 +369,7 @@ static void expand_stp(size_t new_size)	/* BYPASSOK */
 		ESTABLISH(stp_gcol_ch);
 	assert(IS_GTM_IMAGE || IS_MUPIP_IMAGE);
 	assert(!stringpool_unexpandable);
-	DBGSTPGCOL((stderr, "expand_stp(new_size=%u)\n", new_size));
+	DBGSTPGCOL((stderr, "expand_stp(new_size=%llu)\n", new_size));
 	stp_init(new_size);
 	if (retry_if_expansion_fails)
 		REVERT;
@@ -709,12 +710,13 @@ void stp_gcol(size_t space_asked)	/* BYPASSOK */
 		MVAL_STPG_ADD(&dollar_zgbldir);
 		MVAL_STPG_ADD(&dollar_zinterrupt);
 		MVAL_STPG_ADD(&(TREF(dollar_zstep)));
-		MVAL_STPG_ADD(&zstep_action);
+		MVAL_STPG_ADD(&(TREF(zstep_action)));
 		MVAL_STPG_ADD(&dollar_zerror);
 		MVAL_STPG_ADD(&dollar_ztexit);
 		MVAL_STPG_ADD(&dollar_zyerror);
 #		ifdef GTM_TRIGGER
 		MVAL_STPG_ADD(&dollar_ztwormhole);
+		MVAL_STPG_ADD(&dollar_ztslate);
 #		endif
 		MVAL_STPG_ADD(&(TREF(last_fnquery_return_varname)));
 		for (index = 0; index < TREF(last_fnquery_return_subcnt); index++)
@@ -933,7 +935,7 @@ void stp_gcol(size_t space_asked)	/* BYPASSOK */
 #	endif
 	space_reclaim = space_after_compact - space_before_compact; /* this can be -ve, if alignment causes expansion */
 	space_needed -= (ssize_t)space_after_compact;
-	DBGSTPGCOL((stderr, "space_needed=%i\n", space_needed));
+	DBGSTPGCOL((stderr, "space_needed=%li\n", space_needed));
 	/* After compaction if less than 31.25% of space is avail, consider it a low reclaim pass */
 	if (STP_LOWRECLAIM_LEVEL(stringpool.top - stringpool.base) > space_after_compact) /* BYPASSOK */
 		(*low_reclaim_passes)++;
@@ -981,13 +983,17 @@ void stp_gcol(size_t space_asked)	/* BYPASSOK */
 				break;			/* stp_incr can't get smaller - give up and use what we have */
 			expansion_failed = FALSE;	/* will be set to TRUE by condition handler if can't get memory */
 			assert((stp_incr + stringpool.top - stringpool.base) >= (space_needed + blklen));
-			DBGSTPGCOL((stderr, "incr_factor=%i stp_incr=%i space_needed=%i\n", *incr_factor, stp_incr, space_needed));
+			DBGSTPGCOL((stderr, "incr_factor=%i stp_incr=%i space_needed=%li\n", *incr_factor, stp_incr, space_needed));
 			if ((stringpool.strpllimwarned) /* previously warned */
 				&& ((stp_incr + stringpool.top - stringpool.base) > stringpool.strpllim)) /* expanding larger */
 			{
 				assert(0 < stringpool.strpllim);	/* must have been watching stp limit */
 				rts_error_csa(CSA_ARG(NULL) VARLSTCNT(1) ERR_STPOFLOW);
 			}
+			DBGSTPGCOL((stderr, "base=%p top=%p (size=%llu)\n", stringpool.base, stringpool.top,
+						(size_t)(stringpool.top - stringpool.base)));
+			assert((ssize_t)(stringpool.top - stringpool.base)
+					< ((ssize_t)(stp_incr + stringpool.top - stringpool.base)));
 			expand_stp((ssize_t)(stp_incr + stringpool.top - stringpool.base));
 #			ifdef DEBUG
 			/* If expansion failed and stp_gcol_ch did an UNWIND and we were already in exit handling code,
@@ -1046,6 +1052,7 @@ void stp_gcol(size_t space_asked)	/* BYPASSOK */
 		/* Adjust incr_factor */
 		if (*incr_factor > 1)
 			*incr_factor = *incr_factor - 1;
+		DBGSTPGCOL((stderr, "incr_factor=%i space_needed=%li\n", *incr_factor, space_needed));
 		if (topstr != array)
 		{
 #			ifdef STP_MOVE
