@@ -1,6 +1,6 @@
 /****************************************************************
  *								*
- * Copyright (c) 2001-2019 Fidelity National Information	*
+ * Copyright (c) 2001-2021 Fidelity National Information	*
  * Services, Inc. and/or its subsidiaries. All rights reserved.	*
  *								*
  *	This source code contains the intellectual property	*
@@ -31,22 +31,19 @@
 #include "tp_frame.h"
 #endif
 
-GBLREF	short			crash_count;
-GBLREF	trans_num		start_tn;
+GBLREF	boolean_t		mu_reorg_process, mu_reorg_upgrd_dwngrd_in_prog, need_kip_incr;
 GBLREF	cw_set_element		cw_set[];
-GBLREF	unsigned char		cw_set_depth, cw_map_depth;
-GBLREF	unsigned int		t_tries;
-GBLREF	uint4			t_err;
-GBLREF	uint4			update_trans;
 GBLREF	gv_namehead		*gv_target;
-GBLREF	sgmnt_addrs		*cs_addrs;
-GBLREF	uint4			dollar_tlevel;
 GBLREF	jnl_format_buffer	*non_tp_jfb_ptr;
 GBLREF	jnl_gbls_t		jgbl;
-GBLREF	volatile int4		fast_lock_count;
 GBLREF	sgm_info		*first_sgm_info;
-GBLREF	boolean_t		need_kip_incr;
-GBLREF	boolean_t		mu_reorg_process;
+GBLREF	sgmnt_addrs		*cs_addrs;
+GBLREF	short			crash_count;
+GBLREF	trans_num		start_tn;
+GBLREF	uint4			dollar_tlevel, t_err, update_trans;
+GBLREF	unsigned char		cw_set_depth, cw_map_depth;
+GBLREF	unsigned int		t_tries;
+GBLREF	volatile int4		fast_lock_count;
 #ifdef DEBUG
 GBLREF	sgmnt_addrs		*reorg_encrypt_restart_csa;
 GBLREF	uint4			bml_save_dollar_tlevel;
@@ -63,7 +60,7 @@ void t_begin(uint4 err, uint4 upd_trans) 	/* err --> error code for current gvcs
 
 	SETUP_THREADGBL_ACCESS;
 	assert(!dollar_tlevel); /* if in TP, the T_BEGIN_xxx_NONTP_OR_TP macro should have been used and we will not be here */
-	assert(INTRPT_OK_TO_INTERRUPT == intrpt_ok_state);
+	assert((INTRPT_OK_TO_INTERRUPT == intrpt_ok_state) || (mu_reorg_process && (INTRPT_IN_KILL_CLEANUP == intrpt_ok_state)));
 	assert(NULL == reorg_encrypt_restart_csa);
 	/* The wcs_wtstart and dsk_read functions rely on the update_trans global to detect whether they are inside a read-write
 	 * non-TP transaction, in which case they may trigger a restart if encryption settings have been concurrently modified by
@@ -82,8 +79,8 @@ void t_begin(uint4 err, uint4 upd_trans) 	/* err --> error code for current gvcs
 	csa = cs_addrs;
 	if ((NULL == csa->db_addrs[0]) && (dba_mm == csa->hdr->acc_meth))
 	{
-		rts_error_csa(CSA_ARG(csa) VARLSTCNT(6) ERR_MMREGNOACCESS, 4, REG_LEN_STR(csa->region),
-					DB_LEN_STR(csa->region));
+		RTS_ERROR_CSA_ABT(csa, VARLSTCNT(6) ERR_MMREGNOACCESS, 4, REG_LEN_STR(csa->region),
+			DB_LEN_STR(csa->region));
 	}
 	/* If we use a clue then we must consider the oldest tn in the search history to be the start tn for this transaction */
         /* start_tn manipulation for TP taken care of in tp_hist */
@@ -109,7 +106,7 @@ void t_begin(uint4 err, uint4 upd_trans) 	/* err --> error code for current gvcs
 			 *	   so gv_target->clue and/or gv_target->hist.h[x].cse does not matter. The global variable
 			 *	   bml_save_dollar_tlevel exactly identifies this scenario.
 			 */
-			assert(mu_reorg_process || (NULL == s->cse) || bml_save_dollar_tlevel);
+			assert(mu_reorg_process || mu_reorg_upgrd_dwngrd_in_prog || (NULL == s->cse) || bml_save_dollar_tlevel);
 			if (start_tn > histtn)
 				start_tn = histtn;
 		}
