@@ -1,6 +1,6 @@
 /****************************************************************
  *								*
- * Copyright (c) 2001-2019 Fidelity National Information	*
+ * Copyright (c) 2001-2020 Fidelity National Information	*
  * Services, Inc. and/or its subsidiaries. All rights reserved.	*
  *								*
  *	This source code contains the intellectual property	*
@@ -26,7 +26,6 @@
 #include "interlock.h"
 #include "jnl.h"
 #include "buddy_list.h"		/* needed for tp.h */
-#include "hashtab_int4.h"	/* needed for tp.h */
 #include "tp.h"
 #include "repl_msg.h"		/* for gtmsource.h */
 #include "gtmsource.h"		/* for jnlpool_addrs structure definition */
@@ -211,7 +210,7 @@ boolean_t t_commit_cleanup(enum cdb_sc status, int signal)
 			 */
 			/* Make sure we have crit when updating early_tn */
 			if (FALSE == ((was_crit = cs_addrs->now_crit)))
-				grab_crit(gv_cur_region);
+				grab_crit(gv_cur_region, WS_44);
 			cs_addrs->ti->early_tn--;
 			if (!was_crit)
 				rel_crit(gv_cur_region);
@@ -331,7 +330,7 @@ boolean_t t_commit_cleanup(enum cdb_sc status, int signal)
 		 * we need to revisit the commit logic and fix the error as we are past the point where we no longer can roll-back
 		 * the transaction.
 		 */
-		assert(0 == signal);
+		assert(signal || gtm_white_box_test_case_enabled);
 		send_msg_csa(CSA_ARG(NULL) VARLSTCNT(8) ERR_DBCOMMITCLNUP, 6, process_id, process_id, signal, trstr,
 				DB_LEN_STR(xactn_err_region));
 		/* if t_ch() (a condition handler) was driving this routine, then doing send_msg() here is not a good idea
