@@ -1,6 +1,6 @@
 /****************************************************************
  *								*
- * Copyright (c) 2001-2020 Fidelity National Information	*
+ * Copyright (c) 2001-2022 Fidelity National Information	*
  * Services, Inc. and/or its subsidiaries. All rights reserved.	*
  *								*
  *	This source code contains the intellectual property	*
@@ -47,7 +47,6 @@
 #include "min_max.h"
 #include "gtmimagename.h"
 #include "anticipatory_freeze.h"
-
 #include "gtmrecv.h"
 #include "deferred_signal_handler.h"
 #include "repl_instance.h"
@@ -85,6 +84,7 @@
 #include "deferred_events.h"
 #include "error_trap.h"
 #include "ztimeout_routines.h"
+#include "jnl_file_close_timer.h"
 
 GBLREF	bool			rc_locked;
 GBLREF	unsigned char		t_fail_hist[CDB_MAX_TRIES];
@@ -249,8 +249,6 @@ trans_num t_end(srch_hist *hist1, srch_hist *hist2, trans_num ctn)
 	th_index_ptr_t		cti;
 	jbuf_rsrv_struct_t	*jrs;
 	jrec_rsrv_elem_t	*first_jre, *jre, *jre_top;
-	int4			event_type, param_val;
-	void (*set_fn)(int4 param);
 	DCL_THREADGBL_ACCESS;
 
 	SETUP_THREADGBL_ACCESS;
@@ -291,7 +289,7 @@ trans_num t_end(srch_hist *hist1, srch_hist *hist2, trans_num ctn)
 	 * Since we don't expect hold_onto_crit to be set by any other utility/function, the below assert is valid and is intended
 	 * to catch cases where the field is inadvertently set to TRUE.
 	 */
-	assert(!csa->hold_onto_crit || IS_DSE_IMAGE
+	assert(!csa->hold_onto_crit || IS_DSE_IMAGE || mu_reorg_upgrd_dwngrd_in_prog
 		|| jgbl.onlnrlbk || TREF(in_gvcst_redo_root_search) || TREF(in_trigger_upgrade));
 	assert(cs_data == csd);
 	assert((t_tries < CDB_STAGNATE) || csa->now_crit);
@@ -527,7 +525,7 @@ trans_num t_end(srch_hist *hist1, srch_hist *hist2, trans_num ctn)
 								assert(!jnlpool_csa || (jnlpool_csa == csa));
 								jnlpool = save_jnlpool;
 							}
-							rts_error_csa(CSA_ARG(csa) VARLSTCNT(4) ERR_GBLOFLOW, 2,
+							RTS_ERROR_CSA_ABT(csa, VARLSTCNT(4) ERR_GBLOFLOW, 2,
 								DB_LEN_STR(csa->region));
 						}
 					}
@@ -578,7 +576,9 @@ trans_num t_end(srch_hist *hist1, srch_hist *hist2, trans_num ctn)
 				 */
 				assert((CDB_STAGNATE > t_tries) || (cs->blk < cti->total_blks));
 				cs->mode = gds_t_acquired;
-				assert(GDSVCURR == cs->ondsk_blkver);
+				assert((GDSVCURR == cs->ondsk_blkver) || (GDSV7m == cs->ondsk_blkver)
+					|| (BLK_ID_32_VER > cs->ondsk_blkver)
+					|| (mu_reorg_upgrd_dwngrd_in_prog && (GDSV6p == cs->ondsk_blkver)));
 			} else if (reorg_ss_in_prog && WAS_FREE(cs->blk_prior_state))
 			{
 				assert((gds_t_acquired == cs->mode) && (NULL == cs->old_block));
@@ -720,12 +720,12 @@ trans_num t_end(srch_hist *hist1, srch_hist *hist2, trans_num ctn)
 	assert((cdb_sc_normal == prev_status) || ((cdb_sc_onln_rlbk1 != prev_status) && (cdb_sc_onln_rlbk2 != prev_status))
 		|| (!TREF(in_gvcst_bmp_mark_free) || mu_reorg_process));
 	if (is_mm && ((csa->hdr != csd) || (csa->total_blks != cti->total_blks)))
-        {       /* If MM, check if wcs_mm_recover was invoked as part of the grab_crit done above OR if
-                 * the file has been extended. If so, restart.
-                 */
-                status = cdb_sc_helpedout;      /* force retry with special status so philanthropy isn't punished */
-                goto failed;
-        }
+	{	/* If MM, check if wcs_mm_recover was invoked as part of the grab_crit done above OR if
+		 * the file has been extended. If so, restart.
+		 */
+		status = cdb_sc_helpedout;	/* force retry with special status so philanthropy isn't punished */
+		goto failed;
+	}
 #	ifdef GTM_TRIGGER
 	if (!skip_dbtriggers)
 	{
@@ -1102,7 +1102,7 @@ trans_num t_end(srch_hist *hist1, srch_hist *hist2, trans_num ctn)
 			PIN_CACHE_RECORD(cr, cr_array, cr_array_index);
 		}
 	}
-	if ((0 != cw_map_depth) && (mu_reorg_upgrd_dwngrd_in_prog || mu_reorg_encrypt_in_prog))
+	if ((0 != cw_map_depth) && mu_reorg_encrypt_in_prog)
 	{	/* Bit maps from mu_reorg_upgrd_dwngrd. Bitmap history has been validated.
 		 * But we do not want bitmap cse to be considered for bg_update. Reset cw_set_depth accordingly.
 		 */
@@ -1786,7 +1786,6 @@ skip_cr_array:
 	assert(!csa->now_crit || csa->hold_onto_crit);
 	assert(cdb_sc_normal == status);
 	REVERT;	/* no need for t_ch to be invoked if any errors occur after this point */
-	CALL_ZTIMEOUT_IF_DEFERRED;
 	DEFERRED_EXIT_HANDLING_CHECK; /* now that all crits are released, check if deferred signal/exit handling needs to be done */
 	assert(update_trans);
 	if (REPL_ALLOWED(csa) && IS_DSE_IMAGE)
