@@ -1,6 +1,6 @@
 /****************************************************************
  *								*
- * Copyright (c) 2001-2020 Fidelity National Information	*
+ * Copyright (c) 2001-2022 Fidelity National Information	*
  * Services, Inc. and/or its subsidiaries. All rights reserved.	*
  *								*
  *	This source code contains the intellectual property	*
@@ -65,49 +65,38 @@
 #include "gtmci.h"
 #include "gvt_inline.h"
 
+GBLREF	boolean_t		caller_id_flag, is_updproc, mupip_jnl_recover;
+GBLREF	int			dollar_truth, mumps_status;
 GBLDEF	int4			n_pvtmods, n_blkmods;
-
-GBLREF	uint4			dollar_tlevel;
-GBLREF	uint4			dollar_trestart;
-GBLREF	int			dollar_truth;
-GBLREF	mval			dollar_zgbldir;
 GBLREF	gd_addr			*gd_header;
 GBLREF	gv_key			*gv_currkey;
 GBLREF	gv_namehead		*gv_target;
-GBLREF	stack_frame		*frame_pointer;
-GBLREF	tp_frame		*tp_pointer;
-GBLREF	sgm_info		*sgm_info_ptr;
-GBLREF	tp_region		*tp_reg_list;	/* Chained list of regions used in this transaction not cleared on tp_restart */
-GBLREF	mv_stent		*mv_chain;
-GBLREF	unsigned char		*msp, *stackbase, *stacktop, t_fail_hist[CDB_MAX_TRIES];
-GBLREF	sgm_info		*first_sgm_info;
-GBLREF	unsigned int		t_tries;
-GBLREF	int			process_id;
 GBLREF	gd_region		*gv_cur_region;
-GBLREF	jnlpool_addrs_ptr_t	jnlpool;
-GBLREF	jnlpool_addrs_ptr_t	jnlpool_head;
-GBLREF	boolean_t		caller_id_flag;
-GBLREF	unsigned char		*tpstackbase, *tpstacktop;
-GBLREF	trans_num		local_tn;	/* transaction number for THIS PROCESS */
-GBLREF	sgmnt_addrs		*cs_addrs;
-GBLREF	sgmnt_data		*cs_data;
-GBLREF	symval			*curr_symval;
-GBLREF	trans_num		tstart_local_tn;	/* copy of global variable "local_tn" at op_tstart time */
-GBLREF	boolean_t		mupip_jnl_recover;
-GBLREF	mstr			extnam_str;
 GBLREF	jnl_gbls_t		jgbl;
-GBLREF	sgmnt_addrs		*cs_addrs_list;
-GBLREF	boolean_t		is_updproc;
-GBLREF	sgmnt_addrs		*reorg_encrypt_restart_csa;
+GBLREF	jnlpool_addrs_ptr_t	jnlpool, jnlpool_head;
+GBLREF	mstr			extnam_str;
+GBLREF	mval			dollar_zgbldir;
+GBLREF	mv_stent		*mv_chain;
+GBLREF	sgm_info		*sgm_info_ptr;
+GBLREF	sgmnt_addrs		*cs_addrs, *cs_addrs_list, *reorg_encrypt_restart_csa;
+GBLREF	sgmnt_data		*cs_data;
+GBLREF	stack_frame		*frame_pointer;
+GBLREF	symval			*curr_symval;
+GBLREF	tp_frame		*tp_pointer;
+GBLREF	tp_region		*tp_reg_list;		/* chained list of regions in this transaction not cleared on tp_restart */
+GBLREF	trans_num		local_tn;		/* transaction number for THIS PROCESS */
+GBLREF	uint4			dollar_tlevel, dollar_trestart, t_err;
+GBLREF	unsigned char		*msp, *stackbase, *stacktop, t_fail_hist[CDB_MAX_TRIES], *tpstackbase, *tpstacktop;
+GBLREF	unsigned int		t_tries;
 #ifdef GTM_TRIGGER
 GBLREF	int			tprestart_state;	/* When triggers restart, multiple states possible. See tp_restart.h */
-GBLREF	mval			dollar_ztwormhole;	/* Previous value (mval) restored on restart */
 GBLREF	mval			dollar_ztslate;
+GBLREF	mval			dollar_ztwormhole;	/* Previous value (mval) restored on restart */
 LITREF	mval			literal_null;
 #endif
-GBLREF int  			mumps_status;
 
 error_def(ERR_GVFAILCORE);
+error_def(ERR_GVIS);
 error_def(ERR_REPLONLNRLBK);
 error_def(ERR_TLVLZERO);
 error_def(ERR_TPFAIL);
@@ -140,23 +129,22 @@ CONDITION_HANDLER(tp_restart_ch)
  */
 int tp_restart(int newlevel, boolean_t handle_errors_internally)
 {
-	unsigned char		*cp;
-	unsigned char		buff[MAX_ZWR_KEY_SZ], *end, fail_hist[RESTART_CODE_EXPANSION_FACTOR];
-	unsigned int		c, hist_index, local_t_tries;
-	tp_frame		*tf;
+	boolean_t		reset_clues_done = FALSE;
+	boolean_t		tp_tend_status;
+	enum cdb_sc		status;
+	int			tprestart_rc, len;
+	int4			num_closed = 0;
+	gd_region		*restart_reg, *reg;
+	gv_namehead		*gvt;
+	jnlpool_addrs_ptr_t	save_jnlpool;
+	mstr			gvname_mstr, reg_mstr;
 	mv_stent		*mvc;
-	tp_region		*tr;
 	mval			beganHere;
 	sgmnt_addrs		*csa, *jpl_csa;
-	int4			num_closed = 0;
-	boolean_t		tp_tend_status;
-	boolean_t		reset_clues_done = FALSE;
-	mstr			gvname_mstr, reg_mstr;
-	gd_region		*restart_reg, *reg;
-	jnlpool_addrs_ptr_t	save_jnlpool, local_jnlpool;
-	int			tprestart_rc, len;
-	gv_namehead		*gvt;
-	enum cdb_sc		status;
+	tp_frame		*tf;
+	tp_region		*tr;
+	unsigned char		buff[MAX_ZWR_KEY_SZ], *end, fail_hist[RESTART_CODE_EXPANSION_FACTOR];
+	unsigned int		c, hist_index, local_t_tries;
 	DCL_THREADGBL_ACCESS;
 
 	SETUP_THREADGBL_ACCESS;
@@ -234,10 +222,13 @@ int tp_restart(int newlevel, boolean_t handle_errors_internally)
 		 */
 		assert(!mupip_jnl_recover || WB_COMMIT_ERR_ENABLED ||
 				(WBTEST_TP_HIST_CDB_SC_BLKMOD == gtm_white_box_test_case_number));
-		if (TREF(tprestart_syslog_delta) && (((TREF(tp_restart_count))++ < TREF(tprestart_syslog_first))
-			|| (0 == ((TREF(tp_restart_count) - TREF(tprestart_syslog_first)) % TREF(tprestart_syslog_delta)))))
+		if ((CDB_STAGNATE <= t_tries)
+			|| (TREF(tprestart_syslog_delta) && (((TREF(tp_restart_count)) < TREF(tprestart_syslog_first))
+			|| (0 == ((TREF(tp_restart_count) - TREF(tprestart_syslog_first) + 1) % TREF(tprestart_syslog_delta))))))
 		{
 			gvt = TAREF1(tp_fail_hist, t_tries);
+			if ((CDB_STAGNATE <= t_tries) && ((NULL == gvt) || (NULL == gvt->gd_csa)))
+				gvt = gv_target;
 			if ((NULL == gvt) || (NULL == gvt->gd_csa))
 			{
 				gvname_mstr.addr = (char *)gvname_unknown;
@@ -250,6 +241,7 @@ int tp_restart(int newlevel, boolean_t handle_errors_internally)
 			{
 				if (0 == gvt->clue.end)
 				{	/* the clue has been invalidated - just use the unsubscripted name */
+					assert(NULL != &gvt->gvname);
 					gvname_mstr.addr = gvt->gvname.var_name.addr;
 					gvname_mstr.len = gvt->gvname.var_name.len;
 				} else
@@ -261,6 +253,10 @@ int tp_restart(int newlevel, boolean_t handle_errors_internally)
 					gvname_mstr.len = end - buff - 1;
 				}
 			}
+		}
+		if (TREF(tprestart_syslog_delta) && (((TREF(tp_restart_count))++ < TREF(tprestart_syslog_first))
+			|| (0 == ((TREF(tp_restart_count) - TREF(tprestart_syslog_first)) % TREF(tprestart_syslog_delta)))))
+		{
 			assert(0 == cdb_sc_normal);
 			if (cdb_sc_normal == status)
 				t_fail_hist[t_tries] = '0';	/* temporarily reset just for pretty printing */
@@ -290,8 +286,8 @@ int tp_restart(int newlevel, boolean_t handle_errors_internally)
 			for (c = local_t_tries = 0; local_t_tries <= t_tries; local_t_tries++)
 			{	/* in case of non-printable code provide hex representation */
 				if (ISALPHA_ASCII(t_fail_hist[local_t_tries]) || ISDIGIT_ASCII(t_fail_hist[local_t_tries])
-					|| ISPUNCT_ASCII(t_fail_hist[local_t_tries]))	/* currently, only is alpha needed */
-						fail_hist[c++] = t_fail_hist[local_t_tries++];
+						|| ISPUNCT_ASCII(t_fail_hist[local_t_tries])) /* currently, only is alpha needed */
+					fail_hist[c++] = t_fail_hist[local_t_tries++];
 				else
 				{
 					assert((c + 4) < RESTART_CODE_EXPANSION_FACTOR);
@@ -306,7 +302,7 @@ int tp_restart(int newlevel, boolean_t handle_errors_internally)
 			if (cdb_sc_blkmod != status)
 			{
 				send_msg_csa(CSA_ARG(NULL) VARLSTCNT(18) ERR_TPRESTART, 16, reg_mstr.len, reg_mstr.addr,
-					c, &fail_hist, TAREF1(t_fail_hist_blk, t_tries), gvname_mstr.len,
+					c, &fail_hist, &(TAREF1(t_fail_hist_blk, t_tries)), gvname_mstr.len,
 					gvname_mstr.addr, 0, 0, 0, tp_blkmod_nomod,
 					(NULL != sgm_info_ptr) ? sgm_info_ptr->num_of_blks : 0,
 					(NULL != sgm_info_ptr) ? sgm_info_ptr->cw_set_depth : 0, &local_tn,
@@ -314,7 +310,7 @@ int tp_restart(int newlevel, boolean_t handle_errors_internally)
 			} else
 			{
 				send_msg_csa(CSA_ARG(NULL) VARLSTCNT(18) ERR_TPRESTART, 16, reg_mstr.len, reg_mstr.addr,
-					c, &fail_hist, TAREF1(t_fail_hist_blk, t_tries), gvname_mstr.len,
+					c, &fail_hist, &(TAREF1(t_fail_hist_blk, t_tries)), gvname_mstr.len,
 					gvname_mstr.addr, n_pvtmods, n_blkmods, TREF(blkmod_fail_level), TREF(blkmod_fail_type),
 					sgm_info_ptr->num_of_blks, sgm_info_ptr->cw_set_depth, &local_tn,
 					(TREF(tp_restart_entryref)).str.len, (TREF(tp_restart_entryref)).str.addr);
@@ -339,7 +335,7 @@ int tp_restart(int newlevel, boolean_t handle_errors_internally)
 		switch (status)
 		{
 			case cdb_sc_helpedout:
-				assert(IS_FINAL_RETRY_CODE(status));
+				assert(is_final_retry_code(status));
 				csa = sgm_info_ptr->tp_csa;
 				if (dba_bg == csa->hdr->acc_meth)
 				{
@@ -386,7 +382,7 @@ int tp_restart(int newlevel, boolean_t handle_errors_internally)
 				 * virtue of it having now been referenced) has been added to tp_reg_list so all we need
 				 * now is a retry.
 				 */
-				assert(IS_FINAL_RETRY_CODE(status));
+				assert(is_final_retry_code(status));
 				assert(CDB_STAGNATE == t_tries);
 				for (tr = tp_reg_list; NULL != tr; tr = tr->fPtr)
 				{	/* regions might not have been opened if we t_retried in gvcst_init(). dont
@@ -431,7 +427,7 @@ int tp_restart(int newlevel, boolean_t handle_errors_internally)
 				/* If retry due to M-locks, sleep so needed locks have a chance to get released */
 				break;
 			case cdb_sc_reorg_encrypt:
-				assert(IS_FINAL_RETRY_CODE(status));
+				assert(is_final_retry_code(status));
 				/* Even though the failure code is "cdb_sc_reorg_encrypt", it is possible
 				 * "process_reorg_encrypt_restart" was already called from "t_qread" which then
 				 * returned "cdb_sc_reorg_encrypt" that ended up in tp_restart. "reorg_encrypt_restart_csa" would
@@ -505,7 +501,7 @@ int tp_restart(int newlevel, boolean_t handle_errors_internally)
 			case cdb_sc_phase2waitfail:
 			/* cdb_sc_wcs_recover is possible in final retry in TP (see comment in "tp_hist" */
 			case cdb_sc_wcs_recover:
-				assert(IS_FINAL_RETRY_CODE(status));
+				assert(is_final_retry_code(status));
 				if (CDB_STAGNATE <= t_tries)
 				{
 					t_tries--;
@@ -520,14 +516,17 @@ int tp_restart(int newlevel, boolean_t handle_errors_internally)
 			default:
 				if (CDB_STAGNATE < ++t_tries)
 				{
-					assert(!IS_FINAL_RETRY_CODE(status));
+					assert(!is_final_retry_code(status));
 					hist_index = t_tries;
 					t_tries = 0;
+					assert(NULL != sgm_info_ptr);
+					csa = sgm_info_ptr->tp_csa;
+					assert(NULL != csa);
 					assert(0 != have_crit(CRIT_HAVE_ANY_REG)); /* we should still be holding crit */
 					assert(gtm_white_box_test_case_enabled
 					    && (WBTEST_TP_HIST_CDB_SC_BLKMOD == gtm_white_box_test_case_number));
-					send_msg_csa(CSA_ARG(NULL) VARLSTCNT(5) ERR_TPFAIL, 2, hist_index, t_fail_hist,
-							ERR_GVFAILCORE);
+					send_msg_csa(CSA_ARG(csa) VARLSTCNT(11) ERR_TPFAIL, 0, t_err, 2, hist_index, t_fail_hist,
+						ERR_GVIS, 2, gvname_mstr.len, gvname_mstr.addr, ERR_GVFAILCORE);
 					/* Generate core only if not triggering this codepath using white-box tests */
 					DEBUG_ONLY(
 						if (!gtm_white_box_test_case_enabled
@@ -536,7 +535,8 @@ int tp_restart(int newlevel, boolean_t handle_errors_internally)
 							gtm_fork_n_core();
 					if (save_jnlpool != jnlpool)
 						jnlpool = save_jnlpool;
-					rts_error_csa(CSA_ARG(NULL) VARLSTCNT(4) ERR_TPFAIL, 2, hist_index, t_fail_hist);
+					rts_error_csa(CSA_ARG(csa) VARLSTCNT(10) ERR_TPFAIL, 0, t_err, 2, hist_index, t_fail_hist,
+						      ERR_GVIS, 2, gvname_mstr.len, gvname_mstr.addr);
 					return 0; /* for the compiler only -- never executed */
 				} else
 				{
@@ -839,9 +839,9 @@ int tp_restart(int newlevel, boolean_t handle_errors_internally)
 			send_msg_csa(CSA_ARG(NULL) VARLSTCNT(1) ERR_TRESTNOT);		/* Separate msgs so we get both */
 			send_msg_csa(CSA_ARG(NULL) VARLSTCNT(6) ERR_TRESTLOC, 4, beganHere.str.len, beganHere.str.addr,
 				(TREF(tp_restart_entryref)).str.len, (TREF(tp_restart_entryref)).str.addr);
-			rts_error_csa(CSA_ARG(NULL) VARLSTCNT(8) ERR_TRESTNOT, 0, ERR_TRESTLOC, 4,
-					beganHere.str.len, beganHere.str.addr,
-					(TREF(tp_restart_entryref)).str.len, (TREF(tp_restart_entryref)).str.addr);
+			RTS_ERROR_CSA_ABT(NULL, VARLSTCNT(8) ERR_TRESTNOT, 0, ERR_TRESTLOC, 4,
+				beganHere.str.len, beganHere.str.addr,
+				(TREF(tp_restart_entryref)).str.len, (TREF(tp_restart_entryref)).str.addr);
 		} else
 			rts_error_csa(CSA_ARG(NULL) VARLSTCNT(1) ERR_TRESTNOT);
 		return 0; /* for the compiler only -- never executed */
@@ -864,7 +864,7 @@ int tp_restart(int newlevel, boolean_t handle_errors_internally)
 	 */
 	assert(cdb_sc_normal != status);
 	if (is_updproc && ((cdb_sc_onln_rlbk1 == status) || (cdb_sc_onln_rlbk2 == status)))
-		rts_error_csa(CSA_ARG(NULL) VARLSTCNT(1) ERR_REPLONLNRLBK);
+		RTS_ERROR_CSA_ABT(NULL, VARLSTCNT(1) ERR_REPLONLNRLBK);
 	if (handle_errors_internally)
 		REVERT;
 	TREF(expand_prev_key) = FALSE;	/* in case we did a "t_retry" in the middle of "gvcst_zprevious2" */
