@@ -1,6 +1,6 @@
 /****************************************************************
  *								*
- * Copyright (c) 2001-2020 Fidelity National Information	*
+ * Copyright (c) 2001-2022 Fidelity National Information	*
  * Services, Inc. and/or its subsidiaries. All rights reserved.	*
  *								*
  *	This source code contains the intellectual property	*
@@ -278,8 +278,6 @@ boolean_t	tp_tend()
 	jnl_tm_t		save_gbl_jrec_time;
 	uint4			max_upd_num, prev_upd_num, upd_num, upd_num_end, upd_num_start;
 #	endif
-        int4			event_type, param_val;
-        void (*set_fn)(int4 param);
 	DCL_THREADGBL_ACCESS;
 
 	SETUP_THREADGBL_ACCESS;
@@ -329,7 +327,11 @@ boolean_t	tp_tend()
 		{	/* If blocked, or we have MM and file has been extended, force repair */
 			status = cdb_sc_helpedout; /* special status to prevent punishing altruism */
 			assert((CDB_STAGNATE > t_tries) || !is_mm || (csa->total_blks == csd->trans_hist.total_blks));
-			TP_TRACE_HIST(CR_BLKEMPTY, NULL);
+			if (IS_STATSDB_CSA(csa) && (NULL != si) && (NULL != si->first_tp_hist))
+			{	/* Report ^%YGS restart when the region is statsdb */
+				TP_TRACE_HIST(si->first_tp_hist->blk_num, si->first_tp_hist->blk_target);
+			} else
+				TP_TRACE_HIST(CR_BLKEMPTY, NULL);
 			goto failed_skip_revert;
 		}
 		/* Note that there are three ways a deadlock can occur.
@@ -415,7 +417,7 @@ boolean_t	tp_tend()
 				/* check if current TP transaction's jnl size needs are greater than max jnl file size */
 				if (si->tot_jrec_size > csd->autoswitchlimit)
 					/* can't fit in current transaction's journal records into one journal file */
-					rts_error_csa(CSA_ARG(csa) VARLSTCNT(6) ERR_JNLTRANS2BIG, 4, si->tot_jrec_size,
+					RTS_ERROR_CSA_ABT(csa, VARLSTCNT(6) ERR_JNLTRANS2BIG, 4, si->tot_jrec_size,
 						JNL_LEN_STR(csd), csd->autoswitchlimit);
 			}
 			if (REPL_ALLOWED(csa))
@@ -468,8 +470,8 @@ boolean_t	tp_tend()
 			{
 				if (save_jnlpool != jnlpool)
 					jnlpool = save_jnlpool;
-				rts_error_csa(CSA_ARG(REG2CSA(save_gv_cur_region)) VARLSTCNT(4) ERR_REPLOFFJNLON, 2,
-						DB_LEN_STR(save_gv_cur_region));
+				RTS_ERROR_CSA_ABT(REG2CSA(save_gv_cur_region), VARLSTCNT(4) ERR_REPLOFFJNLON, 2,
+					DB_LEN_STR(save_gv_cur_region));
 			}
 			/* If caller does NOT want this update to be replicated, turn "replication" local variable off.
 			 * The only such caller known at this time is "trigger_upgrade" - 2014/05/02.
@@ -1391,15 +1393,6 @@ boolean_t	tp_tend()
 		ctn = csd->trans_hist.curr_tn;
 		ASSERT_CURR_TN_EQUALS_EARLY_TN(csa, ctn);
 		csd->trans_hist.early_tn = ctn + 1;	/* Step CMT04 */
-		/* If this process had used "cnl->tp_hint" and done some block allocations in "bm_getfree", now that
-		 * all history validation is complete and this transaction is going to commit, take this opportunity
-		 * to update the shared memory hint to reflect this process' allocations in this TP transaction.
-		 */
-		if (csa->tp_hint)
-		{
-			cnl = csa->nl;
-			cnl->tp_hint = csa->tp_hint;	/* update the region hint to reflect any (successful) allocations */
-		}
 		is_mm = (dba_mm == csd->acc_meth);
 		csa->t_commit_crit = T_COMMIT_CRIT_PHASE0;	/* phase0 : write journal records. Step CMT05 */
 		if (JNL_ALLOWED(csa))
@@ -1840,7 +1833,6 @@ failed:
 		 */
 		assert(jgbl.onlnrlbk || !release_crit || (0 == have_crit(CRIT_HAVE_ANY_REG | CRIT_IN_COMMIT)));
 	}
-	CALL_ZTIMEOUT_IF_DEFERRED;
 	/* We have finished validation on this region. Reset transaction numbers in the gv_target
 	 * histories so they will be valid for a future access utilizing the clue field. This occurs
 	 * to improve performance (of next tn in case of commit of current tn) or the chances of commit
@@ -1966,7 +1958,9 @@ enum cdb_sc	reallocate_bitmap(sgm_info *si, cw_set_element *bml_cse)
 		if ((gds_t_acquired != cse->mode) || (ROUND_DOWN2(cse->blk, BLKS_PER_LMAP) != bml))
 			continue;
 		assert(gds_t_acquired == cse->mode);
-		assert(GDSVCURR == cse->ondsk_blkver);
+		assert((GDSVCURR == cse->ondsk_blkver) || (BLK_ID_32_VER > cse->ondsk_blkver));
+		assert((GDSVCURR == cse->ondsk_blkver) || (GDSV7m == cse->ondsk_blkver)
+			|| ((bml == (block_id_32)bml) && (total_blks == (block_id_32)total_blks)));
 		assert(*b_ptr == (cse->blk - bml));
 		do
 		{
@@ -2041,7 +2035,8 @@ enum cdb_sc	reallocate_bitmap(sgm_info *si, cw_set_element *bml_cse)
 				 * therefore, fix it to point to the reallocated block's buffer address
 				 */
 				cse->old_block = t_qread(cse->blk, (sm_int_ptr_t)&cse->cycle, &cse->cr);
-				assert(GDSVCURR == cse->ondsk_blkver);	/* should have been already initialized in t_write_map */
+				/* should have been already initialized in t_write_map */
+				assert((GDSV7 >= cse->ondsk_blkver) || (BLK_ID_32_VER > cse->ondsk_blkver));
 				old_block = (blk_hdr_ptr_t)cse->old_block;
 				if (NULL == old_block)
 					return ((enum cdb_sc)rdfail_detail);
