@@ -1,3 +1,14 @@
+/****************************************************************
+ *								*
+ * Copyright (c) 2016-2020 Fidelity National Information	*
+ * Services, Inc. and/or its subsidiaries. All rights reserved.	*
+ *								*
+ *	This source code contains the intellectual property	*
+ *	of its copyright holder(s), and is made available	*
+ *	under a license.  If you do not know the terms of	*
+ *	the license, please stop and do not read further.	*
+ *								*
+ ****************************************************************/
 #include "mdef.h"
 
 #include "mdq.h"
@@ -11,10 +22,11 @@ GBLREF hash_table_str	*complits_hashtab;
 
 boolean_t unuse_literal(mval *x)
 {
+	boolean_t	in_hashtab = FALSE;
+	ht_ent_str	*litent;
 	mliteral	*a = NULL;
 	stringkey	litkey;
-	ht_ent_str	*litent;
-	bool		in_hashtab = FALSE;
+
 	if (complits_hashtab && complits_hashtab->base)
 	{
 		litkey.str = x->str;
