@@ -1,7 +1,7 @@
 /****************************************************************
  *								*
 <<<<<<< HEAD
- * Copyright (c) 2001-2020 Fidelity National Information	*
+ * Copyright (c) 2001-2022 Fidelity National Information	*
  * Services, Inc. and/or its subsidiaries. All rights reserved.	*
  *								*
  *	This source code contains the intellectual property	*
@@ -112,6 +112,7 @@ GBLREF 	sgmnt_data_ptr_t 	cs_data;
 GBLREF	recvpool_addrs		recvpool;
 GBLREF	jnlpool_addrs_ptr_t	jnlpool;
 GBLREF	boolean_t		is_updproc;
+GBLREF	boolean_t		first_syslog;
 GBLREF	seq_num			seq_num_zero, seq_num_one;
 GBLREF  gd_addr                 *gd_header;
 GBLREF	FILE	 		*updproc_log_fp;
@@ -149,7 +150,6 @@ error_def(ERR_REC2BIG);
 error_def(ERR_RECVPOOLSETUP);
 error_def(ERR_REPEATERROR);
 error_def(ERR_REPLONLNRLBK);
-error_def(ERR_SECONDAHEAD);
 error_def(ERR_STRMSEQMISMTCH);
 error_def(ERR_SYSCALL);
 error_def(ERR_TEXT);
@@ -391,7 +391,9 @@ int updproc(void)
 	 * a runtime error and does not turn journaling off.
 	 */
 	TREF(error_on_jnl_file_lost) = JNL_FILE_LOST_ERRORS;
-	is_updproc = TRUE;
+	/* The process might have sent a syslog message already, so set first_syslog here to force setting image type
+	 * with the new value of is_updproc. */
+	is_updproc = first_syslog = TRUE;
 	trap_env_init();
 	is_replicator = TRUE;	/* as update process goes through t_end() and can write jnl recs to the jnlpool for replicated db */
 	TREF(ok_to_see_statsdb_regs) = TRUE;
@@ -400,7 +402,7 @@ int updproc(void)
 	memset((uchar_ptr_t)&recvpool, 0, SIZEOF(recvpool)); /* For util_base_ch and mupip_exit */
 	if (updproc_init(&gld_db_files, &start_jnl_seqno) == UPDPROC_EXISTS) /* we got the global directory header already */
 	{
-		rts_error_csa(CSA_ARG(NULL) VARLSTCNT(6) ERR_RECVPOOLSETUP, 0,
+		RTS_ERROR_CSA_ABT(NULL, VARLSTCNT(6) ERR_RECVPOOLSETUP, 0,
 			ERR_TEXT, 2, RTS_ERROR_LITERAL("Update Process already exists"));
 	}
 	OPERATOR_LOG_MSG;
@@ -494,14 +496,14 @@ int updproc(void)
 		/* We should be the only one acquiring the lock, so a single try should be sufficient. */
 		status = pthread_mutex_trylock(&recvpool.recvpool_ctl->write_updated_ctl);
 		if (0 != status)
-			rts_error_csa(CSA_ARG(NULL) VARLSTCNT(8) ERR_SYSCALL, 5,
-					LEN_AND_LIT("pthread_mutex_trylock"), CALLFROM, status, 0);
+			RTS_ERROR_CSA_ABT(NULL, VARLSTCNT(8) ERR_SYSCALL, 5,
+				LEN_AND_LIT("pthread_mutex_trylock"), CALLFROM, status, 0);
 		while (updproc_continue && !set_onln_rlbk_flg)
 			updproc_actions(gld_db_files);
 		status = pthread_mutex_unlock(&recvpool.recvpool_ctl->write_updated_ctl);
 		if (0 != status)
-			rts_error_csa(CSA_ARG(NULL) VARLSTCNT(8) ERR_SYSCALL, 5,
-					LEN_AND_LIT("pthread_mutex_unlock"), CALLFROM, status, 0);
+			RTS_ERROR_CSA_ABT(NULL, VARLSTCNT(8) ERR_SYSCALL, 5,
+				LEN_AND_LIT("pthread_mutex_unlock"), CALLFROM, status, 0);
 		if (set_onln_rlbk_flg)
 		{	/* A concurrent online rollback happened which drove the updproc_ch and called us. Need to let the receiver
 			 * server know about it and set up the sequence numbers
@@ -689,8 +691,8 @@ void updproc_actions(gld_dbname_list *gld_db_files)
 				/* the 0 == check takes care of the startup case where jnl_seqno is 0 in the recvpool_ctl */
 			status = clock_gettime(CLOCK_REALTIME, &waketime);
 			if (0 != status)
-				rts_error_csa(CSA_ARG(NULL) VARLSTCNT(8) ERR_SYSCALL, 5,
-						LEN_AND_LIT("clock_gettime"), CALLFROM, errno, 0);
+				RTS_ERROR_CSA_ABT(NULL, VARLSTCNT(8) ERR_SYSCALL, 5,
+					LEN_AND_LIT("clock_gettime"), CALLFROM, errno, 0);
 			waketime.tv_sec += 4;
 			PTHREAD_COND_TIMEDWAIT(&recvpool.recvpool_ctl->write_updated,
 						&recvpool.recvpool_ctl->write_updated_ctl,
@@ -698,8 +700,8 @@ void updproc_actions(gld_dbname_list *gld_db_files)
 			if ((0 != status) && (ETIMEDOUT != status))
 			{
 				assert(EINVAL != status);
-				rts_error_csa(CSA_ARG(NULL) VARLSTCNT(8) ERR_SYSCALL, 5,
-						LEN_AND_LIT("pthread_cond_timedwait"), CALLFROM, status, 0);
+				RTS_ERROR_CSA_ABT(NULL, VARLSTCNT(8) ERR_SYSCALL, 5,
+					LEN_AND_LIT("pthread_cond_timedwait"), CALLFROM, status, 0);
 			}
 			if (!upd_proc_local->onln_rlbk_flg && (repl_csa->onln_rlbk_cycle != jnlpool->jnlpool_ctl->onln_rlbk_cycle))
 			{	/* A concurrent online rollback happened. Start afresh */
@@ -1027,8 +1029,8 @@ void updproc_actions(gld_dbname_list *gld_db_files)
 					&& (repl_csa->onln_rlbk_cycle == jnlpool->jnlpool_ctl->onln_rlbk_cycle))
 				{
 					assert(FALSE);
-					rts_error_csa(CSA_ARG(NULL) VARLSTCNT(5) ERR_STRMSEQMISMTCH, 3,
-							strm_index, &rec_strm_seqno, &strm_seqno);
+					RTS_ERROR_CSA_ABT(NULL, VARLSTCNT(5) ERR_STRMSEQMISMTCH, 3,
+						strm_index, &rec_strm_seqno, &strm_seqno);
 				}
 			}
 			if (JRT_NULL == rectype)
@@ -1352,8 +1354,8 @@ void updproc_actions(gld_dbname_list *gld_db_files)
 						if (NULL != gv_failed_key)	/* Free memory if it has been used */
 							free(gv_failed_key);
 						assert(NULL != tmpcsa);
-						rts_error_csa(CSA_ARG(tmpcsa) VARLSTCNT(6) ERR_GVSUBOFLOW, 0,
-									ERR_GVIS, 2, endBuff - fmtBuff, fmtBuff);
+						RTS_ERROR_CSA_ABT(tmpcsa, VARLSTCNT(6) ERR_GVSUBOFLOW, 0,
+							ERR_GVIS, 2, endBuff - fmtBuff, fmtBuff);
 						break;
 					case upd_bad_val_size:
 						if (0 == (end = format_targ_key(buff, MAX_ZWR_KEY_SZ,
@@ -1362,7 +1364,7 @@ void updproc_actions(gld_dbname_list *gld_db_files)
 						if (NULL != gv_failed_key)	/* Free memory if it has been used */
 							free(gv_failed_key);
 						assert(NULL != tmpcsa);
-						rts_error_csa(CSA_ARG(tmpcsa) VARLSTCNT(10) ERR_REC2BIG, 4, val_mv.str.len,
+						RTS_ERROR_CSA_ABT(tmpcsa, VARLSTCNT(10) ERR_REC2BIG, 4, val_mv.str.len,
 							(int4)(tmpcsa->region)->max_rec_size, REG_LEN_STR(tmpcsa->region),
 							ERR_GVIS, 2, end - buff, buff);
 						break;
