@@ -1,6 +1,6 @@
 /****************************************************************
  *								*
- * Copyright (c) 2001-2019 Fidelity National Information	*
+ * Copyright (c) 2001-2021 Fidelity National Information	*
  * Services, Inc. and/or its subsidiaries. All rights reserved.	*
  *								*
  *	This source code contains the intellectual property	*
@@ -21,6 +21,7 @@ urx_rtnref *urx_putrtn (char *rtn, int rtnlen, urx_rtnref *anchor)
 	urx_rtnref	*rp0, *rp1, *tmp;
 	boolean_t	found;
 	int		c;
+	size_t		cpylen;
 
 	assert(anchor->len == 0);
 	assert(0 < rtnlen);
@@ -36,6 +37,7 @@ urx_rtnref *urx_putrtn (char *rtn, int rtnlen, urx_rtnref *anchor)
 	while (rp1 != 0)
 	{	/* Find routine or insertion point */
 		c = rtnlen - rp1->len;
+		assert(0 < rtnlen);
 		if (!c)
 			c = memcmp(rtn, &rp1->name[0], rtnlen);
 		if (c > 0)
@@ -49,12 +51,16 @@ urx_rtnref *urx_putrtn (char *rtn, int rtnlen, urx_rtnref *anchor)
 			break;
 		}
 	}
+	assert((0 < rtnlen) && ((MAXPOSINT4 - sizeof(urx_rtnref)) > rtnlen));
 	assert(rp0->next == rp1);
 	if (!found)
 	{
 		tmp = (urx_rtnref *)malloc(SIZEOF(urx_rtnref) + rtnlen);
-		tmp->len = rtnlen;
-		memcpy(&tmp->name[0], rtn, rtnlen);
+		assert(NULL != tmp);
+		tmp->len = (unsigned int)rtnlen;
+		assert(0 <= rtnlen);
+		cpylen = (size_t) rtnlen;
+		memcpy(tmp->name, rtn, cpylen);
 		tmp->addr = NULL;
 		tmp->lab = NULL;
 		tmp->next = rp1;
