@@ -1,6 +1,6 @@
 /****************************************************************
  *								*
- * Copyright (c) 2001-2019 Fidelity National Information	*
+ * Copyright (c) 2001-2022 Fidelity National Information	*
  * Services, Inc. and/or its subsidiaries. All rights reserved.	*
  *								*
  *	This source code contains the intellectual property	*
@@ -22,6 +22,7 @@ VIEWTAB("DEBUG1",		VTP_VALUE | VTP_NULL,		VTK_DEBUG1,		MV_STR),
 VIEWTAB("DEBUG2",		VTP_VALUE | VTP_NULL,		VTK_DEBUG2,		MV_STR),
 VIEWTAB("DEBUG3",		VTP_VALUE | VTP_NULL,		VTK_DEBUG3,		MV_STR),
 VIEWTAB("DEBUG4",		VTP_VALUE | VTP_NULL,		VTK_DEBUG4,		MV_STR),
+VIEWTAB("DEVICE",		VTP_VALUE,			VTK_DEVICE,		MV_STR),
 VIEWTAB("DMTERM",		VTP_NULL,			VTK_DMTERM,		MV_NM),
 VIEWTAB("EPOCH",		VTP_DBREGION | VTP_NULL,	VTK_EPOCH,		MV_STR),
 VIEWTAB("FILL_FACTOR",		VTP_VALUE | VTP_NULL,		VTK_FILLFACTOR,		MV_NM),
@@ -134,6 +135,8 @@ VIEWTAB("YGVN2GDS",		VTP_VALUE,			VTK_YGVN2GDS,		MV_STR), /* nodoc : GDE related
 												  * undocumented feature */
 VIEWTAB("YLCT",			VTP_NULL | VTP_VALUE,		VTK_YLCT,		MV_NM), /* nodoc : collation related
 												 * undocumented feature */
+VIEWTAB("YLGDE",		VTP_NULL,			VTK_YLGDE,		MV_NM),/* nodoc : gde logging related
+												 * undocumented feature */
 VIEWTAB("ZDEFBUFFER",		VTP_DBREGION,			VTK_ZDEFBUFF,		MV_NM), /* nodoc : seems to be an archaic
 												 * feature of GT.CM GNP */
 VIEWTAB("ZDEFCOUNT",		VTP_DBREGION,			VTK_ZDEFCNT,		MV_NM), /* nodoc : seems to be an archaic
