@@ -1,6 +1,7 @@
 /****************************************************************
  *								*
- *	Copyright 2001, 2014 Fidelity Information Services, Inc	*
+ * Copyright (c) 2001-2015 Fidelity National Information 	*
+ * Services, Inc. and/or its subsidiaries. All rights reserved.	*
  *								*
  *	This source code contains the intellectual property	*
  *	of its copyright holder(s), and is made available	*
@@ -15,6 +16,7 @@
 #include <zbreaksp.h>
 #include "cache.h"
 
+/* Structure for each ZBREAK set in the process */
 typedef struct
 {
 	zb_code		*mpc;		/* MUMPS address for ZBREAK */
@@ -23,9 +25,12 @@ typedef struct
 	int		offset;
 	int		count;		/* # of time ZBREAK encountered */
 	cache_entry	*action;	/* action associated with ZBREAK (indirect cache entry) */
+	rhdtyp		*rtnhdr;	/* Routine header of routine containing ZBREAK */
 	zb_code 	m_opcode;	/* MUMPS op_code replaced */
+	char		filler[SIZEOF(char *) - SIZEOF(zb_code)];
 } zbrk_struct;
 
+/* ZBREAK anchoring structure */
 typedef struct
 {
 	zbrk_struct	*beg;
@@ -45,10 +50,13 @@ typedef struct
 
 #define SIZEOF_LA	0
 
-zbrk_struct *zr_find(z_records *zrecs, zb_code *addr);
-zbrk_struct *zr_get_free(z_records *zrecs, zb_code *addr);
+#define	RETURN_CLOSEST_MATCH_FALSE	FALSE
+#define	RETURN_CLOSEST_MATCH_TRUE	TRUE
+
+zbrk_struct *zr_find(z_records *zrecs, zb_code *addr, boolean_t return_closest_match);
+zbrk_struct *zr_add_zbreak(z_records *zrecs, zb_code *addr);
 void zr_init(z_records *zrecs, int4 count);
-void zr_put_free(z_records *zrecs, zbrk_struct *z_ptr);
+void zr_remove_zbreak(z_records *zrecs, zbrk_struct *z_ptr);
 zb_code *find_line_call(void *addr);
 void zr_remove_zbrks(rhdtyp *rtn, boolean_t notify_is_trigger);
 
