@@ -1,6 +1,6 @@
 /****************************************************************
  *								*
- * Copyright (c) 2001-2019 Fidelity National Information	*
+ * Copyright (c) 2001-2021 Fidelity National Information	*
  * Services, Inc. and/or its subsidiaries. All rights reserved.	*
  *								*
  *	This source code contains the intellectual property	*
@@ -13,6 +13,7 @@
 #include "mdef.h"
 
 #include "gtm_string.h"
+#include "gtm_limits.h"
 
 #include <rtnhdr.h>
 #include "stack_frame.h"
@@ -47,6 +48,7 @@ void zshow_stack(zshow_out *output, boolean_t show_checksum)
 	stack_frame	*fp;
 	mstr 		v;
 	unsigned char	buff[MAX_ENTRYREF_LEN + MAX_ROUTINE_CHECKSUM_DIGITS + SIZEOF(INDR_OVERFLOW)];
+	ptrdiff_t	pdiff;
 
 	v.addr = (char *)&buff[0];
 	flush_pio();
@@ -82,7 +84,10 @@ void zshow_stack(zshow_out *output, boolean_t show_checksum)
 				line_reset = FALSE;
 			} else
 				addr = fp->mpc;
-			v.len = INTCAST(symb_line(addr, &buff[0], SIZEOF(buff) - 1, 0, fp->rvector) - &buff[0]);
+			pdiff = INTCAST(symb_line(addr, &buff[0], SIZEOF(buff) - 1, 0, fp->rvector) - &buff[0]);
+			assert((0 <= pdiff) && (MSTR_LEN_MAX >= pdiff));
+			v.len = pdiff;
+			assert(0 <= v.len); /* 4SCA */
 			if (v.len == 0)
 			{
 				MEMCPY_LIT(&buff[0], UNK_LOC_MESS);
