@@ -1,6 +1,6 @@
 /****************************************************************
  *								*
- * Copyright (c) 2001-2020 Fidelity National Information	*
+ * Copyright (c) 2001-2022 Fidelity National Information	*
  * Services, Inc. and/or its subsidiaries. All rights reserved.	*
  *								*
  *	This source code contains the intellectual property	*
@@ -103,6 +103,7 @@ static readonly char zinterrupt_text[] = "$ZINTERRUPT";
 static readonly char zio_text[] = "$ZIO";
 static readonly char zjob_text[] = "$ZJOB";
 static readonly char zlevel_text[] = "$ZLEVEL";
+static readonly char zmalloclim_text[] = "$ZMALLOCLIM";
 static readonly char zmaxtptime_text[] = "$ZMAXTPTIME";
 static readonly char zmlkhash_text[] = "$ZMLKHASH";
 static readonly char zmode_text[] = "$ZMODE";
@@ -146,49 +147,25 @@ static readonly char zclose_text[] = "$ZCLOSE";
 static readonly char zkey_text[] = "$ZKEY";
 static readonly char arrow_text[] = "->";
 
-GBLREF mval		dollar_zdir;
-GBLREF mval		dollar_zproc;
-GBLREF stack_frame	*frame_pointer;
-GBLREF io_pair		io_curr_device;
-GBLREF io_pair		*io_std_device;
-GBLREF io_log_name	*io_root_log_name;
-GBLREF io_log_name	*dollar_principal;
-GBLREF mval		dollar_zgbldir;
-GBLREF mval		dollar_job;
-GBLREF uint4		dollar_zjob;
-GBLREF mval		dollar_zstatus;
-GBLREF mval		dollar_zsource;
-GBLREF int4		dollar_zsystem;
-GBLREF boolean_t	dollar_zaudit;
-GBLREF int4		dollar_zeditor;
-GBLREF uint4		dollar_tlevel;
-GBLREF uint4		dollar_trestart;
-GBLREF mval		dollar_estack_delta, dollar_zerror, dollar_zyerror, dollar_system;
-GBLREF mval		dollar_zinterrupt, dollar_ztexit;
-GBLREF boolean_t	dollar_zininterrupt;
-GBLREF int4		zdir_form;
-GBLREF size_t		totalAlloc;
-GBLREF size_t		totalRmalloc;
-GBLREF size_t		totalUsed;
-GBLREF mstr		dollar_zchset;
-GBLREF mstr		dollar_zpatnumeric;
-GBLREF boolean_t	dollar_zquit_anyway;
-#ifdef GTM_TRIGGER
-GBLREF mstr		*dollar_ztname;
-GBLREF mval		*dollar_ztdata;
-GBLREF mval		*dollar_ztdelim;
-GBLREF mval		*dollar_ztoldval;
-GBLREF mval		*dollar_ztriggerop;
-GBLREF mval		dollar_ztslate;
-GBLREF mval		*dollar_ztupdate;
-GBLREF mval		*dollar_ztvalue;
-GBLREF mval		dollar_ztwormhole;
-GBLREF int4		gtm_trigger_depth;
-#endif
-GBLREF spdesc		stringpool;
-GBLREF mstr		dollar_zpin;
-GBLREF mstr		dollar_zpout;
+GBLREF boolean_t		dollar_zquit_anyway;
+GBLREF int4			dollar_zeditor, dollar_zsystem, zdir_form;
+GBLREF io_log_name		*dollar_principal, *io_root_log_name;
+GBLREF io_pair			io_curr_device, *io_std_device;;
 GBLREF mlk_subhash_val_t	mlk_last_hash;
+GBLREF mstr			dollar_zchset, dollar_zpatnumeric, dollar_zpin, dollar_zpout;
+GBLREF mval			dollar_estack_delta, dollar_job, dollar_system, dollar_zdir, dollar_zerror, dollar_zgbldir;
+GBLREF mval			dollar_zinterrupt, dollar_zproc, dollar_ztexit, dollar_zsource, dollar_zstatus, dollar_zyerror;
+GBLREF size_t			totalAlloc, totalRmalloc, totalUsed, zmalloclim;
+GBLREF stack_frame		*frame_pointer;
+GBLREF spdesc			stringpool;
+GBLREF uint4			dollar_tlevel, dollar_trestart, dollar_zjob;
+GBLREF volatile boolean_t	dollar_zininterrupt;
+#ifdef GTM_TRIGGER
+GBLREF int4			gtm_trigger_depth;
+GBLREF mstr			*dollar_ztname;
+GBLREF mval			*dollar_ztdata, *dollar_ztdelim, *dollar_ztoldval, *dollar_ztriggerop, dollar_ztslate;
+GBLREF mval			*dollar_ztupdate, *dollar_ztvalue, dollar_ztwormhole;
+#endif
 
 LITREF mval		literal_zero, literal_one, literal_null;
 LITREF char		gtm_release_name[];
@@ -251,7 +228,7 @@ void zshow_svn(zshow_out *output, int one_sv)
 	int 		count, save_dollar_zlevel;
 	char		*c1, *c2;
 	char		zdir_error[ZDIR_ERR_LEN];
-	size_t		zdir_error_rem_len;
+	size_t		tmp, zdir_error_rem_len;
 	DCL_THREADGBL_ACCESS;
 
 	SETUP_THREADGBL_ACCESS;
@@ -430,7 +407,7 @@ void zshow_svn(zshow_out *output, int one_sv)
 				break;
 		/* CAUTION: fall through */
 		case SV_ZAUDIT:
-			MV_FORCE_MVAL(&var, dollar_zaudit);
+			MV_FORCE_MVAL(&var, TREF(dollar_zaudit));
 			ZS_VAR_EQU(&x, zaudit_text);
 			mval_write(output, &var, TRUE);
 			if (SV_ALL != one_sv)
@@ -607,6 +584,14 @@ void zshow_svn(zshow_out *output, int one_sv)
 			if (SV_ALL != one_sv)
 				break;
 		/* CAUTION: fall through */
+		case SV_ZMALLOCLIM:
+			tmp = zmalloclim;
+			MV_FORCE_UMVAL(&var, (unsigned int)tmp);
+			ZS_VAR_EQU(&x, zmalloclim_text);
+			mval_write(output, &var, TRUE);
+			if (SV_ALL != one_sv)
+				break;
+		/* CAUTION: fall through */
 		case SV_ZMAXTPTIME:
 			MV_FORCE_MVAL(&var, TREF(dollar_zmaxtptime));
 			ZS_VAR_EQU(&x, zmaxtptime_text);
@@ -917,6 +902,6 @@ void zshow_svn(zshow_out *output, int one_sv)
 			break;
 		/* NOTE: fall through ended */
 		default:
-			rts_error_csa(CSA_ARG(NULL) VARLSTCNT(1) ERR_INVSVN);
+			RTS_ERROR_CSA_ABT(NULL, VARLSTCNT(1) ERR_INVSVN);
 	}
 }
