@@ -1,6 +1,7 @@
 /****************************************************************
  *								*
- *	Copyright 2001 Sanchez Computer Associates, Inc.	*
+ * Copyright (c) 2001-2018 Fidelity National Information	*
+ * Services, Inc. and/or its subsidiaries. All rights reserved.	*
  *								*
  *	This source code contains the intellectual property	*
  *	of its copyright holder(s), and is made available	*
@@ -9,8 +10,8 @@
  *								*
  ****************************************************************/
 
-#ifndef __LOCK_CMTIMEOUT_H__
-#define __LOCK_CMTIMEOUT_H__
+#ifndef LOCK_CMTIMEOUT_H_INCLUDED
+#define LOCK_CMTIMEOUT_H_INCLUDED
 
 void lock_cmtimeout(void);
 
