@@ -1,6 +1,6 @@
 /****************************************************************
  *								*
- * Copyright (c) 2001-2019 Fidelity National Information	*
+ * Copyright (c) 2001-2022 Fidelity National Information	*
  * Services, Inc. and/or its subsidiaries. All rights reserved.	*
  *								*
  *	This source code contains the intellectual property	*
@@ -399,7 +399,7 @@ int 	parse_arg(CLI_ENTRY *pcmd_parms, int *eof)
 				 */
 				if (pparm->parm_values)
 				{
-					MALLOC_CPY_STR(pparm->pval_str, pparm->parm_values->prompt);
+					pparm->pval_str = malloc_cpy_str(pparm->parm_values->prompt);
 					if (!cli_get_sub_quals(pparm))
 						return (-1);
 				}
@@ -446,7 +446,7 @@ int 	parse_arg(CLI_ENTRY *pcmd_parms, int *eof)
 			 * Allocate memory and save value
 			 * -------------------------------
 			 */
-			MALLOC_CPY_STR(pparm->pval_str, cli_token_buf);
+			pparm->pval_str = malloc_cpy_str(cli_token_buf);
 			if (!cli_get_sub_quals(pparm))
 				return (-1);
 		}
@@ -495,18 +495,18 @@ boolean_t cli_numeric_check(CLI_ENTRY *pparm, char *val_str)
 	if (VAL_NUM == pparm->val_type)
 	{
 		if (pparm->hex_num)
-		{
+		{	/* It can ONLY be a HEX number. */
 			if (!cli_is_hex(val_str))
 			{
 				SNPRINTF(cli_err_str, MAX_CLI_ERR_STR,
-				  "Unrecognized value: %s, HEX number expected",
+				  "Unrecognized value: %s, A non-negative hexadecimal integer required",
 				  val_str);
 				retval = FALSE;
 			}
-		} else if (!cli_is_dcm(val_str))
+		} else if (!cli_is_dcm(val_str) && !cli_is_hex_explicit(val_str))
 		{
 			SNPRINTF(cli_err_str, MAX_CLI_ERR_STR,
-			  "Unrecognized value: %s, Decimal number expected",
+			  "Unrecognized value: %s, A non-negative decimal or hexadecimal integer required",
 			  val_str);
 			retval = FALSE;
 		}
@@ -577,9 +577,10 @@ boolean_t cli_get_sub_quals(CLI_ENTRY *pparm)
 {
 	CLI_ENTRY 	*pparm_qual, *pparm1;
 	char		local_str[MAX_LINE], tmp_str[MAX_LINE + 1], *tmp_str_ptr;
-	char 		*ptr_next_val, *ptr_next_comma, *ptr_equal;
-	int		len_str, neg_flg, ptr_equal_len;
+	char 		*ptr_next_val, *ptr_prev_val, *ptr_next_comma, *ptr_equal;
+	int		neg_flg;
 	boolean_t	val_flg, has_a_qual;
+	size_t		len_str, ptr_equal_len, neg_adj_len_str;
 
 	has_a_qual = FALSE;
 	pparm_qual = pparm->qual_vals;
@@ -587,11 +588,15 @@ boolean_t cli_get_sub_quals(CLI_ENTRY *pparm)
 		return TRUE;
 	if ((VAL_STR == pparm->val_type) || (VAL_LIST == pparm->val_type))
 	{
-		STRNCPY_STR(local_str, pparm->pval_str, SIZEOF(local_str) - 1);
+		len_str = strlen(pparm->pval_str);
+		if (len_str >= sizeof(local_str))	/* BYPASSOK: both size_t */
+			len_str = sizeof(local_str) - 1;
+		memcpy(local_str, pparm->pval_str, len_str);
+		local_str[len_str] = '\0';
 		ptr_next_val = local_str;
 		while (NULL != ptr_next_val)
-		{	/* WARNING assignment below */
-			STRNCPY_STR(tmp_str, ptr_next_val, (len_str = MIN(STRLEN(ptr_next_val), SIZEOF(tmp_str))));
+		{
+			STRNCPY_STR(tmp_str, ptr_next_val, len_str);
 			tmp_str[len_str] = 0;
 			tmp_str_ptr = tmp_str;
 			ptr_next_comma = strchr(tmp_str_ptr, ',');
@@ -616,8 +621,7 @@ boolean_t cli_get_sub_quals(CLI_ENTRY *pparm)
 			 */
 			if (-1 == (neg_flg = cli_check_negated( &tmp_str_ptr, pparm_qual, &pparm1)))
 				return FALSE;
-			if (1 == neg_flg)
-				len_str -=  STRLEN(NO_STRING);
+			neg_adj_len_str = len_str - ((1 == neg_flg) ? strlen(NO_STRING) : 0);
 
 			if ((ptr_equal) && (ptr_equal + 1 < ptr_next_comma))
 				val_flg = TRUE;
@@ -666,11 +670,12 @@ boolean_t cli_get_sub_quals(CLI_ENTRY *pparm)
 						free(pparm1->pval_str);
 				}
 				if ((!val_flg) && (VAL_NOT_REQ == pparm1->required) && pparm1->parm_values)
-					MALLOC_CPY_STR(pparm1->pval_str, pparm1->parm_values->prompt);
+					pparm1->pval_str = malloc_cpy_str(pparm1->parm_values->prompt);
 				if (val_flg)
 				{
-					ptr_equal_len = STRLEN(ptr_equal + 1);
+					ptr_equal_len = strlen(ptr_equal + 1);
 					pparm1->pval_str = malloc(ptr_equal_len + 1);
+					assert(NULL != pparm1->pval_str);
 					STRNCPY_STR(pparm1->pval_str, ptr_next_val + (ptr_equal - tmp_str_ptr) + 1, ptr_equal_len);
 					pparm1->pval_str[ptr_equal_len] = 0;
 				}
@@ -681,8 +686,9 @@ boolean_t cli_get_sub_quals(CLI_ENTRY *pparm)
 			pparm1->negated = neg_flg;
 			pparm1->present = CLI_PRESENT;
 			has_a_qual = TRUE;
-			if ((tmp_str_ptr + len_str) != ptr_next_comma)
+			if ((tmp_str_ptr + neg_adj_len_str) != ptr_next_comma)
 			{
+				ptr_prev_val = ptr_next_val;
 				ptr_next_val = strchr(ptr_next_val, ',') + 1;
 				if (!ptr_next_val || !*ptr_next_val)
 				{
@@ -691,6 +697,10 @@ boolean_t cli_get_sub_quals(CLI_ENTRY *pparm)
 					return FALSE;
 
 				}
+				assert(ptr_next_val > ptr_prev_val);
+				len_str -= ptr_next_val - ptr_prev_val;
+				assert(0 <= len_str);
+				assert(MAX_LINE >= len_str);
 			} else
 				ptr_next_val = NULL;
 		}
