@@ -1,6 +1,7 @@
 /****************************************************************
  *								*
- *	Copyright 2001, 2012 Fidelity Information Services, Inc	*
+ * Copyright (c) 2001-2022 Fidelity National Information	*
+ * Services, Inc. and/or its subsidiaries. All rights reserved.	*
  *								*
  *	This source code contains the intellectual property	*
  *	of its copyright holder(s), and is made available	*
@@ -26,7 +27,7 @@ void dec_err(uint4 argcnt, ...)
 	uint4		i, j, count, err;
 	const err_ctl 	*ec;
 	const err_msg	*em;
-	char		msgbuff[2048];
+	char		msgbuff[OUT_BUFF_SIZE];
 	mstr		msgstr;
 	DCL_THREADGBL_ACCESS;
 
