@@ -17,17 +17,6 @@ error_def(ERR_NOMORESEMCNT);
 
 int do_semop(int sems, int num, int op, int flg);
 
-/* Set the flag to ignore a counter semaphore */
-#define SEM_COUNTER_OFFLINE(STYPE, TSD, CSA, REG)										\
-{																\
-	if ((NULL != TSD) && (TSD)->mumps_can_bypass)										\
-	{															\
-		(TSD)->STYPE##_counter_halted = TRUE;										\
-		send_msg_csa(CSA_ARG(CSA) VARLSTCNT(7) ERR_NOMORESEMCNT, 5, LEN_AND_LIT(#STYPE), FILE_TYPE_DB, DB_LEN_STR(REG));\
-	}															\
-}
-
-
 /* Check whether a counter semaphore is ignored or not */
 #define IS_SEM_COUNTER_ONLINE(HDR, COUNTER_HALTED) (!((HDR)->mumps_can_bypass) || !(COUNTER_HALTED))
 
