@@ -1,6 +1,7 @@
 /****************************************************************
  *								*
- *	Copyright 2001 Sanchez Computer Associates, Inc.	*
+ * Copyright (c) 2001-2015 Fidelity National Information 	*
+ * Services, Inc. and/or its subsidiaries. All rights reserved.	*
  *								*
  *	This source code contains the intellectual property	*
  *	of its copyright holder(s), and is made available	*
@@ -12,6 +13,6 @@
 #ifndef DOLLARH_INCLUDED
 #define DOLLARH_INCLUDED
 
-void dollarh(time_t intime, uint4 *days, time_t *seconds);
+long dollarh(time_t intime, uint4 *days, time_t *seconds);
 
 #endif /* DOLLARH_INCLUDED */
