@@ -1,6 +1,6 @@
 /****************************************************************
  *								*
- * Copyright (c) 2001-2018 Fidelity National Information	*
+ * Copyright (c) 2001-2021 Fidelity National Information	*
  * Services, Inc. and/or its subsidiaries. All rights reserved.	*
  *								*
  *	This source code contains the intellectual property	*
@@ -33,7 +33,7 @@
 #include "gtmio.h"
 #include "eintr_wrappers.h"
 #include "stringpool.h"
-#include "is_file_identical.h"
+#include "is_gdid.h"
 #include "dpgbldir.h"
 #include "dpgbldir_sysops.h"
 #include "gtm_logicals.h"
@@ -54,6 +54,9 @@ error_def(ERR_TEXT);
 error_def(ERR_BADTAG);
 #endif
 
+/* 30 millisec is an arbitrarily chosen value yielding a wait that seems sufficient, but not too annoying */
+#define WAIT_OUT_RENAME_GAP	30
+
 mstr *get_name(mstr *ms)
 {
 	int4	status;
@@ -68,7 +71,7 @@ mstr *get_name(mstr *ms)
 	pblk.def1_size = SIZEOF(DEF_GDR_EXT) - 1;
 	status = parse_file(ms,&pblk);
 	if (!(status & 1))
-		rts_error_csa(CSA_ARG(NULL)
+		RTS_ERROR_CSA_ABT(NULL,
 			VARLSTCNT(9) ERR_ZGBLDIRACC, 6, ms->len, ms->addr, LEN_AND_LIT(""), LEN_AND_LIT(""), status);
 	new = (mstr *)malloc(SIZEOF(mstr));
 	new->len = pblk.b_esl;
@@ -81,6 +84,7 @@ void *open_gd_file(mstr *v)
 
 {
 	file_pointer	*fp;
+	int4		lcnt;
 	mstr		temp;
 	ZOS_ONLY(int	realfiletag;)
 
@@ -89,7 +93,13 @@ void *open_gd_file(mstr *v)
 	fp->v.addr = (char *)malloc(v->len + 1);
 	memcpy(fp->v.addr, v->addr, v->len);
 	*((char*)((char*)fp->v.addr + v->len)) = 0;	/* Null terminate string */
-	if (FD_INVALID == (fp->fd = OPEN(fp->v.addr, O_RDONLY)))
+	for (lcnt = WAIT_OUT_RENAME_GAP; lcnt; lcnt--)
+	{	/* try OPEN for enough time to get past any possibility the file has gone missing due to a GDE making a revision */
+		if (FD_INVALID != (fp->fd = OPEN(fp->v.addr, O_RDONLY)))
+			break;
+		SHORT_SLEEP(lcnt);
+	}
+	if (FD_INVALID == fp->fd)
 	{	/* v gets passed down through a few levels, but should be freed */
 		/* Copy the values into the stringpool so they get cleaned up later */
 		ENSURE_STP_FREE_SPACE(fp->v.len);
@@ -108,7 +118,7 @@ void *open_gd_file(mstr *v)
 				LEN_AND_LIT(".  Cannot continue"), LEN_AND_LIT(""), errno);
 			assert(FALSE);
 		}
-		rts_error_csa(CSA_ARG(NULL) VARLSTCNT(9) ERR_ZGBLDIRACC, 6, temp.len, temp.addr, LEN_AND_LIT(".  Retaining "),
+		RTS_ERROR_CSA_ABT(NULL, VARLSTCNT(9) ERR_ZGBLDIRACC, 6, temp.len, temp.addr, LEN_AND_LIT(".  Retaining "),
 			dollar_zgbldir.str.len, dollar_zgbldir.str.addr, errno);
 	}
 #ifdef __MVS__
@@ -138,7 +148,7 @@ void fill_gd_addr_id(gd_addr *gd_ptr, file_pointer *file_ptr)
 	gd_ptr->id = (gd_id *)malloc(SIZEOF(gd_id));
 	FSTAT_FILE(file_ptr->fd, &buf, fstat_res);
 	if (-1 == fstat_res)
-		rts_error_csa(CSA_ARG(NULL) VARLSTCNT(9) ERR_ZGBLDIRACC, 6, file_ptr->v.len, file_ptr->v.addr,
+		RTS_ERROR_CSA_ABT(NULL, VARLSTCNT(9) ERR_ZGBLDIRACC, 6, file_ptr->v.len, file_ptr->v.addr,
 			LEN_AND_LIT(""), LEN_AND_LIT(""), errno);
 	set_gdid_from_stat(gd_ptr->id, &buf);
 	return;
@@ -160,7 +170,7 @@ void file_read(file_pointer *file_ptr, int4 size, uchar_ptr_t buff, int4 pos)
 	LSEEKREAD(file_ptr->fd, (off_t)(pos - 1 ) * DISK_BLOCK_SIZE, buff, size, save_errno);
 	if (0 != save_errno)
 		if (-1 == save_errno)
-			rts_error_csa(CSA_ARG(NULL) VARLSTCNT(1) ERR_IOEOF);
+			RTS_ERROR_CSA_ABT(NULL, VARLSTCNT(1) ERR_IOEOF);
 		else
 			rts_error_csa(CSA_ARG(NULL) VARLSTCNT(9) ERR_ZGBLDIRACC, 6, file_ptr->v.len, file_ptr->v.addr,
 				LEN_AND_LIT(""), LEN_AND_LIT(""), save_errno);
