@@ -1,6 +1,6 @@
 /****************************************************************
  *								*
- * Copyright (c) 2001-2020 Fidelity National Information	*
+ * Copyright (c) 2001-2022 Fidelity National Information	*
  * Services, Inc. and/or its subsidiaries. All rights reserved.	*
  *								*
  *	This source code contains the intellectual property	*
@@ -131,13 +131,13 @@ STATICFNDEF int extend_wait_for_write(unix_db_info *udi, int blk_size, off_t new
 	return save_errno;
 }
 
-uint4	 gdsfilext(block_id blocks, block_id filesize, boolean_t trans_in_prog)
+int4 gdsfilext(block_id blocks, block_id filesize, boolean_t trans_in_prog)
 {
 	sm_uc_ptr_t		old_base[2], mmap_retaddr;
 	boolean_t		was_crit, is_mm;
 	int			fd, result, save_errno, status, to_msg, to_wait, wait_period;
 	DEBUG_ONLY(int		first_save_errno);
-	block_id		new_bit_maps, bplmap, map, new_blocks, new_total, max_tot_blks, old_total;
+	block_id		new_bit_maps, bplmap, map, new_blocks, new_total, max_tot_blks, old_total, temp_blk;
 	uint4			jnl_status;
 	gtm_uint64_t		avail_blocks, mmap_sz;
 	off_t			new_eof, new_size, old_size;
@@ -162,14 +162,12 @@ uint4	 gdsfilext(block_id blocks, block_id filesize, boolean_t trans_in_prog)
 	is_mm = (dba_mm == cs_addrs->hdr->acc_meth);
 #	if !defined(MM_FILE_EXT_OK)
 	if (!udi->grabbed_access_sem && is_mm)
-		return (uint4)(NO_FREE_SPACE); /* should this be changed to show extension not allowed ? */
+		return NO_FREE_SPACE; /* should this be changed to show extension not allowed ? */
 #	endif
-	/* Both blocks and total blocks are unsigned ints so make sure we aren't asking for huge numbers that will
-	 * overflow and end up doing silly things.
-	 */
+	/* Make sure that there is enough space left in the database to add the requested blocks */
 	assert((blocks <= (MAXTOTALBLKS(cs_data) - cs_data->trans_hist.total_blks)) || WBTEST_ENABLED(WBTEST_FILE_EXTEND_ERROR));
 	if (!blocks && (cs_data->defer_allocate || (TRANS_IN_PROG_TRUE == trans_in_prog)))
-		return (uint4)(NO_FREE_SPACE); /* should this be changed to show extension not enabled ? */
+		return NO_FREE_SPACE; /* should this be changed to show extension not enabled ? */
 	bplmap = cs_data->bplmap;
 	/* New total of non-bitmap blocks will be number of current, non-bitmap blocks, plus new blocks desired
 	 * There are (bplmap - 1) non-bitmap blocks per bitmap, so add (bplmap - 2) to number of non-bitmap blocks
@@ -187,12 +185,12 @@ uint4	 gdsfilext(block_id blocks, block_id filesize, boolean_t trans_in_prog)
 	{
 		assert(WBTEST_ENABLED(WBTEST_FILE_EXTEND_ERROR));
 		send_msg_csa(CSA_ARG(cs_addrs) VARLSTCNT(1) ERR_TOTALBLKMAX);
-		return (uint4)(NO_FREE_SPACE);
+		return NO_FREE_SPACE;
 	}
 	if (0 != (save_errno = disk_block_available(udi->fd, &avail_blocks, FALSE)))
 	{
 		send_msg_csa(CSA_ARG(cs_addrs) VARLSTCNT(5) ERR_DBFILERR, 2, DB_LEN_STR(gv_cur_region), save_errno);
-		rts_error_csa(CSA_ARG(cs_addrs) VARLSTCNT(5) ERR_DBFILERR, 2, DB_LEN_STR(gv_cur_region), save_errno);
+		RTS_ERROR_CSA_ABT(cs_addrs, VARLSTCNT(5) ERR_DBFILERR, 2, DB_LEN_STR(gv_cur_region), save_errno);
 	} else
 	{
 		if (!(gtmDebugLevel & GDL_IgnoreAvailSpace))
@@ -202,24 +200,27 @@ uint4	 gdsfilext(block_id blocks, block_id filesize, boolean_t trans_in_prog)
 			avail_blocks = avail_blocks / (cs_data->blk_size / DISK_BLOCK_SIZE);
 			if ((blocks * EXTEND_WARNING_FACTOR) > avail_blocks)
 			{
-				if (blocks > (uint4)avail_blocks)
+				if (blocks > avail_blocks)
 				{
 					if (!INST_FREEZE_ON_NOSPC_ENABLED(cs_addrs, local_jnlpool))
-						return (uint4)(NO_FREE_SPACE);
+						return NO_FREE_SPACE;
 					else
 						send_msg_csa(CSA_ARG(cs_addrs) VARLSTCNT(6) MAKE_MSG_WARNING(ERR_NOSPACEEXT), 4,
-							DB_LEN_STR(gv_cur_region), new_blocks, (uint4)avail_blocks);
+							DB_LEN_STR(gv_cur_region), &new_blocks, &avail_blocks);
 				} else
+				{
+					temp_blk = avail_blocks - ((new_blocks <= avail_blocks) ? new_blocks : 0);
 					send_msg_csa(CSA_ARG(cs_addrs) VARLSTCNT(5) ERR_DSKSPACEFLOW, 3, DB_LEN_STR(gv_cur_region),
-						 (uint4)(avail_blocks - ((new_blocks <= avail_blocks) ? new_blocks : 0)));
+						&temp_blk);
+				}
 			}
 		}
 	}
 #	ifdef DEBUG
-	if ( (WBTEST_ENABLED(WBTEST_MM_CONCURRENT_FILE_EXTEND) && dollar_tlevel && !MEMCMP_LIT(gv_cur_region->rname, "DEFAULT")) ||
+	if (((WBTEST_ENABLED(WBTEST_MM_CONCURRENT_FILE_EXTEND) && dollar_tlevel && !MEMCMP_LIT(gv_cur_region->rname, "DEFAULT")) ||
 	     (WBTEST_ENABLED(WBTEST_WSSTATS_PAUSE) && (10 == gtm_white_box_test_case_count) &&
-	     !MEMCMP_LIT(gv_cur_region->rname, "DEFAULT")) )
-	{
+	     !MEMCMP_LIT(gv_cur_region->rname, "DEFAULT"))) && !cs_addrs->now_crit)
+	{	/* Sync with copy in bm_getfree() */
 		/* Not clear to me why the WBTEST_MM_CONCURRENT_FILE_EXTEND doesn't need something similar, but we don't want our
 		 * child to come here.  Unsetting the env shouldn't affect the parent, it reads env just once at process startup*/
 		if(WBTEST_ENABLED(WBTEST_WSSTATS_PAUSE))
@@ -278,15 +279,15 @@ uint4	 gdsfilext(block_id blocks, block_id filesize, boolean_t trans_in_prog)
 		 */
 		assert(CDB_STAGNATE <= t_tries);
 		GDSFILEXT_CLNUP;
-		return (uint4)FINAL_RETRY_FREEZE_PROG;
+		return FINAL_RETRY_FREEZE_PROG;
 	} else
 		WAIT_FOR_REGION_TO_UNCHILL(cs_addrs, cs_data);
 	assert(!cs_addrs->jnlpool || (cs_addrs->jnlpool == jnlpool));
 	if (IS_REPL_INST_FROZEN && trans_in_prog)
 	{
-		assert(CDB_STAGNATE <= t_tries);
+		assert((CDB_STAGNATE <= t_tries) || TREF(in_bm_getfree_gdsfilext));
 		GDSFILEXT_CLNUP;
-		return (uint4)FINAL_RETRY_INST_FREEZE;
+		return FINAL_RETRY_INST_FREEZE;
 	}
 	assert(cs_addrs->ti->total_blks == cs_data->trans_hist.total_blks);
 	old_total = cs_data->trans_hist.total_blks;
@@ -297,19 +298,16 @@ uint4	 gdsfilext(block_id blocks, block_id filesize, boolean_t trans_in_prog)
 		assert((old_total > filesize) || !is_mm);
 		/* For BG, someone else could have truncated or extended - we have no idea */
 		GDSFILEXT_CLNUP;
-		return (SS_NORMAL);
+		return SS_NORMAL;
 	}
 	if (trans_in_prog && SUSPICIOUS_EXTEND)
 	{
+		assertpro(is_free_blks_ctr_ok());
 		if (!was_crit)
 		{
 			GDSFILEXT_CLNUP;
-			return (uint4)(EXTEND_SUSPECT);
+			return EXTEND_SUSPECT;
 		}
-		/* If free_blocks counter is not ok, then correct it. Do the check again. If still fails, then it means we held
-		 * crit through bm_getfree into gdsfilext and still didn't get it right.
-		 */
-		assertpro(!is_free_blks_ctr_ok() && !SUSPICIOUS_EXTEND);
 	}
 	if (JNL_ENABLED(cs_data))
 	{
@@ -327,7 +325,7 @@ uint4	 gdsfilext(block_id blocks, block_id filesize, boolean_t trans_in_prog)
 		{
 			GDSFILEXT_CLNUP;
 			send_msg_csa(CSA_ARG(cs_addrs) VARLSTCNT(6) jnl_status, 4, JNL_LEN_STR(cs_data), DB_LEN_STR(gv_cur_region));
-			return (uint4)(NO_FREE_SPACE);	/* should have better return status */
+			return NO_FREE_SPACE;	/* should have better return status */
 		}
 	}
 	if (is_mm)
@@ -350,7 +348,7 @@ uint4	 gdsfilext(block_id blocks, block_id filesize, boolean_t trans_in_prog)
 			GDSFILEXT_CLNUP;
 			send_msg_csa(CSA_ARG(cs_addrs) VARLSTCNT(12) ERR_DBFILERR, 2, DB_LEN_STR(gv_cur_region),
 					ERR_SYSCALL, 5, LEN_AND_STR(MEM_UNMAP_SYSCALL), CALLFROM, save_errno);
-			return (uint4)(NO_FREE_SPACE);
+			return NO_FREE_SPACE;
 		}
 	} else
 	{	/* Due to concurrency issues, it is possible some process had issued a disk read of the GDS block# corresponding
@@ -381,7 +379,7 @@ uint4	 gdsfilext(block_id blocks, block_id filesize, boolean_t trans_in_prog)
 		 * the file is more than half the size of the partition, the next posix_fallocate will fail
 		 * for a 0 increase use the documented way
 		 */
-		save_errno = posix_fallocate(fd, old_size, new_size - old_size);
+		POSIX_FALLOCATE(fd, old_size, new_size - old_size, save_errno);
 		DEBUG_ONLY(first_save_errno = save_errno);
 		if ((ENOSPC == save_errno) && IS_GTM_IMAGE)
 		{
@@ -394,7 +392,7 @@ uint4	 gdsfilext(block_id blocks, block_id filesize, boolean_t trans_in_prog)
 					ISSUE_WAITDSKSPACE(to_wait, wait_period, send_msg_csa);
 				hiber_start(1000);
 				to_wait--;
-				save_errno = posix_fallocate(fd, old_size, new_size - old_size);
+				POSIX_FALLOCATE(fd, old_size, new_size - old_size, save_errno);
 			} while ((to_wait > 0) && (ENOSPC == save_errno));
 		}
 		if (0 != save_errno)
@@ -404,7 +402,7 @@ uint4	 gdsfilext(block_id blocks, block_id filesize, boolean_t trans_in_prog)
 			if (ENOSPC != save_errno)
 				send_msg_csa(CSA_ARG(cs_addrs) VARLSTCNT(5) ERR_PREALLOCATEFAIL, 2, DB_LEN_STR(gv_cur_region),
 					save_errno);
-			return (uint4)(NO_FREE_SPACE);
+			return NO_FREE_SPACE;
 		}
 	}
 	save_errno = db_write_eof_block(udi, udi->fd, cs_data->blk_size, new_eof, &(TREF(dio_buff)));
@@ -415,7 +413,7 @@ uint4	 gdsfilext(block_id blocks, block_id filesize, boolean_t trans_in_prog)
 		GDSFILEXT_CLNUP;
 		if (ENOSPC != save_errno)
 			send_msg_csa(CSA_ARG(cs_addrs) VARLSTCNT(5) ERR_DBFILERR, 2, DB_LEN_STR(gv_cur_region), save_errno);
-		return (uint4)(NO_FREE_SPACE);
+		return NO_FREE_SPACE;
 	}
 	if (WBTEST_ENABLED(WBTEST_FILE_EXTEND_INTERRUPT_1))
 	{
@@ -434,7 +432,7 @@ uint4	 gdsfilext(block_id blocks, block_id filesize, boolean_t trans_in_prog)
 			GDSFILEXT_CLNUP;
 			send_msg_csa(CSA_ARG(cs_addrs) VARLSTCNT(8) ERR_DBFILERR, 5,
 				RTS_ERROR_LITERAL("fsync1()"), CALLFROM, status);
-			return (uint4)(NO_FREE_SPACE);
+			return NO_FREE_SPACE;
 		}
 	}
 	if (WBTEST_ENABLED(WBTEST_FILE_EXTEND_INTERRUPT_2))
@@ -482,7 +480,7 @@ uint4	 gdsfilext(block_id blocks, block_id filesize, boolean_t trans_in_prog)
 			{
 				GDSFILEXT_CLNUP;
 				send_msg_csa(CSA_ARG(cs_addrs) VARLSTCNT(5) ERR_DBFILERR, 2, DB_LEN_STR(gv_cur_region), status);
-				return (uint4)(NO_FREE_SPACE);
+				return NO_FREE_SPACE;
 			}
 		}
 		assert(0 == new_bit_maps);
@@ -504,7 +502,7 @@ uint4	 gdsfilext(block_id blocks, block_id filesize, boolean_t trans_in_prog)
 			GDSFILEXT_CLNUP;
 			send_msg_csa(CSA_ARG(cs_addrs) VARLSTCNT(8) ERR_DBFILERR, 5, RTS_ERROR_LITERAL("fsync2()"), CALLFROM,
 				     status);
-			return (uint4)(NO_FREE_SPACE);
+			return NO_FREE_SPACE;
 		}
 	}
 	if (WBTEST_ENABLED(WBTEST_FILE_EXTEND_INTERRUPT_4))
@@ -513,8 +511,8 @@ uint4	 gdsfilext(block_id blocks, block_id filesize, boolean_t trans_in_prog)
 		assert(FALSE); /* Should be killed before that */
 	}
 	assert(cs_data->blks_to_upgrd == (inctn_detail.blks2upgrd_struct.blks_to_upgrd_delta + prev_extend_blks_to_upgrd));
-	assert(0 < (int)blocks || (!cs_data->defer_allocate && (0 == new_blocks)));
-	assert(0 < (int)(cs_addrs->ti->free_blocks + blocks));
+	assert((0 < blocks) || (!cs_data->defer_allocate && (0 == new_blocks)));
+	assert(0 < (cs_addrs->ti->free_blocks + blocks));
 	cs_addrs->ti->free_blocks += blocks;
 	cs_addrs->total_blks = cs_addrs->ti->total_blks = new_total;
 	blocks = old_total;
@@ -564,7 +562,7 @@ uint4	 gdsfilext(block_id blocks, block_id filesize, boolean_t trans_in_prog)
 			GDSFILEXT_CLNUP;
 			send_msg_csa(CSA_ARG(cs_addrs) VARLSTCNT(12) ERR_DBFILERR, 2, DB_LEN_STR(gv_cur_region),
 					ERR_SYSCALL, 5, LEN_AND_STR(MEM_MAP_SYSCALL), CALLFROM, save_errno);
-			return (uint4)(NO_FREE_SPACE);
+			return NO_FREE_SPACE;
 		}
 		/* In addition to updating the internal map values, gds_map_moved sets cs_data to point to the remapped file */
 #		if defined(_AIX)
@@ -578,13 +576,13 @@ uint4	 gdsfilext(block_id blocks, block_id filesize, boolean_t trans_in_prog)
 	INCR_GVSTATS_COUNTER(cs_addrs, cs_addrs->nl, n_db_extends, 1);
 	if (!gtm_dbfilext_syslog_disable)
 	{
-		send_msg_csa(CSA_ARG(cs_addrs) VARLSTCNT(7) ERR_DBFILEXT, 5, DB_LEN_STR(gv_cur_region), blocks, new_total,
+		send_msg_csa(CSA_ARG(cs_addrs) VARLSTCNT(7) ERR_DBFILEXT, 5, DB_LEN_STR(gv_cur_region), &blocks, &new_total,
 			&curr_tn);
-		if ((NULL != gv_cur_region) && (NULL != gv_cur_region->dyn.addr) && (NULL != gv_cur_region->dyn.addr->fname))
+		if ((NULL != gv_cur_region) && (NULL != gv_cur_region->dyn.addr) && (0 != gv_cur_region->dyn.addr->fname_len))
 			db_file_name = (char *)gv_cur_region->dyn.addr->fname;
 		else
 			db_file_name = "";
 		warn_db_sz(db_file_name, blocks, new_total, MAXTOTALBLKS(cs_data));
 	}
-	return (SS_NORMAL);
+	return SS_NORMAL;
 }
