@@ -1,6 +1,6 @@
 /****************************************************************
  *								*
- * Copyright (c) 2006-2019 Fidelity National Information	*
+ * Copyright (c) 2006-2021 Fidelity National Information	*
  * Services, Inc. and/or its subsidiaries. All rights reserved.	*
  *								*
  *	This source code contains the intellectual property	*
@@ -200,7 +200,7 @@ static boolean_t parse_gtm_icu_version(char *icu_ver_buf, int len, char *icusymv
 		/* Construct the first part of the ICUVERLT36 error message. */
 		SNPRINTF(tmp_errstr, SIZEOF(GTM_ICU_VERSION) + STR_LIT_LEN(GTM_ICU_VERSION_SUFFIX), "%s%s", GTM_ICU_VERSION,
 			GTM_ICU_VERSION_SUFFIX);
-		rts_error_csa(CSA_ARG(NULL) VARLSTCNT(6) ERR_ICUVERLT36, 4, LEN_AND_STR(tmp_errstr), major_ver, minor_ver);
+		RTS_ERROR_CSA_ABT(NULL, VARLSTCNT(6) ERR_ICUVERLT36, 4, LEN_AND_STR(tmp_errstr), major_ver, minor_ver);
 	}
 	return TRUE;
 }
@@ -248,9 +248,9 @@ void gtm_icu_init(void)
 	ENABLE_INTERRUPTS(INTRPT_IN_FUNC_WITH_MALLOC, prev_intrpt_state);
 	chset = nl_langinfo(CODESET);
 	if (NULL == chset)	/* 4SCA : null return nl_langinfo not possible */
-		rts_error_csa(CSA_ARG(NULL) VARLSTCNT(4) ERR_NONUTF8LOCALE, 2, LEN_AND_LIT("<undefined>"));
+		RTS_ERROR_CSA_ABT(NULL, VARLSTCNT(4) ERR_NONUTF8LOCALE, 2, LEN_AND_LIT("<undefined>"));
 	if ((NULL == locale) || (0 != strcasecmp(chset, "utf-8")) && (0 != strcasecmp(chset, "utf8")))
-		rts_error_csa(CSA_ARG(NULL) VARLSTCNT(4) ERR_NONUTF8LOCALE, 2, LEN_AND_STR(chset));
+		RTS_ERROR_CSA_ABT(NULL, VARLSTCNT(4) ERR_NONUTF8LOCALE, 2, LEN_AND_STR(chset));
 	/* By default, GT.M will henceforth expect that ICU has been built with symbol renaming disabled. If that is not the case,
 	 * GT.M can be notified of this through an environment variable (macro GTM_ICU_VERSION).  The variable should contain the
 	 * icu version number formatted as "<major-ver>.<minor-ver>". Example would be "3.6" to indicate ICU 3.6.
@@ -317,7 +317,7 @@ void gtm_icu_init(void)
 		{
 			ENABLE_INTERRUPTS(INTRPT_IN_FUNC_WITH_MALLOC, prev_intrpt_state);
 			SNPRINTF(err_msg, MAX_ERRSTR_LEN, "dlopen(%s)", libname);
-			rts_error_csa(CSA_ARG(NULL) VARLSTCNT(3) ERR_RESTRICTEDOP, 1, err_msg);
+			RTS_ERROR_CSA_ABT(NULL, VARLSTCNT(3) ERR_RESTRICTEDOP, 1, err_msg);
 		}
 #		ifndef _AIX
 		if (NULL == realpath(librarypath, real_path))
@@ -406,8 +406,8 @@ void gtm_icu_init(void)
 		if (NULL == each_libpath)
 		{
 			ENABLE_INTERRUPTS(INTRPT_IN_FUNC_WITH_MALLOC, prev_intrpt_state);
-			rts_error_csa(CSA_ARG(NULL) VARLSTCNT(8) ERR_DLLNOOPEN, 2, LEN_AND_STR(libname),
-					ERR_TEXT, 2, LEN_AND_LIT(ICU_NOT_FOUND_ERR));
+			RTS_ERROR_CSA_ABT(NULL, VARLSTCNT(8) ERR_DLLNOOPEN, 2, LEN_AND_STR(libname),
+				ERR_TEXT, 2, LEN_AND_LIT(ICU_NOT_FOUND_ERR));
 		}
 		libname = buf;
 		handle = dlopen(libname, iculdflags);
@@ -431,8 +431,8 @@ void gtm_icu_init(void)
 			ENABLE_INTERRUPTS(INTRPT_IN_FUNC_WITH_MALLOC, prev_intrpt_state);
 			libname = buf;
 #		endif
-			rts_error_csa(CSA_ARG(NULL) VARLSTCNT(8) ERR_DLLNOOPEN, 2, LEN_AND_STR(libname),
-					ERR_TEXT, 2, LEN_AND_STR(err_msg));
+			RTS_ERROR_CSA_ABT(NULL, VARLSTCNT(8) ERR_DLLNOOPEN, 2, LEN_AND_STR(libname),
+				ERR_TEXT, 2, LEN_AND_STR(err_msg));
 #		ifdef _AIX
 		}
 #		endif
@@ -474,7 +474,7 @@ void gtm_icu_init(void)
 			if (NULL == fptr)
 			{
 				COPY_DLLERR_MSG(err_str, err_msg);
-				rts_error_csa(CSA_ARG(NULL) VARLSTCNT(8) ERR_ICUSYMNOTFOUND, 2, LEN_AND_STR(cur_icu_fname),
+				RTS_ERROR_CSA_ABT(NULL, VARLSTCNT(8) ERR_ICUSYMNOTFOUND, 2, LEN_AND_STR(cur_icu_fname),
 					ERR_TEXT, 2, LEN_AND_STR(err_msg));
 			}
 			if (0 == findx)	/* record the fact that the symbols ARE renamed */
@@ -501,8 +501,8 @@ void gtm_icu_init(void)
 				/* Construct the first part of the ICUVERLT36 error message. */
 				SNPRINTF(tmp_errstr, SIZEOF(ICU_LIBNAME) + STR_LIT_LEN(ICU_LIBNAME_SUFFIX), "%s%s", ICU_LIBNAME,
 					ICU_LIBNAME_SUFFIX);
-				rts_error_csa(CSA_ARG(NULL) VARLSTCNT(6) ERR_ICUVERLT36, 4,
-						LEN_AND_STR(tmp_errstr), icu_version[0], icu_version[1]);
+				RTS_ERROR_CSA_ABT(NULL, VARLSTCNT(6) ERR_ICUVERLT36, 4,
+					LEN_AND_STR(tmp_errstr), icu_version[0], icu_version[1]);
 			}
 		}
 	}
