@@ -1,6 +1,6 @@
 /****************************************************************
  *								*
- * Copyright (c) 2001-2020 Fidelity National Information	*
+ * Copyright (c) 2001-2022 Fidelity National Information	*
  * Services, Inc. and/or its subsidiaries. All rights reserved.	*
  *								*
  *	This source code contains the intellectual property	*
@@ -70,10 +70,12 @@
 #define	GTM_ZROUTINES			"$gtmroutines"
 #define	GTM_BOOLEAN			"$gtm_boolean"
 #define	DISABLE_ALIGN_STRINGS		"$gtm_disable_alignstr"
+#define	GTM_HUGETLB_SHM			"$gtm_hugetlb_shm"
 #define	GTM_MAX_SOCKETS			"$gtm_max_sockets"
 #define	GTM_MEMORY_RESERVE		"$gtm_memory_reserve"
 #define	GTM_NOFFLF			"$gtm_nofflf"
 #define	GTM_NOUNDEF			"$gtm_noundef"
+#define	GTM_PINSHM			"$gtm_pinshm"
 #define	GTM_PRINCIPAL			"$gtm_principal"
 #define	GTM_PROMPT			"$gtm_prompt"
 #define	GTM_SIDE_EFFECT			"$gtm_side_effects"
@@ -100,6 +102,7 @@
 #ifdef USE_LIBAIO
 #define GTM_AIO_NR_EVENTS		"$gtm_aio_nr_events"
 #endif
+#define	GTM_DB_CREATE_VER		"$gtm_db_create_ver"
 
 /* Replication */
 #define	GTM_REPL_INSTANCE		"$gtm_repl_instance"
@@ -144,7 +147,7 @@
 #define	GTM_EXTRACT_NOCOL		"$gtm_extract_nocol"
 #define	GTMDBGFLAGS			"$gtmdbgflags"
 #define	GTMDBGFLAGS_FREQ		"$gtmdbgflags_freq"
-#define GTM_MAX_STORALLOC		"$gtm_max_storalloc"
+#define GTM_MALLOC_LIMIT		"$gtm_malloc_limit"
 #define GTM_MSTACK_SIZE			"$gtm_mstack_size"
 #define GTM_MSTACK_CRIT_THRESH		"$gtm_mstack_crit_threshold"
 #define GTM_IPV4_ONLY			"$gtm_ipv4_only"
@@ -155,4 +158,3 @@
 #define GTM_UTFCGR_STRING_GROUPS	"$gtm_utfcgr_string_groups"
 #define GTM_STRPLLIM			"$gtm_string_pool_limit"
 #define GTM_HUPENABLE			"$gtm_hupenable"
-
