@@ -1,6 +1,6 @@
 /****************************************************************
  *								*
- * Copyright (c) 2013-2020 Fidelity National Information	*
+ * Copyright (c) 2013-2022 Fidelity National Information	*
  * Services, Inc. and/or its subsidiaries. All rights reserved.	*
  *								*
  *	This source code contains the intellectual property	*
@@ -16,9 +16,11 @@
 #ifndef GTM_TLS_INTERFACE_DEFINITIONS_INCLUDED
 #define GTM_TLS_INTERFACE_DEFINITIONS_INCLUDED
 
-#define GTM_TLS_API_VERSION		0x00000004
+#define GTM_TLS_API_VERSION		0x00000006
 #define GTM_TLS_API_VERSION_SOCK	0x00000002	/* when TLS sockets added */
 #define GTM_TLS_API_VERSION_RENEGOPT	0x00000004	/* WRITE /TLS renegotiate with options */
+#define GTM_TLS_API_VERSION_NONBLOCK	0x00000005	/* WRITE /BLOCK("OFF") */
+#define GTM_TLS_API_VERSION_GET_ERROR	0x00000006	/* gtm_tls_get_error() socket ptr for errbuf */
 
 #define MAX_X509_LEN			256
 #define MAX_ALGORITHM_LEN		64
@@ -71,7 +73,7 @@
 #define GTMTLS_OP_REPL_CIPHER		0x00000080
 /* No verify mode specifed at top tls level */
 #define GTMTLS_OP_ABSENT_VERIFYMODE	0x00000100
-/* Server requested renegotiation without waiting for handshake */
+/* This is SSL_OP_ALLOW_CLIENT_RENEGOTIATION; Previously set between V62001-V62002A, but not used */
 #define GTMTLS_OP_RENEGOTIATE_REQUESTED	0x00000200
 /* No gtmcrypt_config  or tls in config needed for client only use */
 #define GTMTLS_OP_ABSENT_CONFIG		0x00000400
@@ -79,6 +81,7 @@
 #define GTMTLS_OP_NOPWDENVVAR		0x00000800
 /* Bit mask for VERIFY_LEVEL options - one now and one planned but allow two more */
 #define GTMTLS_OP_VERIFY_LEVEL_MASK	0x0000F000
+#define GTMTLS_OP_VERIFY_LEVEL_CMPLMNT	0x7FFF0FFF /* 4SCA: 32bit complement */
 /* Check SSL_get_verify_result after SSL_connect and other useful places */
 #define GTMTLS_OP_VERIFY_LEVEL_CHECK	0x00001000
 /* Default VERIFY_LEVEL options */
@@ -87,7 +90,9 @@
 #define GTMTLS_OP_CLIENT_CA	 	0x00010000
 /* CAfile or CApath processed */
 #define GTMTLS_OP_CA_LOADED	 	0x00020000
-#define GTMTMLS_OP_NOSHUTDOWN		0x00040000
+#define GTMTLS_OP_NOSHUTDOWN		0x00040000
+/* NON BLOCKing WRITE so set SSL_MODE_ENABLE_PARTIAL_WRITE	*/
+#define GTMTLS_OP_NONBLOCK_WRITE	0x00080000
 
 #define GTMTLS_IS_FIPS_MODE(CTX)	(TRUE == CTX->fips_mode)
 #define GTMTLS_RUNTIME_LIB_VERSION(CTX)	(CTX->runtime_version)
@@ -124,7 +129,8 @@ typedef struct gtm_tls_ctx_struct
 	unsigned long		compile_time_version;	/* OpenSSL version that this library is compiled with. */
 	unsigned long		runtime_version;	/* OpenSSL version that this library is currently running with. */
 	void			*ctx;
-	int			version;		/* GTM_TLS_API_VERSION */
+	int			version;		/* caller GTM_TLS_API_VERSION */
+	int			plugin_version;		/* added with GTM_TLS_API_VERSION_NONBLOCK */
 } gtm_tls_ctx_t;
 
 typedef struct gtm_tls_session_struct
@@ -134,6 +140,7 @@ typedef struct gtm_tls_session_struct
 	void			*session;
 	char			tlsid[MAX_TLSID_LEN + 1];
 	gtm_tls_ctx_t		*gtm_ctx;
+	char			*errstr;
 } gtm_tls_socket_t;
 
 #endif	/* GTM_TLS_INTERFACE_DEFINITIONS_INCLUDED */
@@ -143,7 +150,7 @@ typedef struct gtm_tls_session_struct
  */
 
 /* Returns the most recent error (null-terminated) related to the workings of the SSL/TLS reference implementation. */
-extern const char		*gtm_tls_get_error(void);
+extern const char		*gtm_tls_get_error(gtm_tls_socket_t *);
 
 /* If the most recent invocation of the SSL/TLS reference implementation resulted in a system call error, `gtm_tls_errno' returns
  * the value of `errno'. Otherwise, -1 is returned in which case `gtm_tls_get_error' provides more information.
@@ -154,8 +161,8 @@ extern int		gtm_tls_errno(void);
  * necessary to initialize the SSL/TLS context are obtained from the configuration file pointed to by `$gtmcrypt_config'.
  *
  * Arguments:
- *   `version' : The API version that the caller understands. Current version is 0x1.
- *   `flags'   : Initialization flags as a bitmask. Currently, the only one the API understands is GTMTLS_OP_INTERACTIVE_MODE.
+ *   `version' : The API version that the caller understands. Current version is 0x5.
+ *   `flags'   : Initialization flags as a bitmask. The first one the API understands is GTMTLS_OP_INTERACTIVE_MODE.
  *               Set this bitmask if the process doing the SSL/TLS initialization is run in an interactive mode. This lets
  *               the API decide if it can prompt for a password if a need arises while decrypting the private key.
  *
@@ -372,4 +379,14 @@ extern void		gtm_tls_session_close(gtm_tls_socket_t **socket);
  */
 extern void		gtm_tls_fini(gtm_tls_ctx_t **ctx);
 
+/* Return the value of GTM_TLS_API_VERSION for the gtm tls plugin.  This can be
+ * used to check if the plugin supports features needed by the caller.
+ *
+ * Argument:
+ *    caller_version	:  GTM_TLS_API_VERSION of caller - not used currently.
+ *
+ * Returns the GTM_TLS_API_VERSION of the plugin.
+ */
+extern int		gtm_tls_version(int caller_version);
+
 #endif
