@@ -1,6 +1,6 @@
 /****************************************************************
  *								*
- * Copyright (c) 2009-2019 Fidelity National Information	*
+ * Copyright (c) 2009-2022 Fidelity National Information	*
  * Services, Inc. and/or its subsidiaries. All rights reserved.	*
  *								*
  *	This source code contains the intellectual property	*
@@ -49,7 +49,7 @@ GBLREF	boolean_t					gtmcrypt_initialized;
 GBLREF	mstr						pvt_crypt_buf;
 GBLREF	char						dl_err[];
 GBLREF	char						*gtmcrypt_badhash_size_msg;
-
+GBLREF	void						(*primary_exit_handler)(void);
 LITREF	char						gtmcrypt_repeat_msg[];
 LITREF	gtm_string_t					null_iv;
 
@@ -199,8 +199,8 @@ MBSTART {															\
 #define INIT_DB_OR_JNL_ENCRYPTION(CSA, CSD, FILENAME_LEN, FILENAME, RC)								\
 {																\
 	RC = 0;															\
-	(CSA)->encr_key_handle = GTMCRYPT_INVALID_KEY_HANDLE;									\
-	(CSA)->encr_key_handle2 = GTMCRYPT_INVALID_KEY_HANDLE;									\
+	(CSA)->encr_key_handle = (void *)GTMCRYPT_INVALID_KEY_HANDLE;								\
+	(CSA)->encr_key_handle2 = (void *)GTMCRYPT_INVALID_KEY_HANDLE;								\
 	if (IS_ENCRYPTED((CSD)->is_encrypted))											\
 	{															\
 		GTMCRYPT_INIT_BOTH_CIPHER_CONTEXTS(CSA, (CSD)->encryption_hash,							\
@@ -229,6 +229,8 @@ MBSTART {															\
 			else													\
 				gtmcrypt_initialized = TRUE; /* Intialization is done for this process. */			\
 			ENABLE_INTERRUPTS(INTRPT_IN_CRYPT_SECTION, prev_intrpt_state);						\
+			if (gtmcrypt_initialized && primary_exit_handler)							\
+				atexit(primary_exit_handler);									\
 		} else														\
 			RC = SET_CRYPTERR_MASK(RC);										\
 	}															\
@@ -250,6 +252,7 @@ MBSTART {															\
 		hash_string.address = (gtm_char_t *)(HASH);									\
 		db_path_string.length = (gtm_long_t)DB_PATH_LENGTH;								\
 		db_path_string.address = (gtm_char_t *)(DB_PATH);								\
+		assert(0 <= db_path_string.length);										\
 		DEFER_INTERRUPTS(INTRPT_IN_CRYPT_SECTION, prev_intrpt_state);							\
 		if (0 != gtmcrypt_init_db_cipher_context_by_hash(&(ENCRYPT_KEY_HANDLE), hash_string, db_path_string, null_iv))	\
 			RC = SET_CRYPTERR_MASK(ERR_CRYPTKEYFETCHFAILED);							\
@@ -261,6 +264,8 @@ MBSTART {															\
 		RC = SET_REPEAT_MSG_MASK((SET_CRYPTERR_MASK(ERR_CRYPTOPFAILED)));						\
 		DEBUG_ONLY(CORE_ON_CRYPTOPFAILED);										\
 	}															\
+	if (gtmcrypt_initialized && primary_exit_handler)									\
+		atexit(primary_exit_handler);											\
 }
 
 /* Ensure that the symmetric key corresponding to the specified hash exists and that a handle is created. */
@@ -336,8 +341,10 @@ MBSTART {															\
 		assert(NULL != HASH);												\
 		filename_string.length = (gtm_long_t)FILENAME_LENGTH;								\
 		filename_string.address = (gtm_char_t *)(FILENAME);								\
+		assert(0 <= filename_string.length);										\
 		key_path_string.length = (gtm_long_t)KEY_PATH_LENGTH;								\
 		key_path_string.address = (gtm_char_t *)(KEY_PATH);								\
+		assert(0 <= key_path_string.length);										\
 		DEFER_INTERRUPTS(INTRPT_IN_CRYPT_SECTION, prev_intrpt_state);							\
 		if (0 != gtmcrypt_obtain_db_key_hash_by_keyname(filename_string, key_path_string, &hash_string))		\
 			RC = SET_CRYPTERR_MASK(ERR_CRYPTKEYFETCHFAILED);							\
