@@ -1,6 +1,6 @@
 /****************************************************************
  *								*
- * Copyright (c) 2013-2019 Fidelity National Information	*
+ * Copyright (c) 2013-2022 Fidelity National Information	*
  * Services, Inc. and/or its subsidiaries. All rights reserved.	*
  *								*
  *	This source code contains the intellectual property	*
@@ -38,6 +38,7 @@
  * implementation.
  */
 GBLDEF	char	gtmcrypt_err_string[MAX_GTMCRYPT_ERR_STRLEN + 1];
+GBLDEF	char	*gtmtls_err_string = NULL;
 #ifndef USE_SYSLIB_FUNCS
 GBLDEF gtm_malloc_fnptr_t		gtm_malloc_fnptr;
 GBLDEF gtm_free_fnptr_t			gtm_free_fnptr;
@@ -141,6 +142,7 @@ int gc_read_passwd(char *prompt, char *buf, int maxlen, void *tty)
 			 * updating the passed-in pointer.
 			 */
 			tty_copy = (struct termios *)MALLOC(SIZEOF(struct termios));
+			assert(NULL != tty_copy);
 			memcpy(tty_copy, &old_tty, SIZEOF(struct termios));
 			*((struct termios **)tty) = tty_copy;
 		}
@@ -354,7 +356,9 @@ int gc_update_passwd(char *name, passwd_entry_t **ppwent, char *prompt, int inte
 		if (NULL != pwent)
 			gc_freeup_pwent(pwent);
 		pwent = MALLOC(SIZEOF(passwd_entry_t));
+		assert(NULL != pwent);
 		pwent->env_value = MALLOC(len ? len + 1 : GTM_PASSPHRASE_MAX * 2 + 1);
+		assert(NULL != pwent->env_value);
 		env_name = pwent->env_name;
 		strncpy(env_name, name, SIZEOF(pwent->env_name));
 		env_name[SIZEOF(pwent->env_name) - 1] = '\0';
@@ -362,6 +366,7 @@ int gc_update_passwd(char *name, passwd_entry_t **ppwent, char *prompt, int inte
 		env_name = pwent->env_name;
 	pwent->passwd_len = len ? len / 2 + 1 : GTM_PASSPHRASE_MAX + 1;
 	pwent->passwd = MALLOC(pwent->passwd_len);
+	assert(NULL != pwent->passwd);
 	env_value = pwent->env_value;
 	passwd = pwent->passwd;
 	if (0 < len)
