@@ -64,6 +64,7 @@
 #define	REL_PATH_TO_GTMSECSHR "./gtmsecshr"
 #define	GTMSECSHR_BASENAME "/gtmsecshr"
 #define MAX_ENV_NAME_LEN 2048
+#define	PERMALL		07777
 
 /* Build up some defines for use with AIX and reading /etc/environment to pick up a default TZ value plus some defines
  * for testing AIX errors in this wrapper with white box test cases.
@@ -306,23 +307,23 @@ int main()
 	/* get the ones we need */
 	if (env_var_ptr = getenv(GTM_DIST))		/* Warning - assignment */
 	{
-		if (MAX_ALLOWABLE_LEN < (strlen(env_var_ptr) + STR_LIT_LEN(SUB_PATH_TO_GTMSECSHRDIR)
-		    + STR_LIT_LEN(GTMSECSHR_BASENAME)))
+		if (MAX_ALLOWABLE_LEN <
+				(strlen(env_var_ptr) + STR_LIT_LEN(SUB_PATH_TO_GTMSECSHRDIR) + STR_LIT_LEN(GTMSECSHR_BASENAME)))
 		{
 			SYSLOG(LOG_USER | LOG_INFO, ERR_SECSHRGTMDIST2LONG);
 			ret = -1;
 		} else
 		{
-			strcpy(gtm_dist_val, env_var_ptr);
+			snprintf(gtm_dist_val, MAX_ENV_VAR_VAL_LEN, "%s", env_var_ptr);
 			/* point the path to the real gtmsecshr - for display purposes only */
-			strcpy(gtm_secshr_path, env_var_ptr);
-			strcat(gtm_secshr_path, SUB_PATH_TO_GTMSECSHRDIR);
-			strcat(gtm_secshr_path, GTMSECSHR_BASENAME);
+			snprintf(gtm_secshr_path, MAX_ENV_VAR_VAL_LEN, "%s%s%s",
+					env_var_ptr, SUB_PATH_TO_GTMSECSHRDIR, GTMSECSHR_BASENAME);
 			strsanitize(gtm_secshr_path, gtm_secshr_path_display);
 			/* point the path to the real gtmsecshrdir */
-			strcpy(gtm_secshrdir_path, env_var_ptr);
-			strcat(gtm_secshrdir_path, SUB_PATH_TO_GTMSECSHRDIR);
+			snprintf(gtm_secshrdir_path, MAX_ENV_VAR_VAL_LEN, "%s%s", env_var_ptr, SUB_PATH_TO_GTMSECSHRDIR);
 			strsanitize(gtm_secshrdir_path, gtm_secshrdir_path_display);
+			/* the path to gtmsecshr wrapper that we want to display in a PS listing */
+			snprintf(gtm_secshr_orig_path, MAX_ENV_VAR_VAL_LEN, "%s%s", env_var_ptr, GTMSECSHR_BASENAME);
 		}
 	} else
 	{
@@ -404,7 +405,7 @@ int main()
 			SYSLOG(LOG_USER | LOG_INFO, ERR_SECSHRNOTOWNEDBYROOT, gtm_secshrdir_path_display);
 		else if (gtm_secshrdir_stat.st_mode & 0277)
 			SYSLOG(LOG_USER | LOG_INFO, ERR_SECSHRPERMINCRCT, gtm_secshrdir_path_display,
-					gtm_secshrdir_stat.st_mode & 0777);
+					gtm_secshrdir_stat.st_mode & PERMALL);
 		else if (-1 == Stat(REL_PATH_TO_GTMSECSHR, &gtm_secshr_stat))
 			SYSLOG(LOG_USER | LOG_INFO, ERR_SECSHRSTATFAILED, gtm_secshr_path_display, errno);
 		else if (ROOTUID != gtm_secshr_stat.st_uid)
@@ -417,8 +418,6 @@ int main()
 			SYSLOG(LOG_USER | LOG_INFO, ERR_SECSHRSETUIDFAILED);
 		else
 		{	/* call the real gtmsecshr, but have ps display the original gtmsecshr location */
-			strcpy(gtm_secshr_orig_path, gtm_dist_val);
-			strcat(gtm_secshr_orig_path, GTMSECSHR_BASENAME);
 			ret = execl(REL_PATH_TO_GTMSECSHR, gtm_secshr_orig_path, NULL);
 			if (-1 == ret)
 				SYSLOG(LOG_USER | LOG_INFO, ERR_SECSHREXECLFAILED, gtm_secshr_path_display);
