@@ -1,6 +1,6 @@
 /****************************************************************
  *								*
- * Copyright (c) 2006-2020 Fidelity National Information	*
+ * Copyright (c) 2006-2022 Fidelity National Information	*
  * Services, Inc. and/or its subsidiaries. All rights reserved.	*
  *								*
  *	This source code contains the intellectual property	*
@@ -47,7 +47,8 @@
 #endif
 
 #define MAX_SECONDARY_LEN 		(MAX_HOST_NAME_LEN + 11) /* +11 for ':' and port number */
-#define DEFAULT_SHUTDOWN_TIMEOUT	30
+#define DEFAULT_SHUTDOWN_TIMEOUT	120
+#define MAX_SHUTDOWN_TIMEOUT		3600 /* 1 hour */
 #define GTMSOURCE_CONN_PARMS_LEN 	((10 + 1) * GTMSOURCE_CONN_PARMS_COUNT - 1)
 
 GBLREF	gtmsource_options_t	gtmsource_options;
@@ -62,6 +63,7 @@ error_def(ERR_BADPARAMCOUNT);
 error_def(ERR_LOGTOOLONG);
 error_def(ERR_REPLINSTSECLEN);
 error_def(ERR_REPLINSTSECUNDF);
+error_def(ERR_INVSHUTDOWN);
 
 int gtmsource_get_opt(void)
 {
@@ -230,7 +232,8 @@ int gtmsource_get_opt(void)
 			     connect_parm_token_str = connect_parms_str;
 			     !connect_parms_badval &&
 			     connect_parms_index <= GTMSOURCE_CONN_PARMS_COUNT &&
-			     (connect_parm = STRTOK_R(connect_parm_token_str, GTMSOURCE_CONN_PARMS_DELIM, &strtokptr)) != NULL;
+			     /* inline assignment */
+			     (NULL != (connect_parm = STRTOK_R(connect_parm_token_str, GTMSOURCE_CONN_PARMS_DELIM, &strtokptr)));
 			     connect_parms_index++,
 			     connect_parm_token_str = NULL)
 			{
@@ -446,10 +449,17 @@ int gtmsource_get_opt(void)
 					return -1;
 				} else if ((0 < renegotiate_interval) && (renegotiate_interval < MIN_RENEGOTIATE_TIMEOUT))
 					renegotiate_interval = MIN_RENEGOTIATE_TIMEOUT;
-				renegotiate_interval = renegotiate_interval * 60;	   /* Convert to seconds. */
+				if (gtmsource_options.connect_parms[GTMSOURCE_CONN_HEARTBEAT_PERIOD] > renegotiate_interval * 60)
+				{	/* MUPIPERR when -CONNECTPARAMS having HEARTBEAT_PERIOD > renegotiate_interval */
+					util_out_print("RENEGOTIATE_INTERVAL [!UL] (in minutes) cannot be less than "
+							"HEARTBEAT_PERIOD [!UL]", TRUE, renegotiate_interval,
+							gtmsource_options.connect_parms[GTMSOURCE_CONN_HEARTBEAT_PERIOD]);
+					return -1;
+				}
+
 			} else
 				renegotiate_interval = DEFAULT_RENEGOTIATE_TIMEOUT * 60; /* Convert to seconds. */
-			gtmsource_options.renegotiate_interval = renegotiate_interval;
+			gtmsource_options.renegotiate_interval = renegotiate_interval * 60;
 			/* Check if plaintext-fallback mode is specified. Default option is NOPLAINTEXTFALLBACK. */
 			if (CLI_PRESENT == (plaintext_fallback = cli_present("PLAINTEXTFALLBACK")))
 				repl_tls.plaintext_fallback = (plaintext_fallback != CLI_NEGATED);
@@ -467,10 +477,10 @@ int gtmsource_get_opt(void)
 				util_out_print("Error parsing TIMEOUT qualifier", TRUE);
 				return(-1);
 			}
-			if (DEFAULT_SHUTDOWN_TIMEOUT < gtmsource_options.shutdown_time || 0 > gtmsource_options.shutdown_time)
+			if (MAX_SHUTDOWN_TIMEOUT < gtmsource_options.shutdown_time || 0 > gtmsource_options.shutdown_time)
 			{
-				gtmsource_options.shutdown_time = DEFAULT_SHUTDOWN_TIMEOUT;
-				util_out_print("shutdown TIMEOUT changed to !UL", TRUE, gtmsource_options.shutdown_time);
+				gtm_putmsg_csa(CSA_ARG(NULL) VARLSTCNT(1) ERR_INVSHUTDOWN);
+				return (-1);
 			}
 		} else if (CLI_NEGATED == timeout_status)
 			gtmsource_options.shutdown_time = -1;
