@@ -1,6 +1,6 @@
 /****************************************************************
  *								*
- * Copyright (c) 2001-2020 Fidelity National Information	*
+ * Copyright (c) 2001-2022 Fidelity National Information	*
  * Services, Inc. and/or its subsidiaries. All rights reserved.	*
  *								*
  *	This source code contains the intellectual property	*
@@ -37,6 +37,7 @@
 #include "fileinfo.h"
 #include "gdsbt.h"
 #include "gdsfhead.h"
+#include "db_header_conversion.h"
 #include "gdsblk.h"
 #include "gdscc.h"
 #include "min_max.h"
@@ -471,7 +472,7 @@ gd_region *dbfilopn(gd_region *reg)
 			free(seg->file_cntl);
 			seg->file_cntl = NULL;
 		}
-		rts_error_csa(CSA_ARG(NULL) VARLSTCNT(5) ERR_DBFILERR, 2, DB_LEN_STR(reg), status);
+		RTS_ERROR_CSA_ABT(NULL, VARLSTCNT(5) ERR_DBFILERR, 2, DB_LEN_STR(reg), status);
 	}
 	assert(((int)pblk.b_esl + 1) <= SIZEOF(seg->fname));
 	memcpy(seg->fname, pblk.buffer, pblk.b_esl);
@@ -515,7 +516,7 @@ gd_region *dbfilopn(gd_region *reg)
 				 */
 				/* First check if we have enough space to do ".new" suffix addition (done a little later) */
 				if (ARRAYSIZE(seg->fname) <= seg->fname_len + STR_LIT_LEN(EXT_NEW))
-					rts_error_csa(CSA_ARG(NULL) VARLSTCNT(4) ERR_DBFILERR, 2, DB_LEN_STR(reg));
+					RTS_ERROR_CSA_ABT(NULL, VARLSTCNT(4) ERR_DBFILERR, 2, DB_LEN_STR(reg));
 				save_gv_cur_region = gv_cur_region;
 				mu_gv_cur_reg_init();
 				tmp_reg = gv_cur_region;
@@ -528,7 +529,7 @@ gd_region *dbfilopn(gd_region *reg)
 				if (!ftok_sem_get(tmp_reg, FALSE, GTM_ID, FALSE, &ftok_counter_halted))
 				{
 					MU_GV_CUR_REG_FREE(tmp_reg, save_gv_cur_region);
-					rts_error_csa(CSA_ARG(NULL) VARLSTCNT(4) ERR_DBFILERR, 2, DB_LEN_STR(reg));
+					RTS_ERROR_CSA_ABT(NULL, VARLSTCNT(4) ERR_DBFILERR, 2, DB_LEN_STR(reg));
 				}
 				assert(!ftok_counter_halted);	/* since we specified FALSE as the 2nd parameter ("incr_cnt") */
 				/* Now that we got the ftok lock, check if the file exists. If so skip the create. */
@@ -552,7 +553,7 @@ gd_region *dbfilopn(gd_region *reg)
 						ftok_sem_release(tmp_reg, FALSE, FALSE);
 						MU_GV_CUR_REG_FREE(tmp_reg, save_gv_cur_region);
 						save_errno = TREF(mu_cre_file_openrc);
-						rts_error_csa(CSA_ARG(NULL) VARLSTCNT(4) ERR_DBFILERR, 2, DB_LEN_STR(reg));
+						RTS_ERROR_CSA_ABT(NULL, VARLSTCNT(4) ERR_DBFILERR, 2, DB_LEN_STR(reg));
 					}
 					memcpy(tmpbuff, seg->fname, seg->fname_len + 1); /* + 1 to include terminating '\0' */
 					seg->fname_len -= STR_LIT_LEN(EXT_NEW);
@@ -563,8 +564,8 @@ gd_region *dbfilopn(gd_region *reg)
 						save_errno = errno;
 						ftok_sem_release(tmp_reg, FALSE, FALSE);
 						MU_GV_CUR_REG_FREE(tmp_reg, save_gv_cur_region);
-						rts_error_csa(CSA_ARG(NULL) VARLSTCNT(5) ERR_DBFILERR, 2, DB_LEN_STR(reg),
-														save_errno);
+						RTS_ERROR_CSA_ABT(NULL, VARLSTCNT(5) ERR_DBFILERR, 2, DB_LEN_STR(reg),
+							save_errno);
 					}
 				}
 				ftok_sem_release(tmp_reg, FALSE, FALSE);
@@ -596,7 +597,7 @@ gd_region *dbfilopn(gd_region *reg)
 					free(seg->file_cntl);
 					seg->file_cntl = NULL;
 				}
-				rts_error_csa(CSA_ARG(NULL) VARLSTCNT(5) ERR_DBFILERR, 2, DB_LEN_STR(reg), save_errno);
+				RTS_ERROR_CSA_ABT(NULL, VARLSTCNT(5) ERR_DBFILERR, 2, DB_LEN_STR(reg), save_errno);
 			}
 			reg->read_only = TRUE;		/* maintain csa->read_write simultaneously */
 			csa->read_write = FALSE;	/* maintain reg->read_only simultaneously */
@@ -643,7 +644,7 @@ gd_region *dbfilopn(gd_region *reg)
         if (-1 == stat_res)
         {
         	save_errno = errno;
-        	rts_error_csa(CSA_ARG(NULL) VARLSTCNT(5) ERR_DBFILERR, 2, DB_LEN_STR(reg), save_errno);
+		RTS_ERROR_CSA_ABT(NULL, VARLSTCNT(5) ERR_DBFILERR, 2, DB_LEN_STR(reg), save_errno);
         }
 	set_gdid_from_stat(&udi->fileid, &buf);
 	if (prev_reg = gv_match(reg))
@@ -673,53 +674,50 @@ void dbsecspc(gd_region *reg, sgmnt_data_ptr_t csd, gtm_uint64_t *sec_size)
 	assert(0 == SHMPOOL_SECTION_SIZE % OS_PAGE_SIZE);
 	assert(0 == CACHE_CONTROL_SIZE(csd) % OS_PAGE_SIZE);
 	/* First compute the size based on sections common to both MM and BG */
-	tmp_sec_size = NODE_LOCAL_SPACE(csd) + JNL_SHARE_SIZE(csd) + SHMPOOL_SECTION_SIZE + LOCK_SPACE_SIZE(csd);
+	csd->free_space = (BLK_ZERO_OFF(csd->start_vbn) - SIZEOF_FILE_HDR(csd));
+	tmp_sec_size = NODE_LOCAL_SPACE(csd) + JNL_SHARE_SIZE(csd) + LOCK_SPACE_SIZE(csd) + SHMPOOL_SECTION_SIZE
+		+ SIZEOF_FILE_HDR(csd);
 	/* Now, add sections specific to MM and BG */
-	if (dba_mm == reg->dyn.addr->acc_meth)
-		tmp_sec_size += SIZEOF_FILE_HDR(csd);
-	else
-	{
-		assertpro(dba_bg == reg->dyn.addr->acc_meth);
-		tmp_sec_size += CACHE_CONTROL_SIZE(csd) + (LOCK_BLOCK(csd) * DISK_BLOCK_SIZE);
-	}
-	ADJUST_SHM_SIZE_FOR_HUGEPAGES(tmp_sec_size, *sec_size);	/* *sec_size is adjusted size */
+	if (dba_bg == reg->dyn.addr->acc_meth)
+		tmp_sec_size += CACHE_CONTROL_SIZE(csd) + (DIVIDE_ROUND_UP(BT_SIZE(csd), OS_PAGE_SIZE) * OS_PAGE_SIZE);
+	ADJUST_SHM_SIZE_FOR_HUGEPAGES(tmp_sec_size, *sec_size); /* *sec_size is adjusted size */
 	return;
 }
 
 int db_init(gd_region *reg, boolean_t ok_to_bypass)
 {
-	boolean_t       	is_bg, read_only, need_stacktrace, have_standalone_access;
+	boolean_t		is_bg, read_only, need_stacktrace, have_standalone_access;
 	boolean_t		shm_setup_ok = FALSE, vermismatch = FALSE, vermismatch_already_printed = FALSE;
 	boolean_t		new_shm_ipc, replinst_mismatch, need_shmctl, need_semctl;
 	boolean_t		gld_do_crypt_init, db_do_crypt_init, semop_success, statsdb_off;
-	char            	machine_name[MAX_MCNAMELEN];
+	char			machine_name[MAX_MCNAMELEN];
 	int			gethostname_res, stat_res, user_id, group_id, perm, save_udi_semid;
-	int4            	status, dblksize, save_errno, wait_time, loopcnt, sem_pid;
-	sm_long_t       	status_l;
-	sgmnt_addrs     	*csa;
+	int4			status, dblksize, save_errno, wait_time, loopcnt, sem_pid;
+	sm_long_t		status_l;
+	sgmnt_addrs		*csa;
 	node_local_ptr_t	cnl;
 	sgmnt_data		tsdbuff;
-	sgmnt_data_ptr_t        csd, tsd;
+	sgmnt_data_ptr_t	csd, tsd;
 	jnlpool_addrs_ptr_t	save_jnlpool, local_jnlpool;
 	replpool_identifier	replpool_id;
 	unsigned int		full_len;	/* for REPL_INST_AVAILABLE */
 	gd_id			replfile_gdid, *tmp_gdid;
 	boolean_t		need_jnlpool_setup;
-	struct sembuf   	sop[3];
-	struct stat     	stat_buf;
+	struct sembuf		sop[3];
+	struct stat		stat_buf;
 	union semun		semarg;
 	struct semid_ds		semstat;
-	struct shmid_ds         shmstat;
+	struct shmid_ds		shmstat;
 	struct statvfs		dbvfs;
-	uint4           	fbwsize, sopcnt;
+	uint4			fbwsize, sopcnt;
 	uint4			max_hrtbt_delta;
 	boolean_t		sem_timedout, *sem_timedoutp = NULL,
 				sem_stacktrace_time, *sem_stacktrace_timep = NULL,
 				indefinite_wait = TRUE;
-	unix_db_info    	*udi;
+	unix_db_info		*udi;
 	char			now_running[MAX_REL_NAME];
 	int			err_ret, init_status;
-	gtm_uint64_t 		sec_size, mmap_sz;
+	gtm_uint64_t		sec_size, mmap_sz;
 	semwait_status_t	retstat;
 	struct perm_diag_data	pdd;
 	boolean_t		bypassed_ftok = FALSE, bypassed_access = FALSE, dummy_ftok_counter_halted,
@@ -733,9 +731,11 @@ int db_init(gd_region *reg, boolean_t ok_to_bypass)
 	gd_region		*baseDBreg;
 	sgmnt_addrs		*baseDBcsa;
 	node_local_ptr_t	baseDBnl;
+	int			slp_cnt = 0;
+	boolean_t		is_logged = FALSE;
 #	ifdef DEBUG
 	int			i;
-	char 			*ptr;
+	char			*ptr;
 #	endif
 
 	DCL_THREADGBL_ACCESS;
@@ -1130,6 +1130,7 @@ int db_init(gd_region *reg, boolean_t ok_to_bypass)
 		udi->shm_created = !tsd->read_only;
 	}
 	assert(udi->grabbed_access_sem || bypassed_access);
+	DO_DB_HDR_CHECK(reg, tsd); /* Basic sanity check on the file header fields */
 	if (udi->fd_opened_with_o_direct)
 	{	/* "tsd" points to dio_buff.aligned, a global variable buffer that will likely be reused by other functions
 		 * inside "db_init" (e.g. "recover_truncate" invoking "db_write_eof_block"). Point it back to tsdbuff
@@ -1139,7 +1140,6 @@ int db_init(gd_region *reg, boolean_t ok_to_bypass)
 		memcpy(&tsdbuff, (sgmnt_data_ptr_t)(TREF(dio_buff)).aligned, SGMNT_HDR_LEN);
 		tsd = &tsdbuff;
 	}
-	DO_DB_HDR_CHECK(reg, tsd); /* Basic sanity check on the file header fields */
 	if (WBTEST_ENABLED(WBTEST_HOLD_ONTO_ACCSEM_IN_DBINIT))
 	{
 		DBGFPF((stderr, "Holding the access control semaphore.. Sleeping for 30 seconds\n"));
@@ -1185,7 +1185,7 @@ int db_init(gd_region *reg, boolean_t ok_to_bypass)
 		dbsecspc(reg, tsd, &sec_size); 	/* Find db segment size */
 		/* Create new shared memory using IPC_PRIVATE. System guarantees a unique id */
 		GTM_WHITE_BOX_TEST(WBTEST_FAIL_ON_SHMGET, sec_size, GTM_UINT64_MAX);
-		if (-1 == (status_l = udi->shmid = shmget(IPC_PRIVATE, sec_size, RWDALL | IPC_CREAT)))
+		if (-1 == (status_l = udi->shmid = gtm_shmget(IPC_PRIVATE, sec_size, RWDALL | IPC_CREAT, TRUE)))
 		{
 			udi->shmid = (int)INVALID_SHMID;
 			status_l = INVALID_SHMID;
@@ -1278,7 +1278,7 @@ int db_init(gd_region *reg, boolean_t ok_to_bypass)
 	DEFAULT_INIT_SS_CTX((SS_CTX_CAST(csa->ss_ctx)));
 	csa->mlkctl = (struct mlk_ctldata_struct *) ((sm_uc_ptr_t)csa->shmpool_buffer + SHMPOOL_SECTION_SIZE);
 	csa->mlkctl_len = LOCK_SPACE_SIZE(tsd);
-	csa->total_blks = tsd->trans_hist.total_blks;   		/* For test to see if file has extended */
+	csa->total_blks = tsd->trans_hist.total_blks;		/* For test to see if file has extended */
 	cnl = csa->nl;
 	if (new_shm_ipc)
 	{
@@ -1306,7 +1306,8 @@ int db_init(gd_region *reg, boolean_t ok_to_bypass)
 		FSTAT_FILE(udi->fd, &stat_buf, stat_res);
 		if (-1 == stat_res)
 			RTS_ERROR(VARLSTCNT(5) ERR_DBFILERR, 2, DB_LEN_STR(reg), errno);
-		mmap_sz = stat_buf.st_size - BLK_ZERO_OFF(tsd->start_vbn);
+		csd = csa->hdr = (sgmnt_data_ptr_t)((sm_uc_ptr_t)csa->mlkctl + csa->mlkctl_len);
+		mmap_sz = stat_buf.st_size - BLK_ZERO_OFF(tsd->start_vbn) - csd->free_space;
 		assert(0 < mmap_sz);
 		CHECK_LARGEFILE_MMAP(reg, mmap_sz); /* can issue rts_error MMFILETOOLARGE */
 #		ifdef _AIX
@@ -1326,7 +1327,6 @@ int db_init(gd_region *reg, boolean_t ok_to_bypass)
 #		endif
 		csa->db_addrs[1] = (sm_uc_ptr_t)((sm_uc_ptr_t)csa->db_addrs[0] + mmap_sz - 1);	/* '- 1' due to 0-based indexing */
 		assert(csa->db_addrs[1] > csa->db_addrs[0]);
-		csd = csa->hdr = (sgmnt_data_ptr_t)((sm_uc_ptr_t)csa->mlkctl + csa->mlkctl_len);
 	}
 	/* At this point, shm_setup_ok is TRUE so we are guaranteed that vermismatch is FALSE.  Therefore, we can safely
 	 * dereference cnl->glob_sec_init without worrying about whether or not it could be at a different offset than
@@ -1384,7 +1384,7 @@ int db_init(gd_region *reg, boolean_t ok_to_bypass)
 			csd->trans_hist.early_tn = csd->trans_hist.curr_tn;
 			csd->max_update_array_size = csd->max_non_bm_update_array_size
 				= (int4)(ROUND_UP2(MAX_NON_BITMAP_UPDATE_ARRAY_SIZE(csd), UPDATE_ARRAY_ALIGN_SIZE));
-			csd->max_update_array_size += (int4)(ROUND_UP2(MAX_BITMAP_UPDATE_ARRAY_SIZE, UPDATE_ARRAY_ALIGN_SIZE));
+			csd->max_update_array_size += (int4)(ROUND_UP2(MAX_BITMAP_UPDATE_ARRAY_SIZE(csd), UPDATE_ARRAY_ALIGN_SIZE));
 			/* add current db_csh counters into the cumulative counters and reset the current counters */
 #			define TAB_DB_CSH_ACCT_REC(COUNTER, DUMMY1, DUMMY2)		\
 				csd->COUNTER.cumul_count += csd->COUNTER.curr_count;	\
@@ -1396,7 +1396,11 @@ int db_init(gd_region *reg, boolean_t ok_to_bypass)
 		gvstats_rec_csd2cnl(csa);	/* should be called before "db_auto_upgrade" */
 		reg->dyn.addr->ext_blk_count = csd->extension_size;
 		mlk_shr_init((sm_uc_ptr_t)csa->mlkctl, csd->lock_space_size, csa, (FALSE == read_only));
-		db_auto_upgrade(reg);		/* should be called before "gtm_mutex_init" to ensure NUM_CRIT_ENTRY is nonzero */
+		/* should be called before "gtm_mutex_init" to ensure NUM_CRIT_ENTRY is nonzero */
+		if (!MEMCMP_LIT(csd->label, GDS_LABEL))
+			db_auto_upgrade(reg);
+		else if(!MEMCMP_LIT(csd->label, V6_GDS_LABEL))
+			v6_db_auto_upgrade(reg);
 		DEBUG_ONLY(locknl = cnl;)	/* for DEBUG_ONLY LOCK_HIST macro */
 		gtm_mutex_init(reg, NUM_CRIT_ENTRY(csd), FALSE);
 		DEBUG_ONLY(locknl = NULL;)	/* restore "locknl" to default value */
@@ -1493,11 +1497,29 @@ int db_init(gd_region *reg, boolean_t ok_to_bypass)
 		 * state. Note that the reissue of a regular Rollback/Recover command will NOT hit this condition because it invokes
 		 * mu_rndwn_file (STANDALONE) that removes the shared memory. The only case in which mu_rndwn_file does NOT remove
 		 * shared memory is if it was invoked by an Online Rollback in which case the below check should be bypassed
+		 * However, if there is an online rollback active (that is not us) this should be a transient condition
+		 * and if we wait a little bit, it should go away.
 		 */
 		if (cnl->donotflush_dbjnl && !jgbl.onlnrlbk)
 		{
-			PRINT_CRASH_MESSAGE(0, cnl, ERR_TEXT, 2,
-				LEN_AND_LIT("mupip recover/rollback created shared memory. Needs MUPIP RUNDOWN"));
+			/* If we can see online rollback is out there, spin a bit */
+			while (cnl->donotflush_dbjnl && !jgbl.onlnrlbk && cnl->onln_rlbk_pid)
+			{
+				if (!(slp_cnt % 100))
+					is_logged = FALSE;
+				if (! is_logged)
+				{
+					send_msg_csa(CSA_ARG(csa) VARLSTCNT(4) ERR_TEXT, 2,
+						LEN_AND_LIT("Spinning for clean DB header"));
+					is_logged = TRUE;
+				}
+				wcs_sleep(++slp_cnt);
+				if (1000 < slp_cnt)
+					break;
+			}
+			if (cnl->donotflush_dbjnl && !jgbl.onlnrlbk)
+				PRINT_CRASH_MESSAGE(0, cnl, ERR_TEXT, 2,
+					LEN_AND_LIT("mupip recover/rollback created shared memory. Needs MUPIP RUNDOWN"));
 		}
 		/* verify pointers from our calculation vs. the copy in shared memory */
 		if (cnl->critical != (sm_off_t)((sm_uc_ptr_t)csa->critical - (sm_uc_ptr_t)cnl))
@@ -1652,6 +1674,8 @@ int db_init(gd_region *reg, boolean_t ok_to_bypass)
 			assert(csd->semid == tsd->semid);
 			assert(!memcmp(&csd->gt_sem_ctime, &tsd->gt_sem_ctime, SIZEOF(tsd->gt_sem_ctime)));
 			assert(!memcmp(&csd->gt_shm_ctime, &tsd->gt_shm_ctime, SIZEOF(tsd->gt_shm_ctime)));
+			if (0 == memcmp(csd->label, V6_GDS_LABEL, GDS_LABEL_SZ - 1))
+				db_header_dwnconv(csd);
 			DB_LSEEKWRITE(csa, udi, udi->fn, udi->fd, (off_t)0, (sm_uc_ptr_t)csd, SGMNT_HDR_LEN, save_errno);
 			if (0 != save_errno)
 			{
