@@ -1,6 +1,6 @@
 /****************************************************************
  *								*
- * Copyright (c) 2008-2018 Fidelity National Information	*
+ * Copyright (c) 2008-2021 Fidelity National Information	*
  * Services, Inc. and/or its subsidiaries. All rights reserved.	*
  *								*
  *	This source code contains the intellectual property	*
@@ -68,8 +68,20 @@ enum
 	PARSE_FAIL_NOPATH	/* the parse failed and $PATH is undefined in the environment */
 };
 
-#define FREE_ALL { if (NULL != copy_cmd_string) free(copy_cmd_string); if (NULL != temp) free(temp);\
-		if (NULL != buf)  free(buf); if (NULL != dir_in_path) free(dir_in_path);if (NULL != command2) free(command2); }
+#define FREE_ALL				\
+MBSTART {					\
+	if (NULL != copy_cmd_string)		\
+		free(copy_cmd_string);		\
+	if (NULL != temp)			\
+		free(temp);			\
+	if (NULL != buf)			\
+		free(buf);			\
+	if (NULL != dir_in_path)		\
+		free(dir_in_path);		\
+	if (NULL != command2)			\
+		free(command2);			\
+} MBEND
+
 
 int parse_pipe(char *cmd_string, char *ret_token);
 
@@ -330,35 +342,35 @@ short iopi_open(io_log_name *dev_name, mval *pp, int fd, mval *mspace, int4 time
 		assert((params) *(pp->str.addr + p_offset) < (params)n_iops);
 		switch ((ch = *(pp->str.addr + p_offset++)))
 		{
-			case iop_exception:
-				DEF_EXCEPTION(pp, p_offset, iod);
-				break;
-			case iop_shell:
-				slen[PSHELL] = (unsigned int)(unsigned char)*(pp->str.addr + p_offset);
-				sparams[PSHELL] = (char *)(pp->str.addr + p_offset + 1);
-				param_cnt++;
-				break;
-			case iop_command:
-				slen[PCOMMAND] = (unsigned int)(unsigned char)*(pp->str.addr + p_offset);
-				sparams[PCOMMAND] = (char *)(pp->str.addr + p_offset + 1);
-				param_cnt++;
-				break;
-			case iop_stderr:
-				slen[PSTDERR] = (unsigned int)(unsigned char)*(pp->str.addr + p_offset);
-				sparams[PSTDERR] = (char *)(pp->str.addr + p_offset + 1);
-				param_cnt++;
-				break;
-			case iop_independent:
-				independent = TRUE;
-				break;
-			case iop_parse:
-				parse = TRUE;
-				break;
-			case iop_writeonly:
-				return_stdout = FALSE;
-				break;
-			default:
-				break;
+		case iop_exception:
+			DEF_EXCEPTION(pp, p_offset, iod);
+			break;
+		case iop_shell:
+			slen[PSHELL] = (unsigned int)(unsigned char)*(pp->str.addr + p_offset);
+			sparams[PSHELL] = (char *)(pp->str.addr + p_offset + 1);
+			param_cnt++;
+			break;
+		case iop_command:
+			slen[PCOMMAND] = (unsigned int)(unsigned char)*(pp->str.addr + p_offset);
+			sparams[PCOMMAND] = (char *)(pp->str.addr + p_offset + 1);
+			param_cnt++;
+			break;
+		case iop_stderr:
+			slen[PSTDERR] = (unsigned int)(unsigned char)*(pp->str.addr + p_offset);
+			sparams[PSTDERR] = (char *)(pp->str.addr + p_offset + 1);
+			param_cnt++;
+			break;
+		case iop_independent:
+			independent = TRUE;
+			break;
+		case iop_parse:
+			parse = TRUE;
+			break;
+		case iop_writeonly:
+			return_stdout = FALSE;
+			break;
+		default:
+			break;
 		}
 		p_offset += ((IOP_VAR_SIZE == io_params_size[ch]) ?
 			     (unsigned char)*(pp->str.addr + p_offset) + 1 : io_params_size[ch]);
@@ -371,11 +383,11 @@ short iopi_open(io_log_name *dev_name, mval *pp, int fd, mval *mspace, int4 time
 	{
 		PIPE_ERROR_INIT();
 		if (0 == sparams[PCOMMAND])
-		    rts_error_csa(CSA_ARG(NULL) VARLSTCNT(8) ERR_DEVOPENFAIL, 2, dev_name->len, dev_name->dollar_io,
-			      ERR_TEXT, 2, LEN_AND_LIT("Missing command string"));
+	    	RTS_ERROR_CSA_ABT(NULL, VARLSTCNT(8) ERR_DEVOPENFAIL, 2, dev_name->len, dev_name->dollar_io,
+	    		ERR_TEXT, 2, LEN_AND_LIT("Missing command string"));
 		else
-			rts_error_csa(CSA_ARG(NULL) VARLSTCNT(8) ERR_DEVOPENFAIL, 2, dev_name->len, dev_name->dollar_io,
-				  ERR_TEXT, 2, LEN_AND_LIT("Command string has no value"));
+			RTS_ERROR_CSA_ABT(NULL, VARLSTCNT(8) ERR_DEVOPENFAIL, 2, dev_name->len, dev_name->dollar_io,
+				ERR_TEXT, 2, LEN_AND_LIT("Command string has no value"));
 	} else
 	{
 		pcommand = malloc(slen[PCOMMAND] + 1);
@@ -392,14 +404,14 @@ short iopi_open(io_log_name *dev_name, mval *pp, int fd, mval *mspace, int4 time
 				else
 					SNPRINTF(error_str, MAXDEVPARLEN + STR_LIT_LEN(INVALID_CMD2), "%s%s", INVALID_CMD2,
 						ret_token);
-				rts_error_csa(CSA_ARG(NULL) VARLSTCNT(8) ERR_DEVOPENFAIL, 2, dev_name->len, dev_name->dollar_io,
-					  ERR_TEXT, 2, LEN_AND_STR(error_str));
+				RTS_ERROR_CSA_ABT(NULL, VARLSTCNT(8) ERR_DEVOPENFAIL, 2, dev_name->len, dev_name->dollar_io,
+					ERR_TEXT, 2, LEN_AND_STR(error_str));
 			}
 		}
 	}
 	/* The above code is just syntax checking. Any actual operation should be below here. */
 	if (RESTRICTED(pipe_open))
-		rts_error_csa(CSA_ARG(NULL) VARLSTCNT(3) ERR_RESTRICTEDOP, 1, "OPEN PIPE");
+		RTS_ERROR_CSA_ABT(NULL, VARLSTCNT(3) ERR_RESTRICTEDOP, 1, "OPEN PIPE");
 	/*Filter the command first, if required*/
 	if (RESTRICTED(pipe_filter))
 	{
@@ -411,8 +423,8 @@ short iopi_open(io_log_name *dev_name, mval *pp, int fd, mval *mspace, int4 time
 				PIPE_ERROR_INIT();
 				send_msg_csa(CSA_ARG(NULL) VARLSTCNT(6) ERR_COMMFILTERERR, 4, LEN_AND_LIT(PIPEOPENSTR),
 					LEN_AND_LIT("Empty return command"));
-				rts_error_csa(CSA_ARG(NULL) VARLSTCNT(6) ERR_COMMFILTERERR, 4,
-						LEN_AND_LIT(PIPEOPENSTR),LEN_AND_LIT("Empty return command"));
+				RTS_ERROR_CSA_ABT(NULL, VARLSTCNT(6) ERR_COMMFILTERERR, 4,
+					LEN_AND_LIT(PIPEOPENSTR),LEN_AND_LIT("Empty return command"));
 			}
 			free(pcommand);
 			pcommand = (char*)malloc(filtered_command.length+1);
@@ -431,8 +443,8 @@ short iopi_open(io_log_name *dev_name, mval *pp, int fd, mval *mspace, int4 time
 	if (sparams[PSHELL] && (0 == slen[PSHELL]))
 	{
 		PIPE_ERROR_INIT();
-		rts_error_csa(CSA_ARG(NULL) VARLSTCNT(8) ERR_DEVOPENFAIL, 2, dev_name->len, dev_name->dollar_io,
-			  ERR_TEXT, 2, LEN_AND_LIT("SHELL parameter has no value"));
+		RTS_ERROR_CSA_ABT(NULL, VARLSTCNT(8) ERR_DEVOPENFAIL, 2, dev_name->len, dev_name->dollar_io,
+			ERR_TEXT, 2, LEN_AND_LIT("SHELL parameter has no value"));
 	} else if (0 != slen[PSHELL])
 	{
 		pshell = malloc(slen[PSHELL] + 1);
@@ -446,8 +458,8 @@ short iopi_open(io_log_name *dev_name, mval *pp, int fd, mval *mspace, int4 time
 			assert(GTM_MAX_DIR_LEN - 1 >= STRLEN(pshell));
 			SNPRINTF(error_str, MAXDEVPARLEN + STR_LIT_LEN(INVALID_CMD2), "Invalid shell: %s", pshell);
 			PIPE_ERROR_INIT();
-			rts_error_csa(CSA_ARG(NULL) VARLSTCNT(8) ERR_DEVOPENFAIL, 2, dev_name->len, dev_name->dollar_io,
-				  ERR_TEXT, 2, LEN_AND_STR(error_str));
+			RTS_ERROR_CSA_ABT(NULL, VARLSTCNT(8) ERR_DEVOPENFAIL, 2, dev_name->len, dev_name->dollar_io,
+				ERR_TEXT, 2, LEN_AND_STR(error_str));
 		}
 		pshell_name = basename(pshell);
 	}
@@ -457,8 +469,8 @@ short iopi_open(io_log_name *dev_name, mval *pp, int fd, mval *mspace, int4 time
 	if (sparams[PSTDERR] && (0 == slen[PSTDERR]))
 	{
 		PIPE_ERROR_INIT();
-		rts_error_csa(CSA_ARG(NULL) VARLSTCNT(8) ERR_DEVOPENFAIL, 2, dev_name->len, dev_name->dollar_io,
-			  ERR_TEXT, 2, LEN_AND_LIT("STDERR parameter has no value"));
+		RTS_ERROR_CSA_ABT(NULL, VARLSTCNT(8) ERR_DEVOPENFAIL, 2, dev_name->len, dev_name->dollar_io,
+			ERR_TEXT, 2, LEN_AND_LIT("STDERR parameter has no value"));
 	} else if (0 != slen[PSTDERR])
 	{
 		pstderr = malloc(slen[PSTDERR] + 1);
@@ -472,8 +484,8 @@ short iopi_open(io_log_name *dev_name, mval *pp, int fd, mval *mspace, int4 time
 	{
 		save_errno = errno;
 		PIPE_ERROR_INIT();
-		rts_error_csa(CSA_ARG(NULL) VARLSTCNT(9) ERR_DEVOPENFAIL, 2, dev_name->len, dev_name->dollar_io,
-			  ERR_TEXT, 2, LEN_AND_LIT("PIPE - pipe(pfd_write) failed"), save_errno);
+		RTS_ERROR_CSA_ABT(NULL, VARLSTCNT(9) ERR_DEVOPENFAIL, 2, dev_name->len, dev_name->dollar_io,
+			ERR_TEXT, 2, LEN_AND_LIT("PIPE - pipe(pfd_write) failed"), save_errno);
 	} else
 	{
 #ifdef __MVS__
@@ -492,8 +504,8 @@ short iopi_open(io_log_name *dev_name, mval *pp, int fd, mval *mspace, int4 time
 		{
 			save_errno = errno;
 			PIPE_ERROR_INIT();
-			rts_error_csa(CSA_ARG(NULL) VARLSTCNT(9) ERR_DEVOPENFAIL, 2, dev_name->len, dev_name->dollar_io,
-				  ERR_TEXT, 2, LEN_AND_LIT("PIPE - pipe(pfd_read) failed"), save_errno);
+			RTS_ERROR_CSA_ABT(NULL, VARLSTCNT(9) ERR_DEVOPENFAIL, 2, dev_name->len, dev_name->dollar_io,
+				ERR_TEXT, 2, LEN_AND_LIT("PIPE - pipe(pfd_read) failed"), save_errno);
 		} else
 		{
 #ifdef __MVS__
@@ -512,8 +524,8 @@ short iopi_open(io_log_name *dev_name, mval *pp, int fd, mval *mspace, int4 time
 		{
 			save_errno = errno;
 			PIPE_ERROR_INIT();
-			rts_error_csa(CSA_ARG(NULL) VARLSTCNT(9) ERR_DEVOPENFAIL, 2, dev_name->len, dev_name->dollar_io,
-				  ERR_TEXT, 2, LEN_AND_LIT("PIPE - pipe(pfd_read_stderr) failed"), save_errno);
+			RTS_ERROR_CSA_ABT(NULL, VARLSTCNT(9) ERR_DEVOPENFAIL, 2, dev_name->len, dev_name->dollar_io,
+				ERR_TEXT, 2, LEN_AND_LIT("PIPE - pipe(pfd_read_stderr) failed"), save_errno);
 		} else
 		{
 #ifdef __MVS__
@@ -535,8 +547,8 @@ short iopi_open(io_log_name *dev_name, mval *pp, int fd, mval *mspace, int4 time
 	{
 		save_errno = errno;
 		PIPE_ERROR_INIT();
-		rts_error_csa(CSA_ARG(NULL) VARLSTCNT(9) ERR_DEVOPENFAIL, 2, dev_name->len, dev_name->dollar_io,
-			  ERR_TEXT, 2, LEN_AND_LIT("PIPE - fork() failed"), save_errno);	/* BYPASSOK */
+		RTS_ERROR_CSA_ABT(NULL, VARLSTCNT(9) ERR_DEVOPENFAIL, 2, dev_name->len, dev_name->dollar_io,
+			ERR_TEXT, 2, LEN_AND_LIT("PIPE - fork() failed"), save_errno);	/* BYPASSOK */
 	}
 	if (0 == cpid)
 	{
@@ -676,8 +688,8 @@ short iopi_open(io_log_name *dev_name, mval *pp, int fd, mval *mspace, int4 time
 			{
 				save_errno = errno;
 				PIPE_ERROR_INIT();
-				rts_error_csa(CSA_ARG(NULL) VARLSTCNT(9) ERR_DEVOPENFAIL, 2, dev_name->len, dev_name->dollar_io,
-					  ERR_TEXT, 2, LEN_AND_LIT("Error in stream open"), save_errno);
+				RTS_ERROR_CSA_ABT(NULL, VARLSTCNT(9) ERR_DEVOPENFAIL, 2, dev_name->len, dev_name->dollar_io,
+					ERR_TEXT, 2, LEN_AND_LIT("Error in stream open"), save_errno);
 			}
 			d_rm->read_fildes = file_des_read;
 		} else
@@ -788,7 +800,7 @@ short iopi_open(io_log_name *dev_name, mval *pp, int fd, mval *mspace, int4 time
 	flags = 0;
 	FCNTL2(file_des_write, F_GETFL, flags);
 	if (0 > flags)
-		rts_error_csa(CSA_ARG(NULL) VARLSTCNT(8) ERR_SYSCALL, 5, LEN_AND_LIT("fcntl"), CALLFROM, errno);
+		RTS_ERROR_CSA_ABT(NULL, VARLSTCNT(8) ERR_SYSCALL, 5, LEN_AND_LIT("fcntl"), CALLFROM, errno);
 	FCNTL3(file_des_write, F_SETFL, (flags | O_NONBLOCK), fcntl_res);
 	if (0 > fcntl_res)
 		rts_error_csa(CSA_ARG(NULL) VARLSTCNT(8) ERR_SYSCALL, 5, LEN_AND_LIT("fcntl"), CALLFROM, errno);
