@@ -1,6 +1,6 @@
 /****************************************************************
  *								*
- * Copyright (c) 2014-2019 Fidelity National Information	*
+ * Copyright (c) 2014-2022 Fidelity National Information	*
  * Services, Inc. and/or its subsidiaries. All rights reserved.	*
  *								*
  *	This source code contains the intellectual property	*
@@ -28,7 +28,8 @@
 #include "gtm_string.h"
 #include "eintr_wrappers.h"
 #include "stringpool.h"
-#include "outofband.h"
+#include "have_crit.h"
+#include "deferred_events_queue.h"
 #include "error.h"
 
 #define MAX_PASS_FDS			256
@@ -154,7 +155,7 @@ void iosocket_pass_local(io_desc *iod, pid_t pid, int4 msec_timeout, int argcnt,
 		return;
 	}
 	if (dsocketptr->mupintr)
-		rts_error_csa(CSA_ARG(NULL) VARLSTCNT(1) ERR_ZINTRECURSEIO);
+		RTS_ERROR_CSA_ABT(NULL, VARLSTCNT(1) ERR_ZINTRECURSEIO);
 	socketptr = dsocketptr->socket[dsocketptr->current_socket];
 	if (socket_local != socketptr->protocol)
 	{
@@ -170,7 +171,7 @@ void iosocket_pass_local(io_desc *iod, pid_t pid, int4 msec_timeout, int argcnt,
 	ENSURE_PASS_SOCKET(socketptr);
 	out_of_time = FALSE;
 #	if PID_CHECKING_SUPPORTED
-	if (-1 != pid)
+	if (-1 < pid)
 	{
 		peerpid = get_peer_pid(socketptr->sd);
 		if (-1 == peerpid)
@@ -278,11 +279,11 @@ void iosocket_pass_local(io_desc *iod, pid_t pid, int4 msec_timeout, int argcnt,
 	{
 		if ((NO_M_TIMEOUT != msec_timeout) && !out_of_time)
 			cancel_timer(timer_id);
-		iod->dollar.za = 9;
+		iod->dollar.za = ZA_IO_ERR;
 		errptr = PROTOCOL_ERROR;
 		SET_DOLLARDEVICE_ONECOMMA_ERRSTR(iod, errptr, errlen);
 		if (socketptr->ioerror)
-			rts_error_csa(CSA_ARG(NULL) VARLSTCNT(4) ERR_SOCKPASS, 0, ERR_TEXT, 2, errlen, errptr);
+			RTS_ERROR_CSA_ABT(NULL, VARLSTCNT(4) ERR_SOCKPASS, 0, ERR_TEXT, 2, errlen, errptr);
 		REVERT_GTMIO_CH(&iod->pair, ch_set);
 		return;
 	}
@@ -309,10 +310,10 @@ ioerr:
 	}
 	if ((NO_M_TIMEOUT != msec_timeout) && !out_of_time)
 		cancel_timer(timer_id);
-	iod->dollar.za = 9;
+	iod->dollar.za = ZA_IO_ERR;
 	SET_DOLLARDEVICE_ONECOMMA_STRERROR(iod, save_errno);
 	if (socketptr->ioerror)
-		rts_error_csa(CSA_ARG(NULL) VARLSTCNT(4) ERR_SOCKPASS, 0, save_errno, 0);
+		RTS_ERROR_CSA_ABT(NULL, VARLSTCNT(4) ERR_SOCKPASS, 0, save_errno, 0);
 	REVERT_GTMIO_CH(&iod->pair, ch_set);
 	return;
 }
@@ -358,7 +359,7 @@ void iosocket_accept_local(io_desc *iod, mval *handlesvar, pid_t pid, int4 msec_
 		return;
 	}
 	if (dsocketptr->mupintr)
-		rts_error_csa(CSA_ARG(NULL) VARLSTCNT(1) ERR_ZINTRECURSEIO);
+		RTS_ERROR_CSA_ABT(NULL, VARLSTCNT(1) ERR_ZINTRECURSEIO);
 	socketptr = dsocketptr->socket[dsocketptr->current_socket];
 	if (socket_local != socketptr->protocol)
 	{
@@ -374,7 +375,7 @@ void iosocket_accept_local(io_desc *iod, mval *handlesvar, pid_t pid, int4 msec_
 	ENSURE_PASS_SOCKET(socketptr);
 	out_of_time = FALSE;
 #	if PID_CHECKING_SUPPORTED
-	if (-1 != pid)
+	if (-1 < pid)
 	{
 		peerpid = get_peer_pid(socketptr->sd);
 		if (-1 == peerpid)
@@ -440,11 +441,11 @@ void iosocket_accept_local(io_desc *iod, mval *handlesvar, pid_t pid, int4 msec_
 	assert(rval == iov.iov_len);
 	if ((MSG_MAGIC != mdata.magic) || (MSG_PROTO_VERSION != mdata.proto_version))
 	{
-		iod->dollar.za = 9;
+		iod->dollar.za = ZA_IO_ERR;
 		errptr = PROTOCOL_ERROR;
 		SET_DOLLARDEVICE_ONECOMMA_ERRSTR(iod, errptr, errlen);
 		if (socketptr->ioerror)
-			rts_error_csa(CSA_ARG(NULL) VARLSTCNT(6) ERR_SOCKACCEPT, 0, ERR_TEXT, 2, errlen, errptr);
+			RTS_ERROR_CSA_ABT(NULL, VARLSTCNT(6) ERR_SOCKACCEPT, 0, ERR_TEXT, 2, errlen, errptr);
 	}
 	cmsg = CMSG_FIRSTHDR(&msg);
 	while((cmsg != NULL) && ((SOL_SOCKET != cmsg->cmsg_level) || (SCM_RIGHTS != cmsg->cmsg_type)))
@@ -545,11 +546,11 @@ void iosocket_accept_local(io_desc *iod, mval *handlesvar, pid_t pid, int4 msec_
 		{
 			CLOSE(fds[fdn], rval);
 		}
-		iod->dollar.za = 9;
+		iod->dollar.za = ZA_IO_ERR;
 		errptr = PROTOCOL_ERROR;
 		SET_DOLLARDEVICE_ONECOMMA_ERRSTR(iod, errptr, errlen);
 		if (socketptr->ioerror)
-			rts_error_csa(CSA_ARG(NULL) VARLSTCNT(6) ERR_SOCKACCEPT, 0, ERR_TEXT, 2, errlen, errptr);
+			RTS_ERROR_CSA_ABT(NULL, VARLSTCNT(6) ERR_SOCKACCEPT, 0, ERR_TEXT, 2, errlen, errptr);
 		REVERT_GTMIO_CH(&iod->pair, ch_set);
 		return;
 	}
@@ -600,10 +601,10 @@ ioerr:
 	}
 	if ((EINTR != save_errno) || outofband)
 	{
-		iod->dollar.za = 9;
+		iod->dollar.za = ZA_IO_ERR;
 		SET_DOLLARDEVICE_ONECOMMA_STRERROR(iod, save_errno);
 		if (socketptr->ioerror)
-			rts_error_csa(CSA_ARG(NULL) VARLSTCNT(4) ERR_SOCKACCEPT, 0, save_errno, 0);
+			RTS_ERROR_CSA_ABT(NULL, VARLSTCNT(4) ERR_SOCKACCEPT, 0, save_errno, 0);
 	}
 	REVERT_GTMIO_CH(&iod->pair, ch_set);
 	return;
