@@ -1,6 +1,6 @@
 /****************************************************************
  *								*
- * Copyright (c) 2014-2019 Fidelity National Information	*
+ * Copyright (c) 2014-2022 Fidelity National Information	*
  * Services, Inc. and/or its subsidiaries. All rights reserved.	*
  *								*
  *	This source code contains the intellectual property	*
@@ -29,6 +29,7 @@
 #include "gt_timer.h"
 #include "iosocketdef.h"
 #include "gtm_conv.h"
+#include "gtmio.h"
 #include "gtm_utf8.h"
 #include "rel_quant.h"
 #include "send_msg.h"
@@ -42,6 +43,7 @@
 GBLREF io_pair			io_curr_device;
 GBLREF char			dl_err[MAX_ERRSTR_LEN];
 GBLREF int			dollar_truth;
+GBLREF void 			(*primary_exit_handler)(void);
 
 GBLREF	gtm_tls_ctx_t		*tls_ctx;
 
@@ -56,6 +58,9 @@ error_def(ERR_NOSOCKETINDEV);
 error_def(ERR_SOCKPASSDATAMIX);
 error_def(ERR_TEXT);
 error_def(ERR_ZINTRECURSEIO);
+error_def(ERR_GETSOCKOPTERR);
+error_def(ERR_SETSOCKOPTERR);
+
 
 #define	MAX_TLSOPTION	12
 #define TLSLABEL	"tls: { "
@@ -77,13 +82,14 @@ void	iosocket_tls(mval *optionmval, int4 msec_timeout, mval *tlsid, mval *passwo
 {
 	int4			devlen, extrastr_len, flags, len, length, save_errno, status, status2, timeout, tls_errno;
 	int4			errlen, errlen2;
+	int			fcntl_flags, fcntl_res;
 	io_desc			*iod;
 	d_socket_struct 	*dsocketptr;
 	socket_struct		*socketptr;
 	char			idstr[MAX_TLSID_LEN + SIZEOF(RENEGOTIATE) + 1], optionstr[MAX_TLSOPTION];
 	char			passwordstr[GTM_PASSPHRASE_MAX_ASCII + 1];
 	const char		*errp, *errp2;
-	char			*charptr, *extraptr, *extrastr;
+	char			*charptr, *extraptr, *extrastr, *errptr;
 	tls_option		option;
 	gtm_tls_socket_t	*tlssocket;
 	ABS_TIME		cur_time, end_time;
@@ -109,7 +115,7 @@ void	iosocket_tls(mval *optionmval, int4 msec_timeout, mval *tlsid, mval *passwo
 		return;
 	}
 	if (dsocketptr->mupintr)
-		rts_error_csa(CSA_ARG(NULL) VARLSTCNT(1) ERR_ZINTRECURSEIO);
+		RTS_ERROR_CSA_ABT(NULL, VARLSTCNT(1) ERR_ZINTRECURSEIO);
 	socketptr = dsocketptr->socket[dsocketptr->current_socket];
 	ESTABLISH_GTMIO_CH(&iod->pair, ch_set);
 	ENSURE_DATA_SOCKET(socketptr);
@@ -189,15 +195,17 @@ void	iosocket_tls(mval *optionmval, int4 msec_timeout, mval *tlsid, mval *passwo
 			}
 			if (NULL == (tls_ctx = (gtm_tls_init(GTM_TLS_API_VERSION, GTMTLS_OP_INTERACTIVE_MODE))))
 			{
-				errp = gtm_tls_get_error();
+				errp = gtm_tls_get_error(NULL);
 				SET_DOLLARDEVICE_ONECOMMA_ERRSTR(iod, errp, errlen);
 				if (socketptr->ioerror)
-					rts_error_csa(CSA_ARG(NULL) VARLSTCNT(6) ERR_TLSINIT, 0, ERR_TEXT, 2, errlen, errp);
+					RTS_ERROR_CSA_ABT(NULL, VARLSTCNT(6) ERR_TLSINIT, 0, ERR_TEXT, 2, errlen, errp);
 				if (NO_M_TIMEOUT != msec_timeout)
 					dollar_truth = FALSE;
 				REVERT_GTMIO_CH(&iod->pair, ch_set);
 				return;
 			}
+			if (primary_exit_handler)
+				atexit(primary_exit_handler);
 		}
 		socketptr->tlsenabled = TRUE;
 		flags = GTMTLS_OP_SOCKET_DEV | ((tlsopt_client == option) ? GTMTLS_OP_CLIENT_MODE : 0);
@@ -227,7 +235,7 @@ void	iosocket_tls(mval *optionmval, int4 msec_timeout, mval *tlsid, mval *passwo
 			{	/* error string available */
 				socketptr->tlsenabled = FALSE;
 				free(extrastr);
-				errp = gtm_tls_get_error();
+				errp = gtm_tls_get_error(NULL);
 				SET_DOLLARDEVICE_ONECOMMA_ERRSTR(iod, errp, errlen);
 				rts_error_csa(CSA_ARG(NULL) VARLSTCNT(6) ERR_TLSCONVSOCK, 0, ERR_TEXT, 2, errlen, errp);
 				return;
@@ -245,20 +253,73 @@ void	iosocket_tls(mval *optionmval, int4 msec_timeout, mval *tlsid, mval *passwo
 			if (0 > gtm_tls_store_passwd(tls_ctx, idstr, passwordstr))
 			{	/* error string available */
 				socketptr->tlsenabled = FALSE;
-				errp = gtm_tls_get_error();
+				errp = gtm_tls_get_error(NULL);
 				SET_DOLLARDEVICE_ONECOMMA_ERRSTR(iod, errp, errlen);
 				rts_error_csa(CSA_ARG(NULL) VARLSTCNT(6) ERR_TLSCONVSOCK, 0, ERR_TEXT, 2, errlen, errp);
 				return;
 			}
 		}
+		if (socketptr->nonblocked_output)
+		{
+			if (GTM_TLS_API_VERSION_NONBLOCK > tls_ctx->plugin_version)
+			{	/* Due to lack of gtm_tls_version(), old pre GTM_TLS_API_VERSION_NONBLOCK plugins will not even
+				 * load but include this test as an example of how to check if the plugin loaded is new enough */
+				socketptr->tlsenabled = FALSE;
+				errp = gtm_tls_get_error(NULL);
+				SET_DOLLARDEVICE_ONECOMMA_ERRSTR(iod, errp, errlen);
+				rts_error_csa(CSA_ARG(NULL) VARLSTCNT(6) ERR_TLSCONVSOCK, 0, ERR_TEXT, 2, errlen, errp);
+				return;
+			}
+			flags |= GTMTLS_OP_NONBLOCK_WRITE;
+			if (!socketptr->nonblocking)
+			{	/* set O_NONBLOCK if needed before calling plugin */
+				FCNTL2(socketptr->sd, F_GETFL, fcntl_flags);
+				if (fcntl_flags < 0)
+				{
+					iod->dollar.za = 9;
+					save_errno = errno;
+					errptr = (char *)STRERROR(save_errno);
+					rts_error_csa(CSA_ARG(NULL) VARLSTCNT(7) ERR_GETSOCKOPTERR, 5,
+						LEN_AND_LIT("F_GETFL FOR NON BLOCKING WRITE"),
+						save_errno, LEN_AND_STR(errptr));
+					return;
+				}
+				FCNTL3(socketptr->sd, F_SETFL, fcntl_flags | (O_NDELAY | O_NONBLOCK), fcntl_res);
+				if (fcntl_res < 0)
+				{
+					iod->dollar.za = 9;
+					save_errno = errno;
+					errptr = (char *)STRERROR(save_errno);
+					rts_error_csa(CSA_ARG(NULL) VARLSTCNT(7) ERR_SETSOCKOPTERR, 5,
+						LEN_AND_LIT("F_SETFL FOR NON BLOCKING WRITE"),
+						save_errno, LEN_AND_STR(errptr));
+					return;
+				}
+				socketptr->nonblocking = TRUE;
+#				ifdef	DEBUG
+				if (WBTEST_ENABLED(WBTEST_SOCKET_NONBLOCK) && (0 < gtm_white_box_test_case_count))
+				{
+					if (-1 == setsockopt(socketptr->sd, SOL_SOCKET, SO_SNDBUF,
+						&gtm_white_box_test_case_count, SIZEOF(gtm_white_box_test_case_count)))
+					{
+						save_errno = errno;
+						errptr = (char *)STRERROR(save_errno);
+						rts_error_csa(CSA_ARG(NULL) VARLSTCNT(7) ERR_SETSOCKOPTERR, 5,
+							LEN_AND_LIT("SO_SNDBUF"), save_errno, LEN_AND_STR(errptr));
+						return;
+					}
+                                }
+#				endif
+			}
+		}
 		socketptr->tlssocket = gtm_tls_socket(tls_ctx, NULL, socketptr->sd, idstr, flags);
 		if (NULL == socketptr->tlssocket)
 		{
 			socketptr->tlsenabled = FALSE;
-			errp = gtm_tls_get_error();
+			errp = gtm_tls_get_error(NULL);
 			SET_DOLLARDEVICE_ONECOMMA_ERRSTR(iod, errp, errlen);
 			if (socketptr->ioerror)
-				rts_error_csa(CSA_ARG(NULL) VARLSTCNT(6) ERR_TLSCONVSOCK, 0, ERR_TEXT, 2, errlen, errp);
+				RTS_ERROR_CSA_ABT(NULL, VARLSTCNT(6) ERR_TLSCONVSOCK, 0, ERR_TEXT, 2, errlen, errp);
 			if (NO_M_TIMEOUT != msec_timeout)
 				dollar_truth = FALSE;
 			REVERT_GTMIO_CH(&iod->pair, ch_set);
@@ -318,7 +379,7 @@ void	iosocket_tls(mval *optionmval, int4 msec_timeout, mval *tlsid, mval *passwo
 				{
 					tls_errno = gtm_tls_errno();
 					if (0 > tls_errno)
-						errp = gtm_tls_get_error();
+						errp = gtm_tls_get_error((gtm_tls_socket_t *)socketptr->tlssocket);
 					else
 						errp = STRERROR(tls_errno);
 				} else
@@ -326,7 +387,7 @@ void	iosocket_tls(mval *optionmval, int4 msec_timeout, mval *tlsid, mval *passwo
 				socketptr->tlsenabled = FALSE;
 				SET_DOLLARDEVICE_ONECOMMA_ERRSTR(iod, errp, errlen);
 				if (socketptr->ioerror)
-					rts_error_csa(CSA_ARG(NULL) VARLSTCNT(6) ERR_TLSHANDSHAKE, 0,
+					RTS_ERROR_CSA_ABT(NULL, VARLSTCNT(6) ERR_TLSHANDSHAKE, 0,
 						ERR_TEXT, 2, errlen, errp);
 				if (NO_M_TIMEOUT != msec_timeout)
 					dollar_truth = FALSE;
@@ -368,12 +429,8 @@ void	iosocket_tls(mval *optionmval, int4 msec_timeout, mval *tlsid, mval *passwo
 			}
 		} while ((GTMTLS_WANT_READ == status) || (GTMTLS_WANT_WRITE == status));
 		/* turn on output buffering */
-		if (0 == socketptr->obuffer_size)
-			socketptr->obuffer_size = socketptr->buffer_size;
-		socketptr->obuffer_length = socketptr->obuffer_offset = 0;
-		socketptr->obuffer_wait_time = DEFAULT_WRITE_WAIT;
-		socketptr->obuffer_flush_time = DEFAULT_WRITE_WAIT * 2;	/* until add device parameter */
-		socketptr->obuffer = malloc(socketptr->obuffer_size);
+		SOCKET_OBUFFER_INIT(socketptr, socketptr->buffer_size, DEFAULT_WRITE_WAIT, (DEFAULT_WRITE_WAIT * 2));
+		socketptr->obuffer_in_use = TRUE;
 	} else if (tlsopt_renegotiate == option)
 	{
 		if (!socketptr->tlsenabled)
@@ -394,7 +451,7 @@ void	iosocket_tls(mval *optionmval, int4 msec_timeout, mval *tlsid, mval *passwo
 			errp = UNREAD_INPUT;
 			SET_DOLLARDEVICE_ONECOMMA_ERRSTR(iod, errp, errlen);
 			if (socketptr->ioerror)
-				rts_error_csa(CSA_ARG(NULL) VARLSTCNT(6) ERR_TLSRENEGOTIATE, 0,
+				RTS_ERROR_CSA_ABT(NULL, VARLSTCNT(6) ERR_TLSRENEGOTIATE, 0,
 					ERR_TEXT, 2, errlen, errp);
 			if (NO_M_TIMEOUT != msec_timeout)
 				dollar_truth = FALSE;
@@ -408,12 +465,12 @@ void	iosocket_tls(mval *optionmval, int4 msec_timeout, mval *tlsid, mval *passwo
 			{ /* get obuffer_error as additional ERR_TEXT */
 				errp = FLUSH_OUTPUT_ERROR;
 				if (-1 == socketptr->obuffer_errno)
-					errp2 = gtm_tls_get_error();
+					errp2 = gtm_tls_get_error((gtm_tls_socket_t *)socketptr->tlssocket);
 				else
 					errp2 = (char *)STRERROR(socketptr->obuffer_errno);
 				SET_DOLLARDEVICE_ONECOMMA_ERRSTR1_ERRSTR2(iod, errp, errlen, errp2, errlen2);
 				if (socketptr->ioerror)
-					rts_error_csa(CSA_ARG(NULL) VARLSTCNT(10) ERR_TLSRENEGOTIATE, 0,
+					RTS_ERROR_CSA_ABT(NULL, VARLSTCNT(10) ERR_TLSRENEGOTIATE, 0,
 						ERR_TEXT, 2, errlen, errp, ERR_TEXT, 2, errlen2, errp2);
 				if (NO_M_TIMEOUT != msec_timeout)
 					dollar_truth = FALSE;
@@ -452,12 +509,12 @@ void	iosocket_tls(mval *optionmval, int4 msec_timeout, mval *tlsid, mval *passwo
 		{
 			tls_errno = gtm_tls_errno();
 			if (0 > tls_errno)
-				errp = gtm_tls_get_error();
+				errp = gtm_tls_get_error((gtm_tls_socket_t *)socketptr->tlssocket);
 			else
 				errp = STRERROR(tls_errno);
 			SET_DOLLARDEVICE_ONECOMMA_ERRSTR(iod, errp, errlen);
 			if (socketptr->ioerror)
-				rts_error_csa(CSA_ARG(NULL) VARLSTCNT(6) ERR_TLSRENEGOTIATE, 0,
+				RTS_ERROR_CSA_ABT(NULL, VARLSTCNT(6) ERR_TLSRENEGOTIATE, 0,
 					ERR_TEXT, 2, errlen, errp);
 			if (NO_M_TIMEOUT != msec_timeout)
 				dollar_truth = FALSE;
@@ -465,7 +522,7 @@ void	iosocket_tls(mval *optionmval, int4 msec_timeout, mval *tlsid, mval *passwo
 			return;
 		}
 	} else
-		rts_error_csa(CSA_ARG(NULL) VARLSTCNT(6) ERR_TLSPARAM, 4, LEN_AND_STR(optionstr),
+		RTS_ERROR_CSA_ABT(NULL, VARLSTCNT(6) ERR_TLSPARAM, 4, LEN_AND_STR(optionstr),
 			LEN_AND_LIT("not a valid option"));
 	if (NO_M_TIMEOUT != msec_timeout)
 		dollar_truth = TRUE;
