@@ -1,6 +1,6 @@
 /****************************************************************
  *								*
- * Copyright (c) 2001-2019 Fidelity National Information	*
+ * Copyright (c) 2001-2022 Fidelity National Information	*
  * Services, Inc. and/or its subsidiaries. All rights reserved.	*
  *								*
  *	This source code contains the intellectual property	*
@@ -29,7 +29,8 @@
 #include "gtmio.h"
 #include "eintr_wrappers.h"
 #include "send_msg.h"
-#include "outofband.h"
+#include "have_crit.h"
+#include "deferred_events_queue.h"
 #include "error.h"
 #include "std_dev_outbndset.h"
 #include "wake_alarm.h"
@@ -41,55 +42,54 @@
 #include "gtm_utf8.h"
 #endif
 
-GBLREF	bool		out_of_time;
-GBLREF	boolean_t	dollar_zininterrupt, gtm_utf8_mode, prin_in_dev_failure, prin_out_dev_failure;
-GBLREF	int4		ctrap_action_is, exi_condition;
-GBLREF	io_pair		io_curr_device, io_std_device;
-GBLREF	mval		dollar_zstatus;
-GBLREF	mv_stent	*mv_chain;
-GBLREF	stack_frame	*frame_pointer;
-GBLREF	unsigned char	*msp, *stackbase, *stacktop, *stackwarn;
-GBLREF	volatile int4	outofband;
+GBLREF	bool			out_of_time;
+GBLREF	boolean_t		gtm_utf8_mode, hup_on, prin_in_dev_failure, prin_out_dev_failure;
+GBLREF	int4			exi_condition;
+GBLREF	io_pair			io_curr_device, io_std_device;
+GBLREF	mval			dollar_zstatus;
+GBLREF	mv_stent		*mv_chain;
+GBLREF	stack_frame		*frame_pointer;
+GBLREF	unsigned char		*msp, *stackbase, *stacktop, *stackwarn;
+GBLREF	volatile boolean_t	dollar_zininterrupt;
+GBLREF	volatile int4		outofband;
 
 LITREF	unsigned char	lower_to_upper_table[];
 
-error_def(ERR_CTRAP);
 error_def(ERR_IOEOF);
 error_def(ERR_NOPRINCIO);
-error_def(ERR_STACKOFLOW);
-error_def(ERR_STACKCRIT);
 error_def(ERR_TERMHANGUP);
 error_def(ERR_ZINTRECURSEIO);
 
 int	iott_rdone (mint *v, int4 msec_timeout)	/* timeout in milliseconds */
 {
-	boolean_t	ret = FALSE, timed, utf8_active, zint_restart, first_time;
-	unsigned char	inbyte;
+	ABS_TIME	cur_time, end_time;
+	boolean_t	ch_set, first_time, ret = FALSE, timed, utf8_active, zint_restart;
+	char		dc1, dc3;
+	d_tt_struct	*tt_ptr;
+	fd_set		input_fd;
+	int		inchar_width, msk_in, msk_num, rdlen, selstat, status, utf8_more;
+	io_desc		*io_ptr;
+	mv_stent	*mv_zintdev;
+	short int	i;
+	struct timeval	input_timeval;
+	TID		timer_id;
+	tt_interrupt	*tt_state;
+	uint4		mask;
+	unsigned char	inbyte, *zb_ptr, *zb_top;
+	unsigned char	more_buf[GTM_MB_LEN_MAX + 1], *more_ptr;	/* to build up multi byte for character */
 	wint_t		inchar;
 #ifdef __MVS__
 	wint_t		asc_inchar;
 #endif
-	char		dc1, dc3;
-	short int	i;
-	io_desc		*io_ptr;
-	d_tt_struct	*tt_ptr;
-	tt_interrupt	*tt_state;
-	TID		timer_id;
-	int		rdlen, selstat, status, utf8_more, inchar_width;
-	uint4		mask;
-	int		msk_in, msk_num;
-	unsigned char	*zb_ptr, *zb_top;
-	unsigned char	more_buf[GTM_MB_LEN_MAX + 1], *more_ptr;	/* to build up multi byte for character */
-	fd_set		input_fd;
-	struct timeval	input_timeval;
-	ABS_TIME	cur_time, end_time;
-	mv_stent	*mv_zintdev;
-	boolean_t	ch_set;
 
 	io_ptr = io_curr_device.in;
-	ESTABLISH_RET_GTMIO_CH(&io_curr_device, -1, ch_set);
-	if ((sighup == outofband) || ((int)ERR_TERMHANGUP == SIGNAL))
+	if (ERR_TERMHANGUP == error_condition)
+	{
 		TERMHUP_NOPRINCIO_CHECK(FALSE);				/* FALSE for READ */
+		io_ptr->dollar.za = ZA_IO_ERR;
+		return FALSE;
+	}
+	ESTABLISH_RET_GTMIO_CH(&io_curr_device, -1, ch_set);
 	assert(io_ptr->state == dev_open);
 	iott_flush(io_curr_device.out);
 	tt_ptr = (d_tt_struct*) io_ptr->dev_sp;
@@ -108,7 +108,7 @@ int	iott_rdone (mint *v, int4 msec_timeout)	/* timeout in milliseconds */
 		{
 			tt_ptr->mupintr = FALSE;
 			tt_state->who_saved = ttwhichinvalid;
-			rts_error_csa(CSA_ARG(NULL) VARLSTCNT(1) ERR_ZINTRECURSEIO);
+			RTS_ERROR_CSA_ABT(NULL, VARLSTCNT(1) ERR_ZINTRECURSEIO);
 		}
 		assertpro(ttrdone == tt_state->who_saved);	/* ZINTRECURSEIO should have caught */
 		mv_zintdev = io_find_mvstent(io_ptr, FALSE);
@@ -154,7 +154,7 @@ int	iott_rdone (mint *v, int4 msec_timeout)	/* timeout in milliseconds */
 			DOWRITERC(tt_ptr->fildes, &dc1, 1, status);
 			if (0 != status)
 			{
-				io_ptr->dollar.za = 9;
+				io_ptr->dollar.za = ZA_IO_ERR;
 				rts_error_csa(CSA_ARG(NULL) VARLSTCNT(1) status);
 			}
 		}
@@ -210,7 +210,7 @@ int	iott_rdone (mint *v, int4 msec_timeout)	/* timeout in milliseconds */
 					iott_rterm(io_ptr);
 			}
 			REVERT_GTMIO_CH(&io_curr_device, ch_set);
-			outofband_action(FALSE);
+			async_action(FALSE);
 			break;
 		}
 		if (!first_time)
@@ -251,7 +251,7 @@ int	iott_rdone (mint *v, int4 msec_timeout)	/* timeout in milliseconds */
 		{
 			if (EINTR != errno)
 			{
-				io_ptr->dollar.za = 9;
+				io_ptr->dollar.za = ZA_IO_ERR;
 				if (timed && (0 == msec_timeout))
 					iott_rterm(io_ptr);
 				rts_error_csa(CSA_ARG(NULL) VARLSTCNT(1) errno);
@@ -291,7 +291,7 @@ int	iott_rdone (mint *v, int4 msec_timeout)	/* timeout in milliseconds */
 					UTF8_MBTOWC(more_buf, more_ptr, inchar);
 					if (WEOF == inchar)
 					{	/* invalid char */
-						io_ptr->dollar.za = 9;
+						io_ptr->dollar.za = ZA_IO_ERR;
 						/* No data to return */
 						iott_readfl_badchar(NULL, NULL, 0,
 								    (int)((more_ptr - more_buf)), more_buf, more_ptr, NULL);
@@ -305,7 +305,7 @@ int	iott_rdone (mint *v, int4 msec_timeout)	/* timeout in milliseconds */
 					{
 						if ((0 > utf8_more) || (GTM_MB_LEN_MAX < utf8_more))
 						{	/* invalid character */
-							io_ptr->dollar.za = 9;
+							io_ptr->dollar.za = ZA_IO_ERR;
 							*more_ptr++ = inbyte;
 							 /* No data to return */
 							iott_readfl_badchar(NULL, NULL, 0, 1, more_buf, more_ptr, NULL);
@@ -322,7 +322,7 @@ int	iott_rdone (mint *v, int4 msec_timeout)	/* timeout in milliseconds */
 						UTF8_MBTOWC(more_buf, more_ptr, inchar);
 						if (WEOF == inchar)
 						{	/* invalid char */
-							io_ptr->dollar.za = 9;
+							io_ptr->dollar.za = ZA_IO_ERR;
 							 /* No data to return */
 							iott_readfl_badchar(NULL, NULL, 0, 1, more_buf, more_ptr, NULL);
 							utf8_badchar(1, more_buf, more_ptr, 0, NULL);   /* ERR_BADCHAR */
@@ -352,7 +352,7 @@ int	iott_rdone (mint *v, int4 msec_timeout)	/* timeout in milliseconds */
 					if (0 == msec_timeout)
 				  		iott_rterm(io_ptr);
 				}
-				rts_error_csa(CSA_ARG(NULL) VARLSTCNT(3) ERR_CTRAP, 1, ctrap_action_is);
+				async_action(FALSE);
 				ret = FALSE;
 				break;
 			}
@@ -396,7 +396,7 @@ int	iott_rdone (mint *v, int4 msec_timeout)	/* timeout in milliseconds */
 				} while (1 == rdlen);
 				*zb_ptr++ = 0;
 				if (rdlen != 1  &&  io_ptr->dollar.za == 0)
-					io_ptr->dollar.za = 9;
+					io_ptr->dollar.za = ZA_IO_ERR;
 				/* -------------------------------------------------
 				 * End of escape sequence...do not process further.
 				 * -------------------------------------------------
@@ -418,7 +418,7 @@ int	iott_rdone (mint *v, int4 msec_timeout)	/* timeout in milliseconds */
 					if (0 >= status)
 					{
 						status = errno;
-						io_ptr->dollar.za = 9;
+						io_ptr->dollar.za = ZA_IO_ERR;
 						if (timed)
 						{
 							if (0 == msec_timeout)
@@ -433,7 +433,7 @@ int	iott_rdone (mint *v, int4 msec_timeout)	/* timeout in milliseconds */
 		{
 			if (errno != EINTR)
 			{
-				io_ptr->dollar.za = 9;
+				io_ptr->dollar.za = ZA_IO_ERR;
 				if (timed && (0 == msec_timeout))
 					iott_rterm(io_ptr);
 				rts_error_csa(CSA_ARG(NULL) VARLSTCNT(1) errno);
@@ -452,14 +452,14 @@ int	iott_rdone (mint *v, int4 msec_timeout)	/* timeout in milliseconds */
 			ISSUE_NOPRINCIO_IF_NEEDED(io_ptr, FALSE, FALSE);	/* FALSE, FALSE: READ, not socket*/
 			if (io_ptr->dollar.zeof)
 			{
-				io_ptr->dollar.za = 9;
-				rts_error_csa(CSA_ARG(NULL) VARLSTCNT(1) ERR_IOEOF);
+				io_ptr->dollar.za = ZA_IO_ERR;
+				RTS_ERROR_CSA_ABT(NULL, VARLSTCNT(1) ERR_IOEOF);
 			} else
 			{
 				io_ptr->dollar.zeof = TRUE;
 				io_ptr->dollar.za   = 0;
 				if (io_ptr->error_handler.len > 0)
-					rts_error_csa(CSA_ARG(NULL) VARLSTCNT(1) ERR_IOEOF);
+					RTS_ERROR_CSA_ABT(NULL, VARLSTCNT(1) ERR_IOEOF);
 			}
 			break;
 		}
@@ -474,13 +474,13 @@ int	iott_rdone (mint *v, int4 msec_timeout)	/* timeout in milliseconds */
 		DOWRITERC(tt_ptr->fildes, &dc3, 1, status);
 		if (0 != status)
 		{
-			io_ptr->dollar.za = 9;
+			io_ptr->dollar.za = ZA_IO_ERR;
 			rts_error_csa(CSA_ARG(NULL) VARLSTCNT(1) status);
 		}
 	}
 	if (outofband && jobinterrupt != outofband)
 	{
-		io_ptr->dollar.za = 9;
+		io_ptr->dollar.za = ZA_IO_ERR;
 		REVERT_GTMIO_CH(&io_curr_device, ch_set);
 		return FALSE;
 	}
