@@ -1,6 +1,6 @@
 /****************************************************************
  *								*
- * Copyright (c) 2001-2019 Fidelity National Information	*
+ * Copyright (c) 2001-2022 Fidelity National Information	*
  * Services, Inc. and/or its subsidiaries. All rights reserved.	*
  *								*
  *	This source code contains the intellectual property	*
@@ -30,7 +30,8 @@
 #include "gtmio.h"
 #include "eintr_wrappers.h"
 #include "send_msg.h"
-#include "outofband.h"
+#include "have_crit.h"
+#include "deferred_events_queue.h"
 #include "error.h"
 #include "std_dev_outbndset.h"
 #include "wake_alarm.h"
@@ -43,27 +44,27 @@
 #include "gtm_utf8.h"
 #endif
 
-GBLREF	boolean_t	dollar_zininterrupt, gtm_utf8_mode, prin_in_dev_failure, prin_out_dev_failure;
-GBLDEF	int		term_error_line;		/* record for cores */
-GBLREF	int4		ctrap_action_is, exi_condition;
-GBLREF	io_pair		io_curr_device, io_std_device;
-GBLREF	spdesc		stringpool;
-GBLREF	mval		dollar_zstatus;
-GBLREF	mv_stent	*mv_chain;
-GBLREF	volatile int4	outofband;
-GBLREF	stack_frame	*frame_pointer;
-GBLREF	unsigned char	*msp, *stackbase, *stacktop, *stackwarn;
+GBLREF	boolean_t		gtm_utf8_mode, hup_on, prin_in_dev_failure, prin_out_dev_failure;
+GBLREF	char			*KEY_BACKSPACE, *KEY_DC, *KEY_DOWN, *KEY_INSERT, *KEY_LEFT, *KEYPAD_LOCAL, *KEYPAD_XMIT, *KEY_RIGHT,
+					*KEY_UP;
+GBLREF	int			AUTO_RIGHT_MARGIN, EAT_NEWLINE_GLITCH;
+GBLDEF	int			term_error_line;		/* record for cores */
+GBLREF	int4			exi_condition;
+GBLREF	io_pair			io_curr_device, io_std_device;
+GBLREF	mval			dollar_zstatus;
+GBLREF	mv_stent		*mv_chain;
+GBLREF	spdesc			stringpool;
+GBLREF	stack_frame		*frame_pointer;
+GBLREF	unsigned char		*msp, *stackbase, *stacktop, *stackwarn;
+GBLREF	volatile boolean_t	dollar_zininterrupt;
+GBLREF	volatile int4		outofband;
+
+LITREF	unsigned char		lower_to_upper_table[];
 
 #ifdef UTF8_SUPPORTED
 LITREF	UChar32		u32_line_term[];
 #endif
 
-GBLREF	int		AUTO_RIGHT_MARGIN, EAT_NEWLINE_GLITCH;
-GBLREF	char		*KEY_BACKSPACE, *KEY_DC;
-GBLREF	char		*KEY_DOWN, *KEY_LEFT, *KEY_RIGHT, *KEY_UP;
-GBLREF	char		*KEY_INSERT;
-GBLREF	char		*KEYPAD_LOCAL, *KEYPAD_XMIT;
-
 #ifdef __MVS__
 #	define SEND_KEYPAD_LOCAL
 #else
@@ -72,18 +73,13 @@ GBLREF	char		*KEYPAD_LOCAL, *KEYPAD_XMIT;
 			DOWRITE(tt_ptr->fildes, KEYPAD_LOCAL, keypad_len);
 #endif
 
-LITREF	unsigned char	lower_to_upper_table[];
-
 /* dc1 & dc3 have the same value in ASCII and EBCDIC */
 static readonly char		dc1 = 17;
 static readonly char		dc3 = 19;
 static readonly unsigned char	eraser[3] = { NATIVE_BS, NATIVE_SP, NATIVE_BS };
 
-error_def(ERR_CTRAP);
 error_def(ERR_IOEOF);
 error_def(ERR_NOPRINCIO);
-error_def(ERR_STACKOFLOW);
-error_def(ERR_STACKCRIT);
 error_def(ERR_TERMHANGUP);
 error_def(ERR_ZINTRECURSEIO);
 
@@ -138,45 +134,47 @@ void iott_readfl_badchar(mval *vmvalptr, wint_t *dataptr32, int datalen,
 
 int	iott_readfl(mval *v, int4 length, int4 msec_timeout)	/* timeout in milliseconds */
 {
-	boolean_t	ret, nonzerotimeout, timed, insert_mode, edit_mode, utf8_active, zint_restart, buffer_moved;
-	uint4		mask;
-	wint_t		inchar, *current_32_ptr, *buffer_32_start, switch_char;
-	unsigned char	inbyte, *outptr, *outtop;
-#ifdef __MVS__
-	wint_t		asc_inchar;
-#endif
-	unsigned char	more_buf[GTM_MB_LEN_MAX + 1], *more_ptr;	/* to build up multi byte for character */
-	unsigned char	*current_ptr;		/* insert next character into buffer here */
-	unsigned char	*buffer_start;		/* beginning of non UTF8 buffer */
-	int		msk_in, msk_num, rdlen, save_errno, selstat, status, ioptr_width, i, utf8_more;
-	int		exp_length;
-	int		inchar_width;		/* display width of inchar */
+	ABS_TIME	cur_time, end_time;
+	boolean_t	buffer_moved, ch_set, edit_mode, empterm, escape_edit, insert_mode, nonzerotimeout, ret, timed, utf8_active,
+				zint_restart;
+	d_tt_struct	*tt_ptr;
+	fd_set		input_fd;
+	int		backspace, delete, down, i, insert_key, ioptr_width, exp_length, keypad_len, left, msk_in, msk_num,
+				rdlen, right, save_errno, selstat, status, up, utf8_more;
 	int		delchar_width;		/* display width of deleted char */
 	int		delta_width;		/* display width change for replaced char */
 	int		dx, dx_start;		/* local dollar X, starting value */
 	int		dx_instr, dx_outlen;	/* wcwidth of string to insert point, whole string */
 	int		dx_prev, dx_cur, dx_next;/* wcwidth of string to char BEFORE, AT and AFTER the insert point */
+	int		inchar_width;		/* display width of inchar */
 	int		instr;			/* insert point in input string */
 	int		outlen;			/* total characters in line so far */
-	int		keypad_len, backspace, delete;
-	int		up, down, right, left, insert_key;
-	boolean_t	escape_edit, empterm;
 	io_desc		*io_ptr;
-	d_tt_struct	*tt_ptr;
 	io_terminator	outofbands;
 	io_termmask	mask_term;
-	tt_interrupt	*tt_state;
 	mv_stent	*mvc, *mv_zintdev;
-	unsigned char	*zb_ptr, *zb_top;
-	ABS_TIME	cur_time, end_time;
-	fd_set		input_fd;
+	tt_interrupt	*tt_state;
+	uint4		mask;
+	unsigned char	inbyte, *outptr, *outtop, *zb_ptr, *zb_top;
+	unsigned char	more_buf[GTM_MB_LEN_MAX + 1], *more_ptr;	/* to build up multi byte for character */
+	unsigned char	*current_ptr;		/* insert next character into buffer here */
+	unsigned char	*buffer_start;		/* beginning of non UTF8 buffer */
 	struct timeval	input_timeval;
 	struct timeval	save_input_timeval;
-	boolean_t	ch_set;
+	wint_t		inchar, *current_32_ptr, *buffer_32_start, switch_char;
+#ifdef __MVS__
+	wint_t		asc_inchar;
+#endif
 
 	assert(stringpool.free >= stringpool.base);
 	assert(stringpool.free <= stringpool.top);
 	io_ptr = io_curr_device.in;
+	if (ERR_TERMHANGUP == error_condition)
+	{
+		TERMHUP_NOPRINCIO_CHECK(FALSE);				/* FALSE for READ */
+		io_ptr->dollar.za = ZA_IO_ERR;
+		return FALSE;
+	}
 	ESTABLISH_RET_GTMIO_CH(&io_curr_device, -1, ch_set);
 	tt_ptr = (d_tt_struct *)(io_ptr->dev_sp);
 	assert(dev_open == io_ptr->state);
@@ -196,7 +194,7 @@ int	iott_readfl(mval *v, int4 length, int4 msec_timeout)	/* timeout in milliseco
 		{
 			tt_ptr->mupintr = FALSE;
 			tt_state->who_saved = ttwhichinvalid;
-			rts_error_csa(CSA_ARG(NULL) VARLSTCNT(1) ERR_ZINTRECURSEIO);
+			RTS_ERROR_CSA_ABT(NULL, VARLSTCNT(1) ERR_ZINTRECURSEIO);
 		}
 		assert(length == tt_state->length);
 		assertpro(ttread == tt_state->who_saved);	/* ZINTRECURSEIO should have caught */
@@ -266,8 +264,6 @@ int	iott_readfl(mval *v, int4 length, int4 msec_timeout)	/* timeout in milliseco
 		io_ptr->dollar.zeof = FALSE;
 		dx_start = (int)io_ptr->dollar.x;
 	}
-	if ((sighup == outofband) || ((int)ERR_TERMHANGUP == SIGNAL))
-		TERMHUP_NOPRINCIO_CHECK(FALSE);				/* FALSE for READ */
 	v->str.len = 0;
 	ret = TRUE;
 	mask = tt_ptr->term_ctrl;
@@ -283,7 +279,7 @@ int	iott_readfl(mval *v, int4 length, int4 msec_timeout)	/* timeout in milliseco
 			DOWRITERC(tt_ptr->fildes, &dc1, 1, status);
 			if (0 != status)
 			{
-				io_ptr->dollar.za = 9;
+				io_ptr->dollar.za = ZA_IO_ERR;
 				rts_error_csa(CSA_ARG(NULL) VARLSTCNT(1) status);
 			}
 		}
@@ -307,7 +303,7 @@ int	iott_readfl(mval *v, int4 length, int4 msec_timeout)	/* timeout in milliseco
 				DOWRITERC(tt_ptr->fildes, KEYPAD_XMIT, keypad_len, status);
 				if (0 != status)
 				{
-					io_ptr->dollar.za = 9;
+					io_ptr->dollar.za = ZA_IO_ERR;
 					rts_error_csa(CSA_ARG(NULL) VARLSTCNT(1) status);
 				}
 			}
@@ -384,7 +380,7 @@ int	iott_readfl(mval *v, int4 length, int4 msec_timeout)	/* timeout in milliseco
 				if (!msec_timeout)
 					iott_rterm(io_ptr);
 			}
-			outofband_action(FALSE);
+			async_action(FALSE);
 			break;
 		}
 		errno = 0;
@@ -427,11 +423,11 @@ int	iott_readfl(mval *v, int4 length, int4 msec_timeout)	/* timeout in milliseco
 					 */
 					io_ptr->dollar.zeof = TRUE;
 					io_ptr->dollar.x = 0;
-					io_ptr->dollar.za = 9;
+					io_ptr->dollar.za = ZA_IO_ERR;
 					io_ptr->dollar.y++;
 					tt_ptr->discard_lf = FALSE;
 					if (io_ptr->error_handler.len > 0)
-						rts_error_csa(CSA_ARG(NULL) VARLSTCNT(1) ERR_IOEOF);
+						RTS_ERROR_CSA_ABT(NULL, VARLSTCNT(1) ERR_IOEOF);
 					break;
 				} else
 					io_ptr->dollar.zeof = FALSE;
@@ -453,7 +449,7 @@ int	iott_readfl(mval *v, int4 length, int4 msec_timeout)	/* timeout in milliseco
 					UTF8_MBTOWC(more_buf, more_ptr, inchar);
 					if (WEOF == inchar)
 					{	/* invalid char */
-						io_ptr->dollar.za = 9;
+						io_ptr->dollar.za = ZA_IO_ERR;
 						iott_readfl_badchar(v, buffer_32_start, outlen,
 								    (int)(more_ptr - more_buf), more_buf, more_ptr, buffer_start);
 						utf8_badchar((int)(more_ptr - more_buf), more_buf, more_ptr, 0, NULL); /* BADCHAR */
@@ -464,7 +460,7 @@ int	iott_readfl(mval *v, int4 length, int4 msec_timeout)	/* timeout in milliseco
 					more_ptr = more_buf;
 					if (0 > utf8_more)
 					{	/* invalid character */
-						io_ptr->dollar.za = 9;
+						io_ptr->dollar.za = ZA_IO_ERR;
 						*more_ptr++ = inbyte;
 						iott_readfl_badchar(v, buffer_32_start, outlen,
 								1, more_buf, more_ptr, buffer_start);
@@ -472,7 +468,7 @@ int	iott_readfl(mval *v, int4 length, int4 msec_timeout)	/* timeout in milliseco
 						break;
 					} else if (GTM_MB_LEN_MAX < utf8_more)
 					{	/* too big to be valid */
-						io_ptr->dollar.za = 9;
+						io_ptr->dollar.za = ZA_IO_ERR;
 						*more_ptr++ = inbyte;
 						iott_readfl_badchar(v, buffer_32_start, outlen,
 								1, more_buf, more_ptr, buffer_start);
@@ -490,7 +486,7 @@ int	iott_readfl(mval *v, int4 length, int4 msec_timeout)	/* timeout in milliseco
 					UTF8_MBTOWC(more_buf, more_ptr, inchar);
 					if (WEOF == inchar)
 					{	/* invalid char */
-						io_ptr->dollar.za = 9;
+						io_ptr->dollar.za = ZA_IO_ERR;
 						iott_readfl_badchar(v, buffer_32_start, outlen,
 								1, more_buf, more_ptr, buffer_start);
 						utf8_badchar(1, more_buf, more_ptr, 0, NULL);	/* ERR_BADCHAR */
@@ -525,12 +521,13 @@ int	iott_readfl(mval *v, int4 length, int4 msec_timeout)	/* timeout in milliseco
 			if ((' ' > INPUT_CHAR) && (tt_ptr->enbld_outofbands.mask & (1 << INPUT_CHAR)))
 			{	/* ctrap supercedes editing so check first */
 				instr = outlen = 0;
-				io_ptr->dollar.za = 9;
+				io_ptr->dollar.za = ZA_IO_ERR;
 				std_dev_outbndset(INPUT_CHAR);	/* it needs ASCII?	*/
 				SEND_KEYPAD_LOCAL
 				if (!msec_timeout)
 					iott_rterm(io_ptr);
-				rts_error_csa(CSA_ARG(NULL) VARLSTCNT(3) ERR_CTRAP, 1, ctrap_action_is);
+				outofband = ctrap;
+				async_action(FALSE);
 				break;
 			}
 			if (((0 != (mask & TRM_ESCAPE)) || edit_mode)
@@ -919,16 +916,16 @@ int	iott_readfl(mval *v, int4 length, int4 msec_timeout)	/* timeout in milliseco
 				ISSUE_NOPRINCIO_IF_NEEDED(io_ptr, FALSE, FALSE);	/* FALSE, FALSE: READ, not socket*/
 				if (io_ptr->dollar.zeof)
 				{
-					io_ptr->dollar.za = 9;
+					io_ptr->dollar.za = ZA_IO_ERR;
 					SEND_KEYPAD_LOCAL
-					rts_error_csa(CSA_ARG(NULL) VARLSTCNT(1) ERR_IOEOF);
+					RTS_ERROR_CSA_ABT(NULL, VARLSTCNT(1) ERR_IOEOF);
 				} else
 				{
 					io_ptr->dollar.zeof = TRUE;
 					io_ptr->dollar.za = 0;
 					SEND_KEYPAD_LOCAL
 					if (0 < io_ptr->error_handler.len)
-						rts_error_csa(CSA_ARG(NULL) VARLSTCNT(1) ERR_IOEOF);
+						RTS_ERROR_CSA_ABT(NULL, VARLSTCNT(1) ERR_IOEOF);
 				}
 				break;
 			}
@@ -940,7 +937,7 @@ int	iott_readfl(mval *v, int4 length, int4 msec_timeout)	/* timeout in milliseco
 				io_ptr->dollar.y++;
 				SEND_KEYPAD_LOCAL
 				if (0 < io_ptr->error_handler.len)
-					rts_error_csa(CSA_ARG(NULL) VARLSTCNT(1) ERR_IOEOF);
+					RTS_ERROR_CSA_ABT(NULL, VARLSTCNT(1) ERR_IOEOF);
 				break;
 			}
 		} else if (EINTR != errno)	/* rdlen < 0 */
@@ -1116,7 +1113,7 @@ int	iott_readfl(mval *v, int4 length, int4 msec_timeout)	/* timeout in milliseco
 		DOWRITERC(tt_ptr->fildes, &dc3, 1, status);
 		if (0 != status)
 		{
-			io_ptr->dollar.za = 9;
+			io_ptr->dollar.za = ZA_IO_ERR;
 			rts_error_csa(CSA_ARG(NULL) VARLSTCNT(1) status);
 		}
 	}
@@ -1124,7 +1121,7 @@ int	iott_readfl(mval *v, int4 length, int4 msec_timeout)	/* timeout in milliseco
 	if (outofband && (jobinterrupt != outofband))
 	{
 		v->str.len = 0;
-		io_ptr->dollar.za = 9;
+		io_ptr->dollar.za = ZA_IO_ERR;
 		REVERT_GTMIO_CH(&io_curr_device, ch_set);
 		return(FALSE);
 	}
@@ -1171,7 +1168,7 @@ int	iott_readfl(mval *v, int4 length, int4 msec_timeout)	/* timeout in milliseco
 
 term_error:
 	save_errno = errno;
-	io_ptr->dollar.za = 9;
+	io_ptr->dollar.za = ZA_IO_ERR;
 	tt_ptr->discard_lf = FALSE;
 	SEND_KEYPAD_LOCAL	/* to turn keypad off if possible */
 	if (!msec_timeout)
