@@ -1,6 +1,6 @@
 /****************************************************************
  *								*
- * Copyright (c) 2001-2019 Fidelity National Information	*
+ * Copyright (c) 2001-2022 Fidelity National Information	*
  * Services, Inc. and/or its subsidiaries. All rights reserved.	*
  *								*
  *	This source code contains the intellectual property	*
@@ -39,7 +39,7 @@ GBLREF	int		process_exiting;
 
 void jnl_file_close_timer(void)
 {
-	boolean_t		any_jnl_open = FALSE;
+	boolean_t		do_timer_restart = FALSE;
 	gd_addr			*addr_ptr;
 	gd_region		*r_local, *r_top;
 	int			rc;
@@ -73,7 +73,7 @@ void jnl_file_close_timer(void)
 				jpc = csa->jnl;
 				if (csa->now_crit)
 				{
-					any_jnl_open |= ((NULL != jpc) && (NOJNL != jpc->channel));
+					do_timer_restart |= csa->snapshot_in_prog || ((NULL != jpc) && (NOJNL != jpc->channel));
 					continue;
 				}
 				if ((NULL != jpc) && (NOJNL != jpc->channel) && JNL_FILE_SWITCHED(jpc))
@@ -83,18 +83,18 @@ void jnl_file_close_timer(void)
 					JNL_FD_CLOSE(jpc->channel, rc);	/* sets jpc->channel to NOJNL */
 					jpc->pini_addr = 0;
 				}
-				any_jnl_open |= ((NULL != jpc) && (NOJNL != jpc->channel));
+				do_timer_restart |= csa->snapshot_in_prog || ((NULL != jpc) && (NOJNL != jpc->channel));
 			}
 		}
 	}
 	/* Only restart the timer if there are still journal files open, or if we didn't check because it wasn't safe.
 	 * Otherwise, it will be restarted on the next successful jnl_file_open(), or never, if we are exiting.
 	 */
-	if (any_jnl_open || (INTRPT_OK_TO_INTERRUPT != intrpt_ok_state))
+	if (do_timer_restart || (INTRPT_OK_TO_INTERRUPT != intrpt_ok_state))
 		start_timer((TID)jnl_file_close_timer, OLDERJNL_CHECK_INTERVAL, jnl_file_close_timer, 0, NULL);
 	else
 	{
-		assert(!any_jnl_open || process_exiting);
+		assert(!do_timer_restart || process_exiting);
 		oldjnlclose_started = FALSE;
 	}
 }
