@@ -1,6 +1,6 @@
 /****************************************************************
  *								*
- * Copyright (c) 2001-2019 Fidelity National Information	*
+ * Copyright (c) 2001-2022 Fidelity National Information	*
  * Services, Inc. and/or its subsidiaries. All rights reserved.	*
  *								*
  *	This source code contains the intellectual property	*
@@ -26,6 +26,7 @@
 #include "iosp.h"
 #include "error.h"
 #include "cli.h"
+#include "io.h"
 #include "stringpool.h"
 #include "interlock.h"
 #include "gtmimagename.h"
@@ -61,6 +62,8 @@
 #include "have_crit.h"
 #include "gt_timers_add_safe_hndlrs.h"
 #include "continue_handler.h"
+#include "restrict.h"
+#include "dm_audit_log.h"
 
 #ifdef UTF8_SUPPORTED
 # include "gtm_icu_api.h"
@@ -73,22 +76,22 @@ GBLREF VSIG_ATOMIC_T		util_interrupt;
 GBLREF bool			licensed;
 GBLREF void			(*func)(void);
 GBLREF spdesc			rts_stringpool, stringpool;
-GBLREF global_latch_t		defer_latch;
 GBLREF char			cli_err_str[];
 GBLREF CLI_ENTRY		lke_cmd_ary[];
 GBLREF ch_ret_type		(*stpgc_ch)();			/* Function pointer to stp_gcol_ch */
-
+GBLREF void 			(*primary_exit_handler)(void);
 GBLDEF CLI_ENTRY		*cmd_ary = &lke_cmd_ary[0];	/* Define cmd_ary to be the LKE specific cmd table */
+GBLREF IN_PARMS			*cli_lex_in_ptr;
 
 static bool lke_process(int argc);
 static void display_prompt(void);
 
 error_def(ERR_CTRLC);
+error_def(ERR_RESTRICTEDOP);
 
 int main (int argc, char *argv[])
 {
 	DCL_THREADGBL_ACCESS;
-
 	GTM_THREADGBL_INIT;
 	common_startup_init(LKE_IMAGE);
 	licensed = TRUE;
@@ -96,7 +99,7 @@ int main (int argc, char *argv[])
 	UTF8_ONLY(gtm_strToTitle_ptr = &gtm_strToTitle);
 	sig_init(generic_signal_handler, lke_ctrlc_handler, suspsigs_handler, continue_handler);
 	atexit(util_exit_handler);
-	SET_LATCH_GLOBAL(&defer_latch, LOCK_AVAILABLE);
+	primary_exit_handler = util_exit_handler;
 	stp_init(STP_INITSIZE);
 	stpgc_ch = &stp_gcol_ch;
 	rts_stringpool = stringpool;
@@ -106,10 +109,13 @@ int main (int argc, char *argv[])
 	prealloc_gt_timers();
 	gt_timers_add_safe_hndlrs();
 	initialize_pattern_table();
+	if (RESTRICTED(lke))
+		RTS_ERROR_CSA_ABT(NULL, VARLSTCNT(3) ERR_RESTRICTEDOP, 1, "LKE");
 	gvinit();
 	region_init(FALSE);	/* Was TRUE, but that doesn't actually work if there are GTCM regions in the GLD,
 				 * at least in DEBUG, so leave it off for now to allow LKE to work in this situation.
 				 */
+	io_init_name();
 	cli_lex_setup(argc, argv);
 	/*      this should be after cli_lex_setup() due to S390 A/E conversion    */
 	OPERATOR_LOG_MSG;
@@ -125,7 +131,7 @@ int main (int argc, char *argv[])
 static bool lke_process(int argc)
 {
 	bool		flag = FALSE;
-	int		res;
+	int		res, status;
 	static int	save_stderr = SYS_STDERR;
 
 	ESTABLISH_RET(util_ch, TRUE);
@@ -134,12 +140,18 @@ static bool lke_process(int argc)
 	if (SYS_STDERR != save_stderr)  /* necesary in case of rts_error */
 		close_fileio(&save_stderr);
 	assert(SYS_STDERR == save_stderr);
-
 	func = 0;
 	util_interrupt = 0;
  	if (argc < 2)
 		display_prompt();
-	if ( EOF == (res = parse_cmd()))
+	res = parse_cmd();
+	if (EOF != res)
+	{
+		status = log_cmd_if_needed(cli_lex_in_ptr->in_str);
+		if (status)
+			return FALSE;
+	}
+	if (EOF == res)
 	{
 		if (util_interrupt)
 		{
@@ -178,4 +190,3 @@ static void display_prompt(void)
 	PRINTF("LKE> ");
 	FFLUSH(stdout);
 }
-
