@@ -1,6 +1,6 @@
 /****************************************************************
  *                                                              *
- * Copyright (c) 2001-2018 Fidelity National Information	*
+ * Copyright (c) 2001-2022 Fidelity National Information	*
  * Services, Inc. and/or its subsidiaries. All rights reserved.	*
  *                                                              *
  *    This source code contains the intellectual property       *
@@ -22,6 +22,7 @@ typedef uint64_t	uint8;	  	/* 8-byte unsigned integer */
 #endif
 typedef uint64_t 	gtm_uint8;	/* these two datatypes are defined because */
 typedef	int64_t		gtm_int8;	/* int8 and uint8 are system defined in AIX_64 */
+typedef	int64_t		gtm_timet;	/* Differentiate a type for this so we can dump epochs intelligently to a string */
 #define INT8_NATIVE
 
 #ifdef __s390__
@@ -229,13 +230,11 @@ typedef struct
 
 #define malloc gtm_malloc
 #define free gtm_free
-/* gtm_shmget calls either the native shmget or libhugetlbfs's shmget which uses Huge Pages
- * to back the shared segment if possible. This is a Linux only library.
+/* gtm_shmget either calls the native shmget or requests Huge Pages to back the
+ * shared memory segment if the environment variable gtm_hugetlb_shm is set.
+ * Huge Pages are only supported on Linux.
  */
-#if defined(__linux__) && (defined(__x86_64__) || defined(__i386__))
-#	define shmget	gtm_shmget
-	extern int gtm_shmget(key_t , size_t , int);
-#endif
+extern int gtm_shmget(key_t , size_t , int , bool);
 
 #ifndef __ia64
 #define CODE_ADDRESS(func)	(unsigned char *)func
