@@ -1,6 +1,6 @@
 /****************************************************************
  *								*
- * Copyright (c) 2001-2020 Fidelity National Information	*
+ * Copyright (c) 2001-2022 Fidelity National Information	*
  * Services, Inc. and/or its subsidiaries. All rights reserved.	*
  *								*
  *	This source code contains the intellectual property	*
@@ -50,6 +50,7 @@
 #include "gtm_permissions.h"
 #include "getzposition.h"
 #include "error.h"
+#include "db_header_conversion.h"
 
 #define BLK_SIZE (((gd_segment*)gv_cur_region->dyn.addr)->blk_size)
 
@@ -66,7 +67,10 @@ MBSTART {												\
 		CLOSEFILE_RESET(mu_cre_file_fd, rc); /* resets "mu_cre_file_fd" to FD_INVALID */	\
 	assert(NULL != mu_cre_file_path);								\
 	if (EXIT_ERR == XX)										\
-		UNLINK(mu_cre_file_path);								\
+	{												\
+		rc = UNLINK(mu_cre_file_path);								\
+		assert(0 == rc);										\
+	}												\
 } MBEND
 
 /* Macros to send warning or error messages to the correct destination:
@@ -120,6 +124,7 @@ GBLREF	jnlpool_addrs_ptr_t	jnlpool;
 GBLREF	sgmnt_addrs		*cs_addrs;
 GBLREF	sgmnt_data_ptr_t	cs_data;
 GBLREF	uint4			gtmDebugLevel;
+GBLREF	enum db_ver		gtm_db_create_ver;              /* database creation version */
 #ifdef DEBUG
 GBLREF	boolean_t		in_mu_cre_file;
 #endif
@@ -176,7 +181,8 @@ unsigned char mu_cre_file(void)
 	gd_segment	*seg;
 	gd_region	*baseDBreg;
 	char		hash[GTMCRYPT_HASH_LEN];
-	int		gtmcrypt_errno, retcode, perms, user_id, group_id;
+	int		retcode, perms, user_id, group_id;
+	uint4		gtmcrypt_errno;
 	off_t		new_eof;
 	uint4		fsb_size;
 	boolean_t	cleanup_needed;
@@ -185,12 +191,23 @@ unsigned char mu_cre_file(void)
 	struct perm_diag_data	pdd;
 	uint4		fbwsize;
 	int4		dblksize;
+	uint4 		file_hdr_size;
+	block_id 	start_vbn_target;
 	ZOS_ONLY(int	realfiletag;)
 	DCL_THREADGBL_ACCESS;
 
 	SETUP_THREADGBL_ACCESS;
 	DEBUG_ONLY(mu_cre_file_fd = FD_INVALID);
 	DEBUG_ONLY(mu_cre_file_path = NULL);
+	if (GDSVCURR == gtm_db_create_ver)
+	{	/* Current version defaults */
+		file_hdr_size		= SIZEOF_FILE_HDR_DFLT;
+		start_vbn_target	= START_VBN_CURRENT;
+	} else
+	{	/* Prior version settings */
+		file_hdr_size		= SIZEOF_FILE_HDR_V6;
+		start_vbn_target	= START_VBN_V6;
+	}
 	cleanup_needed = FALSE;
 	DEBUG_ONLY(in_mu_cre_file = TRUE;)
 	assert((-(SIZEOF(uint4) * 2) & SIZEOF_FILE_HDR_DFLT) == SIZEOF_FILE_HDR_DFLT);
@@ -285,7 +302,7 @@ unsigned char mu_cre_file(void)
 		}
 	}
 	/* Blocks_for_create is in the unit of DISK_BLOCK_SIZE */
-	blocks_for_create = (gtm_uint64_t)(DIVIDE_ROUND_UP(SIZEOF_FILE_HDR_DFLT, DISK_BLOCK_SIZE) + 1
+	blocks_for_create = (gtm_uint64_t)(DIVIDE_ROUND_UP(file_hdr_size, DISK_BLOCK_SIZE) + 1
 					   + (seg->blk_size / DISK_BLOCK_SIZE
 					      * (gtm_uint64_t)((DIVIDE_ROUND_UP(seg->allocation, BLKS_PER_LMAP - 1))
 							       + seg->allocation)));
@@ -328,18 +345,18 @@ unsigned char mu_cre_file(void)
 	memset(&fc, 0, SIZEOF(file_control));
 	fc.file_info = (void*)&udi_struct;
 	udi->fd = mu_cre_file_fd;
-	cs_data = (sgmnt_data_ptr_t)malloc(SIZEOF_FILE_HDR_DFLT);
-	memset(cs_data, 0, SIZEOF_FILE_HDR_DFLT);
+	cs_data = (sgmnt_data_ptr_t)malloc(file_hdr_size);
+	memset(cs_data, 0, file_hdr_size);
 	cs_data->createinprogress = TRUE;
 	cs_data->semid = INVALID_SEMID;
 	cs_data->shmid = INVALID_SHMID;
 	/* We want our datablocks to start on what would be a block boundary within the file which will aid I/O
 	 * so pad the fileheader if necessary to make this happen.
 	 */
-	norm_vbn = DIVIDE_ROUND_UP(SIZEOF_FILE_HDR_DFLT, DISK_BLOCK_SIZE) + 1;
-	assert(START_VBN_CURRENT >= norm_vbn);
-	cs_data->start_vbn = START_VBN_CURRENT;
-	cs_data->free_space += (START_VBN_CURRENT - norm_vbn) * DISK_BLOCK_SIZE;
+	norm_vbn = DIVIDE_ROUND_UP(file_hdr_size, DISK_BLOCK_SIZE) + 1;
+	assert(start_vbn_target >= norm_vbn);
+	cs_data->start_vbn = start_vbn_target;
+	cs_data->free_space += (start_vbn_target - norm_vbn) * DISK_BLOCK_SIZE;
 	cs_data->acc_meth = gv_cur_region->dyn.addr->acc_meth;
 	if ((dba_mm == cs_data->acc_meth) && (gv_cur_region->jnl_before_image))
 	{
@@ -394,7 +411,7 @@ unsigned char mu_cre_file(void)
 	/* Check if this file is an encrypted database. If yes, do init */
 	if (IS_ENCRYPTED(gv_cur_region->dyn.addr->is_encrypted))
 	{
-		GTMCRYPT_HASH_GEN(cs_addrs, STRLEN(path), path, 0, NULL, hash, gtmcrypt_errno);
+		GTMCRYPT_HASH_GEN(cs_addrs, strnlen(path, sizeof(path)), path, 0, NULL, hash, gtmcrypt_errno);
 		if (0 != gtmcrypt_errno)
 		{
 			GTMCRYPT_REPORT_ERROR(gtmcrypt_errno, gtm_putmsg, file.len, file.addr);
@@ -404,7 +421,7 @@ unsigned char mu_cre_file(void)
 		}
 		memcpy(cs_data->encryption_hash, hash, GTMCRYPT_HASH_LEN);
 		SET_AS_ENCRYPTED(cs_data->is_encrypted); /* Mark this file as encrypted */
-		INIT_DB_OR_JNL_ENCRYPTION(cs_addrs, cs_data, STRLEN(path), path, gtmcrypt_errno);
+		INIT_DB_OR_JNL_ENCRYPTION(cs_addrs, cs_data, strnlen(path, sizeof(path)), path, gtmcrypt_errno);
 		if (0 != gtmcrypt_errno)
 		{
 			GTMCRYPT_REPORT_ERROR(gtmcrypt_errno, gtm_putmsg, file.len, file.addr);
@@ -419,10 +436,13 @@ unsigned char mu_cre_file(void)
 	cs_data->encryption_hash2_start_tn = 0;
 	cs_data->span_node_absent = TRUE;
 	cs_data->maxkeysz_assured = TRUE;
-	mucregini(cs_data->trans_hist.total_blks);
+	cs_data->problksplit = DEFAULT_PROBLKSPLIT;
+	mucregini(cs_data->trans_hist.total_blks, gtm_db_create_ver);
 	cs_data->createinprogress = FALSE;
+	if (GDSVCURR != gtm_db_create_ver)
+		db_header_dwnconv(cs_data);
 	ASSERT_NO_DIO_ALIGN_NEEDED(udi);	/* because we are creating the database and so effectively have standalone access */
-	DB_LSEEKWRITE(cs_addrs, udi, udi->fn, udi->fd, 0, cs_data, SIZEOF_FILE_HDR_DFLT, status);
+	DB_LSEEKWRITE(cs_addrs, udi, udi->fn, udi->fd, 0, cs_data, file_hdr_size, status);
 	if (0 != status)
 	{
 		PUTMSG_ERROR_CSA(cs_addrs, 7, ERR_FILECREERR, 4, LEN_AND_LIT("writing out file header"), LEN_AND_LIT(path), status);
@@ -441,8 +461,8 @@ unsigned char mu_cre_file(void)
 	}
 	if (!cs_data->defer_allocate)
 	{
-		status = posix_fallocate(udi->fd, 0, BLK_ZERO_OFF(cs_data->start_vbn) +
-					 ((off_t)(cs_data->trans_hist.total_blks + 1) * cs_data->blk_size));
+		POSIX_FALLOCATE(udi->fd, 0, BLK_ZERO_OFF(cs_data->start_vbn) +
+					 ((off_t)(cs_data->trans_hist.total_blks + 1) * cs_data->blk_size), status);
 		if (0 != status)
 		{
 			assert(ENOSPC == status);
