@@ -1,6 +1,6 @@
 /****************************************************************
  *								*
- * Copyright (c) 2001-2019 Fidelity National Information	*
+ * Copyright (c) 2001-2020 Fidelity National Information	*
  * Services, Inc. and/or its subsidiaries. All rights reserved.	*
  *								*
  *	This source code contains the intellectual property	*
@@ -44,13 +44,12 @@ GBLREF	boolean_t	skip_dbtriggers;
 GBLREF	mstr		sys_input;
 GBLDEF	int		onerror;
 
+error_def(ERR_EXTRFMT);
 error_def(ERR_LDBINFMT);
 error_def(ERR_LOADBGSZ);
-error_def(ERR_LOADBGSZ2);
 error_def(ERR_LOADINVCHSET);
 error_def(ERR_LOADEDBG);
 error_def(ERR_LOADEDSZ);
-error_def(ERR_LOADEDSZ2);
 error_def(ERR_MAXSTRLEN);
 error_def(ERR_MUNOFINISH);
 error_def(ERR_MUPCLIERR);
@@ -62,9 +61,9 @@ error_def(ERR_MUPCLIERR);
 void mupip_cvtgbl(void)
 {
 	char		fn[MAX_FN_LEN + 1], *line1_ptr, *line3_ptr;
-	gtm_int64_t	begin_i8, end_i8;
+	gtm_uint64_t	begin, end;
 	int		dos, i, file_format, line1_len, line3_len, utf8;
-	uint4	        begin, cli_status, end, max_rec_size;
+	uint4	        cli_status, max_rec_size;
 	unsigned char	buff[MAX_ONERROR_VALUE_LEN];
 	unsigned short	fn_len, len;
 
@@ -103,31 +102,24 @@ void mupip_cvtgbl(void)
 	mu_outofband_setup();
 	if ((cli_status = cli_present("BEGIN")) == CLI_PRESENT)
 	{
-	        if (!cli_get_int64("BEGIN", &begin_i8))
+	        if (!cli_get_uint64("BEGIN", &begin))
 			mupip_exit(ERR_MUPCLIERR);
-		if (1 > begin_i8)
+		if (1 > begin)
 			mupip_exit(ERR_LOADBGSZ);
-		else if (MAXUINT4 < begin_i8)
-			mupip_exit(ERR_LOADBGSZ2);
-		begin = (uint4) begin_i8;
 	} else
 	{
 		begin = 1;
-		begin_i8 = 1;
 	}
 	if ((cli_status = cli_present("END")) == CLI_PRESENT)
 	{
-	        if (!cli_get_int64("END", &end_i8))
+	        if (!cli_get_uint64("END", &end))
 			mupip_exit(ERR_MUPCLIERR);
-		if (1 > end_i8)
+		if (1 > end)
 			mupip_exit(ERR_LOADEDSZ);
-		else if (MAXUINT4 < end_i8)
-			mupip_exit(ERR_LOADEDSZ2);
-		if (end_i8 < begin_i8)
+		if (end < begin)
 			mupip_exit(ERR_LOADEDBG);
-		end = (uint4) end_i8;
 	} else
-		end = MAXUINT4;
+		end = MAXUINT8;
 	if ((cli_status = cli_present("FILL_FACTOR")) == CLI_PRESENT)
 	{
 		assert(SIZEOF(gv_fillfactor) == SIZEOF(int4));
@@ -299,8 +291,8 @@ int get_load_format(char **line1_ptr, char **line3_ptr, int *line1_len, int *lin
 			{	/* chances of this are small but we are careful not to overflow buffers */
 				ret = MU_FMT_GOQ;	/* abusing this value to mean not working, as we can't discover GOQ */
 				line2_len = 0;
+				/* There is no second line in this extract file */
 				mupip_error_occurred = TRUE;
-				gtm_putmsg_csa(CSA_ARG(NULL) VARLSTCNT(1) ERR_MAXSTRLEN);
 			}
 		}
 		if (0 < line2_len)
@@ -338,7 +330,7 @@ int get_load_format(char **line1_ptr, char **line3_ptr, int *line1_len, int *lin
 				{	/* chances of this are small but we are careful not to overflow buffers */
 					ret = MU_FMT_GOQ;
 					mupip_error_occurred = TRUE;
-					gtm_putmsg_csa(CSA_ARG(NULL) VARLSTCNT(1) ERR_MAXSTRLEN);
+					gtm_putmsg_csa(CSA_ARG(NULL) VARLSTCNT(1) ERR_EXTRFMT);
 				}
 			} else
 			{
