@@ -1,6 +1,6 @@
 /****************************************************************
  *								*
- * Copyright (c) 2001-2020 Fidelity National Information	*
+ * Copyright (c) 2001-2021 Fidelity National Information	*
  * Services, Inc. and/or its subsidiaries. All rights reserved.	*
  *								*
  *	This source code contains the intellectual property	*
@@ -36,7 +36,7 @@
 #include "mupip_ftok.h"
 #include "mupip_exit.h"
 #include "file_head_read.h"	/* for file_head_read() prototype */
-#include "is_file_identical.h"	/* for filename_to_id() prototype */
+#include "is_gdid.h"	/* for filename_to_id() prototype */
 
 GBLREF	boolean_t		in_mupip_ftok;		/* Used by an assert in repl_inst_read */
 
@@ -97,9 +97,9 @@ void mupip_ftok(void)
 				pblk.fop = F_SYNTAXO;
 				file.addr = instfilename;
 				file.len = full_len;
-				status = parse_file(&file, &pblk);	/* this gets us to just the name and extension */
-				fn_len = pblk.b_name + pblk.b_ext;
-				memcpy(fn, pblk.l_name, fn_len);
+				status = parse_file(&file, &pblk);	/* this gets us to directory, name and extension */
+				fn_len = pblk.b_dir + pblk.b_name + pblk.b_ext;
+				memcpy(fn, pblk.l_dir, fn_len);
 				fn[fn_len] = 0;
 				semkey = FTOK(fn, REPLPOOL_ID);
 				semid = shmid = -1;			/* not relevant for the file */
