@@ -1,6 +1,6 @@
 /****************************************************************
  *								*
- * Copyright (c) 2015-2020 Fidelity National Information	*
+ * Copyright (c) 2015-2021 Fidelity National Information	*
  * Services, Inc. and/or its subsidiaries. All rights reserved.	*
  *								*
  *	This source code contains the intellectual property	*
@@ -143,6 +143,7 @@ void mupip_reorg_encrypt(void)
 {
 	char			key[GTM_PATH_MAX], hash[GTMCRYPT_HASH_LEN];
 	char			*db_name, *bml_lcl_buff;
+	enum db_ver		dummy_blkver;		/*  used for dsk_read call in order to simplify a critical path */
 	int			db_name_len, gtmcrypt_errno, status, reg_status, status1;
 	int			reg_count, i, cycle, lcnt, bml_status;
 	int4			blk_seg_cnt, blk_size, mapsize;	/* needed for BLK_INIT,BLK_SEG and BLK_FINI macros */
@@ -490,7 +491,7 @@ void mupip_reorg_encrypt(void)
 			csd->encryption_hash_cutoff = curbmp;
 			bml_sm_buff = t_qread(curbmp, (sm_int_ptr_t)&cycle, &cr); /* now that in crit, note down stable buffer */
 			if (NULL == bml_sm_buff)
-				rts_error_csa(CSA_ARG(csa) VARLSTCNT(1) ERR_DSEBLKRDFAIL);
+				RTS_ERROR_CSA_ABT(csa, VARLSTCNT(1) ERR_DSEBLKRDFAIL);
 			/* Take a copy of the shared memory bitmap buffer into process-private memory before releasing crit. We are
 			 * interested in those blocks that are currently marked as USED in the bitmap. It is possible that once we
 			 * release crit, concurrent updates change the bitmap state of those blocks. In that case, those updates
@@ -544,7 +545,7 @@ void mupip_reorg_encrypt(void)
 					} else
 						buff = (sm_uc_ptr_t)(TREF(dio_buff)).aligned;
 					mu_reorg_encrypt_in_prog = MUPIP_REORG_IN_PROG_LOCAL_DSK_READ;
-					status1 = dsk_read(curblk, buff, NULL, FALSE);
+					status1 = dsk_read(curblk, buff, &dummy_blkver, FALSE);
 					mu_reorg_encrypt_in_prog = MUPIP_REORG_IN_PROG_TRUE;
 					if (SS_NORMAL != status1)
 					{
@@ -875,7 +876,7 @@ void release_ftok_semaphore(gd_region *reg, sgmnt_addrs *csa, sgmnt_data_ptr_t c
 	 * releasing the ftok in the middle of the reorg encrypt process and we do not want to modify the counter in those cases.
 	 */
 	if (!ftok_sem_release(reg, FALSE, FALSE))
-		rts_error_csa(CSA_ARG(csa) VARLSTCNT(4) ERR_DBFILERR, 2, DB_LEN_STR(reg));
+		RTS_ERROR_CSA_ABT(csa, VARLSTCNT(4) ERR_DBFILERR, 2, DB_LEN_STR(reg));
 	FTOK_TRACE(csa, csd->trans_hist.curr_tn, ftok_ops_release, process_id);
 	udi->grabbed_ftok_sem = FALSE;
 }
@@ -898,7 +899,7 @@ void switch_journal_file(sgmnt_addrs *csa, sgmnt_data_ptr_t csd)
 	if (0 == jnl_status)
 	{
 		if (EXIT_ERR == SWITCH_JNL_FILE(jpc))
-			rts_error_csa(CSA_ARG(csa) VARLSTCNT(4) ERR_JNLEXTEND, 2, JNL_LEN_STR(csd));
+			RTS_ERROR_CSA_ABT(csa, VARLSTCNT(4) ERR_JNLEXTEND, 2, JNL_LEN_STR(csd));
 	} else
 	{
 		if (SS_NORMAL != jpc->status)
