@@ -1,6 +1,6 @@
 /****************************************************************
  *								*
- * Copyright (c) 2001-2020 Fidelity National Information	*
+ * Copyright (c) 2001-2021 Fidelity National Information	*
  * Services, Inc. and/or its subsidiaries. All rights reserved.	*
  *								*
  *	This source code contains the intellectual property	*
@@ -137,7 +137,6 @@ MBSTART {															\
 		}														\
 		if (save_jnlpool != jnlpool)											\
 			jnlpool = save_jnlpool;											\
-		UPDATE_PROC_WAIT_STATE(CSA, STATE, -1);										\
 		return STATUS;													\
 	}															\
 } MBEND
@@ -284,14 +283,14 @@ static	void	clean_initialize(mutex_struct_ptr_t addr, int n, bool crash)
 #		  else
 		if ((NULL == (status = msem_init(&q_free_entry->mutex_wake_msem, MSEM_LOCKED))) || ((msemaphore *)-1 == status))
 #		  endif
-			rts_error_csa(CSA_ARG(NULL) VARLSTCNT(7) ERR_MUTEXERR, 0, ERR_TEXT, 2,
+			RTS_ERROR_CSA_ABT(NULL, VARLSTCNT(7) ERR_MUTEXERR, 0, ERR_TEXT, 2,
 				RTS_ERROR_TEXT("Error with mutex wait memory semaphore initialization"), errno);
 #		endif
 		/* Initialize fl,bl links to 0 before INSQTI as it (gtm_insqti in relqueopi.c) asserts this */
 		DEBUG_ONLY(((que_ent_ptr_t)q_free_entry)->fl = 0;)
 		DEBUG_ONLY(((que_ent_ptr_t)q_free_entry)->bl = 0;)
 		if (INTERLOCK_FAIL == INSQTI((que_ent_ptr_t)q_free_entry++, (que_head_ptr_t)&addr->freehead))
-			rts_error_csa(CSA_ARG(NULL) VARLSTCNT(6) ERR_MUTEXERR, 0, ERR_TEXT, 2,
+			RTS_ERROR_CSA_ABT(NULL, VARLSTCNT(6) ERR_MUTEXERR, 0, ERR_TEXT, 2,
 				RTS_ERROR_TEXT("Interlock instruction failure in mutex initialize"));
 	}
 	SET_LATCH_GLOBAL(&addr->semaphore, LOCK_AVAILABLE);
@@ -417,7 +416,7 @@ static	enum cdb_sc mutex_long_sleep(mutex_struct_ptr_t addr, sgmnt_addrs *csa,
 						      process_id, heartbeat_counter);
 				} else
 				{
-					rts_error_csa(CSA_ARG(csa) VARLSTCNT(7) ERR_MUTEXERR, 0, ERR_TEXT, 2,
+					RTS_ERROR_CSA_ABT(csa, VARLSTCNT(7) ERR_MUTEXERR, 0, ERR_TEXT, 2,
 						RTS_ERROR_TEXT("Error with mutex wake msem"), save_errno);
 				}
 			}
@@ -643,42 +642,42 @@ void	gtm_mutex_init(gd_region *reg, int n, bool crash)
 		status = pthread_mutex_destroy(&csa->critical->mutex);
 		if (0 != status)
 		{
-			rts_error_csa(CSA_ARG(csa) VARLSTCNT(8) ERR_SYSCALL, 5, LEN_AND_LIT("pthread_mutex_destroy"),
-					CALLFROM, status);
+			RTS_ERROR_CSA_ABT(csa, VARLSTCNT(8) ERR_SYSCALL, 5, LEN_AND_LIT("pthread_mutex_destroy"),
+				CALLFROM, status);
 		}
 	}
 	/* Set up mutex for new file */
 	status = pthread_mutexattr_init(&crit_attr);
 	if (0 != status)
 	{
-		rts_error_csa(CSA_ARG(csa) VARLSTCNT(8) ERR_SYSCALL, 5, LEN_AND_LIT("pthread_mutexattr_init"),
-				CALLFROM, status);
+		RTS_ERROR_CSA_ABT(csa, VARLSTCNT(8) ERR_SYSCALL, 5, LEN_AND_LIT("pthread_mutexattr_init"),
+			CALLFROM, status);
 	}
 	status = pthread_mutexattr_settype(&crit_attr, PTHREAD_MUTEX_ERRORCHECK);
 	if (0 != status)
 	{
-		rts_error_csa(CSA_ARG(csa) VARLSTCNT(8) ERR_SYSCALL, 5, LEN_AND_LIT("pthread_mutexattr_settype"),
-				CALLFROM, status);
+		RTS_ERROR_CSA_ABT(csa, VARLSTCNT(8) ERR_SYSCALL, 5, LEN_AND_LIT("pthread_mutexattr_settype"),
+			CALLFROM, status);
 	}
 	status = pthread_mutexattr_setpshared(&crit_attr, PTHREAD_PROCESS_SHARED);
 	if (0 != status)
 	{
-		rts_error_csa(CSA_ARG(csa) VARLSTCNT(8) ERR_SYSCALL, 5, LEN_AND_LIT("pthread_mutexattr_setpshared"),
-				CALLFROM, status);
+		RTS_ERROR_CSA_ABT(csa, VARLSTCNT(8) ERR_SYSCALL, 5, LEN_AND_LIT("pthread_mutexattr_setpshared"),
+			CALLFROM, status);
 	}
 #	if PTHREAD_MUTEX_ROBUST_SUPPORTED
 	status = pthread_mutexattr_setrobust(&crit_attr, PTHREAD_MUTEX_ROBUST);
 	if (0 != status)
 	{
-		rts_error_csa(CSA_ARG(csa) VARLSTCNT(8) ERR_SYSCALL, 5, LEN_AND_LIT("pthread_mutexattr_setrobust"),
-				CALLFROM, status);
+		RTS_ERROR_CSA_ABT(csa, VARLSTCNT(8) ERR_SYSCALL, 5, LEN_AND_LIT("pthread_mutexattr_setrobust"),
+			CALLFROM, status);
 	}
 #	endif
 	status = pthread_mutex_init(&csa->critical->mutex, &crit_attr);
 	if (0 != status)
 	{
-		rts_error_csa(CSA_ARG(csa) VARLSTCNT(8) ERR_SYSCALL, 5, LEN_AND_LIT("pthread_mutex_init"),
-				CALLFROM, status);
+		RTS_ERROR_CSA_ABT(csa, VARLSTCNT(8) ERR_SYSCALL, 5, LEN_AND_LIT("pthread_mutex_init"),
+			CALLFROM, status);
 	}
 #	else
 	if (!crash)
@@ -743,7 +742,7 @@ enum cdb_sc gtm_mutex_lock(gd_region *reg,
 	/* Do a trylock first. If we are locking immediate, we are done. Otherwise we have the opportunity to update
 	 * stats before doing a longer timed lock attempt.
 	 */
-	UPDATE_PROC_WAIT_STATE(csa, state, 1);
+	UPDATE_CRIT_COUNTER(csa, state);
 	status = pthread_mutex_trylock(&csa->critical->mutex);
 	do
 	{
@@ -757,8 +756,8 @@ enum cdb_sc gtm_mutex_lock(gd_region *reg,
 			local_stuck_cycle = csa->critical->stuck_cycle;
 			status = clock_gettime(CLOCK_REALTIME, &timeout);
 			if (0 != status)
-				rts_error_csa(CSA_ARG(NULL) VARLSTCNT(8) ERR_SYSCALL, 5,
-						LEN_AND_LIT("clock_gettime"), CALLFROM, errno, 0);
+				RTS_ERROR_CSA_ABT(NULL, VARLSTCNT(8) ERR_SYSCALL, 5,
+					LEN_AND_LIT("clock_gettime"), CALLFROM, errno, 0);
 			timeout.tv_sec += MUTEX_CONST_TIMEOUT_VAL;
 			status = pthread_mutex_timedlock(&csa->critical->mutex, &timeout);
 		}
@@ -784,8 +783,8 @@ enum cdb_sc gtm_mutex_lock(gd_region *reg,
 #				if PTHREAD_MUTEX_CONSISTENT_SUPPORTED
 				status = pthread_mutex_consistent(&csa->critical->mutex);
 				if (0 != status)
-					rts_error_csa(CSA_ARG(csa) VARLSTCNT(8) ERR_SYSCALL, 5,
-							LEN_AND_LIT("pthread_mutex_consistent"), CALLFROM, status);
+					RTS_ERROR_CSA_ABT(csa, VARLSTCNT(8) ERR_SYSCALL, 5,
+						LEN_AND_LIT("pthread_mutex_consistent"), CALLFROM, status);
 #				endif
 				/* fall through */
 			case 0:
@@ -802,11 +801,9 @@ enum cdb_sc gtm_mutex_lock(gd_region *reg,
 				}
 				INCR_GVSTATS_COUNTER(csa, cnl, n_crit_success, 1);
 				csa->critical->crit_cycle++;
-				UPDATE_PROC_WAIT_STATE(csa, state, -1);
 				return cdb_sc_normal;
 			case EBUSY:
 				assert(MUTEX_LOCK_WRITE_IMMEDIATE == mutex_lock_type);
-				UPDATE_PROC_WAIT_STATE(csa, state, -1);
 				return cdb_sc_nolock;
 			case ETIMEDOUT:
 				mutex_deadlock_check(csa->critical, csa); /* Timed out: See if any deadlocks and fix if detected */
@@ -848,7 +845,7 @@ enum cdb_sc gtm_mutex_lock(gd_region *reg,
 	assertpro(!status);
 	return cdb_sc_nolock;		/* To keep compiler happy; should never happen. */
 #	else
-	UPDATE_PROC_WAIT_STATE(csa, state, 1);
+	UPDATE_CRIT_COUNTER(csa, state);
 	save_jnlpool = jnlpool;
 	optimistic_attempts = MUTEX_MAX_OPTIMISTIC_ATTEMPTS;
 	queue_sleeps = csa->probecrit_rec.p_crit_que_full = 0;
@@ -927,7 +924,6 @@ enum cdb_sc gtm_mutex_lock(gd_region *reg,
 				{	/* This is just a precaution - shouldn't ever happen and has no code to maintain gvstats */
 					assert(FALSE);
 					SET_CSA_NOW_CRIT_TRUE(csa);
-					UPDATE_PROC_WAIT_STATE(csa, state, -1);
 					return (cdb_sc_normal);
 				}
 				if (in_crit_pid && (in_crit_pid == cnl->in_crit) && is_proc_alive(in_crit_pid, 0))
@@ -983,7 +979,6 @@ enum cdb_sc gtm_mutex_lock(gd_region *reg,
 				MUTEX_TRACE_CNTR(mutex_trc_mutex_slp_fn_noslp);
 				if (cdb_sc_normal == mutex_wakeup(addr, mutex_spin_parms))
 					continue;
-				UPDATE_PROC_WAIT_STATE(csa, state, -1);
 				return (cdb_sc_dbccerr);
 			}
 		}
@@ -1060,7 +1055,6 @@ enum cdb_sc gtm_mutex_lock(gd_region *reg,
 					{
 						if (save_jnlpool != jnlpool)
 							jnlpool = save_jnlpool;
-						UPDATE_PROC_WAIT_STATE(csa, state, -1);
 						return (cdb_sc_dbccerr);	/* Too many failures */
 					}
 					assert(!csa->now_crit);
@@ -1097,7 +1091,6 @@ enum cdb_sc gtm_mutex_lock(gd_region *reg,
 			}
 		} while (redo_cntr);
 	} while (TRUE);
-	UPDATE_PROC_WAIT_STATE(csa, state, -1);
 #	endif
 }
 
