@@ -1,6 +1,6 @@
 /****************************************************************
  *								*
- * Copyright (c) 2001-2020 Fidelity National Information	*
+ * Copyright (c) 2001-2021 Fidelity National Information	*
  * Services, Inc. and/or its subsidiaries. All rights reserved.	*
  *								*
  *	This source code contains the intellectual property	*
@@ -58,6 +58,10 @@
 #include "zwrite.h"
 #include "gtm_maxstr.h"
 #include "getzdir.h"
+#ifdef DEBUG
+#include <sys/time.h>
+#include <sys/resource.h>
+#endif
 
 GBLREF	bool			jobpid;	/* job's output files should have the pid appended to them. */
 GBLREF	volatile boolean_t	ojtimeout;
@@ -270,7 +274,7 @@ void ojpassvar_hook(void)
 	setup_op = job_set_locals;
 	rc = gtm_fwrite(&setup_op, 1, SIZEOF(setup_op), setup_file, &written);
 	if (0 < rc)
-		rts_error_csa(CSA_ARG(NULL) VARLSTCNT(9) ERR_SYSCALL, 5, RTS_ERROR_LITERAL("fwrite(op)"), CALLFROM, errno, 0);
+		RTS_ERROR_CSA_ABT(NULL, VARLSTCNT(9) ERR_SYSCALL, 5, RTS_ERROR_LITERAL("fwrite(op)"), CALLFROM, errno, 0);
 	/* Crop the ' ;*' string at the end of the aliases */
 	if (zwr_output->buff[buffer_size - 1] == '*' && zwr_output->buff[buffer_size - 2] == ';'
 	    && zwr_output->buff[buffer_size - 3] == ' ')
@@ -278,12 +282,12 @@ void ojpassvar_hook(void)
 	/* Always send the buffer size. If it is bigger than MAX_STRLEN, the child will handle this as a JOBLVN2LONG */
 	rc = gtm_fwrite(&buffer_size, 1, SIZEOF(buffer_size), setup_file, &written);
 	if (0 < rc)
-		rts_error_csa(CSA_ARG(NULL) VARLSTCNT(9) ERR_SYSCALL, 5, RTS_ERROR_LITERAL("fwrite(size)"), CALLFROM, errno, 0);
+		RTS_ERROR_CSA_ABT(NULL, VARLSTCNT(9) ERR_SYSCALL, 5, RTS_ERROR_LITERAL("fwrite(size)"), CALLFROM, errno, 0);
 	if (buffer_size > MAX_STRLEN)
-		rts_error_csa(CSA_ARG(NULL) VARLSTCNT(4) ERR_JOBLVN2LONG, 2, MAX_STRLEN, buffer_size);
+		RTS_ERROR_CSA_ABT(NULL, VARLSTCNT(4) ERR_JOBLVN2LONG, 2, MAX_STRLEN, buffer_size);
 	rc = gtm_fwrite(zwr_output->buff, 1, buffer_size, setup_file, &written);
 	if (0 < rc)
-		rts_error_csa(CSA_ARG(NULL) VARLSTCNT(9) ERR_SYSCALL, 5, RTS_ERROR_LITERAL("fwrite(buf)"), CALLFROM, errno, 0);
+		RTS_ERROR_CSA_ABT(NULL, VARLSTCNT(9) ERR_SYSCALL, 5, RTS_ERROR_LITERAL("fwrite(buf)"), CALLFROM, errno, 0);
 	zwr_output->ptr = zwr_output->buff;
 	return;
 }
@@ -382,8 +386,8 @@ int ojstartchild (job_params_type *jparms, int argcnt, boolean_t *non_exit_retur
 	 * in the child process) does not affect file offsets in this (parent) process' file descriptors
 	 */
 	if (!gtm_dist_ok_to_use)
-		rts_error_csa(CSA_ARG(NULL) VARLSTCNT(6) ERR_GTMDISTUNVERIF, 4, STRLEN(gtm_dist), gtm_dist,
-				gtmImageNames[image_type].imageNameLen, gtmImageNames[image_type].imageName);
+		RTS_ERROR_CSA_ABT(NULL, VARLSTCNT(6) ERR_GTMDISTUNVERIF, 4, STRLEN(gtm_dist), gtm_dist,
+			gtmImageNames[image_type].imageNameLen, gtmImageNames[image_type].imageName);
 	FFLUSH(NULL);
 	FORK_RETRY(child_pid);
 	if (child_pid == 0)
@@ -490,15 +494,28 @@ int ojstartchild (job_params_type *jparms, int argcnt, boolean_t *non_exit_retur
 			assert(FALSE);
 			UNDERSCORE_EXIT(joberr);
 		}
+#		ifdef DEBUG
+		if (WBTEST_ENABLED(WBTEST_JOBFAIL_FILE_LIM))
+		{
+			const struct rlimit	flim = {2, 2};
 
+			if (-1 == setrlimit(RLIMIT_NOFILE, &flim))
+			{	/* the white box test will fail bacause the error reported won't match expectations */
+				job_errno = errno;
+				DOWRITERC(pipe_fds[1], &job_errno, SIZEOF(job_errno), pipe_status);
+				assert(FALSE);	/* core the middle child toward diagnosing why the white box case failed */
+			}
+		}
+#		endif
 		joberr = joberr_sp;
 		if (-1 == (rc = socketpair(AF_UNIX, SOCK_STREAM, 0, setup_fds)))
 		{
 			job_errno = errno;
 			DOWRITERC(pipe_fds[1], &job_errno, SIZEOF(job_errno), pipe_status);
-			assert(FALSE);
+			assert(WBTEST_ENABLED(WBTEST_JOBFAIL_FILE_LIM));
 			UNDERSCORE_EXIT(joberr);
 		}
+		assert(!WBTEST_ENABLED(WBTEST_JOBFAIL_FILE_LIM));
 		/* Kill ourselves before we fork again */
 		if (WBTEST_ENABLED(WBTEST_SIGTERM_IN_JOB_CHILD))
 			kill(getpid(), SIGTERM);
@@ -664,8 +681,8 @@ int ojstartchild (job_params_type *jparms, int argcnt, boolean_t *non_exit_retur
 		DOREADRC(mproc_fds[0], &decision, SIZEOF(decision), pipe_status);
 		if (pipe_status)	 /* We failed to read the communication from middle process */
 		{
-			rts_error_csa(CSA_ARG(NULL) VARLSTCNT(7) ERR_JOBFAIL, 0, ERR_TEXT, 2,
-						LEN_AND_LIT("Error reading from pipe"), errno);
+			RTS_ERROR_CSA_ABT(NULL, VARLSTCNT(7) ERR_JOBFAIL, 0, ERR_TEXT, 2,
+				LEN_AND_LIT("Error reading from pipe"), errno);
 		} else
 		{
 			if (JOB_EXIT == decision)
@@ -718,7 +735,7 @@ int ojstartchild (job_params_type *jparms, int argcnt, boolean_t *non_exit_retur
 		string_len = STRLEN("%s=%d") + STRLEN(CHILD_FLAG_ENV) + MAX_NUM_LEN - 4;
 		if (string_len >= MAX_MUMPS_EXE_PATH_LEN)
 		{
-			rts_error_csa(CSA_ARG(NULL) VARLSTCNT(1) ERR_JOBPARTOOLONG);
+			RTS_ERROR_CSA_ABT(NULL, VARLSTCNT(1) ERR_JOBPARTOOLONG);
 		}
 		c1 = (char *)malloc(string_len + 1);
 #ifdef KEEP_zOS_EBCDIC
@@ -791,7 +808,7 @@ int ojstartchild (job_params_type *jparms, int argcnt, boolean_t *non_exit_retur
 			strcpy(c2, MUMPS_EXE_STR);
 		} else
 		{
-			rts_error_csa(CSA_ARG(NULL) VARLSTCNT(5) ERR_LOGTOOLONG, 3, string_len, c1,
+			RTS_ERROR_CSA_ABT(NULL, VARLSTCNT(5) ERR_LOGTOOLONG, 3, string_len, c1,
 				SIZEOF(tbuff) - SIZEOF(MUMPS_EXE_STR));
 		}
 #		ifdef KEEP_zOS_EBCDIC_	/* use real strcpy to preserve env in native code set */
@@ -807,7 +824,7 @@ int ojstartchild (job_params_type *jparms, int argcnt, boolean_t *non_exit_retur
 		{
 			if (jparms->cmdline.len >= TEMP_BUFF_SIZE)
 			{
-				rts_error_csa(CSA_ARG(NULL) VARLSTCNT(1) ERR_JOBPARTOOLONG);
+				RTS_ERROR_CSA_ABT(NULL, VARLSTCNT(1) ERR_JOBPARTOOLONG);
 			}
 			memcpy(cmdbuff, jparms->cmdline.buffer, jparms->cmdline.len);
 			*(cmdbuff + jparms->cmdline.len) = 0;
@@ -831,7 +848,7 @@ int ojstartchild (job_params_type *jparms, int argcnt, boolean_t *non_exit_retur
 		signal(SIGHUP, SIG_IGN);
 		EXECVE(tbuff, argv, env_ary);
 		/* if we got here, error starting the Job */
-		rts_error_csa(CSA_ARG(NULL) VARLSTCNT(7) ERR_JOBFAIL, 0, ERR_TEXT, 2, LEN_AND_LIT("Exec error in Job"), errno);
+		RTS_ERROR_CSA_ABT(NULL, VARLSTCNT(7) ERR_JOBFAIL, 0, ERR_TEXT, 2, LEN_AND_LIT("Exec error in Job"), errno);
 		REVERT;
 	} else
 	{
