@@ -1,6 +1,6 @@
 /****************************************************************
  *								*
- * Copyright (c) 2006-2018 Fidelity National Information	*
+ * Copyright (c) 2006-2021 Fidelity National Information	*
  * Services, Inc. and/or its subsidiaries. All rights reserved.	*
  *								*
  *	This source code contains the intellectual property	*
@@ -65,7 +65,7 @@ void	op_fnzconvert2(mval *src, mval *kase, mval *dst)
 	if (-1 == (index = verify_case(&kase->str)))
 	{
 		ENABLE_INTERRUPTS(INTRPT_IN_FUNC_WITH_MALLOC, prev_intrpt_state);
-		rts_error_csa(CSA_ARG(NULL) VARLSTCNT(4) ERR_BADCASECODE, 2, kase->str.len, kase->str.addr);
+		RTS_ERROR_CSA_ABT(NULL, VARLSTCNT(4) ERR_BADCASECODE, 2, kase->str.len, kase->str.addr);
 	}
 
 	MV_FORCE_STR(src);
@@ -110,8 +110,8 @@ void	op_fnzconvert2(mval *src, mval *kase, mval *dst)
 			if (U_ILLEGAL_CHAR_FOUND == status || U_INVALID_CHAR_FOUND == status)
 				utf8_len_strict((unsigned char *)src->str.addr, src->str.len);	/* to report BADCHAR error */
 			ENABLE_INTERRUPTS(INTRPT_IN_FUNC_WITH_MALLOC, prev_intrpt_state);
-			rts_error_csa(CSA_ARG(NULL) VARLSTCNT(3) ERR_ICUERROR,
-						1, status); /* ICU said bad, we say good or don't recognize error*/
+			RTS_ERROR_CSA_ABT(NULL, VARLSTCNT(3) ERR_ICUERROR,
+				1, status); /* ICU said bad, we say good or don't recognize error*/
 		}
 		else
 		{	/* BADCHAR should not be possible after the above validations */
@@ -128,8 +128,8 @@ void	op_fnzconvert2(mval *src, mval *kase, mval *dst)
 			{
 				RELEASE_IF_NOT_LOCAL(src_ustr_ptr, src_ustr);
 				ENABLE_INTERRUPTS(INTRPT_IN_FUNC_WITH_MALLOC, prev_intrpt_state);
-				rts_error_csa(CSA_ARG(NULL) VARLSTCNT(3) ERR_ICUERROR,
-					      1, status);
+				RTS_ERROR_CSA_ABT(NULL, VARLSTCNT(3) ERR_ICUERROR,
+					1, status);
 			}
 			/* Fake the conversion from UTF-16 to UTF-8 to compute the required buffer size */
 			status = U_ZERO_ERROR;
@@ -140,7 +140,7 @@ void	op_fnzconvert2(mval *src, mval *kase, mval *dst)
 			{
 				RELEASE_IF_NOT_LOCAL(dst_ustr_ptr, dst_ustr);
 				ENABLE_INTERRUPTS(INTRPT_IN_FUNC_WITH_MALLOC, prev_intrpt_state);
-				rts_error_csa(CSA_ARG(NULL) VARLSTCNT(1) ERR_MAXSTRLEN);
+				RTS_ERROR_CSA_ABT(NULL, VARLSTCNT(1) ERR_MAXSTRLEN);
 			}
 			ENSURE_STP_FREE_SPACE(dstlen);
 			dstbase = (char *)stringpool.free;
@@ -152,8 +152,8 @@ void	op_fnzconvert2(mval *src, mval *kase, mval *dst)
 				RELEASE_IF_NOT_LOCAL(src_ustr_ptr, src_ustr);
 				RELEASE_IF_NOT_LOCAL(dst_ustr_ptr, dst_ustr);
 				ENABLE_INTERRUPTS(INTRPT_IN_FUNC_WITH_MALLOC, prev_intrpt_state);
-				rts_error_csa(CSA_ARG(NULL) VARLSTCNT(3) ERR_ICUERROR,
-					      1, status); /* ICU said bad, but same call above just returned OK */
+				RTS_ERROR_CSA_ABT(NULL, VARLSTCNT(3) ERR_ICUERROR,
+					1, status); /* ICU said bad, but same call above just returned OK */
 			}
 			assertpro(ulen == dstlen);
 			RELEASE_IF_NOT_LOCAL(dst_ustr_ptr, dst_ustr);
@@ -173,16 +173,16 @@ void	op_fnzconvert3(mval *src, mval* ichset, mval* ochset, mval* dst)
 	MV_FORCE_STR(src);
 	if (!gtm_utf8_mode)
 	{ /* UTF8 not enabled, report error rather than silently ignoring the conversion */
-		rts_error_csa(CSA_ARG(NULL) VARLSTCNT(6) ERR_INVFCN, 0, ERR_TEXT, 2,
+		RTS_ERROR_CSA_ABT(NULL, VARLSTCNT(6) ERR_INVFCN, 0, ERR_TEXT, 2,
 			LEN_AND_LIT("Three-argument form of $ZCONVERT() is not allowed in the current $ZCHSET"));
 	}
 	MV_FORCE_STR(ichset);
 	MV_FORCE_STR(ochset);
 	/* The only supported names are: "UTF-8", "UTF-16", "UTF-16LE" and "UTF-16BE */
 	if (NULL == (from = get_chset_desc(&ichset->str)))
-		rts_error_csa(CSA_ARG(NULL) VARLSTCNT(4) ERR_BADCHSET, 2, ichset->str.len, ichset->str.addr);
+		RTS_ERROR_CSA_ABT(NULL, VARLSTCNT(4) ERR_BADCHSET, 2, ichset->str.len, ichset->str.addr);
 	if (NULL == (to = get_chset_desc(&ochset->str)))
-		rts_error_csa(CSA_ARG(NULL) VARLSTCNT(4) ERR_BADCHSET, 2, ochset->str.len, ochset->str.addr);
+		RTS_ERROR_CSA_ABT(NULL, VARLSTCNT(4) ERR_BADCHSET, 2, ochset->str.len, ochset->str.addr);
 
 	dstlen = gtm_conv(from, to, &src->str, NULL, NULL);
 	assert(-1 != dstlen);
