@@ -1,6 +1,6 @@
 /****************************************************************
  *								*
- * Copyright (c) 2001-2019 Fidelity National Information	*
+ * Copyright (c) 2001-2022 Fidelity National Information	*
  * Services, Inc. and/or its subsidiaries. All rights reserved.	*
  *								*
  *	This source code contains the intellectual property	*
@@ -37,6 +37,7 @@
 #include "relinkctl.h"
 #include "toktyp.h"		/* Needed for "valid_mname.h" */
 #include "valid_mname.h"
+#include "restrict.h"
 
 typedef enum
 {
@@ -126,6 +127,7 @@ GBLREF unsigned short		object_name_len;
 
 error_def(ERR_FILENOTFND);
 error_def(ERR_FILEPARSE);
+error_def(ERR_RESTRICTEDOP);
 error_def(ERR_SYSCALL);
 error_def(ERR_TEXT);
 error_def(ERR_VERSION);
@@ -164,19 +166,23 @@ void op_zlink (mval *v, mval *quals)
 	parse_blk		pblk;
 	struct stat		obj_stat, src_stat;
 	unsigned short		objnamelen, srcnamelen;
+	size_t			cpy_len;
 	zro_ent			*srcdir, *objdir;
 	ARLINK_ONLY(zro_hist	*recent_zhist;)
 	DCL_THREADGBL_ACCESS;
 
 	SETUP_THREADGBL_ACCESS;
+	if ((NULL != quals) && RESTRICTED(zlink_op))
+		RTS_ERROR_CSA_ABT(NULL, VARLSTCNT(3) ERR_RESTRICTEDOP, 1, "ZLINK");
 	ARLINK_ONLY(recent_zhist = NULL);
 	MV_FORCE_STR(v);
 	if (!v->str.len)
-		rts_error_csa(CSA_ARG(NULL) VARLSTCNT(8) ERR_ZLINKFILE, 2, v->str.len, v->str.addr, ERR_TEXT, 2,
-			      RTS_ERROR_LITERAL("Filename/path is missing"));
-	if (MAX_FN_LEN < v->str.len)
-		rts_error_csa(CSA_ARG(NULL) VARLSTCNT(8) ERR_ZLINKFILE, 2, v->str.len, v->str.addr, ERR_TEXT, 2,
-			      RTS_ERROR_LITERAL("Filename/path exceeds max length"));
+		RTS_ERROR_CSA_ABT(NULL, VARLSTCNT(8) ERR_ZLINKFILE, 2, v->str.len, v->str.addr, ERR_TEXT, 2,
+			RTS_ERROR_LITERAL("Filename/path is missing"));
+	assert(0 <= v->str.len);
+	if ((0 > v->str.len) || (MAX_FN_LEN < v->str.len))
+		RTS_ERROR_CSA_ABT(NULL, VARLSTCNT(8) ERR_ZLINKFILE, 2, (0 < v->str.len) ? MAX_FN_LEN : 0, v->str.addr,
+			ERR_TEXT, 2, RTS_ERROR_LITERAL("Filename/path exceeds max length"));
 	DBGARLNK((stderr, "op_zlink: Call to (re)link routine %.*s\n", v->str.len, v->str.addr));
 	assert((SIZEOF(DOTM) == SIZEOF(DOTOBJ)) && (SIZEOF(srcnamebuf) == SIZEOF(objnamebuf)));
 	object_file_des = FD_INVALID;
@@ -189,10 +195,10 @@ void op_zlink (mval *v, mval *quals)
 	pblk.fop = F_SYNTAXO;
 	status = parse_file(&v->str, &pblk);
 	if (!(status & 1))
-		rts_error_csa(CSA_ARG(NULL) VARLSTCNT(5) ERR_FILEPARSE, 2, v->str.len, v->str.addr, status);
+		RTS_ERROR_CSA_ABT(NULL, VARLSTCNT(5) ERR_FILEPARSE, 2, v->str.len, v->str.addr, status);
 	if (pblk.fnb & F_WILD)
-		rts_error_csa(CSA_ARG(NULL) VARLSTCNT(8) ERR_ZLINKFILE, 2, v->str.len, v->str.addr,
-			      ERR_WILDCARD, 2, v->str.len, v->str.addr);
+		RTS_ERROR_CSA_ABT(NULL, VARLSTCNT(8) ERR_ZLINKFILE, 2, v->str.len, v->str.addr,
+			ERR_WILDCARD, 2, v->str.len, v->str.addr);
 	/* 4SCA: file.len is bounded by MAX_FN_LEN in parse_file() */
 	file.addr = pblk.buffer;
 	file.len = pblk.b_esl;
@@ -230,8 +236,14 @@ void op_zlink (mval *v, mval *quals)
 		} else if (SRC != type)
 		{	/* if we have an explicit, directory shift the object name to make room and fill in the directory */
 			assert((OBJ == type) || (NOTYPE == type));
-			assert(MAX_FN_LEN > (objnamelen + pblk.b_dir));
-			memmove(&objnamebuf[pblk.b_dir], objnamebuf, objnamelen);
+			cpy_len = objnamelen;
+			assert(sizeof(objnamebuf) >= cpy_len);
+			assert(sizeof(objnamebuf) >= (cpy_len + pblk.b_dir));
+			if (sizeof(objnamebuf) < (cpy_len + pblk.b_dir)) /* 4SCA BYPASSOK */
+				cpy_len = sizeof(objnamebuf) - pblk.b_dir;
+			assert(256 >= cpy_len); /* For Veracode grins */
+			memmove(&objnamebuf[pblk.b_dir], objnamebuf, cpy_len);
+			assert(sizeof(objnamebuf) >= pblk.b_dir);
 			memcpy(objnamebuf, file.addr, pblk.b_dir);
 			objnamelen += pblk.b_dir;
 		}
@@ -263,7 +275,7 @@ void op_zlink (mval *v, mval *quals)
 		if (NOTYPE == type)
 		{	/* No file extension specified - object done above, so do source */
 			if ((file.len + SIZEOF(DOTM)) > SIZEOF(srcnamebuf))
-				rts_error_csa(CSA_ARG(NULL) VARLSTCNT(8) ERR_ZLINKFILE, 2, v->str.len, v->str.addr, ERR_TEXT, 2,
+				RTS_ERROR_CSA_ABT(NULL, VARLSTCNT(8) ERR_ZLINKFILE, 2, v->str.len, v->str.addr, ERR_TEXT, 2,
 					RTS_ERROR_LITERAL("Filename/path exceeds max length"));
 			memcpy(srcnamebuf, file.addr, file.len);
 			srcnamelen = file.len + SIZEOF(DOTM) - 1;
@@ -280,19 +292,19 @@ void op_zlink (mval *v, mval *quals)
 			{	/* Explicit ZLINK of object - don't locate source */
 				zro_search(&objstr, &objdir, NULL, NULL, SKIP_SHLIBS);
 				if (NULL == objdir)
-					rts_error_csa(CSA_ARG(NULL) VARLSTCNT(8) ERR_ZLINKFILE, 2, dollar_zsource.str.len,
-						      dollar_zsource.str.addr, ERR_FILENOTFND, 2, objnamelen, objnamebuf);
+					RTS_ERROR_CSA_ABT(NULL, VARLSTCNT(8) ERR_ZLINKFILE, 2, dollar_zsource.str.len,
+						dollar_zsource.str.addr, ERR_FILENOTFND, 2, objnamelen, objnamebuf);
 			} else if (SRC == type)
 			{	/* Explicit ZLINK of source - locate both source and object*/
 				zro_search(&objstr, &objdir, &srcstr, &srcdir, SKIP_SHLIBS);
 				if (NULL == srcdir)
-					rts_error_csa(CSA_ARG(NULL) VARLSTCNT(8) ERR_ZLINKFILE, 2, dollar_zsource.str.len,
-						      dollar_zsource.str.addr, ERR_FILENOTFND,2, srcnamelen, srcnamebuf);
+					RTS_ERROR_CSA_ABT(NULL, VARLSTCNT(8) ERR_ZLINKFILE, 2, dollar_zsource.str.len,
+						dollar_zsource.str.addr, ERR_FILENOTFND,2, srcnamelen, srcnamebuf);
 			} else
 			{	/* Explicit ZLINK no file type specified - locate both source and object */
 				zro_search(&objstr, &objdir, &srcstr, &srcdir, PROBE_SHLIBS);
 				if ((NULL == objdir) && (NULL == srcdir))
-					rts_error_csa(CSA_ARG(NULL) VARLSTCNT(12) ERR_ZLINKFILE, 2, dollar_zsource.str.len,
+					RTS_ERROR_CSA_ABT(NULL, VARLSTCNT(12) ERR_ZLINKFILE, 2, dollar_zsource.str.len,
 						dollar_zsource.str.addr, ERR_FILENOTFND,2, srcnamelen, srcnamebuf,
 						ERR_FILENOTFND,2, objnamelen, objnamebuf);
 			}
@@ -315,8 +327,8 @@ void op_zlink (mval *v, mval *quals)
 		zro_search(&objstr, &objdir, &srcstr, &srcdir, PROBE_SHLIBS);
 		if (NULL == srcdir)
 			if (NULL == objdir)
-				rts_error_csa(CSA_ARG(NULL) VARLSTCNT(12) ERR_ZLINKFILE, 2, v->str.len, v->str.addr,
-				      ERR_FILENOTFND, 2, srcnamelen, srcnamebuf, ERR_FILENOTFND, 2, objnamelen, objnamebuf);
+				RTS_ERROR_CSA_ABT(NULL, VARLSTCNT(12) ERR_ZLINKFILE, 2, v->str.len, v->str.addr,
+					ERR_FILENOTFND, 2, srcnamelen, srcnamebuf, ERR_FILENOTFND, 2, objnamelen, objnamebuf);
 		else if (NULL == objdir)
 			type = SRC;
 	}
@@ -326,8 +338,8 @@ void op_zlink (mval *v, mval *quals)
 		{	/* Object file found via zro_search() */
 			assert(ZRO_TYPE_OBJLIB != objdir->type);
 			if ((objdir->str.len + objnamelen + 2) > (SIZEOF(objnamebuf) - 1))
-				rts_error_csa(CSA_ARG(NULL) VARLSTCNT(8) ERR_ZLINKFILE, 2, v->str.len, v->str.addr, ERR_TEXT, 2,
-					      RTS_ERROR_LITERAL("Filename/path exceeds max length"));
+				RTS_ERROR_CSA_ABT(NULL, VARLSTCNT(8) ERR_ZLINKFILE, 2, v->str.len, v->str.addr, ERR_TEXT, 2,
+					RTS_ERROR_LITERAL("Filename/path exceeds max length"));
 			if (objdir->str.len)
 			{
 				tslash = ('/' == objdir->str.addr[objdir->str.len - 1]) ? 0 : 1;
@@ -346,13 +358,13 @@ void op_zlink (mval *v, mval *quals)
 		OPEN_OBJECT_FILE(objnamebuf, O_RDONLY, object_file_des);
 		if (FD_INVALID == object_file_des)
 		{	/* Could not find object file */
-			rts_error_csa(CSA_ARG(NULL) VARLSTCNT(5) ERR_ZLINKFILE, 2, objnamelen, objnamebuf, errno);
+			RTS_ERROR_CSA_ABT(NULL, VARLSTCNT(5) ERR_ZLINKFILE, 2, objnamelen, objnamebuf, errno);
 		}
 		/* Note - if explicit ZLINK, objdir can be NULL if link is from a directory not mentioned in $ZROUTINES */
 		CHECK_OBJECT_HISTORY(objnamebuf, objdir, RECENT_ZHIST);
 		if (IL_RECOMPILE == INCR_LINK(&object_file_des, objdir, RECENT_ZHIST, objnamelen, objnamebuf))
 		{
-			rts_error_csa(CSA_ARG(NULL) VARLSTCNT(5) ERR_ZLINKFILE, 2, v->str.len, v->str.addr, ERR_VERSION);
+			RTS_ERROR_CSA_ABT(NULL, VARLSTCNT(5) ERR_ZLINKFILE, 2, v->str.len, v->str.addr, ERR_VERSION);
 		}
 		CLOSE_OBJECT_FD(object_file_des, status);
 	} else
@@ -361,8 +373,8 @@ void op_zlink (mval *v, mval *quals)
 		{	/* A source directory containing routine was found by zro_search() */
 			assert(ZRO_TYPE_OBJLIB != objdir->type);
 			if ((srcdir->str.len + srcnamelen) > (SIZEOF(srcnamebuf) - 1))
-				rts_error_csa(CSA_ARG(NULL) VARLSTCNT(8) ERR_ZLINKFILE, 2, v->str.len, v->str.addr, ERR_TEXT, 2,
-					      RTS_ERROR_LITERAL("Filename/path exceeds max length"));
+				RTS_ERROR_CSA_ABT(NULL, VARLSTCNT(8) ERR_ZLINKFILE, 2, v->str.len, v->str.addr, ERR_TEXT, 2,
+					RTS_ERROR_LITERAL("Filename/path exceeds max length"));
 			if (srcdir->str.len)
 			{
 				tslash = ('/' == srcdir->str.addr[srcdir->str.len - 1]) ? 0 : 1;
@@ -390,8 +402,8 @@ void op_zlink (mval *v, mval *quals)
 				return;
 			}
 			if ((objdir->str.len + objnamelen) > (SIZEOF(objnamebuf) - 1))
-				rts_error_csa(CSA_ARG(NULL) VARLSTCNT(8) ERR_ZLINKFILE, 2, v->str.len, v->str.addr, ERR_TEXT, 2,
-					      RTS_ERROR_LITERAL("Filename/path exceeds max length"));
+				RTS_ERROR_CSA_ABT(NULL, VARLSTCNT(8) ERR_ZLINKFILE, 2, v->str.len, v->str.addr, ERR_TEXT, 2,
+					RTS_ERROR_LITERAL("Filename/path exceeds max length"));
 			if (objdir->str.len)
 			{
 				tslash = ('/' == objdir->str.addr[objdir->str.len - 1]) ? 0 : 1;
@@ -419,8 +431,8 @@ void op_zlink (mval *v, mval *quals)
 				if (ENOENT != save_errno)
 				{
 					err_code = STRERROR(save_errno);
-					rts_error_csa(CSA_ARG(NULL) VARLSTCNT(8) ERR_ZLINKFILE, 2, objnamelen, objnamebuf,
-						      ERR_TEXT, 2, LEN_AND_STR(err_code));
+					RTS_ERROR_CSA_ABT(NULL, VARLSTCNT(8) ERR_ZLINKFILE, 2, objnamelen, objnamebuf,
+						ERR_TEXT, 2, LEN_AND_STR(err_code));
 				}
 			} else
 				obj_found = TRUE;
@@ -440,7 +452,7 @@ void op_zlink (mval *v, mval *quals)
 				if (FD_INVALID != object_file_des)	/* Chose object file if open, ignore error */
 					CLOSE_OBJECT_FILE(object_file_des, status);
 				err_code = STRERROR(save_errno);
-				rts_error_csa(CSA_ARG(NULL) VARLSTCNT(8) ERR_ZLINKFILE, 2, v->str.len, v->str.addr,
+				RTS_ERROR_CSA_ABT(NULL, VARLSTCNT(8) ERR_ZLINKFILE, 2, v->str.len, v->str.addr,
 					ERR_TEXT, 2, LEN_AND_STR(err_code));
 			}
 		} else
@@ -460,7 +472,7 @@ void op_zlink (mval *v, mval *quals)
 						if (FD_INVALID != object_file_des)	/* Close object file if open */
 							CLOSE_OBJECT_FILE(object_file_des, status);
 						err_code = STRERROR(save_errno);
-						rts_error_csa(CSA_ARG(NULL)VARLSTCNT(8) ERR_ZLINKFILE, 2, v->str.len, v->str.addr,
+						RTS_ERROR_CSA_ABT(NULL, VARLSTCNT(8) ERR_ZLINKFILE, 2, v->str.len, v->str.addr,
 							ERR_TEXT, 2, LEN_AND_STR(err_code));
 					}
 					if ((src_stat.st_mtime > obj_stat.st_mtime) || ((src_stat.st_mtime == obj_stat.st_mtime)
@@ -477,8 +489,8 @@ void op_zlink (mval *v, mval *quals)
 			} else if (!obj_found)
 			{
 				assert(FD_INVALID == object_file_des);	/* Make sure closed */
-				rts_error_csa(CSA_ARG(NULL) VARLSTCNT(8) ERR_ZLINKFILE, 2, v->str.len, v->str.addr,
-					      ERR_FILENOTFND, 2, objnamelen, objnamebuf);
+				RTS_ERROR_CSA_ABT(NULL, VARLSTCNT(8) ERR_ZLINKFILE, 2, v->str.len, v->str.addr,
+					ERR_FILENOTFND, 2, objnamelen, objnamebuf);
 			}
 		}
 		if (compile)
@@ -487,7 +499,7 @@ void op_zlink (mval *v, mval *quals)
 			if (!(qlf & CQ_OBJECT) && (SRC != type))
 			{
 				assert(FD_INVALID == object_file_des);	/* Make sure no object file open */
-				rts_error_csa(CSA_ARG(NULL) VARLSTCNT(5) ERR_ZLINKFILE, 2, v->str.len, v->str.addr, ERR_ZLNOOBJECT);
+				RTS_ERROR_CSA_ABT(NULL, VARLSTCNT(5) ERR_ZLINKFILE, 2, v->str.len, v->str.addr, ERR_ZLNOOBJECT);
 			}
 			COMBINE_OBJ_DIR_W_NAME();
 			zlcompile(srcnamelen, (uchar_ptr_t)srcnamebuf);
@@ -504,7 +516,7 @@ void op_zlink (mval *v, mval *quals)
 			{
 				save_errno = errno;
 				err_code = STRERROR(save_errno);
-				rts_error_csa(CSA_ARG(NULL) VARLSTCNT(10) ERR_ZLINKFILE, 2, v->str.len, v->str.addr,
+				RTS_ERROR_CSA_ABT(NULL, VARLSTCNT(10) ERR_ZLINKFILE, 2, v->str.len, v->str.addr,
 					ERR_ZLNOOBJECT, 0, ERR_TEXT, 2, LEN_AND_STR(err_code));
 			}
 		} else
@@ -521,7 +533,7 @@ void op_zlink (mval *v, mval *quals)
 		{	/* Failure due only to version mismatch, so recompile */
 			assertpro(!compile);
 			if (!src_found)
-				rts_error_csa(CSA_ARG(NULL) VARLSTCNT(9) ERR_ZLINKFILE, 2, objnamelen, objnamebuf, ERR_FILENOTFND,
+				RTS_ERROR_CSA_ABT(NULL, VARLSTCNT(9) ERR_ZLINKFILE, 2, objnamelen, objnamebuf, ERR_FILENOTFND,
 					2, srcnamelen, srcnamebuf, ERR_VERSION);
 			if (!compile)
 			{
