@@ -1,6 +1,6 @@
 /****************************************************************
  *								*
- * Copyright (c) 2001-2020 Fidelity National Information	*
+ * Copyright (c) 2001-2021 Fidelity National Information	*
  * Services, Inc. and/or its subsidiaries. All rights reserved.	*
  *								*
  *	This source code contains the intellectual property	*
@@ -131,12 +131,12 @@ struct gd_addr_struct *repl_inst_get_name(char *fn, unsigned int *fn_len, unsign
 		if (issue_rts_error == error_action)
 		{
 			if (SS_LOG2LONG == status)
-				rts_error_csa(CSA_ARG(NULL) VARLSTCNT(5) ERR_LOGTOOLONG, 3, log_nam.len, log_nam.addr,
-					      SIZEOF(temp_inst_fn) - 1);
+				RTS_ERROR_CSA_ABT(NULL, VARLSTCNT(5) ERR_LOGTOOLONG, 3, log_nam.len, log_nam.addr,
+					SIZEOF(temp_inst_fn) - 1);
 			else if (!inst_from_gld)
-				rts_error_csa(CSA_ARG(NULL) VARLSTCNT(1) ERR_REPLINSTUNDEF);
+				RTS_ERROR_CSA_ABT(NULL, VARLSTCNT(1) ERR_REPLINSTUNDEF);
 			else
-				rts_error_csa(CSA_ARG(NULL) VARLSTCNT(8) ERR_REPLINSTACC, 2, log_nam.len, log_nam.addr,
+				RTS_ERROR_CSA_ABT(NULL, VARLSTCNT(8) ERR_REPLINSTACC, 2, log_nam.len, log_nam.addr,
 					ERR_TEXT, 2, RTS_ERROR_LITERAL("from global directory"));
 		} else if (issue_gtm_putmsg == error_action)
 		{
@@ -146,7 +146,7 @@ struct gd_addr_struct *repl_inst_get_name(char *fn, unsigned int *fn_len, unsign
 			else if (!inst_from_gld)
 				gtm_putmsg_csa(CSA_ARG(NULL) VARLSTCNT(1) ERR_REPLINSTUNDEF);
 			else
-				rts_error_csa(CSA_ARG(NULL) VARLSTCNT(8) ERR_REPLINSTACC, 2, log_nam.len, log_nam.addr,
+				RTS_ERROR_CSA_ABT(NULL, VARLSTCNT(8) ERR_REPLINSTACC, 2, log_nam.len, log_nam.addr,
 					ERR_TEXT, 2, RTS_ERROR_LITERAL("from global directory"));
 		}
 	} else if ('\0' == repl_instfilename[0])
@@ -234,8 +234,8 @@ void	repl_inst_read(char *fn, off_t offset, sm_uc_ptr_t buff, size_t buflen)
 			{	/* Have read the entire label in the instance file header. Check if it is the right version */
 				if (memcmp(buff, GDS_REPL_INST_LABEL, GDS_REPL_INST_LABEL_SZ - 1))
 				{
-					rts_error_csa(CSA_ARG(NULL) VARLSTCNT(8) ERR_REPLINSTFMT, 6, LEN_AND_STR(fn),
-					      GDS_REPL_INST_LABEL_SZ - 1, GDS_REPL_INST_LABEL, GDS_REPL_INST_LABEL_SZ - 1, buff);
+					RTS_ERROR_CSA_ABT(NULL, VARLSTCNT(8) ERR_REPLINSTFMT, 6, LEN_AND_STR(fn),
+						GDS_REPL_INST_LABEL_SZ - 1, GDS_REPL_INST_LABEL, GDS_REPL_INST_LABEL_SZ - 1, buff);
 				}
 			}
 			if (0 == status)
@@ -245,14 +245,14 @@ void	repl_inst_read(char *fn, off_t offset, sm_uc_ptr_t buff, size_t buflen)
 				/* Check endianness match */
 				if (GTM_IS_LITTLE_ENDIAN != replhdr->is_little_endian)
 				{
-					rts_error_csa(CSA_ARG(NULL) VARLSTCNT(8) ERR_REPLINSTFMT, 6, LEN_AND_STR(fn),
-						      LEN_AND_LIT(ENDIANTHIS), LEN_AND_LIT(ENDIANOTHER));
+					RTS_ERROR_CSA_ABT(NULL, VARLSTCNT(8) ERR_REPLINSTFMT, 6, LEN_AND_STR(fn),
+						LEN_AND_LIT(ENDIANTHIS), LEN_AND_LIT(ENDIANOTHER));
 				}
 				/* Check 64bitness match */
 				if (GTM_IS_64BIT != replhdr->is_64bit)
 				{
-					rts_error_csa(CSA_ARG(NULL) VARLSTCNT(8) ERR_REPLINSTFMT, 6, LEN_AND_STR(fn),
-						      LEN_AND_LIT(GTM_BITNESS_THIS), LEN_AND_LIT(GTM_BITNESS_OTHER));
+					RTS_ERROR_CSA_ABT(NULL, VARLSTCNT(8) ERR_REPLINSTFMT, 6, LEN_AND_STR(fn),
+						LEN_AND_LIT(GTM_BITNESS_THIS), LEN_AND_LIT(GTM_BITNESS_OTHER));
 				}
 				/* At the time of this writing, the only minor version supported is 1.
 				 * Whenever this gets updated, we need to add code to do the online upgrade.
@@ -269,15 +269,15 @@ void	repl_inst_read(char *fn, off_t offset, sm_uc_ptr_t buff, size_t buflen)
 	if (0 != status)
 	{
 		if (-1 == status)
-			rts_error_csa(CSA_ARG(NULL) VARLSTCNT(6) ERR_REPLINSTREAD, 4, buflen, (qw_off_t *)&offset, LEN_AND_STR(fn));
+			RTS_ERROR_CSA_ABT(NULL, VARLSTCNT(6) ERR_REPLINSTREAD, 4, buflen, (qw_off_t *)&offset, LEN_AND_STR(fn));
 		else
-			rts_error_csa(CSA_ARG(NULL) VARLSTCNT(7) ERR_REPLINSTREAD, 4, buflen, (qw_off_t *)&offset, LEN_AND_STR(fn),
-				      status);
+			RTS_ERROR_CSA_ABT(NULL, VARLSTCNT(7) ERR_REPLINSTREAD, 4, buflen, (qw_off_t *)&offset, LEN_AND_STR(fn),
+				status);
 	}
 	CLOSEFILE_RESET(fd, status);	/* resets "fd" to FD_INVALID */
 	assert(0 == status);
 	if (0 != status)
-		rts_error_csa(CSA_ARG(NULL) VARLSTCNT(5) ERR_REPLINSTCLOSE, 2, LEN_AND_STR(fn), status);
+		RTS_ERROR_CSA_ABT(NULL, VARLSTCNT(5) ERR_REPLINSTCLOSE, 2, LEN_AND_STR(fn), status);
 }
 
 /* Description:
@@ -334,9 +334,9 @@ void	repl_inst_write(char *fn, off_t offset, sm_uc_ptr_t buff, size_t buflen)
 	if (FD_INVALID == fd)
 	{
 		if (!in_repl_inst_create)
-			rts_error_csa(CSA_ARG(NULL) VARLSTCNT(5) ERR_REPLINSTOPEN, 2, LEN_AND_STR(fn), errno);
+			RTS_ERROR_CSA_ABT(NULL, VARLSTCNT(5) ERR_REPLINSTOPEN, 2, LEN_AND_STR(fn), errno);
 		else
-			rts_error_csa(CSA_ARG(NULL) VARLSTCNT(5) ERR_REPLINSTCREATE, 2, LEN_AND_STR(fn), errno);
+			RTS_ERROR_CSA_ABT(NULL, VARLSTCNT(5) ERR_REPLINSTCREATE, 2, LEN_AND_STR(fn), errno);
 	}
 #ifdef __MVS__
 	if (-1 == (in_repl_inst_create ? gtm_zos_set_tag(fd, TAG_BINARY, TAG_NOTTEXT, TAG_FORCE, &realfiletag) :
@@ -347,12 +347,12 @@ void	repl_inst_write(char *fn, off_t offset, sm_uc_ptr_t buff, size_t buflen)
 	REPL_INST_LSEEKWRITE(fd, offset, buff, buflen, status);
 	assert(0 == status);
 	if (0 != status)
-		rts_error_csa(CSA_ARG(NULL) VARLSTCNT(7) ERR_REPLINSTWRITE, 4, buflen, (qw_off_t *)&offset, LEN_AND_STR(fn),
-			      status);
+		RTS_ERROR_CSA_ABT(NULL, VARLSTCNT(7) ERR_REPLINSTWRITE, 4, buflen, (qw_off_t *)&offset, LEN_AND_STR(fn),
+			status);
 	CLOSEFILE_RESET(fd, status);	/* resets "fd" to FD_INVALID */
 	assert(0 == status);
 	if (0 != status)
-		rts_error_csa(CSA_ARG(NULL) VARLSTCNT(5) ERR_REPLINSTCLOSE, 2, LEN_AND_STR(fn), status);
+		RTS_ERROR_CSA_ABT(NULL, VARLSTCNT(5) ERR_REPLINSTCLOSE, 2, LEN_AND_STR(fn), status);
 }
 
 /* Description:
@@ -389,15 +389,15 @@ void	repl_inst_sync(char *fn)
 	oflag = O_RDWR;
 	OPENFILE3_CLOEXEC(fn, oflag, 0666, fd);
 	if (FD_INVALID == fd)
-		rts_error_csa(CSA_ARG(NULL) VARLSTCNT(5) ERR_REPLINSTOPEN, 2, LEN_AND_STR(fn), errno);
+		RTS_ERROR_CSA_ABT(NULL, VARLSTCNT(5) ERR_REPLINSTOPEN, 2, LEN_AND_STR(fn), errno);
 	GTM_REPL_INST_FSYNC(fd, status);
 	assert(0 == status);
 	if (0 != status)
-		rts_error_csa(CSA_ARG(NULL) VARLSTCNT(8) ERR_SYSCALL, 5, RTS_ERROR_LITERAL("fsync()"), CALLFROM, errno);
+		RTS_ERROR_CSA_ABT(NULL, VARLSTCNT(8) ERR_SYSCALL, 5, RTS_ERROR_LITERAL("fsync()"), CALLFROM, errno);
 	CLOSEFILE_RESET(fd, status);	/* resets "fd" to FD_INVALID */
 	assert(0 == status);
 	if (0 != status)
-		rts_error_csa(CSA_ARG(NULL) VARLSTCNT(5) ERR_REPLINSTCLOSE, 2, LEN_AND_STR(fn), status);
+		RTS_ERROR_CSA_ABT(NULL, VARLSTCNT(5) ERR_REPLINSTCLOSE, 2, LEN_AND_STR(fn), status);
 }
 
 /* Description:
@@ -492,7 +492,7 @@ void	repl_inst_ftok_sem_lock(void)
 		if (!ftok_sem_lock(reg, FALSE))
 		{
 			assert(FALSE);
-			rts_error_csa(CSA_ARG(NULL) VARLSTCNT(4) ERR_REPLFTOKSEM, 2, LEN_AND_STR(udi->fn));
+			RTS_ERROR_CSA_ABT(NULL, VARLSTCNT(4) ERR_REPLFTOKSEM, 2, LEN_AND_STR(udi->fn));
 		}
 	}
 	assert(udi->grabbed_ftok_sem);
@@ -520,7 +520,7 @@ void	repl_inst_ftok_sem_release(void)
 		if (!ftok_sem_release(reg, FALSE, FALSE))
 		{
 			assert(FALSE);
-			rts_error_csa(CSA_ARG(NULL) VARLSTCNT(4) ERR_REPLFTOKSEM, 2, LEN_AND_STR(udi->fn));
+			RTS_ERROR_CSA_ABT(NULL, VARLSTCNT(4) ERR_REPLFTOKSEM, 2, LEN_AND_STR(udi->fn));
 		}
 	}
 	assert(!udi->grabbed_ftok_sem);
@@ -719,8 +719,8 @@ void	repl_inst_histinfo_add(repl_histinfo *histinfo)
 		assert(jnlpool->jnlpool_ctl->last_histinfo_seqno == last_histinfo->start_seqno);
 		if (histinfo->start_seqno < last_histinfo->start_seqno)
 		{	/* cannot create histinfo with out-of-order start_seqno */
-			rts_error_csa(CSA_ARG(NULL) VARLSTCNT(8) ERR_REPLINSTSEQORD, 6, LEN_AND_LIT("New history record"),
-				      &histinfo->start_seqno, &last_histinfo->start_seqno, LEN_AND_STR(udi->fn));
+			RTS_ERROR_CSA_ABT(NULL, VARLSTCNT(8) ERR_REPLINSTSEQORD, 6, LEN_AND_LIT("New history record"),
+				&histinfo->start_seqno, &last_histinfo->start_seqno, LEN_AND_STR(udi->fn));
 		}
 	}
 	strm_histinfo_num = jnlpool->repl_inst_filehdr->last_histinfo_num[strm_idx];
@@ -1319,7 +1319,7 @@ int4	repl_inst_reset_zqgblmod_seqno_and_tn(void)
 	/* We use the same code dse uses to open all regions but we must make sure they are all open before proceeding. */
 	all_files_open = region_init(FALSE);
 	if (!all_files_open)
-		rts_error_csa(CSA_ARG(NULL) VARLSTCNT(1) ERR_NOTALLDBOPN);
+		RTS_ERROR_CSA_ABT(NULL, VARLSTCNT(1) ERR_NOTALLDBOPN);
 	repl_csa = &FILE_INFO(jnlpool->jnlpool_dummy_reg)->s_addrs;
 	for (reg = gd_header->regions, reg_top = reg + gd_header->n_regions;  reg < reg_top;  reg++)
 	{
