@@ -1,6 +1,6 @@
 /****************************************************************
  *								*
- * Copyright (c) 2009-2020 Fidelity National Information	*
+ * Copyright (c) 2009-2022 Fidelity National Information	*
  * Services, Inc. and/or its subsidiaries. All rights reserved.	*
  *								*
  *	This source code contains the intellectual property	*
@@ -29,6 +29,7 @@
 #include "gdsbt.h"
 #include "gdsblk.h"
 #include "gdsfhead.h"
+#include "db_header_conversion.h"
 #include "filestruct.h"
 #include "iosp.h"
 #include "error.h"
@@ -61,6 +62,7 @@
 #include "have_crit.h"
 #include "ss_lock_facility.h"
 #include "gtmimagename.h"
+#include "jnl_file_close_timer.h"
 
 GBLREF	boolean_t		muint_fast, ointeg_this_reg, online_specified;
 GBLREF	gd_region		*gv_cur_region;
@@ -148,7 +150,7 @@ ZOS_ONLY(error_def(ERR_TEXT);)
 					LEN_AND_LIT("Error with shmctl"), CALLFROM, RC);		\
 		}											\
 	}												\
-	SS_SHMID = shmget(IPC_PRIVATE, SS_SHMSIZE, RWDALL | IPC_CREAT);					\
+	SS_SHMID = gtm_shmget(IPC_PRIVATE, SS_SHMSIZE, RWDALL | IPC_CREAT, FALSE);			\
 	if (-1 == SS_SHMID)										\
 	{												\
 		RC = errno;										\
@@ -685,6 +687,8 @@ boolean_t	ss_initiate(gd_region *reg,			/* Region in which snapshot has to be st
 	 * Write the database file header at an offset equal to the database shared memory size.
 	 * This is because if we want to preserve the snapshot then we would want to write the
 	 * shared memory information at the beginning of the file. */
+	if (0 == memcmp(util_ss_ptr->header->label, V6_GDS_LABEL, GDS_LABEL_SZ - 1))
+		db_header_dwnconv(util_ss_ptr->header);
 	LSEEKWRITE(shdw_fd, (off_t)ss_shmsize, (sm_uc_ptr_t)util_ss_ptr->header, SGMNT_HDR_LEN, pwrite_res);
 	if (0 != pwrite_res)
 	{
