@@ -1,6 +1,7 @@
 /****************************************************************
  *								*
- *	Copyright 2001, 2004 Sanchez Computer Associates, Inc.	*
+ * Copyright (c) 2001-2018 Fidelity National Information	*
+ * Services, Inc. and/or its subsidiaries. All rights reserved.	*
  *								*
  *	This source code contains the intellectual property	*
  *	of its copyright holder(s), and is made available	*
@@ -9,8 +10,8 @@
  *								*
  ****************************************************************/
 
-#ifndef __SUSPSIGS_HANDLER_H__
-#define __SUSPSIGS_HANDLER_H__
+#ifndef SUSPSIGS_HANDLER_H_INCLUDED
+#define SUSPSIGS_HANDLER_H_INCLUDED
 
 void suspsigs_handler(int sig, siginfo_t *info, void *context);
 
