@@ -1,6 +1,6 @@
 /****************************************************************
  *								*
- * Copyright (c) 2010-2020 Fidelity National Information	*
+ * Copyright (c) 2010-2022 Fidelity National Information	*
  * Services, Inc. and/or its subsidiaries. All rights reserved.	*
  *								*
  *	This source code contains the intellectual property	*
@@ -242,14 +242,15 @@ STATICFNDEF void write_out_trigger(char *gbl_name, uint4 gbl_name_len, int nam_i
 	{
 		mv_trig_cnt_ptr = &trigger_count;
 		count = MV_FORCE_INT(mv_trig_cnt_ptr);
+		assert(0 < count);
 		BUILD_HASHT_SUB_SUB_CURRKEY(gbl_name, gbl_name_len, LITERAL_HASHLABEL, STRLEN(LITERAL_HASHLABEL));
 		if (!gvcst_get(&trigger_value))
 		{	/* There has to be a #LABEL */
 			if (UPDATE_CAN_RETRY(t_tries, t_fail_hist[t_tries]))
 				t_retry(cdb_sc_triggermod);
 			assert(WBTEST_HELPOUT_TRIGDEFBAD == gtm_white_box_test_case_number);
-			rts_error_csa(CSA_ARG(REG2CSA(gv_cur_region)) VARLSTCNT(8) ERR_TRIGDEFBAD, 6, gbl_name_len,
-					gbl_name, gbl_name_len, gbl_name, LEN_AND_LIT("\"#LABEL\""));
+			RTS_ERROR_CSA_ABT(REG2CSA(gv_cur_region), VARLSTCNT(8) ERR_TRIGDEFBAD, 6, gbl_name_len,
+				gbl_name, gbl_name_len, gbl_name, LEN_AND_LIT("\"#LABEL\""));
 		}
 		skip_chars = 1;
 		if ((trigger_value.str.len != STRLEN(HASHT_GBL_CURLABEL))
@@ -265,8 +266,8 @@ STATICFNDEF void write_out_trigger(char *gbl_name, uint4 gbl_name_len, int nam_i
 			if (UPDATE_CAN_RETRY(t_tries, t_fail_hist[t_tries]))
 				t_retry(cdb_sc_triggermod);
 			assert(WBTEST_HELPOUT_TRIGDEFBAD == gtm_white_box_test_case_number);
-			rts_error_csa(CSA_ARG(REG2CSA(gv_cur_region)) VARLSTCNT(8) ERR_TRIGDEFBAD, 6, gbl_name_len,
-					gbl_name, gbl_name_len, gbl_name, LEN_AND_LIT("\"#CYCLE\""));
+			RTS_ERROR_CSA_ABT(REG2CSA(gv_cur_region), VARLSTCNT(8) ERR_TRIGDEFBAD, 6, gbl_name_len,
+				gbl_name, gbl_name_len, gbl_name, LEN_AND_LIT("\"#CYCLE\""));
 		}
 		cycle_len = MIN(trigger_value.str.len, MAX_DIGITS_IN_INT);
 		memcpy(cycle, trigger_value.str.addr, cycle_len);
@@ -287,8 +288,8 @@ STATICFNDEF void write_out_trigger(char *gbl_name, uint4 gbl_name_len, int nam_i
 				if (UPDATE_CAN_RETRY(t_tries, t_fail_hist[t_tries]))
 					t_retry(cdb_sc_triggermod);
 				assert(WBTEST_HELPOUT_TRIGDEFBAD == gtm_white_box_test_case_number);
-				rts_error_csa(CSA_ARG(REG2CSA(gv_cur_region)) VARLSTCNT(8) ERR_TRIGDEFBAD, 6, gbl_name_len,
-						gbl_name, gbl_name_len, gbl_name, LEN_AND_LIT("\"#NAME\""));
+				RTS_ERROR_CSA_ABT(REG2CSA(gv_cur_region), VARLSTCNT(8) ERR_TRIGDEFBAD, 6, gbl_name_len,
+					gbl_name, gbl_name_len, gbl_name, LEN_AND_LIT("\"#NAME\""));
 
 			}
 			*out_rec_ptr++ = COMMENT_LITERAL;
@@ -479,9 +480,9 @@ STATICFNDEF void write_gbls_or_names(char *gbl_name, uint4 gbl_name_len, boolean
 				if (UPDATE_CAN_RETRY(t_tries, t_fail_hist[t_tries]))
 					t_retry(cdb_sc_triggermod);
 				assert(WBTEST_HELPOUT_TRIGDEFBAD == gtm_white_box_test_case_number);
-				rts_error_csa(CSA_ARG(REG2CSA(gv_cur_region)) VARLSTCNT(8) ERR_TRIGDEFBAD, 6,
-						LEN_AND_LIT("\"#TNAME\""), curr_name_len, curr_name,
-						mv_trigger_val.str.len, mv_trigger_val.str.addr);
+				RTS_ERROR_CSA_ABT(REG2CSA(gv_cur_region), VARLSTCNT(8) ERR_TRIGDEFBAD, 6,
+					LEN_AND_LIT("\"#TNAME\""), curr_name_len, curr_name,
+					mv_trigger_val.str.len, mv_trigger_val.str.addr);
 			}
 			ptr++;
 			A2I(ptr, mv_trigger_val.str.addr + mv_trigger_val.str.len, indx);
@@ -490,9 +491,9 @@ STATICFNDEF void write_gbls_or_names(char *gbl_name, uint4 gbl_name_len, boolean
 				if (UPDATE_CAN_RETRY(t_tries, t_fail_hist[t_tries]))
 					t_retry(cdb_sc_triggermod);
 				assert(WBTEST_HELPOUT_TRIGDEFBAD == gtm_white_box_test_case_number);
-				rts_error_csa(CSA_ARG(REG2CSA(gv_cur_region)) VARLSTCNT(8) ERR_TRIGDEFBAD, 6,
-						LEN_AND_LIT("\"#TNAME\""), curr_name_len, curr_name,
-						mv_trigger_val.str.len, mv_trigger_val.str.addr);
+				RTS_ERROR_CSA_ABT(REG2CSA(gv_cur_region), VARLSTCNT(8) ERR_TRIGDEFBAD, 6,
+					LEN_AND_LIT("\"#TNAME\""), curr_name_len, curr_name,
+					mv_trigger_val.str.len, mv_trigger_val.str.addr);
 			}
 			/* Use a local buffer to avoid possible garbage collection issues from write_out_trigger below */
 			memcpy(curr_gbl, mv_trigger_val.str.addr, trigvn_len);
@@ -550,7 +551,7 @@ STATICFNDEF void dump_all_triggers(void)
 		{
 			/* If ^#t needs to be upgraded, issue error. Cannot read older ^#t format */
 			if (cs_addrs->hdr->hasht_upgrade_needed)
-				rts_error_csa(CSA_ARG(cs_addrs) VARLSTCNT(4) ERR_NEEDTRIGUPGRD, 2, DB_LEN_STR(gv_cur_region));
+				RTS_ERROR_CSA_ABT(cs_addrs, VARLSTCNT(4) ERR_NEEDTRIGUPGRD, 2, DB_LEN_STR(gv_cur_region));
 			op_gvdata(&val);
 			if ((literal_ten.m[0] == val.m[0]) && (literal_ten.m[1] == val.m[1]))
 			{	/* $DATA(^#t) is 10 - get first subscript (trigger's global) */
@@ -661,7 +662,7 @@ boolean_t trigger_select_tpwrap(char *select_list, uint4 select_list_len, char *
 				|| !gv_target || !gv_target->root);
 			assert((cdb_sc_onln_rlbk2 != failure) || !IS_GTM_IMAGE || TREF(dollar_zonlnrlbk));
 			if (cdb_sc_onln_rlbk2 == failure)
-				rts_error_csa(CSA_ARG(NULL) VARLSTCNT(1) ERR_DBROLLEDBACK);
+				RTS_ERROR_CSA_ABT(NULL, VARLSTCNT(1) ERR_DBROLLEDBACK);
 			/* else if (cdb_sc_onln_rlbk1 == status) we don't need to do anything other than trying again. Since this
 			 * is ^#t global, we don't need to GVCST_ROOT_SEARCH before continuing with the next restart because the
 			 * trigger load logic already takes care of doing INITIAL_HASHT_ROOT_SEARCH_IF_NEEDED before doing the
@@ -768,12 +769,14 @@ STATICFNDEF boolean_t trigger_select(char *select_list, uint4 select_list_len)
 	}
 	if (dump_all)
 		dump_all_triggers();
-	else
+	else if (0 < select_list_len)
 	{
 		len = select_list_len;
 		sel_ptr = STRTOK_R(save_select_list, ",", &strtok_ptr);
 		do
 		{
+			if (NULL == sel_ptr)	/* Unlikely event that the first return is NULL */
+				break;
 			trig_name = ('^' != *sel_ptr);
 			ptr1 = sel_ptr;
 			len1 = STRLEN(ptr1);
