@@ -1,6 +1,6 @@
 /****************************************************************
  *								*
- * Copyright (c) 2010-2020 Fidelity National Information	*
+ * Copyright (c) 2010-2022 Fidelity National Information	*
  * Services, Inc. and/or its subsidiaries. All rights reserved.	*
  *								*
  *	This source code contains the intellectual property	*
@@ -150,7 +150,7 @@ static	boolean_t		promptanswer = TRUE;
 	memcpy(lcl_cmds, COMMANDS, STRLEN(COMMANDS) + 1);							\
 	BITMAP = 0;												\
 	lcl_ptr = STRTOK_R(lcl_cmds, ",", &strtok_ptr);								\
-	do													\
+	while (NULL != lcl_ptr)											\
 	{													\
 		switch (*lcl_ptr)										\
 		{												\
@@ -192,7 +192,8 @@ static	boolean_t		promptanswer = TRUE;
 				assertpro(FALSE && lcl_ptr[0]);							\
 				break;										\
 		}												\
-	} while (lcl_ptr = STRTOK_R(NULL, ",", &strtok_ptr));							\
+		lcl_ptr = STRTOK_R(NULL, ",", &strtok_ptr);							\
+	}													\
 }
 
 #define	COMMAND_BITMAP_TO_STR(COMMANDS, BITMAP, LEN)										\
@@ -224,39 +225,39 @@ static	boolean_t		promptanswer = TRUE;
 	memcpy(lcl_options, OPTIONS, STRLEN(OPTIONS) + 1);							\
 	BITMAP = 0;												\
 	lcl_ptr = STRTOK_R(lcl_options, ",", &strtok_ptr);							\
-	if (NULL != lcl_ptr)											\
-		do												\
+	while (NULL != lcl_ptr)											\
+	{													\
+		switch (*lcl_ptr)										\
 		{												\
-			switch (*lcl_ptr)									\
-			{											\
-				case 'C':									\
-					BITMAP |= OPTIONS_C;							\
-					break;									\
-				case 'I':									\
-					BITMAP |= OPTIONS_I;							\
-					break;									\
-				case 'N':									\
-					assert('O' == *(lcl_ptr + 1));						\
-					switch (*(lcl_ptr + 2))							\
-					{									\
-						case 'C':							\
-							BITMAP |= OPTIONS_NOC;					\
-							break;							\
-						case 'I':							\
-							BITMAP |= OPTIONS_NOI;					\
-							break;							\
-						default:							\
-							/* Parsing should have found invalid command */		\
-							assertpro(FALSE && lcl_ptr[2]);				\
-							break;							\
-					}									\
-					break;									\
-				default:									\
-					/* Parsing should have found invalid command */				\
-					assertpro(FALSE && lcl_ptr[0]);						\
-					break;									\
-			}											\
-		} while (lcl_ptr = STRTOK_R(NULL, ",", &strtok_ptr));						\
+			case 'C':										\
+				BITMAP |= OPTIONS_C;								\
+				break;										\
+			case 'I':										\
+				BITMAP |= OPTIONS_I;								\
+				break;										\
+			case 'N':										\
+				assert('O' == *(lcl_ptr + 1));							\
+				switch (*(lcl_ptr + 2))								\
+				{										\
+					case 'C':								\
+						BITMAP |= OPTIONS_NOC;						\
+						break;								\
+					case 'I':								\
+						BITMAP |= OPTIONS_NOI;						\
+						break;								\
+					default:								\
+						/* Parsing should have found invalid command */			\
+						assertpro(FALSE && lcl_ptr[2]);					\
+						break;								\
+				}										\
+				break;										\
+			default:										\
+				/* Parsing should have found invalid command */					\
+				assertpro(FALSE && lcl_ptr[0]);							\
+				break;										\
+		}												\
+		lcl_ptr = STRTOK_R(NULL, ",", &strtok_ptr);							\
+	}													\
 }
 
 #define	OPTION_BITMAP_TO_STR(OPTIONS, BITMAP, LEN)								\
@@ -498,7 +499,7 @@ boolean_t trigger_name_search(char *trigger_name, uint4 trigger_name_len, mval *
 			if (UPDATE_CAN_RETRY(t_tries, t_fail_hist[t_tries]))
 				t_retry(cdb_sc_triggermod);
 			assert(WBTEST_HELPOUT_TRIGDEFBAD == gtm_white_box_test_case_number);
-			rts_error_csa(CSA_ARG(NULL) VARLSTCNT(6) ERR_TRIGNAMBAD, 4, LEN_AND_LIT("\"#TNAME\""),
+			RTS_ERROR_CSA_ABT(NULL, VARLSTCNT(6) ERR_TRIGNAMBAD, 4, LEN_AND_LIT("\"#TNAME\""),
 				trigger_name_len, trigger_name);
 		}
 		len = ptr2 - ptr;
@@ -1555,7 +1556,7 @@ STATICFNDEF trig_stats_t trigupdrec_reg(char *trigvn, uint4 trigvn_len, boolean_
 
 	csa = cs_addrs;
 	if (NULL == csa)	/* Remote region */
-		rts_error_csa(CSA_ARG(csa) VARLSTCNT(6) ERR_REMOTEDBNOTRIG, 4, trigvn_len, trigvn, REG_LEN_STR(gv_cur_region));
+		RTS_ERROR_CSA_ABT(csa, VARLSTCNT(6) ERR_REMOTEDBNOTRIG, 4, trigvn_len, trigvn, REG_LEN_STR(gv_cur_region));
 	if (gv_cur_region->read_only)
 		rts_error_csa(CSA_ARG(csa) VARLSTCNT(4) ERR_TRIGMODREGNOTRW, 2, REG_LEN_STR(gv_cur_region));
 	assert(cs_addrs == gv_target->gd_csa);
@@ -1583,7 +1584,7 @@ STATICFNDEF trig_stats_t trigupdrec_reg(char *trigvn, uint4 trigvn_len, boolean_
 								 * Note: It is safe to do so even if we dont hold crit.
 								 */
 		} else
-			rts_error_csa(CSA_ARG(csa) VARLSTCNT(4) ERR_NEEDTRIGUPGRD, 2, DB_LEN_STR(gv_cur_region));
+			RTS_ERROR_CSA_ABT(csa, VARLSTCNT(4) ERR_NEEDTRIGUPGRD, 2, DB_LEN_STR(gv_cur_region));
 	}
 	if (!*new_name_check_done)
 	{	/* Make sure below call is done only ONCE for a global spanning multiple regions since this call goes
@@ -2089,7 +2090,7 @@ boolean_t trigger_update(mval *trigger_rec)
 				|| !gv_target || !gv_target->root);
 			assert((cdb_sc_onln_rlbk2 != failure) || !IS_GTM_IMAGE || TREF(dollar_zonlnrlbk));
 			if (cdb_sc_onln_rlbk2 == failure)
-				rts_error_csa(CSA_ARG(gv_target->gd_csa) VARLSTCNT(1) ERR_DBROLLEDBACK);
+				RTS_ERROR_CSA_ABT(gv_target->gd_csa, VARLSTCNT(1) ERR_DBROLLEDBACK);
 			/* else if (cdb_sc_onln_rlbk1 == status) we don't need to do anything other than trying again. Since this
 			 * is ^#t global, we don't need to GVCST_ROOT_SEARCH before continuing with the next restart because the
 			 * trigger load logic already takes care of doing INITIAL_HASHT_ROOT_SEARCH_IF_NEEDED before doing the
