@@ -1,6 +1,6 @@
  /****************************************************************
  *								*
- * Copyright (c) 2001-2020 Fidelity National Information	*
+ * Copyright (c) 2001-2022 Fidelity National Information	*
  * Services, Inc. and/or its subsidiaries. All rights reserved.	*
  *								*
  *	This source code contains the intellectual property	*
@@ -57,7 +57,8 @@ GBLDEF	char			facility[MAX_INSTNAME_LEN + 100];
 GBLDEF	boolean_t		util_out_print_vaparm_flush;
 #endif
 
-GBLREF	boolean_t		blocksig_initialized, err_same_as_out, hup_on, is_rcvr_server, is_src_server, is_updproc;
+GBLREF	boolean_t		blocksig_initialized, err_same_as_out, hup_on, is_rcvr_server, is_src_server, is_updproc,
+					prin_in_dev_failure;
 GBLREF	io_pair			io_std_device;
 GBLREF	gd_addr			*gd_header;
 GBLREF	jnlpool_addrs_ptr_t	jnlpool;
@@ -111,16 +112,16 @@ error_def(ERR_TEXT);
  *
  *		!mSB	!mSW	!mSL
  *
- *		!mUB	!mUW	!mUL    !m@UJ   !m@UQ
+ *		!mUB	!mUW	!mUL	!m@UJ	!m@UQ
  *
- *		!mXB	!mXW	!mXL    !mXJ    !m@XJ   !m@XQ
+ *		!mXB	!mXW	!mXL	!mXJ	!m@XJ	!m@XQ
  *
  *		!mZB	!mZW	!mZL
  *
  *		!n*c
  *
  *		!@ZJ	!@XJ		!@ZJ	!@ZQ
- #
+ *
  *	Where `m' is an optional field width, `n' is a repeat count, and `c' is a single character.
  *	`m' or `n' may be specified as the '#' character, in which case the value is taken from the next parameter.
  *
@@ -143,7 +144,7 @@ error_def(ERR_TEXT);
  *
  *	One important caveat in using !mXJ variant is that the input value is expected to be 4-byte on 32-bit platforms and 8-byte
  *	on 64-bit platforms. Passing an 8-byte quantity on a 32-bit platform can cause SIGSEGV. If a field is always 8-bytes on both
- * 	the 32 and 64 bit platforms (like transaction numbers), use 0x!16@XQ variant instead.
+ *	the 32 and 64 bit platforms (like transaction numbers), use 0x!16@XQ variant instead.
  *
  *	In addition, this routine also implements another set of directives
  *
@@ -257,7 +258,10 @@ caddr_t util_format(caddr_t message, va_list fao, caddr_t buff, ssize_t size, in
 		if (*message == '#')
 		{
 			if (0 < faocnt)
+			{
 				field_width = repeat_count = va_arg(fao, int4);
+				faocnt--;
+			}
 			++message;
 		} else
 		{
@@ -265,11 +269,20 @@ caddr_t util_format(caddr_t message, va_list fao, caddr_t buff, ssize_t size, in
 				;
 			if ((length = (int)(c - message)) > 0)
 			{
-				field_width = repeat_count
-					= asc2i((uchar_ptr_t)message, length);
+				/* Accept the width/repeat only if there are more directives for the length */
+				if (0 < faocnt)
+					field_width = repeat_count = asc2i((uchar_ptr_t)message, length);
+				else
+					field_width = repeat_count = 0;
 				message = c;
 			}
 		}
+		if ((outtop - outptr) < field_width)
+		{
+			/* Requested width exceeds buffer size. Should not happen. Fix it */
+			assert((outtop - outptr) > field_width);
+			field_width = (outtop - outptr);
+		}
 		if ('@' == *message)			/* Indirectly addressed operand */
 		{
 			indirect = TRUE;
@@ -318,7 +331,7 @@ caddr_t util_format(caddr_t message, va_list fao, caddr_t buff, ssize_t size, in
 				{
 					case 'C': /* a string with length in the first byte */
 						GETFAOVALDEF(faocnt, fao, caddr_t, c, NULL);
-						length = c ? *c++ : 0;
+						length = (int)(c ? (unsigned int)*c++ : 0); /* Cast ensures proper sign convert */
 						break;
 					case 'D':
 					case 'F': /* string with length and addr parameters. restrict to 1 MiB length */
@@ -327,12 +340,14 @@ caddr_t util_format(caddr_t message, va_list fao, caddr_t buff, ssize_t size, in
 						GETFAOVALDEF(faocnt, fao, caddr_t, c, NULL);
 						break;
 					case 'S':
-						if (faocnt)
+						if (0 < faocnt)
 						{
 							d = (desc_struct *)va_arg(fao, caddr_t);
 							faocnt--;
 							c = d->addr;
-							length = d->len;
+							/* Unsigned conversion simplifies the MAX_STRLEN check */
+							stringlength = (uint4)d->len;
+							length = (MAX_STRLEN > stringlength) ? (int)stringlength : MAX_STRLEN;
 						} else
 						{
 							c = NULL;
@@ -343,7 +358,14 @@ caddr_t util_format(caddr_t message, va_list fao, caddr_t buff, ssize_t size, in
 						GETFAOVALDEF(faocnt, fao, caddr_t, c, NULL);
 						stringlength = c ? strlen(c) : 0;
 						length = (MAX_STRLEN > stringlength) ? (int)stringlength : MAX_STRLEN;
+						break;
+					default:
+						assert(FALSE);
+						c = NULL;
+						length = 0;
 				}
+				/* Check the parameter and count. The pointer can't be NULL with a non-zero length */
+				assert((NULL != c) || (0 == length));
 				/* Since gtmsecshr does not load ICU libraries (since dlopen() with LD_LIBRARY_PATH
 				 * does not work for root setuid executables), avoid calling gtm_wcswidth() and
 				 * U_ISPRINT() from gtmsecshr and thus non-zero widths used in util_out_print()
@@ -471,6 +493,8 @@ caddr_t util_format(caddr_t message, va_list fao, caddr_t buff, ssize_t size, in
 								break;
 							default:
 								assert(FALSE);
+								GTM64_ONLY(addr_val = (UINTPTR_T)NULL);
+								NON_GTM64_ONLY(int_val = 0);
 						}
 					else
 					{
@@ -496,6 +520,7 @@ caddr_t util_format(caddr_t message, va_list fao, caddr_t buff, ssize_t size, in
 								NON_GTM64_ONLY(int_val = int_val & 0xFFFFFFFF;)
 								break;
 							default:
+								int_val = 0;
 								assert(FALSE);
 						}
 					}
@@ -550,6 +575,7 @@ caddr_t util_format(caddr_t message, va_list fao, caddr_t buff, ssize_t size, in
 						                       	break;
 								default:
 									assert(FALSE);
+									length = 0;
 							}
 							NON_GTM64_ONLY(i2hex(int_val, numptr, length);)
 							GTM64_ONLY(i2hex(('J' == type2) ? addr_val : int_val, numptr, length);)
@@ -594,7 +620,7 @@ caddr_t util_format(caddr_t message, va_list fao, caddr_t buff, ssize_t size, in
 					}
 				}
 				length = (int)(numptr - numa);		/* Length of asciified number */
-				if (length < field_width)
+				if ((0 < length) && (length < field_width))
 				{
 					memset(outptr, (('Z' == type) ? '0' : ' '), field_width - length);
 					outptr += field_width - length;
@@ -611,7 +637,7 @@ caddr_t util_format(caddr_t message, va_list fao, caddr_t buff, ssize_t size, in
 					)
 					NON_GTM64_ONLY(memset(outptr, '*', field_width);)
 					outptr += field_width;
-				} else
+				} else if (0 < length)
 				{
 					memcpy(outptr, numa, length);
 					outptr += length;
@@ -689,9 +715,6 @@ void	util_out_send_oper(char *addr, unsigned int len)
 			case LKE_IMAGE:
 				img_type = "LKE";
 				break;
-			case DBCERTIFY_IMAGE:
-				img_type = "DBCERTIFY";
-				break;
 			case GTCM_SERVER_IMAGE:
 				img_type = "GTCM";
 				break;
@@ -881,6 +904,7 @@ void	util_out_print_vaparm(caddr_t message, int flush, va_list var, int faocnt)
 }
 
 /* #GTM_THREAD_SAFE : The below function (util_out_print) is thread-safe because caller ensures serialization with locks */
+/* Because this function passes MAXPOSINT4 as the faocnt, only internal callers are allowed. */
 void	util_out_print(caddr_t message, int flush, ...)
 {
 	va_list	var;
@@ -889,7 +913,21 @@ void	util_out_print(caddr_t message, int flush, ...)
 	SETUP_THREADGBL_ACCESS;
 	ASSERT_SAFE_TO_UPDATE_THREAD_GBLS;
 	va_start(var, flush);
-	util_out_print_vaparm(message, flush, var, MAXPOSINT4);
+	util_out_print_vaparm(message, flush, var, MAXPOSINT4); /* WARNING: UNCHECKED faocnt - internal use only */
+	va_end(TREF(last_va_list_ptr));
+	va_end(var);
+}
+
+/* #GTM_THREAD_SAFE : The below function (util_out_print_args) is thread-safe because caller ensures serialization with locks */
+void	util_out_print_args(caddr_t message, int faocnt, int flush, ...)
+{
+	va_list	var;
+	DCL_THREADGBL_ACCESS;
+
+	SETUP_THREADGBL_ACCESS;
+	ASSERT_SAFE_TO_UPDATE_THREAD_GBLS;
+	va_start(var, flush);
+	util_out_print_vaparm(message, flush, var, faocnt);
 	va_end(TREF(last_va_list_ptr));
 	va_end(var);
 }
@@ -928,7 +966,7 @@ void util_cond_flush(void)
 
 	SETUP_THREADGBL_ACCESS;
 	ASSERT_SAFE_TO_UPDATE_THREAD_GBLS;
-	assert(((int)ERR_TERMHANGUP != SIGNAL) || ((tt == io_std_device.in->type) && hup_on));
+	assert(((int)ERR_TERMHANGUP != SIGNAL) || ((tt == io_std_device.in->type) && prin_in_dev_failure));
 	if ((TREF(util_outptr) != TREF(util_outbuff_ptr)) && ((int)ERR_TERMHANGUP != SIGNAL))	/* if a PRINCIPAL device terminal */
 		util_out_print(NULL, FLUSH);	/* disappeared, sending stuff to it with a PRN_ERROR just complicates things */
 }
