@@ -1,6 +1,6 @@
 /****************************************************************
  *								*
- * Copyright (c) 2016-2020 Fidelity National Information	*
+ * Copyright (c) 2016-2021 Fidelity National Information	*
  * Services, Inc. and/or its subsidiaries. All rights reserved.	*
  *								*
  *	This source code contains the intellectual property	*
@@ -68,7 +68,8 @@ enum dbg_wtfini_lcnt_t {
 	dbg_wtfini_wcs_recover = 32769,
 	dbg_wtfini_wcs_get_space1 = 32770,
 	dbg_wtfini_wcs_get_space2 = 32771,
-	dbg_wtfini_wcs_wtstart = 32772
+	dbg_wtfini_wcs_wtstart = 32772,
+	dbg_wtfini_wcs_get_space3 = 32773
 };
 
 GBLREF	enum dbg_wtfini_lcnt_t	dbg_wtfini_lcnt;	/* "lcnt" value for WCS_OPS_TRACE tracking purposes */
@@ -92,10 +93,10 @@ error_def(ERR_AIOQUEUESTUCK);
 error_def(ERR_DBFILERR);
 
 /* Wait for wip queue to be cleared or a specific CR to become non-dirty. RET is non-zero if "wcs_wtfini"
- * returns a non-zero value in any iteration. We wait a max of 1 minute if we do not see any progress in
+ * returns a non-zero value in any iteration. We wait a max of ~1 minute if we do not see any progress in
  * the WIP queue count. If there is evidence of progress, time waited till now does not count towards the 1
- * minute timeout i.e. the 1 minute wait is started afresh. If we hit a minute we send a message to the syslog
- * and keep trying.
+ * minute timeout i.e. the ~1 minute wait is started afresh. If we hit about a minute we send a message to the
+ * syslog and keep trying.
  */
 inline static int wait_for_wip_queue_to_clear(node_local_ptr_t cnl, cache_que_head_ptr_t crwipq, cache_rec_ptr_t cr, gd_region *reg)
 {
@@ -114,12 +115,13 @@ inline static int wait_for_wip_queue_to_clear(node_local_ptr_t cnl, cache_que_he
 		ret = wcs_wtfini(reg, CHECK_IS_PROC_ALIVE_TRUE_OR_FALSE(lcnt, MAX_WIP_QWAIT), cr);
 		if (ret || (cr && (0 == cr->dirty)))
 			break;
-		if (0 == (lcnt = (lcnt % (MAX_WIP_QWAIT AIX_ONLY(* 4)))))
-		{
+		if (lcnt >= (MAX_WIP_QWAIT AIX_ONLY(* 4)))
+		{	/* The number of 10 ms sleeps to reach 1 min. Issue error and restart the count */
 			send_msg_csa(CSA_ARG(REG2CSA(reg)) VARLSTCNT(7) ERR_DBFILERR, 2, DB_LEN_STR(reg),
-					ERR_AIOQUEUESTUCK, 2, (lcnt / SLEEP_ONE_MIN), (cr ? cr->blk : 0));
+					ERR_AIOQUEUESTUCK, 2, (MAX_WIP_QWAIT AIX_ONLY(* 4)), (cr ? cr->blk : 0));
+			lcnt = 1;	/* Can't sleep for zero seconds */
 		}
-		wcs_sleep(lcnt);
+		wcs_sleep(lcnt);	/* Iterations 10 and above sleep for 10 ms */
 		if ((wip_cnt != cnl->wcs_wip_lvl) && (NULL == cr)) /* only for non-specific crs */
 		{	/* Change in WIP queue size. Restart wait. Note that "CNL->wcs_wip_lvl" could
 			 * have even increased since we last noted it in "wip_cnt". This is because
