@@ -1,6 +1,6 @@
 /****************************************************************
  *								*
- * Copyright (c) 2001-2020 Fidelity National Information	*
+ * Copyright (c) 2001-2022 Fidelity National Information	*
  * Services, Inc. and/or its subsidiaries. All rights reserved.	*
  *								*
  *	This source code contains the intellectual property	*
@@ -44,9 +44,8 @@ STATICDEF uint4		save_qlf;
 
 error_def(ERR_COMPILEQUALS);
 error_def(ERR_FILEPARSE);
+error_def(ERR_NORTN);
 error_def(ERR_NOTMNAME);
-error_def(ERR_WILDCARD);
-error_def(ERR_ZLINKFILE);
 error_def(ERR_ZLNOOBJECT);
 
 void zl_cmd_qlf(mstr *quals, command_qualifier *qualif, char *srcstr, unsigned short *srclen, boolean_t last)
@@ -59,11 +58,13 @@ void zl_cmd_qlf(mstr *quals, command_qualifier *qualif, char *srcstr, unsigned s
 	parse_blk	pblk;
 	short		object_name_mvtype;
 	unsigned short	clen = 0;
+	int		iclen;		/* 4SCA on clen underflows */
 	DCL_THREADGBL_ACCESS;
 
 	SETUP_THREADGBL_ACCESS;
-	if (quals->len + SIZEOF(COMMAND) > MAX_LINE)
-		rts_error_csa(CSA_ARG(NULL) VARLSTCNT(4) ERR_COMPILEQUALS, 2, quals->len, quals->addr);
+	assert(0 <= quals->len);
+	if ((0 > quals->len) || ((MAX_LINE - SIZEOF(COMMAND)) < quals->len))
+		RTS_ERROR_CSA_ABT(NULL, VARLSTCNT(4) ERR_COMPILEQUALS, 2, (0 < quals->len) ? MAX_LINE : 0, quals->addr);
 	MEMCPY_LIT(cbuf, COMMAND);
 	memcpy(cbuf + SIZEOF(COMMAND) - 1, quals->addr, quals->len);
 	cbuf[SIZEOF(COMMAND) - 1 + quals->len] = 0;
@@ -103,17 +104,23 @@ void zl_cmd_qlf(mstr *quals, command_qualifier *qualif, char *srcstr, unsigned s
 		fstr.addr = srcstr;
 		fstr.len = *srclen;
 		status = parse_file(&fstr, &pblk);
-		if (!(status & 1))
+		if (!(status & 1) || !pblk.b_name)
 		{
+			assert(!TREF(trigger_compile_and_link));
 			if (save_qlf)
 				glb_cmd_qlf.qlf = save_qlf;
-			rts_error_csa(CSA_ARG(NULL) VARLSTCNT(5) ERR_FILEPARSE, 2, *srclen, srcstr, status);
+			RTS_ERROR_CSA_ABT(NULL, VARLSTCNT(5) ERR_FILEPARSE, 2, *srclen, srcstr,
+				!(status & 1) ? status : ERR_NORTN);
 		}
-		assert(pblk.b_name);
 		file.addr = pblk.l_name;
 		file.len = pblk.b_name;
-		if ((0 == pblk.b_ext) && ( MAX_FN_LEN >= (*srclen + SIZEOF(DOTM))))
+		if ((0 == pblk.b_ext) && (MAX_FN_LEN >= (*srclen + SIZEOF(DOTM))))
 		{
+			assert(NULL != pblk.l_name);
+			assert((MAX_FN_LEN >= pblk.b_name) && (MAX_FN_LEN >= pblk.buff_size));
+			assert((pblk.buffer < pblk.l_name) && ((pblk.buffer + pblk.buff_size) > pblk.l_name));
+			/* pblk.buff_size is MAX_FN_LEN, but pblk.buffer is allocated with an extra byte for the trailing null */
+			assert((pblk.buffer + pblk.buff_size + 1) >= (pblk.l_name + pblk.b_name + SIZEOF(DOTM)));
 			memcpy(&pblk.l_name[pblk.b_name], DOTM, SIZEOF(DOTM));
 			pblk.b_ext = (SIZEOF(DOTM) - 1);
 		}
@@ -147,17 +154,21 @@ void zl_cmd_qlf(mstr *quals, command_qualifier *qualif, char *srcstr, unsigned s
 		object_name_len = MAX_FN_LEN;
 		cli_get_str("OBJECT", (char *)object_file_name, &object_name_len);
 		cmd_qlf.object_file.mvtype = MV_STR;
+		assert((0 < object_name_len) && (MAX_FN_LEN >= object_name_len));
 		if (!(CQ_NAMEOFRTN & cmd_qlf.qlf))
 		{
-			for (ci = object_name_len - 1; ci && ('/' != object_file_name[ci]); ci--)
+			for (ci = object_name_len - 1; (0 < ci) && ('/' != object_file_name[ci]); ci--)
 				;       /* scan back from end for rtn name & triggerness */
-			ci += ci ? 1 : 0;
-			clen = object_name_len - ci;
-			if (('o' != object_file_name[ci + --clen])
-					|| ('.' != object_file_name[ci + --clen]))
-				clen = object_name_len - ci;
+			ci += (0 < ci) ? 1 : 0;
+			assert(object_name_len >= ci);
+			iclen = object_name_len - ci;
+			assert((iclen >= 0) && (iclen <= object_name_len));
+			if ((2 <= iclen) && ('o' == object_file_name[ci + iclen - 1])
+					&& ('.' == object_file_name[ci + iclen - 2]))
+				iclen -= 2;	/* Strip trailing ".o" */
 			SET_OBJ(object_file_name, object_name_len);
-			clen = object_name_len = MIN(clen, MAX_MIDENT_LEN);
+			assert(0 <= iclen);
+			clen = object_name_len = (unsigned short) MIN(iclen, MAX_MIDENT_LEN);
 			memcpy(routine_name.addr, &object_file_name[ci], clen);
 		}
 	}
@@ -166,7 +177,7 @@ void zl_cmd_qlf(mstr *quals, command_qualifier *qualif, char *srcstr, unsigned s
 	{       /* Routine name specified - name wins over object */
 		clen = MAX_MIDENT_LEN;
 		cli_get_str("NAMEOFRTN", routine_name.addr, &clen);
-		assert(MAX_MIDENT_LEN >= clen);
+		assert((0 < clen) && (MAX_MIDENT_LEN >= clen));
 		cmd_qlf.qlf &= ~CQ_NAMEOFRTN;   /* Can only be used for first module in list */
 		if (CLI_PRESENT != cli_present("OBJECT"))
 		{
@@ -185,7 +196,7 @@ void zl_cmd_qlf(mstr *quals, command_qualifier *qualif, char *srcstr, unsigned s
 		{
 			if (save_qlf)
 				glb_cmd_qlf.qlf = save_qlf;
-			rts_error_csa(CSA_ARG(NULL) VARLSTCNT(5) ERR_NOTMNAME, 2, RTS_ERROR_MSTR(&routine_name), ERR_ZLNOOBJECT);
+			RTS_ERROR_CSA_ABT(NULL, VARLSTCNT(5) ERR_NOTMNAME, 2, RTS_ERROR_MSTR(&routine_name), ERR_ZLNOOBJECT);
 		}
 		module_name.len = int_module_name.len = clen;
 		memcpy(int_module_name.addr, routine_name.addr, clen);
