@@ -60,6 +60,11 @@ int rc_prc_getr(rc_q_hdr *qhdr)
 	rc_rsp_page	*rsp;
 	srch_blk_status	*bh;
 	gvnh_reg_t	*gvnh_reg;
+
+	/*This code is probably not being maintained as part of the v7 change so put this assert here to see
+	 * if we are even hitting this code during testing*/
+	assert(FALSE);
+
 	DCL_THREADGBL_ACCESS;
 
 	SETUP_THREADGBL_ACCESS;
