@@ -1,6 +1,6 @@
 /****************************************************************
  *								*
- * Copyright (c) 2001-2020 Fidelity National Information	*
+ * Copyright (c) 2001-2022 Fidelity National Information	*
  * Services, Inc. and/or its subsidiaries. All rights reserved.	*
  *								*
  *	This source code contains the intellectual property	*
@@ -114,6 +114,7 @@ GBLREF enum gtmImageTypes	image_type;
 GBLREF IN_PARMS			*cli_lex_in_ptr;
 GBLREF char			cli_token_buf[];
 GBLREF CLI_ENTRY		gtcm_gnp_cmd_ary[];
+GBLREF void 			(*primary_exit_handler)(void);
 
 GBLDEF CLI_ENTRY		*cmd_ary = &gtcm_gnp_cmd_ary[0]; /* Define cmd_ary to be the GTCM_GNP server specific cmd table */
 GBLDEF bool			cm_timeout = FALSE;
@@ -185,7 +186,7 @@ static void gtcm_gnp_server_actions(void)
 		gtcm_remove_from_action_queue();
 		CMI_MUTEX_RESTORE(cmi_mutex_rc);
 		if ((connection_struct *)INTERLOCK_FAIL == curr_entry)
-			rts_error_csa(CSA_ARG(NULL) VARLSTCNT(1) CMERR_CMINTQUE);
+			RTS_ERROR_CSA_ABT(NULL, VARLSTCNT(1) CMERR_CMINTQUE);
 		if ((connection_struct *)EMPTY_QUEUE != curr_entry)
 		{
 			if (1 == (curr_entry->int_cancel.laflag & 1))
@@ -290,7 +291,7 @@ static void gtcm_gnp_server_actions(void)
 					if (SS_NORMAL == status)
 					{
 						GET_LONG(status, curr_entry->clb_ptr->mbf);
-                                                rts_error_csa(CSA_ARG(NULL) VARLSTCNT(3) ERR_BADGTMNETMSG, 1, status);
+						RTS_ERROR_CSA_ABT(NULL, VARLSTCNT(3) ERR_BADGTMNETMSG, 1, status);
 					}
 					break;
 			}
@@ -468,6 +469,7 @@ int main(int argc, char **argv, char **envp)
 		EXIT(status);
 	}
 	atexit(gtcm_exi_handler);
+	primary_exit_handler = gtcm_exi_handler;
 	initialize_pattern_table();
 	/* Pre-allocate some timer blocks. */
 	prealloc_gt_timers();
