@@ -1,6 +1,7 @@
 /****************************************************************
  *								*
- *	Copyright 2001, 2014 Fidelity Information Services, Inc	*
+ * Copyright (c) 2001-2021 Fidelity National Information	*
+ * Services, Inc. and/or its subsidiaries. All rights reserved.	*
  *								*
  *	This source code contains the intellectual property	*
  *	of its copyright holder(s), and is made available	*
@@ -36,6 +37,7 @@
 #include "gtm_string.h"
 #include "stringpool.h"
 #include "rtn_src_chksum.h"
+#include "have_crit.h"
 
 GBLREF boolean_t		run_time;
 GBLREF command_qualifier	cmd_qlf;
@@ -49,7 +51,9 @@ GBLREF char			cg_phase;	/* code generation phase */
 GBLREF char			cg_phase_last;	/* previous code generation phase */
 GBLREF int4			curr_addr, code_size;
 GBLREF char			object_file_name[];
+GBLREF int			object_file_des;
 
+error_def(ERR_SYSCALL);
 error_def(ERR_TEXT);
 
 void	cg_lab (mlabel *l, int4 base);
@@ -85,10 +89,12 @@ void	cg_lab (mlabel *l, int4 base);
 
 void	obj_code (uint4 src_lines, void *checksum_ctx)
 {
+	int		status;
 	rhdtyp		rhead;
 	mline		*mlx, *mly;
 	var_tabent	*vptr;
 	int4		lnr_pad_len;
+	intrpt_state_t	prev_intrpt_state;
 	DCL_THREADGBL_ACCESS;
 
 	SETUP_THREADGBL_ACCESS;
@@ -135,7 +141,9 @@ void	obj_code (uint4 src_lines, void *checksum_ctx)
 	code_size += src_lines * SIZEOF(int4);
 	lnr_pad_len = PADLEN(code_size, SECTION_ALIGN_BOUNDARY);
 	code_size += lnr_pad_len;
+	DEFER_INTERRUPTS(INTRPT_IN_OBJECT_FILE_COMPILE, prev_intrpt_state);
 	create_object_file(&rhead);
+	ENABLE_INTERRUPTS(INTRPT_IN_OBJECT_FILE_COMPILE, prev_intrpt_state);
 	cg_phase = CGP_MACHINE;
 	code_gen();
 	/* Variable table: */
@@ -171,13 +179,18 @@ void	obj_code (uint4 src_lines, void *checksum_ctx)
 	}
 	if (0 != lnr_pad_len) /* emit padding so literal text pool starts on proper boundary */
 		emit_immed(PADCHARS, lnr_pad_len);
-#if !defined(__MVS__) && !defined(__s390__)	/* assert not valid for instructions on OS390 */
+#	if !defined(__MVS__) && !defined(__s390__)	/* assert not valid for instructions on OS390 */
 	assert(code_size == psect_use_tab[GTM_CODE]);
-#endif
+#	endif
 	emit_literals();
-	close_object_file();
+	DEFER_INTERRUPTS(INTRPT_IN_OBJECT_FILE_COMPILE, prev_intrpt_state);
+	finish_object_file();
+	ENABLE_INTERRUPTS(INTRPT_IN_OBJECT_FILE_COMPILE, prev_intrpt_state);
+	CLOSE_OBJECT_FILE(object_file_des, status);
+	if (-1 == status)
+		RTS_ERROR_CSA_ABT(NULL, VARLSTCNT(8) ERR_SYSCALL, 5, RTS_ERROR_LITERAL("close()"), CALLFROM, errno);
 	/* Ready to make object visible. Rename from tmp name to real routine name */
-	rename_tmp_object_file(object_file_name);
+	RENAME_TMP_OBJECT_FILE(object_file_name);
 }
 
 void	cg_lab (mlabel *l, int4 base)
