@@ -1,6 +1,6 @@
 /****************************************************************
  *								*
- * Copyright (c) 2001-2020 Fidelity National Information 	*
+ * Copyright (c) 2001-2022 Fidelity National Information 	*
  * Services, Inc. and/or its subsidiaries. All rights reserved.	*
  *								*
  *	This source code contains the intellectual property	*
@@ -85,7 +85,7 @@ const static readonly int error_ansi[] = {
 	   0,	/* GVIS */
 	   0,	/* GVKILLFAIL */
 	   1,	/* GVNAKED */
-	   0,	/* UNUSEDMSG256 */
+	   0,	/* BACKUPDBFILE */
 	   0,	/* GVORDERFAIL */
 	   0,	/* GVPUTFAIL */
 	   0,	/* PATTABSYNTAX */
@@ -136,17 +136,17 @@ const static readonly int error_ansi[] = {
 	   0,	/* MEMORY */
 	   0,	/* DONOBLOCK */
 	   0,	/* ZATRANSCOL */
-	   0,	/* UNUSEDMSG307 */
-	   0,	/* UNUSEDMSG308 */
-	   0,	/* UNUSEDMSG309 */
-	   0,	/* UNUSEDMSG310 */
-	   0,	/* UNUSEDMSG311 */
-	   0,	/* UNUSEDMSG312 */
-	   0,	/* UNUSEDMSG313 */
-	   0,	/* UNUSEDMSG314 */
+	   0,	/* VIEWREGLIST */
+	   0,	/* NUMERR */
+	   0,	/* NUM64ERR */
+	   0,	/* UNUM64ERR */
+	   0,	/* HEXERR */
+	   0,	/* HEX64ERR */
+	   0,	/* CMDERR */
+	   0,	/* BACKUPSUCCESS */
 	   0,	/* JNLTMQUAL3 */
 	  57,	/* MULTLAB */
-	   0,	/* BLKCNT */
+	   0,	/* GTMCURUNSUPP */
 	   0,	/* CCEDUMPOFF */
 	   0,	/* NOPLACE */
 	   0,	/* JNLCLOSE */
@@ -182,7 +182,7 @@ const static readonly int error_ansi[] = {
 	   0,	/* SPOREOL */
 	   0,	/* SRCLIN */
 	   0,	/* SRCLOC */
-	   0,	/* SRCLOCUNKNOWN */
+	   0,	/* RLNKRECNFL */
 	   0,	/* STACKCRIT */
 	   0,	/* STACKOFLOW */
 	   0,	/* STACKUNDERFLO */
@@ -200,7 +200,7 @@ const static readonly int error_ansi[] = {
 	   6,	/* UNDEF */
 	   0,	/* UNIMPLOP */
 	  39,	/* VAREXPECTED */
-	   0,	/* VARRECBLKSZ */
+	   0,	/* BACKUPFAIL */
 	   0,	/* MAXARGCNT */
 	   0,	/* GTMSECSHRSEMGET */
 	   0,	/* VIEWARGCNT */
@@ -348,9 +348,9 @@ const static readonly int error_ansi[] = {
 	   0,	/* AMBISYIPARAM */
 	   0,	/* PREVJNLNOEOF */
 	   0,	/* LKSECINIT */
-	   0,	/* UNUSEDMSG519 */
-	   0,	/* UNUSEDMSG520 */
-	   0,	/* UNUSEDMSG521 */
+	   0,	/* BACKUPREPL */
+	   0,	/* BACKUPSEQNO */
+	   0,	/* DIRACCESS */
 	   0,	/* TXTSRCMAT */
 	   0,	/* CCENOGROUP */
 	   0,	/* BADDBVER */
@@ -373,8 +373,8 @@ const static readonly int error_ansi[] = {
 	   0,	/* NETFAIL */
 	   0,	/* NETLCKFAIL */
 	   0,	/* TTINVFILTER */
-	   0,	/* UNUSEDMSG544 */
-	   0,	/* UNUSEDMSG545 */
+	   0,	/* BACKUPTN */
+	   0,	/* WCSFLUFAIL */
 	   0,	/* BADTRNPARAM */
 	   0,	/* DSEONLYBGMM */
 	   0,	/* DSEINVLCLUSFN */
@@ -464,7 +464,7 @@ const static readonly int error_ansi[] = {
 	  72,	/* RMWIDTHPOS */
 	  13,	/* OFFSETINV */
 	   0,	/* JOBPARTOOLONG */
-	   0,	/* JOBARGMISSING */
+	   0,	/* RLNKINTEGINFO */
 	   0,	/* RUNPARAMERR */
 	   0,	/* FNNAMENEG */
 	   0,	/* ORDER2 */
@@ -591,7 +591,7 @@ const static readonly int error_ansi[] = {
 	   0,	/* DELIMSIZNA */
 	   0,	/* INVCTLMNE */
 	   0,	/* SOCKLISTEN */
-	   0,	/* UNUSEDMSG762 */
+	   0,	/* RESTORESUCCESS */
 	   0,	/* ADDRTOOLONG */
 	   0,	/* GTMSECSHRGETSEMFAIL */
 	   0,	/* CPBEYALLOC */
@@ -607,7 +607,7 @@ const static readonly int error_ansi[] = {
 	   0,	/* KRNLKILL */
 	   0,	/* MEMORYRECURSIVE */
 	   0,	/* FREEZEID */
-	   0,	/* BLKWRITERR */
+	   0,	/* UNUSEDMSG778 */
 	   0,	/* DSEINVALBLKID */
 	   0,	/* PINENTRYERR */
 	   0,	/* BCKUPBUFLUSH */
@@ -632,7 +632,7 @@ const static readonly int error_ansi[] = {
 	   0,	/* GBLMODFAIL */
 	   0,	/* TTLENGTHTOOBIG */
 	   0,	/* TPTIMEOUT */
-	   0,	/* DEFEREVENT */
+	   0,	/* NORTN */
 	   0,	/* JNLFILNOTCHG */
 	   0,	/* EVENTLOGERR */
 	   0,	/* UPDATEFILEOPEN */
@@ -727,7 +727,7 @@ const static readonly int error_ansi[] = {
 	   0,	/* GTMSECSHRSSIDF */
 	   0,	/* GTMSECSHRFORKF */
 	   0,	/* DBFSYNCERR */
-	   0,	/* SECONDAHEAD */
+	   0,	/* UNUSEDMSG898 */
 	   0,	/* SCNDDBNOUPD */
 	   0,	/* MUINFOUINT4 */
 	   0,	/* NLMISMATCHCALC */
@@ -782,7 +782,7 @@ const static readonly int error_ansi[] = {
 	   0,	/* DBMBPFRDLBM */
 	   0,	/* DBMBPFRINT */
 	   0,	/* DBMAXKEYEXC */
-	   0,	/* DBMXRSEXCMIN */
+	   0,	/* REPLAHEAD */
 	   0,	/* MUPIPSET2SML */
 	   0,	/* DBREADBM */
 	   0,	/* DBCOMPTOOLRG */
@@ -839,7 +839,7 @@ const static readonly int error_ansi[] = {
 	   0,	/* NOTALLDBOPN */
 	   0,	/* MUSELFBKUP */
 	   0,	/* DBDANGER */
-	   0,	/* TRUNCATEFAIL */
+	   0,	/* UNUSEDMSG1010 */
 	   0,	/* TCGETATTR */
 	   0,	/* TCSETATTR */
 	   0,	/* IOWRITERR */
@@ -880,7 +880,7 @@ const static readonly int error_ansi[] = {
 	   0,	/* FILTERBADCONV */
 	   0,	/* PRIMARYISROOT */
 	   0,	/* GVQUERYGETFAIL */
-	   0,	/* DBCREC2BIGINBLK */
+	   0,	/* UNUSEDMSG1051 */
 	  19,	/* MERGEDESC */
 	   0,	/* MERGEINCOMPL */
 	   0,	/* DBNAMEMISMATCH */
@@ -944,7 +944,7 @@ const static readonly int error_ansi[] = {
 	   0,	/* JNLALIGNSZCHG */
 	   0,	/* SEFCTNEEDSFULLB */
 	   0,	/* GVFAILCORE */
-	   0,	/* DBCDBNOCERTIFY */
+	   0,	/* UNUSEDMSG1115 */
 	   0,	/* DBFRZRESETSUC */
 	   0,	/* JNLFILEXTERR */
 	   0,	/* JOBEXAMDONE */
@@ -970,7 +970,7 @@ const static readonly int error_ansi[] = {
 	   0,	/* JNLSWITCHSZCHG */
 	   0,	/* NOTRNDMACC */
 	   0,	/* TMPFILENOCRE */
-	   0,	/* SHRMEMEXHAUSTED */
+	   0,	/* DEVICEOPTION */
 	   0,	/* JNLSENDOPER */
 	   0,	/* DDPSUBSNUL */
 	   0,	/* DDPNOCONNECT */
@@ -1041,18 +1041,18 @@ const static readonly int error_ansi[] = {
 	   0,	/* DBBTUWRNG */
 	   0,	/* DBBTUFIXED */
 	   0,	/* DBMAXREC2BIG */
-	   0,	/* DBCSCNNOTCMPLT */
-	   0,	/* DBCBADFILE */
-	   0,	/* DBCNOEXTND */
-	   0,	/* DBCINTEGERR */
+	   0,	/* UNUSEDMSG1212 */
+	   0,	/* UNUSEDMSG1213 */
+	   0,	/* UNUSEDMSG1214 */
+	   0,	/* UNUSEDMSG1215 */
 	   0,	/* DBMINRESBYTES */
-	   0,	/* DBCNOTSAMEDB */
-	   0,	/* DBCDBCERTIFIED */
-	   0,	/* DBCMODBLK2BIG */
-	   0,	/* DBCREC2BIG */
-	   0,	/* DBCCMDFAIL */
-	   0,	/* DBCKILLIP */
-	   0,	/* DBCNOFINISH */
+	   0,	/* UNUSEDMSG1217 */
+	   0,	/* UNUSEDMSG1218 */
+	   0,	/* UNUSEDMSG1219 */
+	   0,	/* UNUSEDMSG1220 */
+	   0,	/* UNUSEDMSG1221 */
+	   0,	/* UNUSEDMSG1222 */
+	   0,	/* UNUSEDMSG1223 */
 	   0,	/* DYNUPGRDFAIL */
 	   0,	/* MMNODYNDWNGRD */
 	   0,	/* MMNODYNUPGRD */
@@ -1071,8 +1071,8 @@ const static readonly int error_ansi[] = {
 	   0,	/* RSVDBYTE2HIGH */
 	   0,	/* BKUPTMPFILOPEN */
 	   0,	/* BKUPTMPFILWRITE */
-	   0,	/* VMSMEMORY2 */
-	   0,	/* UNUSEDMSG1243 */
+	   0,	/* SHMHUGETLB */
+	   0,	/* SHMLOCK */
 	   0,	/* UNUSEDMSG1244 */
 	   0,	/* REPLINSTMISMTCH */
 	   0,	/* REPLINSTREAD */
@@ -1352,14 +1352,14 @@ const static readonly int error_ansi[] = {
 	   0,	/* NULLENTRYREF */
 	   0,	/* ZPEEKNORPLINFO */
 	   0,	/* MMREGNOACCESS */
-	   0,	/* MALLOCMAXUNIX */
-	   0,	/* MALLOCMAXVMS */
+	   0,	/* UNUSEDMSG1525 */
+	   0,	/* MALLOCCRIT */
 	   0,	/* HOSTCONFLICT */
 	   0,	/* GETADDRINFO */
 	   0,	/* GETNAMEINFO */
 	   0,	/* SOCKBIND */
 	   0,	/* INSTFRZDEFER */
-	   0,	/* UNUSEDMSG1532 */
+	   0,	/* VIEWARGTOOLONG */
 	   0,	/* REGOPENFAIL */
 	   0,	/* REPLINSTNOSHM */
 	   0,	/* DEVPARMTOOSMALL */
@@ -1545,4 +1545,20 @@ const static readonly int error_ansi[] = {
 	   0,	/* BADCONNECTPARAM */
 	   0,	/* BADPARAMCOUNT */
 	   0,	/* REPLALERT */
+	   0,	/* SHUT2QUICK */
+	   0,	/* REPLNORESP */
+	   0,	/* REPL0BACKLOG */
+	   0,	/* REPLBACKLOG */
+	   0,	/* INVSHUTDOWN */
+	   0,	/* SOCKBLOCKERR */
+	   0,	/* SOCKWAITARG */
+	   0,	/* LASTTRANS */
+	   0,	/* SRCBACKLOGSTATUS */
+	   0,	/* BKUPRETRY */
+	   0,	/* BKUPPROGRESS */
+	   0,	/* BKUPFILEPERM */
+	   0,	/* AUDINITFAIL */
+	   0,	/* AUDCONNFAIL */
+	   0,	/* AUDLOGFAIL */
+	  80,	/* SOCKCLOSE */
 	};
